//
// Copyright (c) 2003, 2010, Oracle and/or its affiliates. All rights reserved.
// DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
//
// This code is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 2 only, as
// published by the Free Software Foundation.
//
// This code is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// version 2 for more details (a copy is included in the LICENSE file that
// accompanied this code).
//
// You should have received a copy of the GNU General Public License version
// 2 along with this work; if not, write to the Free Software Foundation,
// Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
//
// Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
// or visit www.oracle.com if you need additional information or have any
// questions.
//
//

// Machine Generated File.  Do Not Edit!

#include "precompiled.hpp"
#include "adfiles/adGlobals_x86_64.hpp"
#include "adfiles/ad_x86_64.hpp"
#include "memory/allocation.inline.hpp"
#include "asm/assembler.hpp"
#include "code/vmreg.hpp"
#include "gc_interface/collectedHeap.inline.hpp"
#include "oops/compiledICHolderOop.hpp"
#include "oops/markOop.hpp"
#include "oops/methodOop.hpp"
#include "oops/oop.inline.hpp"
#include "oops/oop.inline2.hpp"
#include "opto/cfgnode.hpp"
#include "opto/locknode.hpp"
#include "opto/opcodes.hpp"
#include "opto/regalloc.hpp"
#include "opto/regmask.hpp"
#include "opto/runtime.hpp"
#include "runtime/biasedLocking.hpp"
#include "runtime/sharedRuntime.hpp"
#include "runtime/stubRoutines.hpp"
#include "utilities/growableArray.hpp"
#include "assembler_x86.inline.hpp"
#include "nativeInst_x86.hpp"
#include "vmreg_x86.inline.hpp"

//SourceForm

#line 548 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

#define   RELOC_IMM64    Assembler::imm_operand
#define   RELOC_DISP32   Assembler::disp32_operand

#define __ _masm.

static int preserve_SP_size() {
  return LP64_ONLY(1 +) 2;  // [rex,] op, rm(reg/reg)
}

// !!!!! Special hack to get all types of calls to specify the byte offset
//       from the start of the call to the point where the return address
//       will point.
int MachCallStaticJavaNode::ret_addr_offset()
{
  int offset = 5; // 5 bytes from start of call to where return address points
  if (_method_handle_invoke)
    offset += preserve_SP_size();
  return offset;
}

int MachCallDynamicJavaNode::ret_addr_offset()
{
  return 15; // 15 bytes from start of call to where return address points
}

// In os_cpu .ad file
// int MachCallRuntimeNode::ret_addr_offset()

// Indicate if the safepoint node needs the polling page as an input.
// Since amd64 does not have absolute addressing but RIP-relative
// addressing and the polling page is within 2G, it doesn't.
bool SafePointNode::needs_polling_address_input()
{
  return false;
}

//
// Compute padding required for nodes which need alignment
//

// The address of the call instruction needs to be 4-byte aligned to
// ensure that it does not span a cache line so that it can be patched.
int CallStaticJavaDirectNode::compute_padding(int current_offset) const
{
  current_offset += 1; // skip call opcode byte
  return round_to(current_offset, alignment_required()) - current_offset;
}

// The address of the call instruction needs to be 4-byte aligned to
// ensure that it does not span a cache line so that it can be patched.
int CallStaticJavaHandleNode::compute_padding(int current_offset) const
{
  current_offset += preserve_SP_size();   // skip mov rbp, rsp
  current_offset += 1; // skip call opcode byte
  return round_to(current_offset, alignment_required()) - current_offset;
}

// The address of the call instruction needs to be 4-byte aligned to
// ensure that it does not span a cache line so that it can be patched.
int CallDynamicJavaDirectNode::compute_padding(int current_offset) const
{
  current_offset += 11; // skip movq instruction + call opcode byte
  return round_to(current_offset, alignment_required()) - current_offset;
}

#ifndef PRODUCT
void MachBreakpointNode::format(PhaseRegAlloc*, outputStream* st) const
{
  st->print("INT3");
}
#endif

// EMIT_RM()
void emit_rm(CodeBuffer &cbuf, int f1, int f2, int f3) {
  unsigned char c = (unsigned char) ((f1 << 6) | (f2 << 3) | f3);
  cbuf.insts()->emit_int8(c);
}

// EMIT_CC()
void emit_cc(CodeBuffer &cbuf, int f1, int f2) {
  unsigned char c = (unsigned char) (f1 | f2);
  cbuf.insts()->emit_int8(c);
}

// EMIT_OPCODE()
void emit_opcode(CodeBuffer &cbuf, int code) {
  cbuf.insts()->emit_int8((unsigned char) code);
}

// EMIT_OPCODE() w/ relocation information
void emit_opcode(CodeBuffer &cbuf,
                 int code, relocInfo::relocType reloc, int offset, int format)
{
  cbuf.relocate(cbuf.insts_mark() + offset, reloc, format);
  emit_opcode(cbuf, code);
}

// EMIT_D8()
void emit_d8(CodeBuffer &cbuf, int d8) {
  cbuf.insts()->emit_int8((unsigned char) d8);
}

// EMIT_D16()
void emit_d16(CodeBuffer &cbuf, int d16) {
  cbuf.insts()->emit_int16(d16);
}

// EMIT_D32()
void emit_d32(CodeBuffer &cbuf, int d32) {
  cbuf.insts()->emit_int32(d32);
}

// EMIT_D64()
void emit_d64(CodeBuffer &cbuf, int64_t d64) {
  cbuf.insts()->emit_int64(d64);
}

// emit 32 bit value and construct relocation entry from relocInfo::relocType
void emit_d32_reloc(CodeBuffer& cbuf,
                    int d32,
                    relocInfo::relocType reloc,
                    int format)
{
  assert(reloc != relocInfo::external_word_type, "use 2-arg emit_d32_reloc");
  cbuf.relocate(cbuf.insts_mark(), reloc, format);
  cbuf.insts()->emit_int32(d32);
}

// emit 32 bit value and construct relocation entry from RelocationHolder
void emit_d32_reloc(CodeBuffer& cbuf, int d32, RelocationHolder const& rspec, int format) {
#ifdef ASSERT
  if (rspec.reloc()->type() == relocInfo::oop_type &&
      d32 != 0 && d32 != (intptr_t) Universe::non_oop_word()) {
    assert(oop((intptr_t)d32)->is_oop() && (ScavengeRootsInCode || !oop((intptr_t)d32)->is_scavengable()), "cannot embed scavengable oops in code");
  }
#endif
  cbuf.relocate(cbuf.insts_mark(), rspec, format);
  cbuf.insts()->emit_int32(d32);
}

void emit_d32_reloc(CodeBuffer& cbuf, address addr) {
  address next_ip = cbuf.insts_end() + 4;
  emit_d32_reloc(cbuf, (int) (addr - next_ip),
                 external_word_Relocation::spec(addr),
                 RELOC_DISP32);
}


// emit 64 bit value and construct relocation entry from relocInfo::relocType
void emit_d64_reloc(CodeBuffer& cbuf, int64_t d64, relocInfo::relocType reloc, int format) {
  cbuf.relocate(cbuf.insts_mark(), reloc, format);
  cbuf.insts()->emit_int64(d64);
}

// emit 64 bit value and construct relocation entry from RelocationHolder
void emit_d64_reloc(CodeBuffer& cbuf, int64_t d64, RelocationHolder const& rspec, int format) {
#ifdef ASSERT
  if (rspec.reloc()->type() == relocInfo::oop_type &&
      d64 != 0 && d64 != (int64_t) Universe::non_oop_word()) {
    assert(oop(d64)->is_oop() && (ScavengeRootsInCode || !oop(d64)->is_scavengable()),
           "cannot embed scavengable oops in code");
  }
#endif
  cbuf.relocate(cbuf.insts_mark(), rspec, format);
  cbuf.insts()->emit_int64(d64);
}

// Access stack slot for load or store
void store_to_stackslot(CodeBuffer &cbuf, int opcode, int rm_field, int disp)
{
  emit_opcode(cbuf, opcode);                  // (e.g., FILD   [RSP+src])
  if (-0x80 <= disp && disp < 0x80) {
    emit_rm(cbuf, 0x01, rm_field, RSP_enc);   // R/M byte
    emit_rm(cbuf, 0x00, RSP_enc, RSP_enc);    // SIB byte
    emit_d8(cbuf, disp);     // Displacement  // R/M byte
  } else {
    emit_rm(cbuf, 0x02, rm_field, RSP_enc);   // R/M byte
    emit_rm(cbuf, 0x00, RSP_enc, RSP_enc);    // SIB byte
    emit_d32(cbuf, disp);     // Displacement // R/M byte
  }
}

   // rRegI ereg, memory mem) %{    // emit_reg_mem
void encode_RegMem(CodeBuffer &cbuf,
                   int reg,
                   int base, int index, int scale, int disp, bool disp_is_oop)
{
  assert(!disp_is_oop, "cannot have disp");
  int regenc = reg & 7;
  int baseenc = base & 7;
  int indexenc = index & 7;

  // There is no index & no scale, use form without SIB byte
  if (index == 0x4 && scale == 0 && base != RSP_enc && base != R12_enc) {
    // If no displacement, mode is 0x0; unless base is [RBP] or [R13]
    if (disp == 0 && base != RBP_enc && base != R13_enc) {
      emit_rm(cbuf, 0x0, regenc, baseenc); // *
    } else if (-0x80 <= disp && disp < 0x80 && !disp_is_oop) {
      // If 8-bit displacement, mode 0x1
      emit_rm(cbuf, 0x1, regenc, baseenc); // *
      emit_d8(cbuf, disp);
    } else {
      // If 32-bit displacement
      if (base == -1) { // Special flag for absolute address
        emit_rm(cbuf, 0x0, regenc, 0x5); // *
        if (disp_is_oop) {
          emit_d32_reloc(cbuf, disp, relocInfo::oop_type, RELOC_DISP32);
        } else {
          emit_d32(cbuf, disp);
        }
      } else {
        // Normal base + offset
        emit_rm(cbuf, 0x2, regenc, baseenc); // *
        if (disp_is_oop) {
          emit_d32_reloc(cbuf, disp, relocInfo::oop_type, RELOC_DISP32);
        } else {
          emit_d32(cbuf, disp);
        }
      }
    }
  } else {
    // Else, encode with the SIB byte
    // If no displacement, mode is 0x0; unless base is [RBP] or [R13]
    if (disp == 0 && base != RBP_enc && base != R13_enc) {
      // If no displacement
      emit_rm(cbuf, 0x0, regenc, 0x4); // *
      emit_rm(cbuf, scale, indexenc, baseenc);
    } else {
      if (-0x80 <= disp && disp < 0x80 && !disp_is_oop) {
        // If 8-bit displacement, mode 0x1
        emit_rm(cbuf, 0x1, regenc, 0x4); // *
        emit_rm(cbuf, scale, indexenc, baseenc);
        emit_d8(cbuf, disp);
      } else {
        // If 32-bit displacement
        if (base == 0x04 ) {
          emit_rm(cbuf, 0x2, regenc, 0x4);
          emit_rm(cbuf, scale, indexenc, 0x04); // XXX is this valid???
        } else {
          emit_rm(cbuf, 0x2, regenc, 0x4);
          emit_rm(cbuf, scale, indexenc, baseenc); // *
        }
        if (disp_is_oop) {
          emit_d32_reloc(cbuf, disp, relocInfo::oop_type, RELOC_DISP32);
        } else {
          emit_d32(cbuf, disp);
        }
      }
    }
  }
}

void encode_copy(CodeBuffer &cbuf, int dstenc, int srcenc)
{
  if (dstenc != srcenc) {
    if (dstenc < 8) {
      if (srcenc >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
        srcenc -= 8;
      }
    } else {
      if (srcenc < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
        srcenc -= 8;
      }
      dstenc -= 8;
    }

    emit_opcode(cbuf, 0x8B);
    emit_rm(cbuf, 0x3, dstenc, srcenc);
  }
}

void encode_CopyXD( CodeBuffer &cbuf, int dst_encoding, int src_encoding ) {
  if( dst_encoding == src_encoding ) {
    // reg-reg copy, use an empty encoding
  } else {
    MacroAssembler _masm(&cbuf);

    __ movdqa(as_XMMRegister(dst_encoding), as_XMMRegister(src_encoding));
  }
}


//=============================================================================
const bool Matcher::constant_table_absolute_addressing = true;
const RegMask& MachConstantBaseNode::_out_RegMask = RegMask::Empty;

void MachConstantBaseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  // Empty encoding
}

uint MachConstantBaseNode::size(PhaseRegAlloc* ra_) const {
  return 0;
}

#ifndef PRODUCT
void MachConstantBaseNode::format(PhaseRegAlloc* ra_, outputStream* st) const {
  st->print("# MachConstantBaseNode (empty encoding)");
}
#endif


//=============================================================================
#ifndef PRODUCT
void MachPrologNode::format(PhaseRegAlloc* ra_, outputStream* st) const
{
  Compile* C = ra_->C;

  int framesize = C->frame_slots() << LogBytesPerInt;
  assert((framesize & (StackAlignmentInBytes-1)) == 0, "frame size not aligned");
  // Remove wordSize for return adr already pushed
  // and another for the RBP we are going to save
  framesize -= 2*wordSize;
  bool need_nop = true;

  // Calls to C2R adapters often do not accept exceptional returns.
  // We require that their callers must bang for them.  But be
  // careful, because some VM calls (such as call site linkage) can
  // use several kilobytes of stack.  But the stack safety zone should
  // account for that.  See bugs 4446381, 4468289, 4497237.
  if (C->need_stack_bang(framesize)) {
    st->print_cr("# stack bang"); st->print("\t");
    need_nop = false;
  }
  st->print_cr("pushq   rbp"); st->print("\t");

  if (VerifyStackAtCalls) {
    // Majik cookie to verify stack depth
    st->print_cr("pushq   0xffffffffbadb100d"
                  "\t# Majik cookie for stack depth check");
    st->print("\t");
    framesize -= wordSize; // Remove 2 for cookie
    need_nop = false;
  }

  if (framesize) {
    st->print("subq    rsp, #%d\t# Create frame", framesize);
    if (framesize < 0x80 && need_nop) {
      st->print("\n\tnop\t# nop for patch_verified_entry");
    }
  }
}
#endif

void MachPrologNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const
{
  Compile* C = ra_->C;

  // WARNING: Initial instruction MUST be 5 bytes or longer so that
  // NativeJump::patch_verified_entry will be able to patch out the entry
  // code safely. The fldcw is ok at 6 bytes, the push to verify stack
  // depth is ok at 5 bytes, the frame allocation can be either 3 or
  // 6 bytes. So if we don't do the fldcw or the push then we must
  // use the 6 byte frame allocation even if we have no frame. :-(
  // If method sets FPU control word do it now

  int framesize = C->frame_slots() << LogBytesPerInt;
  assert((framesize & (StackAlignmentInBytes-1)) == 0, "frame size not aligned");
  // Remove wordSize for return adr already pushed
  // and another for the RBP we are going to save
  framesize -= 2*wordSize;
  bool need_nop = true;

  // Calls to C2R adapters often do not accept exceptional returns.
  // We require that their callers must bang for them.  But be
  // careful, because some VM calls (such as call site linkage) can
  // use several kilobytes of stack.  But the stack safety zone should
  // account for that.  See bugs 4446381, 4468289, 4497237.
  if (C->need_stack_bang(framesize)) {
    MacroAssembler masm(&cbuf);
    masm.generate_stack_overflow_check(framesize);
    need_nop = false;
  }

  // We always push rbp so that on return to interpreter rbp will be
  // restored correctly and we can correct the stack.
  emit_opcode(cbuf, 0x50 | RBP_enc);

  if (VerifyStackAtCalls) {
    // Majik cookie to verify stack depth
    emit_opcode(cbuf, 0x68); // pushq (sign-extended) 0xbadb100d
    emit_d32(cbuf, 0xbadb100d);
    framesize -= wordSize; // Remove 2 for cookie
    need_nop = false;
  }

  if (framesize) {
    emit_opcode(cbuf, Assembler::REX_W);
    if (framesize < 0x80) {
      emit_opcode(cbuf, 0x83);   // sub  SP,#framesize
      emit_rm(cbuf, 0x3, 0x05, RSP_enc);
      emit_d8(cbuf, framesize);
      if (need_nop) {
        emit_opcode(cbuf, 0x90); // nop
      }
    } else {
      emit_opcode(cbuf, 0x81);   // sub  SP,#framesize
      emit_rm(cbuf, 0x3, 0x05, RSP_enc);
      emit_d32(cbuf, framesize);
    }
  }

  C->set_frame_complete(cbuf.insts_size());

#ifdef ASSERT
  if (VerifyStackAtCalls) {
    Label L;
    MacroAssembler masm(&cbuf);
    masm.push(rax);
    masm.mov(rax, rsp);
    masm.andptr(rax, StackAlignmentInBytes-1);
    masm.cmpptr(rax, StackAlignmentInBytes-wordSize);
    masm.pop(rax);
    masm.jcc(Assembler::equal, L);
    masm.stop("Stack is not properly aligned!");
    masm.bind(L);
  }
#endif
}

uint MachPrologNode::size(PhaseRegAlloc* ra_) const
{
  return MachNode::size(ra_); // too many variables; just compute it
                              // the hard way
}

int MachPrologNode::reloc() const
{
  return 0; // a large enough number
}

//=============================================================================
#ifndef PRODUCT
void MachEpilogNode::format(PhaseRegAlloc* ra_, outputStream* st) const
{
  Compile* C = ra_->C;
  int framesize = C->frame_slots() << LogBytesPerInt;
  assert((framesize & (StackAlignmentInBytes-1)) == 0, "frame size not aligned");
  // Remove word for return adr already pushed
  // and RBP
  framesize -= 2*wordSize;

  if (framesize) {
    st->print_cr("addq\trsp, %d\t# Destroy frame", framesize);
    st->print("\t");
  }

  st->print_cr("popq\trbp");
  if (do_polling() && C->is_method_compilation()) {
    st->print_cr("\ttestl\trax, [rip + #offset_to_poll_page]\t"
                  "# Safepoint: poll for GC");
    st->print("\t");
  }
}
#endif

void MachEpilogNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const
{
  Compile* C = ra_->C;
  int framesize = C->frame_slots() << LogBytesPerInt;
  assert((framesize & (StackAlignmentInBytes-1)) == 0, "frame size not aligned");
  // Remove word for return adr already pushed
  // and RBP
  framesize -= 2*wordSize;

  // Note that VerifyStackAtCalls' Majik cookie does not change the frame size popped here

  if (framesize) {
    emit_opcode(cbuf, Assembler::REX_W);
    if (framesize < 0x80) {
      emit_opcode(cbuf, 0x83); // addq rsp, #framesize
      emit_rm(cbuf, 0x3, 0x00, RSP_enc);
      emit_d8(cbuf, framesize);
    } else {
      emit_opcode(cbuf, 0x81); // addq rsp, #framesize
      emit_rm(cbuf, 0x3, 0x00, RSP_enc);
      emit_d32(cbuf, framesize);
    }
  }

  // popq rbp
  emit_opcode(cbuf, 0x58 | RBP_enc);

  if (do_polling() && C->is_method_compilation()) {
    // testl %rax, off(%rip) // Opcode + ModRM + Disp32 == 6 bytes
    // XXX reg_mem doesn't support RIP-relative addressing yet
    cbuf.set_insts_mark();
    cbuf.relocate(cbuf.insts_mark(), relocInfo::poll_return_type, 0); // XXX
    emit_opcode(cbuf, 0x85); // testl
    emit_rm(cbuf, 0x0, RAX_enc, 0x5); // 00 rax 101 == 0x5
    // cbuf.insts_mark() is beginning of instruction
    emit_d32_reloc(cbuf, os::get_polling_page());
//                    relocInfo::poll_return_type,
  }
}

uint MachEpilogNode::size(PhaseRegAlloc* ra_) const
{
  Compile* C = ra_->C;
  int framesize = C->frame_slots() << LogBytesPerInt;
  assert((framesize & (StackAlignmentInBytes-1)) == 0, "frame size not aligned");
  // Remove word for return adr already pushed
  // and RBP
  framesize -= 2*wordSize;

  uint size = 0;

  if (do_polling() && C->is_method_compilation()) {
    size += 6;
  }

  // count popq rbp
  size++;

  if (framesize) {
    if (framesize < 0x80) {
      size += 4;
    } else if (framesize) {
      size += 7;
    }
  }

  return size;
}

int MachEpilogNode::reloc() const
{
  return 2; // a large enough number
}

const Pipeline* MachEpilogNode::pipeline() const
{
  return MachNode::pipeline_class();
}

int MachEpilogNode::safepoint_offset() const
{
  return 0;
}

//=============================================================================

enum RC {
  rc_bad,
  rc_int,
  rc_float,
  rc_stack
};

static enum RC rc_class(OptoReg::Name reg)
{
  if( !OptoReg::is_valid(reg)  ) return rc_bad;

  if (OptoReg::is_stack(reg)) return rc_stack;

  VMReg r = OptoReg::as_VMReg(reg);

  if (r->is_Register()) return rc_int;

  assert(r->is_XMMRegister(), "must be");
  return rc_float;
}

uint MachSpillCopyNode::implementation(CodeBuffer* cbuf,
                                       PhaseRegAlloc* ra_,
                                       bool do_size,
                                       outputStream* st) const
{

  // Get registers to move
  OptoReg::Name src_second = ra_->get_reg_second(in(1));
  OptoReg::Name src_first = ra_->get_reg_first(in(1));
  OptoReg::Name dst_second = ra_->get_reg_second(this);
  OptoReg::Name dst_first = ra_->get_reg_first(this);

  enum RC src_second_rc = rc_class(src_second);
  enum RC src_first_rc = rc_class(src_first);
  enum RC dst_second_rc = rc_class(dst_second);
  enum RC dst_first_rc = rc_class(dst_first);

  assert(OptoReg::is_valid(src_first) && OptoReg::is_valid(dst_first),
         "must move at least 1 register" );

  if (src_first == dst_first && src_second == dst_second) {
    // Self copy, no move
    return 0;
  } else if (src_first_rc == rc_stack) {
    // mem ->
    if (dst_first_rc == rc_stack) {
      // mem -> mem
      assert(src_second != dst_first, "overlap");
      if ((src_first & 1) == 0 && src_first + 1 == src_second &&
          (dst_first & 1) == 0 && dst_first + 1 == dst_second) {
        // 64-bit
        int src_offset = ra_->reg2offset(src_first);
        int dst_offset = ra_->reg2offset(dst_first);
        if (cbuf) {
          emit_opcode(*cbuf, 0xFF);
          encode_RegMem(*cbuf, RSI_enc, RSP_enc, 0x4, 0, src_offset, false);

          emit_opcode(*cbuf, 0x8F);
          encode_RegMem(*cbuf, RAX_enc, RSP_enc, 0x4, 0, dst_offset, false);

#ifndef PRODUCT
        } else if (!do_size) {
          st->print("pushq   [rsp + #%d]\t# 64-bit mem-mem spill\n\t"
                     "popq    [rsp + #%d]",
                     src_offset,
                     dst_offset);
#endif
        }
        return
          3 + ((src_offset == 0) ? 0 : (src_offset < 0x80 ? 1 : 4)) +
          3 + ((dst_offset == 0) ? 0 : (dst_offset < 0x80 ? 1 : 4));
      } else {
        // 32-bit
        assert(!((src_first & 1) == 0 && src_first + 1 == src_second), "no transform");
        assert(!((dst_first & 1) == 0 && dst_first + 1 == dst_second), "no transform");
        // No pushl/popl, so:
        int src_offset = ra_->reg2offset(src_first);
        int dst_offset = ra_->reg2offset(dst_first);
        if (cbuf) {
          emit_opcode(*cbuf, Assembler::REX_W);
          emit_opcode(*cbuf, 0x89);
          emit_opcode(*cbuf, 0x44);
          emit_opcode(*cbuf, 0x24);
          emit_opcode(*cbuf, 0xF8);

          emit_opcode(*cbuf, 0x8B);
          encode_RegMem(*cbuf,
                        RAX_enc,
                        RSP_enc, 0x4, 0, src_offset,
                        false);

          emit_opcode(*cbuf, 0x89);
          encode_RegMem(*cbuf,
                        RAX_enc,
                        RSP_enc, 0x4, 0, dst_offset,
                        false);

          emit_opcode(*cbuf, Assembler::REX_W);
          emit_opcode(*cbuf, 0x8B);
          emit_opcode(*cbuf, 0x44);
          emit_opcode(*cbuf, 0x24);
          emit_opcode(*cbuf, 0xF8);

#ifndef PRODUCT
        } else if (!do_size) {
          st->print("movq    [rsp - #8], rax\t# 32-bit mem-mem spill\n\t"
                     "movl    rax, [rsp + #%d]\n\t"
                     "movl    [rsp + #%d], rax\n\t"
                     "movq    rax, [rsp - #8]",
                     src_offset,
                     dst_offset);
#endif
        }
        return
          5 + // movq
          3 + ((src_offset == 0) ? 0 : (src_offset < 0x80 ? 1 : 4)) + // movl
          3 + ((dst_offset == 0) ? 0 : (dst_offset < 0x80 ? 1 : 4)) + // movl
          5; // movq
      }
    } else if (dst_first_rc == rc_int) {
      // mem -> gpr
      if ((src_first & 1) == 0 && src_first + 1 == src_second &&
          (dst_first & 1) == 0 && dst_first + 1 == dst_second) {
        // 64-bit
        int offset = ra_->reg2offset(src_first);
        if (cbuf) {
          if (Matcher::_regEncode[dst_first] < 8) {
            emit_opcode(*cbuf, Assembler::REX_W);
          } else {
            emit_opcode(*cbuf, Assembler::REX_WR);
          }
          emit_opcode(*cbuf, 0x8B);
          encode_RegMem(*cbuf,
                        Matcher::_regEncode[dst_first],
                        RSP_enc, 0x4, 0, offset,
                        false);
#ifndef PRODUCT
        } else if (!do_size) {
          st->print("movq    %s, [rsp + #%d]\t# spill",
                     Matcher::regName[dst_first],
                     offset);
#endif
        }
        return
          ((offset == 0) ? 0 : (offset < 0x80 ? 1 : 4)) + 4; // REX
      } else {
        // 32-bit
        assert(!((src_first & 1) == 0 && src_first + 1 == src_second), "no transform");
        assert(!((dst_first & 1) == 0 && dst_first + 1 == dst_second), "no transform");
        int offset = ra_->reg2offset(src_first);
        if (cbuf) {
          if (Matcher::_regEncode[dst_first] >= 8) {
            emit_opcode(*cbuf, Assembler::REX_R);
          }
          emit_opcode(*cbuf, 0x8B);
          encode_RegMem(*cbuf,
                        Matcher::_regEncode[dst_first],
                        RSP_enc, 0x4, 0, offset,
                        false);
#ifndef PRODUCT
        } else if (!do_size) {
          st->print("movl    %s, [rsp + #%d]\t# spill",
                     Matcher::regName[dst_first],
                     offset);
#endif
        }
        return
          ((offset == 0) ? 0 : (offset < 0x80 ? 1 : 4)) +
          ((Matcher::_regEncode[dst_first] < 8)
           ? 3
           : 4); // REX
      }
    } else if (dst_first_rc == rc_float) {
      // mem-> xmm
      if ((src_first & 1) == 0 && src_first + 1 == src_second &&
          (dst_first & 1) == 0 && dst_first + 1 == dst_second) {
        // 64-bit
        int offset = ra_->reg2offset(src_first);
        if (cbuf) {
          emit_opcode(*cbuf, UseXmmLoadAndClearUpper ? 0xF2 : 0x66);
          if (Matcher::_regEncode[dst_first] >= 8) {
            emit_opcode(*cbuf, Assembler::REX_R);
          }
          emit_opcode(*cbuf, 0x0F);
          emit_opcode(*cbuf, UseXmmLoadAndClearUpper ? 0x10 : 0x12);
          encode_RegMem(*cbuf,
                        Matcher::_regEncode[dst_first],
                        RSP_enc, 0x4, 0, offset,
                        false);
#ifndef PRODUCT
        } else if (!do_size) {
          st->print("%s  %s, [rsp + #%d]\t# spill",
                     UseXmmLoadAndClearUpper ? "movsd " : "movlpd",
                     Matcher::regName[dst_first],
                     offset);
#endif
        }
        return
          ((offset == 0) ? 0 : (offset < 0x80 ? 1 : 4)) +
          ((Matcher::_regEncode[dst_first] < 8)
           ? 5
           : 6); // REX
      } else {
        // 32-bit
        assert(!((src_first & 1) == 0 && src_first + 1 == src_second), "no transform");
        assert(!((dst_first & 1) == 0 && dst_first + 1 == dst_second), "no transform");
        int offset = ra_->reg2offset(src_first);
        if (cbuf) {
          emit_opcode(*cbuf, 0xF3);
          if (Matcher::_regEncode[dst_first] >= 8) {
            emit_opcode(*cbuf, Assembler::REX_R);
          }
          emit_opcode(*cbuf, 0x0F);
          emit_opcode(*cbuf, 0x10);
          encode_RegMem(*cbuf,
                        Matcher::_regEncode[dst_first],
                        RSP_enc, 0x4, 0, offset,
                        false);
#ifndef PRODUCT
        } else if (!do_size) {
          st->print("movss   %s, [rsp + #%d]\t# spill",
                     Matcher::regName[dst_first],
                     offset);
#endif
        }
        return
          ((offset == 0) ? 0 : (offset < 0x80 ? 1 : 4)) +
          ((Matcher::_regEncode[dst_first] < 8)
           ? 5
           : 6); // REX
      }
    }
  } else if (src_first_rc == rc_int) {
    // gpr ->
    if (dst_first_rc == rc_stack) {
      // gpr -> mem
      if ((src_first & 1) == 0 && src_first + 1 == src_second &&
          (dst_first & 1) == 0 && dst_first + 1 == dst_second) {
        // 64-bit
        int offset = ra_->reg2offset(dst_first);
        if (cbuf) {
          if (Matcher::_regEncode[src_first] < 8) {
            emit_opcode(*cbuf, Assembler::REX_W);
          } else {
            emit_opcode(*cbuf, Assembler::REX_WR);
          }
          emit_opcode(*cbuf, 0x89);
          encode_RegMem(*cbuf,
                        Matcher::_regEncode[src_first],
                        RSP_enc, 0x4, 0, offset,
                        false);
#ifndef PRODUCT
        } else if (!do_size) {
          st->print("movq    [rsp + #%d], %s\t# spill",
                     offset,
                     Matcher::regName[src_first]);
#endif
        }
        return ((offset == 0) ? 0 : (offset < 0x80 ? 1 : 4)) + 4; // REX
      } else {
        // 32-bit
        assert(!((src_first & 1) == 0 && src_first + 1 == src_second), "no transform");
        assert(!((dst_first & 1) == 0 && dst_first + 1 == dst_second), "no transform");
        int offset = ra_->reg2offset(dst_first);
        if (cbuf) {
          if (Matcher::_regEncode[src_first] >= 8) {
            emit_opcode(*cbuf, Assembler::REX_R);
          }
          emit_opcode(*cbuf, 0x89);
          encode_RegMem(*cbuf,
                        Matcher::_regEncode[src_first],
                        RSP_enc, 0x4, 0, offset,
                        false);
#ifndef PRODUCT
        } else if (!do_size) {
          st->print("movl    [rsp + #%d], %s\t# spill",
                     offset,
                     Matcher::regName[src_first]);
#endif
        }
        return
          ((offset == 0) ? 0 : (offset < 0x80 ? 1 : 4)) +
          ((Matcher::_regEncode[src_first] < 8)
           ? 3
           : 4); // REX
      }
    } else if (dst_first_rc == rc_int) {
      // gpr -> gpr
      if ((src_first & 1) == 0 && src_first + 1 == src_second &&
          (dst_first & 1) == 0 && dst_first + 1 == dst_second) {
        // 64-bit
        if (cbuf) {
          if (Matcher::_regEncode[dst_first] < 8) {
            if (Matcher::_regEncode[src_first] < 8) {
              emit_opcode(*cbuf, Assembler::REX_W);
            } else {
              emit_opcode(*cbuf, Assembler::REX_WB);
            }
          } else {
            if (Matcher::_regEncode[src_first] < 8) {
              emit_opcode(*cbuf, Assembler::REX_WR);
            } else {
              emit_opcode(*cbuf, Assembler::REX_WRB);
            }
          }
          emit_opcode(*cbuf, 0x8B);
          emit_rm(*cbuf, 0x3,
                  Matcher::_regEncode[dst_first] & 7,
                  Matcher::_regEncode[src_first] & 7);
#ifndef PRODUCT
        } else if (!do_size) {
          st->print("movq    %s, %s\t# spill",
                     Matcher::regName[dst_first],
                     Matcher::regName[src_first]);
#endif
        }
        return 3; // REX
      } else {
        // 32-bit
        assert(!((src_first & 1) == 0 && src_first + 1 == src_second), "no transform");
        assert(!((dst_first & 1) == 0 && dst_first + 1 == dst_second), "no transform");
        if (cbuf) {
          if (Matcher::_regEncode[dst_first] < 8) {
            if (Matcher::_regEncode[src_first] >= 8) {
              emit_opcode(*cbuf, Assembler::REX_B);
            }
          } else {
            if (Matcher::_regEncode[src_first] < 8) {
              emit_opcode(*cbuf, Assembler::REX_R);
            } else {
              emit_opcode(*cbuf, Assembler::REX_RB);
            }
          }
          emit_opcode(*cbuf, 0x8B);
          emit_rm(*cbuf, 0x3,
                  Matcher::_regEncode[dst_first] & 7,
                  Matcher::_regEncode[src_first] & 7);
#ifndef PRODUCT
        } else if (!do_size) {
          st->print("movl    %s, %s\t# spill",
                     Matcher::regName[dst_first],
                     Matcher::regName[src_first]);
#endif
        }
        return
          (Matcher::_regEncode[src_first] < 8 && Matcher::_regEncode[dst_first] < 8)
          ? 2
          : 3; // REX
      }
    } else if (dst_first_rc == rc_float) {
      // gpr -> xmm
      if ((src_first & 1) == 0 && src_first + 1 == src_second &&
          (dst_first & 1) == 0 && dst_first + 1 == dst_second) {
        // 64-bit
        if (cbuf) {
          emit_opcode(*cbuf, 0x66);
          if (Matcher::_regEncode[dst_first] < 8) {
            if (Matcher::_regEncode[src_first] < 8) {
              emit_opcode(*cbuf, Assembler::REX_W);
            } else {
              emit_opcode(*cbuf, Assembler::REX_WB);
            }
          } else {
            if (Matcher::_regEncode[src_first] < 8) {
              emit_opcode(*cbuf, Assembler::REX_WR);
            } else {
              emit_opcode(*cbuf, Assembler::REX_WRB);
            }
          }
          emit_opcode(*cbuf, 0x0F);
          emit_opcode(*cbuf, 0x6E);
          emit_rm(*cbuf, 0x3,
                  Matcher::_regEncode[dst_first] & 7,
                  Matcher::_regEncode[src_first] & 7);
#ifndef PRODUCT
        } else if (!do_size) {
          st->print("movdq   %s, %s\t# spill",
                     Matcher::regName[dst_first],
                     Matcher::regName[src_first]);
#endif
        }
        return 5; // REX
      } else {
        // 32-bit
        assert(!((src_first & 1) == 0 && src_first + 1 == src_second), "no transform");
        assert(!((dst_first & 1) == 0 && dst_first + 1 == dst_second), "no transform");
        if (cbuf) {
          emit_opcode(*cbuf, 0x66);
          if (Matcher::_regEncode[dst_first] < 8) {
            if (Matcher::_regEncode[src_first] >= 8) {
              emit_opcode(*cbuf, Assembler::REX_B);
            }
          } else {
            if (Matcher::_regEncode[src_first] < 8) {
              emit_opcode(*cbuf, Assembler::REX_R);
            } else {
              emit_opcode(*cbuf, Assembler::REX_RB);
            }
          }
          emit_opcode(*cbuf, 0x0F);
          emit_opcode(*cbuf, 0x6E);
          emit_rm(*cbuf, 0x3,
                  Matcher::_regEncode[dst_first] & 7,
                  Matcher::_regEncode[src_first] & 7);
#ifndef PRODUCT
        } else if (!do_size) {
          st->print("movdl   %s, %s\t# spill",
                     Matcher::regName[dst_first],
                     Matcher::regName[src_first]);
#endif
        }
        return
          (Matcher::_regEncode[src_first] < 8 && Matcher::_regEncode[dst_first] < 8)
          ? 4
          : 5; // REX
      }
    }
  } else if (src_first_rc == rc_float) {
    // xmm ->
    if (dst_first_rc == rc_stack) {
      // xmm -> mem
      if ((src_first & 1) == 0 && src_first + 1 == src_second &&
          (dst_first & 1) == 0 && dst_first + 1 == dst_second) {
        // 64-bit
        int offset = ra_->reg2offset(dst_first);
        if (cbuf) {
          emit_opcode(*cbuf, 0xF2);
          if (Matcher::_regEncode[src_first] >= 8) {
              emit_opcode(*cbuf, Assembler::REX_R);
          }
          emit_opcode(*cbuf, 0x0F);
          emit_opcode(*cbuf, 0x11);
          encode_RegMem(*cbuf,
                        Matcher::_regEncode[src_first],
                        RSP_enc, 0x4, 0, offset,
                        false);
#ifndef PRODUCT
        } else if (!do_size) {
          st->print("movsd   [rsp + #%d], %s\t# spill",
                     offset,
                     Matcher::regName[src_first]);
#endif
        }
        return
          ((offset == 0) ? 0 : (offset < 0x80 ? 1 : 4)) +
          ((Matcher::_regEncode[src_first] < 8)
           ? 5
           : 6); // REX
      } else {
        // 32-bit
        assert(!((src_first & 1) == 0 && src_first + 1 == src_second), "no transform");
        assert(!((dst_first & 1) == 0 && dst_first + 1 == dst_second), "no transform");
        int offset = ra_->reg2offset(dst_first);
        if (cbuf) {
          emit_opcode(*cbuf, 0xF3);
          if (Matcher::_regEncode[src_first] >= 8) {
              emit_opcode(*cbuf, Assembler::REX_R);
          }
          emit_opcode(*cbuf, 0x0F);
          emit_opcode(*cbuf, 0x11);
          encode_RegMem(*cbuf,
                        Matcher::_regEncode[src_first],
                        RSP_enc, 0x4, 0, offset,
                        false);
#ifndef PRODUCT
        } else if (!do_size) {
          st->print("movss   [rsp + #%d], %s\t# spill",
                     offset,
                     Matcher::regName[src_first]);
#endif
        }
        return
          ((offset == 0) ? 0 : (offset < 0x80 ? 1 : 4)) +
          ((Matcher::_regEncode[src_first] < 8)
           ? 5
           : 6); // REX
      }
    } else if (dst_first_rc == rc_int) {
      // xmm -> gpr
      if ((src_first & 1) == 0 && src_first + 1 == src_second &&
          (dst_first & 1) == 0 && dst_first + 1 == dst_second) {
        // 64-bit
        if (cbuf) {
          emit_opcode(*cbuf, 0x66);
          if (Matcher::_regEncode[dst_first] < 8) {
            if (Matcher::_regEncode[src_first] < 8) {
              emit_opcode(*cbuf, Assembler::REX_W);
            } else {
              emit_opcode(*cbuf, Assembler::REX_WR); // attention!
            }
          } else {
            if (Matcher::_regEncode[src_first] < 8) {
              emit_opcode(*cbuf, Assembler::REX_WB); // attention!
            } else {
              emit_opcode(*cbuf, Assembler::REX_WRB);
            }
          }
          emit_opcode(*cbuf, 0x0F);
          emit_opcode(*cbuf, 0x7E);
          emit_rm(*cbuf, 0x3,
                  Matcher::_regEncode[src_first] & 7,
                  Matcher::_regEncode[dst_first] & 7);
#ifndef PRODUCT
        } else if (!do_size) {
          st->print("movdq   %s, %s\t# spill",
                     Matcher::regName[dst_first],
                     Matcher::regName[src_first]);
#endif
        }
        return 5; // REX
      } else {
        // 32-bit
        assert(!((src_first & 1) == 0 && src_first + 1 == src_second), "no transform");
        assert(!((dst_first & 1) == 0 && dst_first + 1 == dst_second), "no transform");
        if (cbuf) {
          emit_opcode(*cbuf, 0x66);
          if (Matcher::_regEncode[dst_first] < 8) {
            if (Matcher::_regEncode[src_first] >= 8) {
              emit_opcode(*cbuf, Assembler::REX_R); // attention!
            }
          } else {
            if (Matcher::_regEncode[src_first] < 8) {
              emit_opcode(*cbuf, Assembler::REX_B); // attention!
            } else {
              emit_opcode(*cbuf, Assembler::REX_RB);
            }
          }
          emit_opcode(*cbuf, 0x0F);
          emit_opcode(*cbuf, 0x7E);
          emit_rm(*cbuf, 0x3,
                  Matcher::_regEncode[src_first] & 7,
                  Matcher::_regEncode[dst_first] & 7);
#ifndef PRODUCT
        } else if (!do_size) {
          st->print("movdl   %s, %s\t# spill",
                     Matcher::regName[dst_first],
                     Matcher::regName[src_first]);
#endif
        }
        return
          (Matcher::_regEncode[src_first] < 8 && Matcher::_regEncode[dst_first] < 8)
          ? 4
          : 5; // REX
      }
    } else if (dst_first_rc == rc_float) {
      // xmm -> xmm
      if ((src_first & 1) == 0 && src_first + 1 == src_second &&
          (dst_first & 1) == 0 && dst_first + 1 == dst_second) {
        // 64-bit
        if (cbuf) {
          emit_opcode(*cbuf, UseXmmRegToRegMoveAll ? 0x66 : 0xF2);
          if (Matcher::_regEncode[dst_first] < 8) {
            if (Matcher::_regEncode[src_first] >= 8) {
              emit_opcode(*cbuf, Assembler::REX_B);
            }
          } else {
            if (Matcher::_regEncode[src_first] < 8) {
              emit_opcode(*cbuf, Assembler::REX_R);
            } else {
              emit_opcode(*cbuf, Assembler::REX_RB);
            }
          }
          emit_opcode(*cbuf, 0x0F);
          emit_opcode(*cbuf, UseXmmRegToRegMoveAll ? 0x28 : 0x10);
          emit_rm(*cbuf, 0x3,
                  Matcher::_regEncode[dst_first] & 7,
                  Matcher::_regEncode[src_first] & 7);
#ifndef PRODUCT
        } else if (!do_size) {
          st->print("%s  %s, %s\t# spill",
                     UseXmmRegToRegMoveAll ? "movapd" : "movsd ",
                     Matcher::regName[dst_first],
                     Matcher::regName[src_first]);
#endif
        }
        return
          (Matcher::_regEncode[src_first] < 8 && Matcher::_regEncode[dst_first] < 8)
          ? 4
          : 5; // REX
      } else {
        // 32-bit
        assert(!((src_first & 1) == 0 && src_first + 1 == src_second), "no transform");
        assert(!((dst_first & 1) == 0 && dst_first + 1 == dst_second), "no transform");
        if (cbuf) {
          if (!UseXmmRegToRegMoveAll)
            emit_opcode(*cbuf, 0xF3);
          if (Matcher::_regEncode[dst_first] < 8) {
            if (Matcher::_regEncode[src_first] >= 8) {
              emit_opcode(*cbuf, Assembler::REX_B);
            }
          } else {
            if (Matcher::_regEncode[src_first] < 8) {
              emit_opcode(*cbuf, Assembler::REX_R);
            } else {
              emit_opcode(*cbuf, Assembler::REX_RB);
            }
          }
          emit_opcode(*cbuf, 0x0F);
          emit_opcode(*cbuf, UseXmmRegToRegMoveAll ? 0x28 : 0x10);
          emit_rm(*cbuf, 0x3,
                  Matcher::_regEncode[dst_first] & 7,
                  Matcher::_regEncode[src_first] & 7);
#ifndef PRODUCT
        } else if (!do_size) {
          st->print("%s  %s, %s\t# spill",
                     UseXmmRegToRegMoveAll ? "movaps" : "movss ",
                     Matcher::regName[dst_first],
                     Matcher::regName[src_first]);
#endif
        }
        return
          (Matcher::_regEncode[src_first] < 8 && Matcher::_regEncode[dst_first] < 8)
          ? (UseXmmRegToRegMoveAll ? 3 : 4)
          : (UseXmmRegToRegMoveAll ? 4 : 5); // REX
      }
    }
  }

  assert(0," foo ");
  Unimplemented();

  return 0;
}

#ifndef PRODUCT
void MachSpillCopyNode::format(PhaseRegAlloc *ra_, outputStream* st) const
{
  implementation(NULL, ra_, false, st);
}
#endif

void MachSpillCopyNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const
{
  implementation(&cbuf, ra_, false, NULL);
}

uint MachSpillCopyNode::size(PhaseRegAlloc *ra_) const
{
  return implementation(NULL, ra_, true, NULL);
}

//=============================================================================
#ifndef PRODUCT
void MachNopNode::format(PhaseRegAlloc*, outputStream* st) const
{
  st->print("nop \t# %d bytes pad for loops and calls", _count);
}
#endif

void MachNopNode::emit(CodeBuffer &cbuf, PhaseRegAlloc*) const
{
  MacroAssembler _masm(&cbuf);
  __ nop(_count);
}

uint MachNopNode::size(PhaseRegAlloc*) const
{
  return _count;
}


//=============================================================================
#ifndef PRODUCT
void BoxLockNode::format(PhaseRegAlloc* ra_, outputStream* st) const
{
  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem());
  int reg = ra_->get_reg_first(this);
  st->print("leaq    %s, [rsp + #%d]\t# box lock",
            Matcher::regName[reg], offset);
}
#endif

void BoxLockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const
{
  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem());
  int reg = ra_->get_encode(this);
  if (offset >= 0x80) {
    emit_opcode(cbuf, reg < 8 ? Assembler::REX_W : Assembler::REX_WR);
    emit_opcode(cbuf, 0x8D); // LEA  reg,[SP+offset]
    emit_rm(cbuf, 0x2, reg & 7, 0x04);
    emit_rm(cbuf, 0x0, 0x04, RSP_enc);
    emit_d32(cbuf, offset);
  } else {
    emit_opcode(cbuf, reg < 8 ? Assembler::REX_W : Assembler::REX_WR);
    emit_opcode(cbuf, 0x8D); // LEA  reg,[SP+offset]
    emit_rm(cbuf, 0x1, reg & 7, 0x04);
    emit_rm(cbuf, 0x0, 0x04, RSP_enc);
    emit_d8(cbuf, offset);
  }
}

uint BoxLockNode::size(PhaseRegAlloc *ra_) const
{
  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem());
  return (offset < 0x80) ? 5 : 8; // REX
}

//=============================================================================

// emit call stub, compiled java to interpreter
void emit_java_to_interp(CodeBuffer& cbuf)
{
  // Stub is fixed up when the corresponding call is converted from
  // calling compiled code to calling interpreted code.
  // movq rbx, 0
  // jmp -5 # to self

  address mark = cbuf.insts_mark();  // get mark within main instrs section

  // Note that the code buffer's insts_mark is always relative to insts.
  // That's why we must use the macroassembler to generate a stub.
  MacroAssembler _masm(&cbuf);

  address base =
  __ start_a_stub(Compile::MAX_stubs_size);
  if (base == NULL)  return;  // CodeBuffer::expand failed
  // static stub relocation stores the instruction address of the call
  __ relocate(static_stub_Relocation::spec(mark), RELOC_IMM64);
  // static stub relocation also tags the methodOop in the code-stream.
  __ movoop(rbx, (jobject) NULL);  // method is zapped till fixup time
  // This is recognized as unresolved by relocs/nativeinst/ic code
  __ jump(RuntimeAddress(__ pc()));

  // Update current stubs pointer and restore insts_end.
  __ end_a_stub();
}

// size of call stub, compiled java to interpretor
uint size_java_to_interp()
{
  return 15;  // movq (1+1+8); jmp (1+4)
}

// relocation entries for call stub, compiled java to interpretor
uint reloc_java_to_interp()
{
  return 4; // 3 in emit_java_to_interp + 1 in Java_Static_Call
}

//=============================================================================
#ifndef PRODUCT
void MachUEPNode::format(PhaseRegAlloc* ra_, outputStream* st) const
{
  if (UseCompressedOops) {
    st->print_cr("movl    rscratch1, [j_rarg0 + oopDesc::klass_offset_in_bytes()]\t# compressed klass");
    if (Universe::narrow_oop_shift() != 0) {
      st->print_cr("\tdecode_heap_oop_not_null rscratch1, rscratch1");
    }
    st->print_cr("\tcmpq    rax, rscratch1\t # Inline cache check");
  } else {
    st->print_cr("\tcmpq    rax, [j_rarg0 + oopDesc::klass_offset_in_bytes()]\t"
                 "# Inline cache check");
  }
  st->print_cr("\tjne     SharedRuntime::_ic_miss_stub");
  st->print_cr("\tnop\t# nops to align entry point");
}
#endif

void MachUEPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const
{
  MacroAssembler masm(&cbuf);
  uint insts_size = cbuf.insts_size();
  if (UseCompressedOops) {
    masm.load_klass(rscratch1, j_rarg0);
    masm.cmpptr(rax, rscratch1);
  } else {
    masm.cmpptr(rax, Address(j_rarg0, oopDesc::klass_offset_in_bytes()));
  }

  masm.jump_cc(Assembler::notEqual, RuntimeAddress(SharedRuntime::get_ic_miss_stub()));

  /* WARNING these NOPs are critical so that verified entry point is properly
     4 bytes aligned for patching by NativeJump::patch_verified_entry() */
  int nops_cnt = 4 - ((cbuf.insts_size() - insts_size) & 0x3);
  if (OptoBreakpoint) {
    // Leave space for int3
    nops_cnt -= 1;
  }
  nops_cnt &= 0x3; // Do not add nops if code is aligned.
  if (nops_cnt > 0)
    masm.nop(nops_cnt);
}

uint MachUEPNode::size(PhaseRegAlloc* ra_) const
{
  return MachNode::size(ra_); // too many variables; just compute it
                              // the hard way
}


//=============================================================================
uint size_exception_handler()
{
  // NativeCall instruction size is the same as NativeJump.
  // Note that this value is also credited (in output.cpp) to
  // the size of the code section.
  return NativeJump::instruction_size;
}

// Emit exception handler code.
int emit_exception_handler(CodeBuffer& cbuf)
{

  // Note that the code buffer's insts_mark is always relative to insts.
  // That's why we must use the macroassembler to generate a handler.
  MacroAssembler _masm(&cbuf);
  address base =
  __ start_a_stub(size_exception_handler());
  if (base == NULL)  return 0;  // CodeBuffer::expand failed
  int offset = __ offset();
  __ jump(RuntimeAddress(OptoRuntime::exception_blob()->entry_point()));
  assert(__ offset() - offset <= (int) size_exception_handler(), "overflow");
  __ end_a_stub();
  return offset;
}

uint size_deopt_handler()
{
  // three 5 byte instructions
  return 15;
}

// Emit deopt handler code.
int emit_deopt_handler(CodeBuffer& cbuf)
{

  // Note that the code buffer's insts_mark is always relative to insts.
  // That's why we must use the macroassembler to generate a handler.
  MacroAssembler _masm(&cbuf);
  address base =
  __ start_a_stub(size_deopt_handler());
  if (base == NULL)  return 0;  // CodeBuffer::expand failed
  int offset = __ offset();
  address the_pc = (address) __ pc();
  Label next;
  // push a "the_pc" on the stack without destroying any registers
  // as they all may be live.

  // push address of "next"
  __ call(next, relocInfo::none); // reloc none is fine since it is a disp32
  __ bind(next);
  // adjust it so it matches "the_pc"
  __ subptr(Address(rsp, 0), __ offset() - offset);
  __ jump(RuntimeAddress(SharedRuntime::deopt_blob()->unpack()));
  assert(__ offset() - offset <= (int) size_deopt_handler(), "overflow");
  __ end_a_stub();
  return offset;
}


const bool Matcher::match_rule_supported(int opcode) {
  if (!has_match_rule(opcode))
    return false;

  return true;  // Per default match rules are supported.
}

int Matcher::regnum_to_fpu_offset(int regnum)
{
  return regnum - 32; // The FP registers are in the second chunk
}

// This is UltraSparc specific, true just means we have fast l2f conversion
const bool Matcher::convL2FSupported(void) {
  return true;
}

// Vector width in bytes
const uint Matcher::vector_width_in_bytes(void) {
  return 8;
}

// Vector ideal reg
const uint Matcher::vector_ideal_reg(void) {
  return Op_RegD;
}

// Is this branch offset short enough that a short branch can be used?
//
// NOTE: If the platform does not provide any short branch variants, then
//       this method should return false for offset 0.
bool Matcher::is_short_branch_offset(int rule, int offset) {
  // the short version of jmpConUCF2 contains multiple branches,
  // making the reach slightly less
  if (rule == jmpConUCF2_rule)
    return (-126 <= offset && offset <= 125);
  return (-128 <= offset && offset <= 127);
}

const bool Matcher::isSimpleConstant64(jlong value) {
  // Will one (StoreL ConL) be cheaper than two (StoreI ConI)?.
  //return value == (int) value;  // Cf. storeImmL and immL32.

  // Probably always true, even if a temp register is required.
  return true;
}

// The ecx parameter to rep stosq for the ClearArray node is in words.
const bool Matcher::init_array_count_is_in_bytes = false;

// Threshold size for cleararray.
const int Matcher::init_array_short_size = 8 * BytesPerLong;

// Should the Matcher clone shifts on addressing modes, expecting them
// to be subsumed into complex addressing expressions or compute them
// into registers?  True for Intel but false for most RISCs
const bool Matcher::clone_shift_expressions = true;

bool Matcher::narrow_oop_use_complex_address() {
  assert(UseCompressedOops, "only for compressed oops code");
  return (LogMinObjAlignmentInBytes <= 3);
}

// Is it better to copy float constants, or load them directly from
// memory?  Intel can load a float constant from a direct address,
// requiring no extra registers.  Most RISCs will have to materialize
// an address into a register first, so they would do better to copy
// the constant from stack.
const bool Matcher::rematerialize_float_constants = true; // XXX

// If CPU can load and store mis-aligned doubles directly then no
// fixup is needed.  Else we split the double into 2 integer pieces
// and move it piece-by-piece.  Only happens when passing doubles into
// C code as the Java calling convention forces doubles to be aligned.
const bool Matcher::misaligned_doubles_ok = true;

// No-op on amd64
void Matcher::pd_implicit_null_fixup(MachNode *node, uint idx) {}

// Advertise here if the CPU requires explicit rounding operations to
// implement the UseStrictFP mode.
const bool Matcher::strict_fp_requires_explicit_rounding = true;

// Are floats conerted to double when stored to stack during deoptimization?
// On x64 it is stored without convertion so we can use normal access.
bool Matcher::float_in_double() { return false; }

// Do ints take an entire long register or just half?
const bool Matcher::int_in_long = true;

// Return whether or not this register is ever used as an argument.
// This function is used on startup to build the trampoline stubs in
// generateOptoStub.  Registers not mentioned will be killed by the VM
// call in the trampoline, and arguments in those registers not be
// available to the callee.
bool Matcher::can_be_java_arg(int reg)
{
  return
    reg ==  RDI_num || reg ==  RDI_H_num ||
    reg ==  RSI_num || reg ==  RSI_H_num ||
    reg ==  RDX_num || reg ==  RDX_H_num ||
    reg ==  RCX_num || reg ==  RCX_H_num ||
    reg ==   R8_num || reg ==   R8_H_num ||
    reg ==   R9_num || reg ==   R9_H_num ||
    reg ==  R12_num || reg ==  R12_H_num ||
    reg == XMM0_num || reg == XMM0_H_num ||
    reg == XMM1_num || reg == XMM1_H_num ||
    reg == XMM2_num || reg == XMM2_H_num ||
    reg == XMM3_num || reg == XMM3_H_num ||
    reg == XMM4_num || reg == XMM4_H_num ||
    reg == XMM5_num || reg == XMM5_H_num ||
    reg == XMM6_num || reg == XMM6_H_num ||
    reg == XMM7_num || reg == XMM7_H_num;
}

bool Matcher::is_spillable_arg(int reg)
{
  return can_be_java_arg(reg);
}

bool Matcher::use_asm_for_ldiv_by_con( jlong divisor ) {
  // In 64 bit mode a code which use multiply when
  // devisor is constant is faster than hardware
  // DIV instruction (it uses MulHiL).
  return false;
}

// Register for DIVI projection of divmodI
RegMask Matcher::divI_proj_mask() {
  return INT_RAX_REG_mask;
}

// Register for MODI projection of divmodI
RegMask Matcher::modI_proj_mask() {
  return INT_RDX_REG_mask;
}

// Register for DIVL projection of divmodL
RegMask Matcher::divL_proj_mask() {
  return LONG_RAX_REG_mask;
}

// Register for MODL projection of divmodL
RegMask Matcher::modL_proj_mask() {
  return LONG_RDX_REG_mask;
}

const RegMask Matcher::method_handle_invoke_SP_save_mask() {
  return PTR_RBP_REG_mask;
}

static Address build_address(int b, int i, int s, int d) {
  Register index = as_Register(i);
  Address::ScaleFactor scale = (Address::ScaleFactor)s;
  if (index == rsp) {
    index = noreg;
    scale = Address::no_scale;
  }
  Address addr(as_Register(b), index, scale, d);
  return addr;
}


#line 1617 "../generated/adfiles/ad_x86_64.cpp"


//SourceForm

#line 152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/os_cpu/linux_x86/vm/linux_x86_64.ad"


int MachCallRuntimeNode::ret_addr_offset() {
  return 13; // movq r10,#addr; callq (r10)
}

// emit an interrupt that is caught by the debugger
void emit_break(CodeBuffer& cbuf) {
  // Debugger doesn't really catch this but best we can do so far QQQ
  MacroAssembler* masm = new MacroAssembler(&cbuf);
  masm->call(RuntimeAddress(CAST_FROM_FN_PTR(address, os::breakpoint)));
}

void MachBreakpointNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  emit_break(cbuf);
}

uint MachBreakpointNode::size(PhaseRegAlloc* ra_) const {
  return 5;
}


#line 1645 "../generated/adfiles/ad_x86_64.cpp"


#ifndef PRODUCT
void Compile::adlc_verification() {

  // Following assertions generated from definition section
}
#endif

// Map from machine-independent register number to register_save_policy
const        char register_save_policy[] = {
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'N',
  'N',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'N',
  'N',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C' // no trailing comma
};

// Map from machine-independent register number to c_reg_save_policy
const        char c_reg_save_policy[] = {
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'E',
  'E',
  'C',
  'C',
  'E',
  'E',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'E',
  'E',
  'E',
  'E',
  'E',
  'E',
  'E',
  'E',
  'N',
  'N',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C',
  'C' // no trailing comma
};

// Map from machine-independent register number to register_save_type
const        int register_save_type[] = {
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  0 // no trailing comma
};


const        int   reduceOp[] = {
  0, // 0
  0, // 1
  0, // 2
  0, // 3
  0, // 4
  0, // 5
  0, // 6
  0, // 7
  immI_rule, // 8
  immI0_rule, // 9
  immI1_rule, // 10
  immI_M1_rule, // 11
  immI2_rule, // 12
  immI8_rule, // 13
  immI16_rule, // 14
  immI_32_rule, // 15
  immI_64_rule, // 16
  immP_rule, // 17
  immP0_rule, // 18
  immN_rule, // 19
  immN0_rule, // 20
  immP31_rule, // 21
  immL_rule, // 22
  immL8_rule, // 23
  immUL32_rule, // 24
  immL32_rule, // 25
  immL0_rule, // 26
  immL1_rule, // 27
  immL_M1_rule, // 28
  immL10_rule, // 29
  immL_127_rule, // 30
  immL_32bits_rule, // 31
  immF0_rule, // 32
  immF_rule, // 33
  immD0_rule, // 34
  immD_rule, // 35
  immI_16_rule, // 36
  immI_24_rule, // 37
  immI_255_rule, // 38
  immI_65535_rule, // 39
  immL_255_rule, // 40
  immL_65535_rule, // 41
  rRegI_rule, // 42
  rax_RegI_rule, // 43
  rbx_RegI_rule, // 44
  rcx_RegI_rule, // 45
  rdx_RegI_rule, // 46
  rdi_RegI_rule, // 47
  no_rcx_RegI_rule, // 48
  no_rax_rdx_RegI_rule, // 49
  any_RegP_rule, // 50
  rRegP_rule, // 51
  rRegN_rule, // 52
  no_rax_RegP_rule, // 53
  no_rbp_RegP_rule, // 54
  no_rax_rbx_RegP_rule, // 55
  rax_RegP_rule, // 56
  rax_RegN_rule, // 57
  rbx_RegP_rule, // 58
  rsi_RegP_rule, // 59
  rdi_RegP_rule, // 60
  rbp_RegP_rule, // 61
  r15_RegP_rule, // 62
  rRegL_rule, // 63
  no_rax_rdx_RegL_rule, // 64
  no_rax_RegL_rule, // 65
  no_rcx_RegL_rule, // 66
  rax_RegL_rule, // 67
  rcx_RegL_rule, // 68
  rdx_RegL_rule, // 69
  rFlagsReg_rule, // 70
  rFlagsRegU_rule, // 71
  rFlagsRegUCF_rule, // 72
  regF_rule, // 73
  regD_rule, // 74
  indirect_rule, // 75
  indOffset8_rule, // 76
  indOffset32_rule, // 77
  indIndexOffset_rule, // 78
  indIndex_rule, // 79
  indIndexScale_rule, // 80
  indIndexScaleOffset_rule, // 81
  indPosIndexScaleOffset_rule, // 82
  indCompressedOopOffset_rule, // 83
  indirectNarrow_rule, // 84
  indOffset8Narrow_rule, // 85
  indOffset32Narrow_rule, // 86
  indIndexOffsetNarrow_rule, // 87
  indIndexNarrow_rule, // 88
  indIndexScaleNarrow_rule, // 89
  indIndexScaleOffsetNarrow_rule, // 90
  indPosIndexScaleOffsetNarrow_rule, // 91
  stackSlotP_rule, // 92
  stackSlotI_rule, // 93
  stackSlotF_rule, // 94
  stackSlotD_rule, // 95
  stackSlotL_rule, // 96
  cmpOp_rule, // 97
  cmpOpU_rule, // 98
  cmpOpUCF_rule, // 99
  cmpOpUCF2_rule, // 100
  // last operand
  memory_rule, // 101
  // last operand class
  _AddP_any_RegP_rRegL_rule, // 102
  _LShiftL_rRegL_immI2_rule, // 103
  _AddP_any_RegP__LShiftL_rRegL_immI2_rule, // 104
  _ConvI2L_rRegI__rule, // 105
  _LShiftL__ConvI2L_rRegI__immI2_rule, // 106
  _AddP_any_RegP__LShiftL__ConvI2L_rRegI__immI2_rule, // 107
  _DecodeN_rRegN__rule, // 108
  _AddP__DecodeN_rRegN__rRegL_rule, // 109
  _AddP__DecodeN_rRegN___LShiftL_rRegL_immI2_rule, // 110
  _AddP__DecodeN_rRegN___LShiftL__ConvI2L_rRegI__immI2_rule, // 111
  _LoadB_memory__rule, // 112
  _LoadUB_memory__rule, // 113
  _AndI__LoadUB_memory__immI8_rule, // 114
  _LoadS_memory__rule, // 115
  _LShiftI__LoadS_memory__immI_24_rule, // 116
  _LoadUS_memory__rule, // 117
  _LShiftI__LoadUS_memory__immI_24_rule, // 118
  _AndI__LoadUS_memory__immI_255_rule, // 119
  _AndI__LoadUS_memory__immI16_rule, // 120
  _LoadI_memory__rule, // 121
  _LShiftI__LoadI_memory__immI_24_rule, // 122
  _LShiftI__LoadI_memory__immI_16_rule, // 123
  _AndI__LoadI_memory__immI_255_rule, // 124
  _AndI__LoadI_memory__immI_65535_rule, // 125
  _AndI__LoadI_memory__immI_rule, // 126
  _LoadL_memory__rule, // 127
  _AddL__LShiftL_rRegL_immI2_immL32_rule, // 128
  _Binary_cmpOp_rFlagsReg_rule, // 129
  _Binary_rRegI_rRegI_rule, // 130
  _Binary_cmpOpU_rFlagsRegU_rule, // 131
  _Binary_cmpOpUCF_rFlagsRegUCF_rule, // 132
  _Binary_rRegI__LoadI_memory__rule, // 133
  _Binary_rRegN_rRegN_rule, // 134
  _Binary_rRegP_rRegP_rule, // 135
  _Binary_rRegL_rRegL_rule, // 136
  _Binary_rRegL__LoadL_memory__rule, // 137
  _Binary_regF_regF_rule, // 138
  _Binary_regD_regD_rule, // 139
  _AddI__LoadI_memory__rRegI_rule, // 140
  _AddI_rRegI__LoadI_memory__rule, // 141
  _AddI__LoadI_memory__immI_rule, // 142
  _AddI__LoadI_memory__immI1_rule, // 143
  _AddI__LoadI_memory__immI_M1_rule, // 144
  _AddL__LoadL_memory__rRegL_rule, // 145
  _AddL_rRegL__LoadL_memory__rule, // 146
  _AddL__LoadL_memory__immL32_rule, // 147
  _AddL__LoadL_memory__immL1_rule, // 148
  _AddL__LoadL_memory__immL_M1_rule, // 149
  _Binary_rax_RegP_rRegP_rule, // 150
  _Binary_rax_RegI_rRegI_rule, // 151
  _Binary_rax_RegL_rRegL_rule, // 152
  _Binary_rax_RegN_rRegN_rule, // 153
  _SubI__LoadI_memory__rRegI_rule, // 154
  _SubI__LoadI_memory__immI_rule, // 155
  _SubL__LoadL_memory__rRegL_rule, // 156
  _SubL__LoadL_memory__immL32_rule, // 157
  _SubI_immI0_rRegI_rule, // 158
  _SubI_immI0__LoadI_memory__rule, // 159
  _SubL_immL0__LoadL_memory__rule, // 160
  _LShiftI__LoadI_memory__immI1_rule, // 161
  _LShiftI__LoadI_memory__immI8_rule, // 162
  _LShiftI__LoadI_memory__rcx_RegI_rule, // 163
  _RShiftI__LoadI_memory__immI1_rule, // 164
  _RShiftI__LoadI_memory__immI8_rule, // 165
  _RShiftI__LoadI_memory__rcx_RegI_rule, // 166
  _URShiftI__LoadI_memory__immI1_rule, // 167
  _URShiftI__LoadI_memory__immI8_rule, // 168
  _URShiftI__LoadI_memory__rcx_RegI_rule, // 169
  _LShiftL__LoadL_memory__immI1_rule, // 170
  _LShiftL__LoadL_memory__immI8_rule, // 171
  _LShiftL__LoadL_memory__rcx_RegI_rule, // 172
  _RShiftL__LoadL_memory__immI1_rule, // 173
  _RShiftL__LoadL_memory__immI8_rule, // 174
  _RShiftL__LoadL_memory__rcx_RegI_rule, // 175
  _URShiftL__LoadL_memory__immI1_rule, // 176
  _URShiftL__LoadL_memory__immI8_rule, // 177
  _URShiftL__LoadL_memory__rcx_RegI_rule, // 178
  _LShiftI_rRegI_immI_24_rule, // 179
  _LShiftI_rRegI_immI_16_rule, // 180
  _LShiftI_rRegI_immI1_rule, // 181
  _URShiftI_rRegI_immI_M1_rule, // 182
  _LShiftI_rRegI_immI8_rule, // 183
  _URShiftI_rRegI_immI8_rule, // 184
  _LShiftI_no_rcx_RegI_rcx_RegI_rule, // 185
  _SubI_immI0_rcx_RegI_rule, // 186
  _URShiftI_no_rcx_RegI__SubI_immI0_rcx_RegI_rule, // 187
  _SubI_immI_32_rcx_RegI_rule, // 188
  _URShiftI_no_rcx_RegI__SubI_immI_32_rcx_RegI_rule, // 189
  _URShiftI_rRegI_immI1_rule, // 190
  _LShiftI_rRegI_immI_M1_rule, // 191
  _URShiftI_no_rcx_RegI_rcx_RegI_rule, // 192
  _LShiftI_no_rcx_RegI__SubI_immI0_rcx_RegI_rule, // 193
  _LShiftI_no_rcx_RegI__SubI_immI_32_rcx_RegI_rule, // 194
  _LShiftL_rRegL_immI1_rule, // 195
  _URShiftL_rRegL_immI_M1_rule, // 196
  _LShiftL_rRegL_immI8_rule, // 197
  _URShiftL_rRegL_immI8_rule, // 198
  _LShiftL_no_rcx_RegL_rcx_RegI_rule, // 199
  _URShiftL_no_rcx_RegL__SubI_immI0_rcx_RegI_rule, // 200
  _SubI_immI_64_rcx_RegI_rule, // 201
  _URShiftL_no_rcx_RegL__SubI_immI_64_rcx_RegI_rule, // 202
  _URShiftL_rRegL_immI1_rule, // 203
  _LShiftL_rRegL_immI_M1_rule, // 204
  _URShiftL_no_rcx_RegL_rcx_RegI_rule, // 205
  _LShiftL_no_rcx_RegL__SubI_immI0_rcx_RegI_rule, // 206
  _LShiftL_no_rcx_RegL__SubI_immI_64_rcx_RegI_rule, // 207
  _AndI_rRegI_immI_255_rule, // 208
  _AndI_rRegI_immI_65535_rule, // 209
  _AndI__LoadI_memory__rRegI_rule, // 210
  _AndI_rRegI__LoadI_memory__rule, // 211
  _OrI__LoadI_memory__rRegI_rule, // 212
  _OrI_rRegI__LoadI_memory__rule, // 213
  _OrI__LoadI_memory__immI_rule, // 214
  _XorI__LoadI_memory__rRegI_rule, // 215
  _XorI_rRegI__LoadI_memory__rule, // 216
  _XorI__LoadI_memory__immI_rule, // 217
  _AndL__LoadL_memory__rRegL_rule, // 218
  _AndL_rRegL__LoadL_memory__rule, // 219
  _AndL__LoadL_memory__immL32_rule, // 220
  _CastP2X_any_RegP__rule, // 221
  _OrL__LoadL_memory__rRegL_rule, // 222
  _OrL_rRegL__LoadL_memory__rule, // 223
  _OrL__LoadL_memory__immL32_rule, // 224
  _XorL__LoadL_memory__rRegL_rule, // 225
  _XorL_rRegL__LoadL_memory__rule, // 226
  _XorL__LoadL_memory__immL32_rule, // 227
  _CmpLTMask_rRegI_rRegI_rule, // 228
  _AndI__CmpLTMask_rRegI_rRegI_rRegI_rule, // 229
  _SubI_rRegI_rRegI_rule, // 230
  _AndI_rRegI__CmpLTMask_rRegI_rRegI_rule, // 231
  _LoadF_memory__rule, // 232
  _LoadD_memory__rule, // 233
  _ConvF2D_regF__rule, // 234
  _SqrtD__ConvF2D_regF___rule, // 235
  _ConvF2D__LoadF_memory___rule, // 236
  _SqrtD__ConvF2D__LoadF_memory____rule, // 237
  _ConvF2D_immF__rule, // 238
  _SqrtD__ConvF2D_immF___rule, // 239
  _ConvI2L__LoadI_memory___rule, // 240
  _Binary_rdi_RegP_rcx_RegI_rule, // 241
  _Binary_rsi_RegP_rdx_RegI_rule, // 242
  _Binary_rdi_RegP_rdx_RegI_rule, // 243
  _Binary_rsi_RegP_rax_RegI_rule, // 244
  _Binary_rdi_RegP_rsi_RegP_rule, // 245
  _AndI_rRegI_immI_rule, // 246
  _LoadP_memory__rule, // 247
  _LoadN_memory__rule, // 248
  _AndL_rRegL_immL32_rule, // 249
  _PartialSubtypeCheck_rsi_RegP_rax_RegP_rule, // 250
  // last internally defined operand
  stackSlotI_rule, // 251
  stackSlotL_rule, // 252
  stackSlotP_rule, // 253
  stackSlotF_rule, // 254
  stackSlotD_rule, // 255
  rRegP_rule, // 256
  rRegP_rule, // 257
  rRegP_rule, // 258
  rRegP_rule, // 259
  rRegP_rule, // 260
  rRegP_rule, // 261
  rRegP_rule, // 262
  rRegP_rule, // 263
  rRegP_rule, // 264
  rRegP_rule, // 265
  rRegP_rule, // 266
  rRegP_rule, // 267
  rRegP_rule, // 268
  rRegI_rule, // 269
  rRegI_rule, // 270
  rRegL_rule, // 271
  rRegL_rule, // 272
  rRegL_rule, // 273
  rRegL_rule, // 274
  rRegP_rule, // 275
  rRegP_rule, // 276
  rRegP_rule, // 277
  regF_rule, // 278
  rRegN_rule, // 279
  rRegN_rule, // 280
  regF_rule, // 281
  regD_rule, // 282
  regD_rule, // 283
  rRegI_rule, // 284
  rRegL_rule, // 285
  rRegP_rule, // 286
  regF_rule, // 287
  regD_rule, // 288
  rRegI_rule, // 289
  rRegI_rule, // 290
  rRegI_rule, // 291
  rRegI_rule, // 292
  Universe_rule, // 293
  rRegP_rule, // 294
  rRegL_rule, // 295
  rRegP_rule, // 296
  rRegP_rule, // 297
  rRegI_rule, // 298
  rFlagsReg_rule, // 299
  rFlagsReg_rule, // 300
  rFlagsReg_rule, // 301
  rFlagsRegU_rule, // 302
  rFlagsRegUCF_rule, // 303
  rFlagsRegU_rule, // 304
  rFlagsRegUCF_rule, // 305
  rFlagsRegU_rule, // 306
  rFlagsRegUCF_rule, // 307
  rFlagsRegU_rule, // 308
  rFlagsRegUCF_rule, // 309
  rFlagsRegU_rule, // 310
  rFlagsRegUCF_rule, // 311
  rFlagsRegU_rule, // 312
  rFlagsRegUCF_rule, // 313
  regF_rule, // 314
  regD_rule, // 315
  regF_rule, // 316
  regD_rule, // 317
  regF_rule, // 318
  regD_rule, // 319
  rRegI_rule, // 320
  regF_rule, // 321
  rRegL_rule, // 322
  regD_rule, // 323
  regD_rule, // 324
  rRegI_rule, // 325
  rRegL_rule, // 326
  regF_rule, // 327
  regD_rule, // 328
  rFlagsReg_rule, // 329
  rFlagsReg_rule, // 330
  rFlagsReg_rule, // 331
  rFlagsReg_rule, // 332
  rFlagsReg_rule, // 333
  rFlagsReg_rule, // 334
  rFlagsReg_rule, // 335
  rFlagsRegU_rule, // 336
  rFlagsRegU_rule, // 337
  rFlagsRegU_rule, // 338
  rFlagsRegU_rule, // 339
  rFlagsRegU_rule, // 340
  rFlagsRegU_rule, // 341
  rFlagsRegU_rule, // 342
  rFlagsReg_rule, // 343
  rFlagsReg_rule, // 344
  rFlagsReg_rule, // 345
  rFlagsRegU_rule, // 346
  rFlagsRegU_rule, // 347
  rFlagsRegU_rule, // 348
  rFlagsRegU_rule, // 349
  rFlagsReg_rule, // 350
  rFlagsReg_rule, // 351
  rFlagsReg_rule, // 352
  rFlagsReg_rule, // 353
  rFlagsReg_rule, // 354
  rFlagsReg_rule, // 355
  rFlagsReg_rule, // 356
  rFlagsReg_rule, // 357
  rFlagsReg_rule, // 358
  rFlagsReg_rule, // 359
  rFlagsReg_rule, // 360
  rFlagsReg_rule, // 361
  rFlagsReg_rule, // 362
  Universe_rule, // 363
  r15_RegP_rule, // 364
  rRegI_rule, // 365
  rRegL_rule, // 366
  rRegI_rule, // 367
  rRegL_rule, // 368
  rRegL_rule, // 369
  rRegI_rule, // 370
  rRegI_rule, // 371
  rRegL_rule, // 372
  rRegI_rule, // 373
  rRegI_rule, // 374
  rRegL_rule, // 375
  rRegL_rule, // 376
  rRegL_rule, // 377
  rRegI_rule, // 378
  rRegI_rule, // 379
  rRegI_rule, // 380
  rRegI_rule, // 381
  rRegI_rule, // 382
  rRegL_rule, // 383
  rRegL_rule, // 384
  rRegL_rule, // 385
  rRegL_rule, // 386
  rRegL_rule, // 387
  rRegL_rule, // 388
  rRegI_rule, // 389
  rRegP_rule, // 390
  rRegN_rule, // 391
  rRegP_rule, // 392
  rRegN_rule, // 393
  regF_rule, // 394
  regD_rule, // 395
  regD_rule, // 396
  regD_rule, // 397
  regD_rule, // 398
  regD_rule, // 399
  regD_rule, // 400
  regD_rule, // 401
  Universe_rule, // 402
  Universe_rule, // 403
  Universe_rule, // 404
  Universe_rule, // 405
  Universe_rule, // 406
  Universe_rule, // 407
  Universe_rule, // 408
  Universe_rule, // 409
  Universe_rule, // 410
  Universe_rule, // 411
  Universe_rule, // 412
  Universe_rule, // 413
  Universe_rule, // 414
  Universe_rule, // 415
  Universe_rule, // 416
  Universe_rule, // 417
  Universe_rule, // 418
  Universe_rule, // 419
  Universe_rule, // 420
  Universe_rule, // 421
  Universe_rule, // 422
  Universe_rule, // 423
  Universe_rule, // 424
  Universe_rule, // 425
  Universe_rule, // 426
  Universe_rule, // 427
  Universe_rule, // 428
  Universe_rule, // 429
  Universe_rule, // 430
  Universe_rule, // 431
  Universe_rule, // 432
  Universe_rule, // 433
  Universe_rule, // 434
  Universe_rule, // 435
  Universe_rule, // 436
  Universe_rule, // 437
  Universe_rule, // 438
  Universe_rule, // 439
  rRegI_rule, // 440
  rRegL_rule, // 441
  rRegI_rule, // 442
  rRegI_rule, // 443
  rRegI_rule, // 444
  rRegI_rule, // 445
  rRegI_rule, // 446
  rRegI_rule, // 447
  rRegI_rule, // 448
  rRegI_rule, // 449
  Universe_rule, // 450
  Universe_rule, // 451
  Universe_rule, // 452
  Universe_rule, // 453
  Universe_rule, // 454
  rRegN_rule, // 455
  rRegN_rule, // 456
  rRegP_rule, // 457
  rRegP_rule, // 458
  Universe_rule, // 459
  Universe_rule, // 460
  Universe_rule, // 461
  rRegI_rule, // 462
  rRegI_rule, // 463
  rRegI_rule, // 464
  rRegI_rule, // 465
  rRegI_rule, // 466
  rRegI_rule, // 467
  rRegN_rule, // 468
  rRegN_rule, // 469
  rRegN_rule, // 470
  rRegP_rule, // 471
  rRegP_rule, // 472
  rRegP_rule, // 473
  rRegL_rule, // 474
  rRegL_rule, // 475
  rRegL_rule, // 476
  rRegL_rule, // 477
  rRegL_rule, // 478
  rRegL_rule, // 479
  regF_rule, // 480
  regF_rule, // 481
  regF_rule, // 482
  regD_rule, // 483
  regD_rule, // 484
  regD_rule, // 485
  rRegI_rule, // 486
  rRegI_rule, // 487
  rRegI_rule, // 488
  rRegI_rule, // 489
  Universe_rule, // 490
  Universe_rule, // 491
  Universe_rule, // 492
  rRegI_rule, // 493
  Universe_rule, // 494
  rRegI_rule, // 495
  Universe_rule, // 496
  rRegI_rule, // 497
  rRegL_rule, // 498
  rRegL_rule, // 499
  rRegL_rule, // 500
  rRegL_rule, // 501
  Universe_rule, // 502
  Universe_rule, // 503
  Universe_rule, // 504
  rRegI_rule, // 505
  Universe_rule, // 506
  rRegL_rule, // 507
  Universe_rule, // 508
  rRegL_rule, // 509
  rRegP_rule, // 510
  rRegP_rule, // 511
  rRegP_rule, // 512
  rRegP_rule, // 513
  rRegL_rule, // 514
  rRegI_rule, // 515
  rRegI_rule, // 516
  rRegI_rule, // 517
  rRegI_rule, // 518
  rRegI_rule, // 519
  rRegI_rule, // 520
  rRegI_rule, // 521
  Universe_rule, // 522
  Universe_rule, // 523
  rRegL_rule, // 524
  rRegI_rule, // 525
  rRegL_rule, // 526
  Universe_rule, // 527
  Universe_rule, // 528
  rRegP_rule, // 529
  rRegI_rule, // 530
  Universe_rule, // 531
  rRegL_rule, // 532
  Universe_rule, // 533
  rRegI_rule, // 534
  rRegI_rule, // 535
  rRegI_rule, // 536
  rRegI_rule, // 537
  rRegI_rule, // 538
  rRegL_rule, // 539
  rRegL_rule, // 540
  rRegL_rule, // 541
  rRegL_rule, // 542
  rRegL_rule, // 543
  rdx_RegL_rule, // 544
  rax_RegI_rule, // 545
  rax_RegL_rule, // 546
  Universe_rule, // 547
  Universe_rule, // 548
  0, // 549
  0, // 550
  0, // 551
  0, // 552
  rdx_RegL_rule, // 553
  rdx_RegI_rule, // 554
  rdx_RegL_rule, // 555
  rRegI_rule, // 556
  Universe_rule, // 557
  rRegI_rule, // 558
  Universe_rule, // 559
  rRegI_rule, // 560
  Universe_rule, // 561
  rRegI_rule, // 562
  Universe_rule, // 563
  rRegI_rule, // 564
  Universe_rule, // 565
  rRegI_rule, // 566
  Universe_rule, // 567
  rRegI_rule, // 568
  Universe_rule, // 569
  rRegI_rule, // 570
  Universe_rule, // 571
  rRegI_rule, // 572
  Universe_rule, // 573
  rRegL_rule, // 574
  Universe_rule, // 575
  rRegL_rule, // 576
  Universe_rule, // 577
  rRegL_rule, // 578
  Universe_rule, // 579
  rRegL_rule, // 580
  Universe_rule, // 581
  rRegL_rule, // 582
  Universe_rule, // 583
  rRegL_rule, // 584
  Universe_rule, // 585
  rRegL_rule, // 586
  Universe_rule, // 587
  rRegL_rule, // 588
  Universe_rule, // 589
  rRegL_rule, // 590
  Universe_rule, // 591
  rRegI_rule, // 592
  rRegI_rule, // 593
  0, // 594
  0, // 595
  0, // 596
  rRegI_rule, // 597
  rRegI_rule, // 598
  rRegI_rule, // 599
  rRegI_rule, // 600
  no_rcx_RegI_rule, // 601
  no_rcx_RegI_rule, // 602
  no_rcx_RegI_rule, // 603
  no_rcx_RegI_rule, // 604
  0, // 605
  0, // 606
  0, // 607
  rRegI_rule, // 608
  rRegI_rule, // 609
  rRegI_rule, // 610
  rRegI_rule, // 611
  no_rcx_RegI_rule, // 612
  no_rcx_RegI_rule, // 613
  no_rcx_RegI_rule, // 614
  no_rcx_RegI_rule, // 615
  0, // 616
  0, // 617
  0, // 618
  rRegL_rule, // 619
  rRegL_rule, // 620
  rRegL_rule, // 621
  rRegL_rule, // 622
  no_rcx_RegL_rule, // 623
  no_rcx_RegL_rule, // 624
  no_rcx_RegL_rule, // 625
  no_rcx_RegL_rule, // 626
  0, // 627
  0, // 628
  0, // 629
  rRegL_rule, // 630
  rRegL_rule, // 631
  rRegL_rule, // 632
  rRegL_rule, // 633
  no_rcx_RegL_rule, // 634
  no_rcx_RegL_rule, // 635
  no_rcx_RegL_rule, // 636
  no_rcx_RegL_rule, // 637
  rRegI_rule, // 638
  rRegI_rule, // 639
  rRegL_rule, // 640
  rRegI_rule, // 641
  rRegL_rule, // 642
  rRegI_rule, // 643
  rRegI_rule, // 644
  rRegI_rule, // 645
  Universe_rule, // 646
  Universe_rule, // 647
  Universe_rule, // 648
  rRegI_rule, // 649
  rRegI_rule, // 650
  rRegI_rule, // 651
  rRegI_rule, // 652
  Universe_rule, // 653
  Universe_rule, // 654
  Universe_rule, // 655
  rRegI_rule, // 656
  rRegI_rule, // 657
  rRegI_rule, // 658
  rRegI_rule, // 659
  rRegI_rule, // 660
  Universe_rule, // 661
  Universe_rule, // 662
  Universe_rule, // 663
  rRegL_rule, // 664
  rRegL_rule, // 665
  rRegL_rule, // 666
  rRegL_rule, // 667
  rRegL_rule, // 668
  rRegL_rule, // 669
  Universe_rule, // 670
  Universe_rule, // 671
  Universe_rule, // 672
  rRegL_rule, // 673
  rRegL_rule, // 674
  rRegL_rule, // 675
  rRegL_rule, // 676
  rRegL_rule, // 677
  rRegL_rule, // 678
  Universe_rule, // 679
  Universe_rule, // 680
  Universe_rule, // 681
  rRegL_rule, // 682
  rRegL_rule, // 683
  rRegL_rule, // 684
  rRegL_rule, // 685
  rRegL_rule, // 686
  Universe_rule, // 687
  Universe_rule, // 688
  Universe_rule, // 689
  rRegI_rule, // 690
  rRegI_rule, // 691
  rRegI_rule, // 692
  rRegI_rule, // 693
  rRegI_rule, // 694
  rRegI_rule, // 695
  rRegI_rule, // 696
  rRegI_rule, // 697
  rRegI_rule, // 698
  rRegI_rule, // 699
  rRegI_rule, // 700
  rRegI_rule, // 701
  rRegI_rule, // 702
  rRegI_rule, // 703
  regF_rule, // 704
  regF_rule, // 705
  regF_rule, // 706
  regF_rule, // 707
  regD_rule, // 708
  regD_rule, // 709
  regD_rule, // 710
  regD_rule, // 711
  regF_rule, // 712
  regF_rule, // 713
  regF_rule, // 714
  regD_rule, // 715
  regD_rule, // 716
  regD_rule, // 717
  regF_rule, // 718
  regF_rule, // 719
  regF_rule, // 720
  regF_rule, // 721
  regD_rule, // 722
  regD_rule, // 723
  regD_rule, // 724
  regD_rule, // 725
  regF_rule, // 726
  regF_rule, // 727
  regF_rule, // 728
  regD_rule, // 729
  regD_rule, // 730
  regD_rule, // 731
  regF_rule, // 732
  regF_rule, // 733
  regF_rule, // 734
  regD_rule, // 735
  regD_rule, // 736
  regD_rule, // 737
  regD_rule, // 738
  regD_rule, // 739
  regD_rule, // 740
  regD_rule, // 741
  regD_rule, // 742
  regD_rule, // 743
  regD_rule, // 744
  regF_rule, // 745
  regF_rule, // 746
  rRegI_rule, // 747
  rRegL_rule, // 748
  rRegI_rule, // 749
  rRegL_rule, // 750
  regF_rule, // 751
  regF_rule, // 752
  regD_rule, // 753
  regD_rule, // 754
  regF_rule, // 755
  regD_rule, // 756
  regF_rule, // 757
  regF_rule, // 758
  regD_rule, // 759
  regD_rule, // 760
  rRegL_rule, // 761
  rRegL_rule, // 762
  rRegL_rule, // 763
  rRegL_rule, // 764
  rRegI_rule, // 765
  stackSlotI_rule, // 766
  stackSlotF_rule, // 767
  stackSlotL_rule, // 768
  stackSlotD_rule, // 769
  regD_rule, // 770
  regD_rule, // 771
  regD_rule, // 772
  regD_rule, // 773
  regD_rule, // 774
  regD_rule, // 775
  regD_rule, // 776
  regD_rule, // 777
  regD_rule, // 778
  regD_rule, // 779
  regD_rule, // 780
  regD_rule, // 781
  regD_rule, // 782
  regD_rule, // 783
  regD_rule, // 784
  Universe_rule, // 785
  rax_RegI_rule, // 786
  rbx_RegI_rule, // 787
  rax_RegI_rule, // 788
  rax_RegI_rule, // 789
  rRegI_rule, // 790
  0, // 791
  rRegI_rule, // 792
  0, // 793
  rRegI_rule, // 794
  Universe_rule, // 795
  Universe_rule, // 796
  Universe_rule, // 797
  Universe_rule, // 798
  Universe_rule, // 799
  Universe_rule, // 800
  Universe_rule, // 801
  Universe_rule, // 802
  rdi_RegP_rule, // 803
  Universe_rule, // 804
  Universe_rule, // 805
  Universe_rule, // 806
  Universe_rule, // 807
  Universe_rule, // 808
  Universe_rule, // 809
  Universe_rule, // 810
  Universe_rule, // 811
  Universe_rule, // 812
  Universe_rule, // 813
  Universe_rule, // 814
  Universe_rule, // 815
  Universe_rule, // 816
  Universe_rule, // 817
  Universe_rule, // 818
  Universe_rule, // 819
  Universe_rule, // 820
  rax_RegP_rule, // 821
  Universe_rule, // 822
  Universe_rule, // 823
  // last instruction
  0 // no trailing comma
};

const        int   leftOp[] = {
  0, // 0
  0, // 1
  0, // 2
  0, // 3
  0, // 4
  0, // 5
  0, // 6
  0, // 7
  0, // 8
  0, // 9
  0, // 10
  0, // 11
  0, // 12
  0, // 13
  0, // 14
  0, // 15
  0, // 16
  0, // 17
  0, // 18
  0, // 19
  0, // 20
  0, // 21
  0, // 22
  0, // 23
  0, // 24
  0, // 25
  0, // 26
  0, // 27
  0, // 28
  0, // 29
  0, // 30
  0, // 31
  0, // 32
  0, // 33
  0, // 34
  0, // 35
  0, // 36
  0, // 37
  0, // 38
  0, // 39
  0, // 40
  0, // 41
  0, // 42
  0, // 43
  0, // 44
  0, // 45
  0, // 46
  0, // 47
  0, // 48
  0, // 49
  0, // 50
  0, // 51
  0, // 52
  0, // 53
  0, // 54
  0, // 55
  0, // 56
  0, // 57
  0, // 58
  0, // 59
  0, // 60
  0, // 61
  0, // 62
  0, // 63
  0, // 64
  0, // 65
  0, // 66
  0, // 67
  0, // 68
  0, // 69
  0, // 70
  0, // 71
  0, // 72
  0, // 73
  0, // 74
  0, // 75
  any_RegP_rule, // 76
  any_RegP_rule, // 77
  _AddP_any_RegP_rRegL_rule, // 78
  any_RegP_rule, // 79
  any_RegP_rule, // 80
  _AddP_any_RegP__LShiftL_rRegL_immI2_rule, // 81
  _AddP_any_RegP__LShiftL__ConvI2L_rRegI__immI2_rule, // 82
  _DecodeN_rRegN__rule, // 83
  rRegN_rule, // 84
  _DecodeN_rRegN__rule, // 85
  _DecodeN_rRegN__rule, // 86
  _AddP__DecodeN_rRegN__rRegL_rule, // 87
  _DecodeN_rRegN__rule, // 88
  _DecodeN_rRegN__rule, // 89
  _AddP__DecodeN_rRegN___LShiftL_rRegL_immI2_rule, // 90
  _AddP__DecodeN_rRegN___LShiftL__ConvI2L_rRegI__immI2_rule, // 91
  0, // 92
  0, // 93
  0, // 94
  0, // 95
  0, // 96
  0, // 97
  0, // 98
  0, // 99
  0, // 100
  // last operand
  0, // 101
  // last operand class
  any_RegP_rule, // 102
  rRegL_rule, // 103
  any_RegP_rule, // 104
  rRegI_rule, // 105
  _ConvI2L_rRegI__rule, // 106
  any_RegP_rule, // 107
  rRegN_rule, // 108
  _DecodeN_rRegN__rule, // 109
  _DecodeN_rRegN__rule, // 110
  _DecodeN_rRegN__rule, // 111
  memory_rule, // 112
  memory_rule, // 113
  _LoadUB_memory__rule, // 114
  memory_rule, // 115
  _LoadS_memory__rule, // 116
  memory_rule, // 117
  _LoadUS_memory__rule, // 118
  _LoadUS_memory__rule, // 119
  _LoadUS_memory__rule, // 120
  memory_rule, // 121
  _LoadI_memory__rule, // 122
  _LoadI_memory__rule, // 123
  _LoadI_memory__rule, // 124
  _LoadI_memory__rule, // 125
  _LoadI_memory__rule, // 126
  memory_rule, // 127
  _LShiftL_rRegL_immI2_rule, // 128
  cmpOp_rule, // 129
  rRegI_rule, // 130
  cmpOpU_rule, // 131
  cmpOpUCF_rule, // 132
  rRegI_rule, // 133
  rRegN_rule, // 134
  rRegP_rule, // 135
  rRegL_rule, // 136
  rRegL_rule, // 137
  regF_rule, // 138
  regD_rule, // 139
  _LoadI_memory__rule, // 140
  rRegI_rule, // 141
  _LoadI_memory__rule, // 142
  _LoadI_memory__rule, // 143
  _LoadI_memory__rule, // 144
  _LoadL_memory__rule, // 145
  rRegL_rule, // 146
  _LoadL_memory__rule, // 147
  _LoadL_memory__rule, // 148
  _LoadL_memory__rule, // 149
  rax_RegP_rule, // 150
  rax_RegI_rule, // 151
  rax_RegL_rule, // 152
  rax_RegN_rule, // 153
  _LoadI_memory__rule, // 154
  _LoadI_memory__rule, // 155
  _LoadL_memory__rule, // 156
  _LoadL_memory__rule, // 157
  immI0_rule, // 158
  immI0_rule, // 159
  immL0_rule, // 160
  _LoadI_memory__rule, // 161
  _LoadI_memory__rule, // 162
  _LoadI_memory__rule, // 163
  _LoadI_memory__rule, // 164
  _LoadI_memory__rule, // 165
  _LoadI_memory__rule, // 166
  _LoadI_memory__rule, // 167
  _LoadI_memory__rule, // 168
  _LoadI_memory__rule, // 169
  _LoadL_memory__rule, // 170
  _LoadL_memory__rule, // 171
  _LoadL_memory__rule, // 172
  _LoadL_memory__rule, // 173
  _LoadL_memory__rule, // 174
  _LoadL_memory__rule, // 175
  _LoadL_memory__rule, // 176
  _LoadL_memory__rule, // 177
  _LoadL_memory__rule, // 178
  rRegI_rule, // 179
  rRegI_rule, // 180
  rRegI_rule, // 181
  rRegI_rule, // 182
  rRegI_rule, // 183
  rRegI_rule, // 184
  no_rcx_RegI_rule, // 185
  immI0_rule, // 186
  no_rcx_RegI_rule, // 187
  immI_32_rule, // 188
  no_rcx_RegI_rule, // 189
  rRegI_rule, // 190
  rRegI_rule, // 191
  no_rcx_RegI_rule, // 192
  no_rcx_RegI_rule, // 193
  no_rcx_RegI_rule, // 194
  rRegL_rule, // 195
  rRegL_rule, // 196
  rRegL_rule, // 197
  rRegL_rule, // 198
  no_rcx_RegL_rule, // 199
  no_rcx_RegL_rule, // 200
  immI_64_rule, // 201
  no_rcx_RegL_rule, // 202
  rRegL_rule, // 203
  rRegL_rule, // 204
  no_rcx_RegL_rule, // 205
  no_rcx_RegL_rule, // 206
  no_rcx_RegL_rule, // 207
  rRegI_rule, // 208
  rRegI_rule, // 209
  _LoadI_memory__rule, // 210
  rRegI_rule, // 211
  _LoadI_memory__rule, // 212
  rRegI_rule, // 213
  _LoadI_memory__rule, // 214
  _LoadI_memory__rule, // 215
  rRegI_rule, // 216
  _LoadI_memory__rule, // 217
  _LoadL_memory__rule, // 218
  rRegL_rule, // 219
  _LoadL_memory__rule, // 220
  any_RegP_rule, // 221
  _LoadL_memory__rule, // 222
  rRegL_rule, // 223
  _LoadL_memory__rule, // 224
  _LoadL_memory__rule, // 225
  rRegL_rule, // 226
  _LoadL_memory__rule, // 227
  rRegI_rule, // 228
  _CmpLTMask_rRegI_rRegI_rule, // 229
  rRegI_rule, // 230
  rRegI_rule, // 231
  memory_rule, // 232
  memory_rule, // 233
  regF_rule, // 234
  _ConvF2D_regF__rule, // 235
  _LoadF_memory__rule, // 236
  _ConvF2D__LoadF_memory___rule, // 237
  immF_rule, // 238
  _ConvF2D_immF__rule, // 239
  _LoadI_memory__rule, // 240
  rdi_RegP_rule, // 241
  rsi_RegP_rule, // 242
  rdi_RegP_rule, // 243
  rsi_RegP_rule, // 244
  rdi_RegP_rule, // 245
  rRegI_rule, // 246
  memory_rule, // 247
  memory_rule, // 248
  rRegL_rule, // 249
  rsi_RegP_rule, // 250
  // last internally defined operand
  rRegI_rule, // 251
  rRegL_rule, // 252
  rRegP_rule, // 253
  regF_rule, // 254
  regD_rule, // 255
  indOffset8_rule, // 256
  indOffset32_rule, // 257
  indIndexOffset_rule, // 258
  indIndexScale_rule, // 259
  indIndexScaleOffset_rule, // 260
  indPosIndexScaleOffset_rule, // 261
  indCompressedOopOffset_rule, // 262
  indOffset8Narrow_rule, // 263
  indOffset32Narrow_rule, // 264
  indIndexOffsetNarrow_rule, // 265
  indIndexScaleNarrow_rule, // 266
  indIndexScaleOffsetNarrow_rule, // 267
  indPosIndexScaleOffsetNarrow_rule, // 268
  immI_rule, // 269
  immI0_rule, // 270
  immL_rule, // 271
  immL0_rule, // 272
  immUL32_rule, // 273
  immL32_rule, // 274
  immP_rule, // 275
  immP0_rule, // 276
  immP31_rule, // 277
  immF_rule, // 278
  immN0_rule, // 279
  immN_rule, // 280
  immF0_rule, // 281
  immD_rule, // 282
  immD0_rule, // 283
  stackSlotI_rule, // 284
  stackSlotL_rule, // 285
  stackSlotP_rule, // 286
  stackSlotF_rule, // 287
  stackSlotD_rule, // 288
  rRegI_rule, // 289
  _LoadI_memory__rule, // 290
  rRegL_rule, // 291
  _LoadL_memory__rule, // 292
  0, // 293
  rRegL_rule, // 294
  rRegP_rule, // 295
  rRegP_rule, // 296
  rRegP_rule, // 297
  rRegI_rule, // 298
  memory_rule, // 299
  memory_rule, // 300
  memory_rule, // 301
  regF_rule, // 302
  regF_rule, // 303
  regF_rule, // 304
  regF_rule, // 305
  regF_rule, // 306
  regF_rule, // 307
  regD_rule, // 308
  regD_rule, // 309
  regD_rule, // 310
  regD_rule, // 311
  regD_rule, // 312
  regD_rule, // 313
  regF_rule, // 314
  regD_rule, // 315
  regF_rule, // 316
  regD_rule, // 317
  regF_rule, // 318
  regD_rule, // 319
  stackSlotF_rule, // 320
  stackSlotI_rule, // 321
  stackSlotD_rule, // 322
  stackSlotL_rule, // 323
  stackSlotL_rule, // 324
  regF_rule, // 325
  regD_rule, // 326
  rRegI_rule, // 327
  rRegL_rule, // 328
  rRegI_rule, // 329
  rRegI_rule, // 330
  rRegI_rule, // 331
  rRegI_rule, // 332
  _AndI_rRegI_immI_rule, // 333
  _AndI_rRegI__LoadI_memory__rule, // 334
  _AndI__LoadI_memory__rRegI_rule, // 335
  rRegI_rule, // 336
  rRegI_rule, // 337
  rRegI_rule, // 338
  rRegI_rule, // 339
  rRegP_rule, // 340
  rRegP_rule, // 341
  rRegP_rule, // 342
  rRegP_rule, // 343
  _LoadP_memory__rule, // 344
  _LoadP_memory__rule, // 345
  rRegN_rule, // 346
  rRegN_rule, // 347
  rRegN_rule, // 348
  immN_rule, // 349
  rRegN_rule, // 350
  _LoadN_memory__rule, // 351
  _LoadN_memory__rule, // 352
  rRegL_rule, // 353
  rRegL_rule, // 354
  rRegL_rule, // 355
  rRegL_rule, // 356
  _AndL_rRegL_immL32_rule, // 357
  _AndL_rRegL__LoadL_memory__rule, // 358
  _AndL__LoadL_memory__rRegL_rule, // 359
  _PartialSubtypeCheck_rsi_RegP_rax_RegP_rule, // 360
  rRegP_rule, // 361
  rRegP_rule, // 362
  0, // 363
  0, // 364
  memory_rule, // 365
  _LoadB_memory__rule, // 366
  memory_rule, // 367
  _LoadUB_memory__rule, // 368
  _AndI__LoadUB_memory__immI8_rule, // 369
  memory_rule, // 370
  _LShiftI__LoadS_memory__immI_24_rule, // 371
  _LoadS_memory__rule, // 372
  memory_rule, // 373
  _LShiftI__LoadUS_memory__immI_24_rule, // 374
  _LoadUS_memory__rule, // 375
  _AndI__LoadUS_memory__immI_255_rule, // 376
  _AndI__LoadUS_memory__immI16_rule, // 377
  memory_rule, // 378
  _LShiftI__LoadI_memory__immI_24_rule, // 379
  _LoadI_memory__rule, // 380
  _LShiftI__LoadI_memory__immI_16_rule, // 381
  _LoadI_memory__rule, // 382
  _LoadI_memory__rule, // 383
  _AndI__LoadI_memory__immI_255_rule, // 384
  _AndI__LoadI_memory__immI_65535_rule, // 385
  _AndI__LoadI_memory__immI_rule, // 386
  memory_rule, // 387
  memory_rule, // 388
  memory_rule, // 389
  memory_rule, // 390
  memory_rule, // 391
  memory_rule, // 392
  memory_rule, // 393
  memory_rule, // 394
  memory_rule, // 395
  memory_rule, // 396
  memory_rule, // 397
  memory_rule, // 398
  memory_rule, // 399
  memory_rule, // 400
  memory_rule, // 401
  memory_rule, // 402
  memory_rule, // 403
  memory_rule, // 404
  memory_rule, // 405
  memory_rule, // 406
  memory_rule, // 407
  memory_rule, // 408
  memory_rule, // 409
  memory_rule, // 410
  memory_rule, // 411
  memory_rule, // 412
  memory_rule, // 413
  memory_rule, // 414
  memory_rule, // 415
  memory_rule, // 416
  memory_rule, // 417
  memory_rule, // 418
  memory_rule, // 419
  memory_rule, // 420
  memory_rule, // 421
  memory_rule, // 422
  memory_rule, // 423
  memory_rule, // 424
  memory_rule, // 425
  memory_rule, // 426
  memory_rule, // 427
  memory_rule, // 428
  memory_rule, // 429
  memory_rule, // 430
  memory_rule, // 431
  memory_rule, // 432
  memory_rule, // 433
  memory_rule, // 434
  memory_rule, // 435
  memory_rule, // 436
  memory_rule, // 437
  memory_rule, // 438
  memory_rule, // 439
  rRegI_rule, // 440
  rRegL_rule, // 441
  rRegI_rule, // 442
  rRegI_rule, // 443
  rRegI_rule, // 444
  rRegI_rule, // 445
  rRegL_rule, // 446
  rRegL_rule, // 447
  rRegI_rule, // 448
  rRegL_rule, // 449
  0, // 450
  0, // 451
  0, // 452
  0, // 453
  0, // 454
  rRegP_rule, // 455
  rRegP_rule, // 456
  rRegN_rule, // 457
  rRegN_rule, // 458
  _LShiftL_rRegL_immI2_rule, // 459
  _AddL__LShiftL_rRegL_immI2_immL32_rule, // 460
  rRegL_rule, // 461
  _Binary_cmpOp_rFlagsReg_rule, // 462
  _Binary_cmpOpU_rFlagsRegU_rule, // 463
  _Binary_cmpOpUCF_rFlagsRegUCF_rule, // 464
  _Binary_cmpOp_rFlagsReg_rule, // 465
  _Binary_cmpOpU_rFlagsRegU_rule, // 466
  _Binary_cmpOpUCF_rFlagsRegUCF_rule, // 467
  _Binary_cmpOp_rFlagsReg_rule, // 468
  _Binary_cmpOpU_rFlagsRegU_rule, // 469
  _Binary_cmpOpUCF_rFlagsRegUCF_rule, // 470
  _Binary_cmpOp_rFlagsReg_rule, // 471
  _Binary_cmpOpU_rFlagsRegU_rule, // 472
  _Binary_cmpOpUCF_rFlagsRegUCF_rule, // 473
  _Binary_cmpOp_rFlagsReg_rule, // 474
  _Binary_cmpOp_rFlagsReg_rule, // 475
  _Binary_cmpOpU_rFlagsRegU_rule, // 476
  _Binary_cmpOpUCF_rFlagsRegUCF_rule, // 477
  _Binary_cmpOpU_rFlagsRegU_rule, // 478
  _Binary_cmpOpUCF_rFlagsRegUCF_rule, // 479
  _Binary_cmpOp_rFlagsReg_rule, // 480
  _Binary_cmpOpU_rFlagsRegU_rule, // 481
  _Binary_cmpOpUCF_rFlagsRegUCF_rule, // 482
  _Binary_cmpOp_rFlagsReg_rule, // 483
  _Binary_cmpOpU_rFlagsRegU_rule, // 484
  _Binary_cmpOpUCF_rFlagsRegUCF_rule, // 485
  rRegI_rule, // 486
  rRegI_rule, // 487
  rRegI_rule, // 488
  _LoadI_memory__rule, // 489
  memory_rule, // 490
  memory_rule, // 491
  memory_rule, // 492
  rRegI_rule, // 493
  memory_rule, // 494
  rRegI_rule, // 495
  memory_rule, // 496
  rRegI_rule, // 497
  rRegL_rule, // 498
  rRegL_rule, // 499
  rRegL_rule, // 500
  _LoadL_memory__rule, // 501
  memory_rule, // 502
  memory_rule, // 503
  memory_rule, // 504
  rRegI_rule, // 505
  memory_rule, // 506
  rRegL_rule, // 507
  memory_rule, // 508
  rRegL_rule, // 509
  rRegP_rule, // 510
  rRegP_rule, // 511
  rRegP_rule, // 512
  memory_rule, // 513
  memory_rule, // 514
  memory_rule, // 515
  memory_rule, // 516
  memory_rule, // 517
  memory_rule, // 518
  rRegI_rule, // 519
  rRegI_rule, // 520
  rRegI_rule, // 521
  memory_rule, // 522
  memory_rule, // 523
  rRegL_rule, // 524
  rRegI_rule, // 525
  rRegL_rule, // 526
  memory_rule, // 527
  memory_rule, // 528
  rRegP_rule, // 529
  immI0_rule, // 530
  memory_rule, // 531
  immL0_rule, // 532
  memory_rule, // 533
  rRegI_rule, // 534
  rRegI_rule, // 535
  rRegI_rule, // 536
  _LoadI_memory__rule, // 537
  _LoadI_memory__rule, // 538
  rRegL_rule, // 539
  rRegL_rule, // 540
  rRegL_rule, // 541
  _LoadL_memory__rule, // 542
  _LoadL_memory__rule, // 543
  no_rax_RegL_rule, // 544
  rax_RegI_rule, // 545
  rax_RegL_rule, // 546
  rax_RegI_rule, // 547
  rax_RegL_rule, // 548
  0, // 549
  0, // 550
  0, // 551
  0, // 552
  no_rax_RegL_rule, // 553
  rax_RegI_rule, // 554
  rax_RegL_rule, // 555
  rRegI_rule, // 556
  memory_rule, // 557
  rRegI_rule, // 558
  memory_rule, // 559
  rRegI_rule, // 560
  memory_rule, // 561
  rRegI_rule, // 562
  memory_rule, // 563
  rRegI_rule, // 564
  memory_rule, // 565
  rRegI_rule, // 566
  memory_rule, // 567
  rRegI_rule, // 568
  memory_rule, // 569
  rRegI_rule, // 570
  memory_rule, // 571
  rRegI_rule, // 572
  memory_rule, // 573
  rRegL_rule, // 574
  memory_rule, // 575
  rRegL_rule, // 576
  memory_rule, // 577
  rRegL_rule, // 578
  memory_rule, // 579
  rRegL_rule, // 580
  memory_rule, // 581
  rRegL_rule, // 582
  memory_rule, // 583
  rRegL_rule, // 584
  memory_rule, // 585
  rRegL_rule, // 586
  memory_rule, // 587
  rRegL_rule, // 588
  memory_rule, // 589
  rRegL_rule, // 590
  memory_rule, // 591
  _LShiftI_rRegI_immI_24_rule, // 592
  _LShiftI_rRegI_immI_16_rule, // 593
  0, // 594
  0, // 595
  0, // 596
  _LShiftI_rRegI_immI1_rule, // 597
  _URShiftI_rRegI_immI_M1_rule, // 598
  _LShiftI_rRegI_immI8_rule, // 599
  _URShiftI_rRegI_immI8_rule, // 600
  _LShiftI_no_rcx_RegI_rcx_RegI_rule, // 601
  _URShiftI_no_rcx_RegI__SubI_immI0_rcx_RegI_rule, // 602
  _LShiftI_no_rcx_RegI_rcx_RegI_rule, // 603
  _URShiftI_no_rcx_RegI__SubI_immI_32_rcx_RegI_rule, // 604
  0, // 605
  0, // 606
  0, // 607
  _URShiftI_rRegI_immI1_rule, // 608
  _LShiftI_rRegI_immI_M1_rule, // 609
  _URShiftI_rRegI_immI8_rule, // 610
  _LShiftI_rRegI_immI8_rule, // 611
  _URShiftI_no_rcx_RegI_rcx_RegI_rule, // 612
  _LShiftI_no_rcx_RegI__SubI_immI0_rcx_RegI_rule, // 613
  _URShiftI_no_rcx_RegI_rcx_RegI_rule, // 614
  _LShiftI_no_rcx_RegI__SubI_immI_32_rcx_RegI_rule, // 615
  0, // 616
  0, // 617
  0, // 618
  _LShiftL_rRegL_immI1_rule, // 619
  _URShiftL_rRegL_immI_M1_rule, // 620
  _LShiftL_rRegL_immI8_rule, // 621
  _URShiftL_rRegL_immI8_rule, // 622
  _LShiftL_no_rcx_RegL_rcx_RegI_rule, // 623
  _URShiftL_no_rcx_RegL__SubI_immI0_rcx_RegI_rule, // 624
  _LShiftL_no_rcx_RegL_rcx_RegI_rule, // 625
  _URShiftL_no_rcx_RegL__SubI_immI_64_rcx_RegI_rule, // 626
  0, // 627
  0, // 628
  0, // 629
  _URShiftL_rRegL_immI1_rule, // 630
  _LShiftL_rRegL_immI_M1_rule, // 631
  _URShiftL_rRegL_immI8_rule, // 632
  _LShiftL_rRegL_immI8_rule, // 633
  _URShiftL_no_rcx_RegL_rcx_RegI_rule, // 634
  _LShiftL_no_rcx_RegL__SubI_immI0_rcx_RegI_rule, // 635
  _URShiftL_no_rcx_RegL_rcx_RegI_rule, // 636
  _LShiftL_no_rcx_RegL__SubI_immI_64_rcx_RegI_rule, // 637
  rRegI_rule, // 638
  rRegI_rule, // 639
  _AndI_rRegI_immI_255_rule, // 640
  rRegI_rule, // 641
  _AndI_rRegI_immI_65535_rule, // 642
  rRegI_rule, // 643
  rRegI_rule, // 644
  _LoadI_memory__rule, // 645
  memory_rule, // 646
  memory_rule, // 647
  memory_rule, // 648
  rRegI_rule, // 649
  rRegI_rule, // 650
  rRegI_rule, // 651
  _LoadI_memory__rule, // 652
  memory_rule, // 653
  memory_rule, // 654
  memory_rule, // 655
  rRegI_rule, // 656
  rRegI_rule, // 657
  rRegI_rule, // 658
  rRegI_rule, // 659
  _LoadI_memory__rule, // 660
  memory_rule, // 661
  memory_rule, // 662
  memory_rule, // 663
  rRegL_rule, // 664
  rRegL_rule, // 665
  rRegL_rule, // 666
  rRegL_rule, // 667
  rRegL_rule, // 668
  _LoadL_memory__rule, // 669
  memory_rule, // 670
  memory_rule, // 671
  memory_rule, // 672
  rRegL_rule, // 673
  rRegL_rule, // 674
  _CastP2X_any_RegP__rule, // 675
  rRegL_rule, // 676
  rRegL_rule, // 677
  _LoadL_memory__rule, // 678
  memory_rule, // 679
  memory_rule, // 680
  memory_rule, // 681
  rRegL_rule, // 682
  rRegL_rule, // 683
  rRegL_rule, // 684
  rRegL_rule, // 685
  _LoadL_memory__rule, // 686
  memory_rule, // 687
  memory_rule, // 688
  memory_rule, // 689
  rRegI_rule, // 690
  rRegP_rule, // 691
  rRegI_rule, // 692
  rRegI_rule, // 693
  _AndI__CmpLTMask_rRegI_rRegI_rRegI_rule, // 694
  _SubI_rRegI_rRegI_rule, // 695
  _AndI_rRegI__CmpLTMask_rRegI_rRegI_rule, // 696
  _SubI_rRegI_rRegI_rule, // 697
  regF_rule, // 698
  regF_rule, // 699
  regF_rule, // 700
  regD_rule, // 701
  regD_rule, // 702
  regD_rule, // 703
  regF_rule, // 704
  regF_rule, // 705
  _LoadF_memory__rule, // 706
  regF_rule, // 707
  regD_rule, // 708
  regD_rule, // 709
  _LoadD_memory__rule, // 710
  regD_rule, // 711
  regF_rule, // 712
  regF_rule, // 713
  regF_rule, // 714
  regD_rule, // 715
  regD_rule, // 716
  regD_rule, // 717
  regF_rule, // 718
  regF_rule, // 719
  _LoadF_memory__rule, // 720
  regF_rule, // 721
  regD_rule, // 722
  regD_rule, // 723
  _LoadD_memory__rule, // 724
  regD_rule, // 725
  regF_rule, // 726
  regF_rule, // 727
  regF_rule, // 728
  regD_rule, // 729
  regD_rule, // 730
  regD_rule, // 731
  _SqrtD__ConvF2D_regF___rule, // 732
  _SqrtD__ConvF2D__LoadF_memory____rule, // 733
  _SqrtD__ConvF2D_immF___rule, // 734
  regD_rule, // 735
  _LoadD_memory__rule, // 736
  immD_rule, // 737
  regD_rule, // 738
  regD_rule, // 739
  regD_rule, // 740
  regD_rule, // 741
  regD_rule, // 742
  regF_rule, // 743
  _LoadF_memory__rule, // 744
  regD_rule, // 745
  _LoadD_memory__rule, // 746
  regF_rule, // 747
  regF_rule, // 748
  regD_rule, // 749
  regD_rule, // 750
  rRegI_rule, // 751
  _LoadI_memory__rule, // 752
  rRegI_rule, // 753
  _LoadI_memory__rule, // 754
  rRegI_rule, // 755
  rRegI_rule, // 756
  rRegL_rule, // 757
  _LoadL_memory__rule, // 758
  rRegL_rule, // 759
  _LoadL_memory__rule, // 760
  rRegI_rule, // 761
  _ConvI2L_rRegI__rule, // 762
  _ConvI2L__LoadI_memory___rule, // 763
  rRegL_rule, // 764
  rRegL_rule, // 765
  regF_rule, // 766
  rRegI_rule, // 767
  regD_rule, // 768
  rRegL_rule, // 769
  regD_rule, // 770
  rRegI_rule, // 771
  immI0_rule, // 772
  regD_rule, // 773
  rRegI_rule, // 774
  immI0_rule, // 775
  regD_rule, // 776
  rRegI_rule, // 777
  immI0_rule, // 778
  regD_rule, // 779
  rRegI_rule, // 780
  immI0_rule, // 781
  regD_rule, // 782
  regF_rule, // 783
  immF0_rule, // 784
  rcx_RegL_rule, // 785
  _Binary_rdi_RegP_rcx_RegI_rule, // 786
  _Binary_rdi_RegP_rdx_RegI_rule, // 787
  _Binary_rdi_RegP_rsi_RegP_rule, // 788
  rdi_RegP_rule, // 789
  rRegL_rule, // 790
  0, // 791
  rRegI_rule, // 792
  0, // 793
  rRegI_rule, // 794
  0, // 795
  cmpOp_rule, // 796
  cmpOp_rule, // 797
  cmpOpU_rule, // 798
  cmpOpUCF_rule, // 799
  cmpOpU_rule, // 800
  cmpOpUCF_rule, // 801
  cmpOpUCF2_rule, // 802
  rsi_RegP_rule, // 803
  0, // 804
  cmpOp_rule, // 805
  cmpOp_rule, // 806
  cmpOpU_rule, // 807
  cmpOpUCF_rule, // 808
  cmpOpU_rule, // 809
  cmpOpUCF_rule, // 810
  cmpOpUCF2_rule, // 811
  0, // 812
  0, // 813
  0, // 814
  0, // 815
  0, // 816
  0, // 817
  0, // 818
  no_rbp_RegP_rule, // 819
  no_rbp_RegP_rule, // 820
  0, // 821
  0, // 822
  0, // 823
  // last instruction
  0 // no trailing comma
};

const        int   rightOp[] = {
  0, // 0
  0, // 1
  0, // 2
  0, // 3
  0, // 4
  0, // 5
  0, // 6
  0, // 7
  0, // 8
  0, // 9
  0, // 10
  0, // 11
  0, // 12
  0, // 13
  0, // 14
  0, // 15
  0, // 16
  0, // 17
  0, // 18
  0, // 19
  0, // 20
  0, // 21
  0, // 22
  0, // 23
  0, // 24
  0, // 25
  0, // 26
  0, // 27
  0, // 28
  0, // 29
  0, // 30
  0, // 31
  0, // 32
  0, // 33
  0, // 34
  0, // 35
  0, // 36
  0, // 37
  0, // 38
  0, // 39
  0, // 40
  0, // 41
  0, // 42
  0, // 43
  0, // 44
  0, // 45
  0, // 46
  0, // 47
  0, // 48
  0, // 49
  0, // 50
  0, // 51
  0, // 52
  0, // 53
  0, // 54
  0, // 55
  0, // 56
  0, // 57
  0, // 58
  0, // 59
  0, // 60
  0, // 61
  0, // 62
  0, // 63
  0, // 64
  0, // 65
  0, // 66
  0, // 67
  0, // 68
  0, // 69
  0, // 70
  0, // 71
  0, // 72
  0, // 73
  0, // 74
  0, // 75
  immL8_rule, // 76
  immL32_rule, // 77
  immL32_rule, // 78
  rRegL_rule, // 79
  _LShiftL_rRegL_immI2_rule, // 80
  immL32_rule, // 81
  immL32_rule, // 82
  immL32_rule, // 83
  0, // 84
  immL8_rule, // 85
  immL32_rule, // 86
  immL32_rule, // 87
  rRegL_rule, // 88
  _LShiftL_rRegL_immI2_rule, // 89
  immL32_rule, // 90
  immL32_rule, // 91
  0, // 92
  0, // 93
  0, // 94
  0, // 95
  0, // 96
  0, // 97
  0, // 98
  0, // 99
  0, // 100
  // last operand
  0, // 101
  // last operand class
  rRegL_rule, // 102
  immI2_rule, // 103
  _LShiftL_rRegL_immI2_rule, // 104
  0, // 105
  immI2_rule, // 106
  _LShiftL__ConvI2L_rRegI__immI2_rule, // 107
  0, // 108
  rRegL_rule, // 109
  _LShiftL_rRegL_immI2_rule, // 110
  _LShiftL__ConvI2L_rRegI__immI2_rule, // 111
  0, // 112
  0, // 113
  immI8_rule, // 114
  0, // 115
  immI_24_rule, // 116
  0, // 117
  immI_24_rule, // 118
  immI_255_rule, // 119
  immI16_rule, // 120
  0, // 121
  immI_24_rule, // 122
  immI_16_rule, // 123
  immI_255_rule, // 124
  immI_65535_rule, // 125
  immI_rule, // 126
  0, // 127
  immL32_rule, // 128
  rFlagsReg_rule, // 129
  rRegI_rule, // 130
  rFlagsRegU_rule, // 131
  rFlagsRegUCF_rule, // 132
  _LoadI_memory__rule, // 133
  rRegN_rule, // 134
  rRegP_rule, // 135
  rRegL_rule, // 136
  _LoadL_memory__rule, // 137
  regF_rule, // 138
  regD_rule, // 139
  rRegI_rule, // 140
  _LoadI_memory__rule, // 141
  immI_rule, // 142
  immI1_rule, // 143
  immI_M1_rule, // 144
  rRegL_rule, // 145
  _LoadL_memory__rule, // 146
  immL32_rule, // 147
  immL1_rule, // 148
  immL_M1_rule, // 149
  rRegP_rule, // 150
  rRegI_rule, // 151
  rRegL_rule, // 152
  rRegN_rule, // 153
  rRegI_rule, // 154
  immI_rule, // 155
  rRegL_rule, // 156
  immL32_rule, // 157
  rRegI_rule, // 158
  _LoadI_memory__rule, // 159
  _LoadL_memory__rule, // 160
  immI1_rule, // 161
  immI8_rule, // 162
  rcx_RegI_rule, // 163
  immI1_rule, // 164
  immI8_rule, // 165
  rcx_RegI_rule, // 166
  immI1_rule, // 167
  immI8_rule, // 168
  rcx_RegI_rule, // 169
  immI1_rule, // 170
  immI8_rule, // 171
  rcx_RegI_rule, // 172
  immI1_rule, // 173
  immI8_rule, // 174
  rcx_RegI_rule, // 175
  immI1_rule, // 176
  immI8_rule, // 177
  rcx_RegI_rule, // 178
  immI_24_rule, // 179
  immI_16_rule, // 180
  immI1_rule, // 181
  immI_M1_rule, // 182
  immI8_rule, // 183
  immI8_rule, // 184
  rcx_RegI_rule, // 185
  rcx_RegI_rule, // 186
  _SubI_immI0_rcx_RegI_rule, // 187
  rcx_RegI_rule, // 188
  _SubI_immI_32_rcx_RegI_rule, // 189
  immI1_rule, // 190
  immI_M1_rule, // 191
  rcx_RegI_rule, // 192
  _SubI_immI0_rcx_RegI_rule, // 193
  _SubI_immI_32_rcx_RegI_rule, // 194
  immI1_rule, // 195
  immI_M1_rule, // 196
  immI8_rule, // 197
  immI8_rule, // 198
  rcx_RegI_rule, // 199
  _SubI_immI0_rcx_RegI_rule, // 200
  rcx_RegI_rule, // 201
  _SubI_immI_64_rcx_RegI_rule, // 202
  immI1_rule, // 203
  immI_M1_rule, // 204
  rcx_RegI_rule, // 205
  _SubI_immI0_rcx_RegI_rule, // 206
  _SubI_immI_64_rcx_RegI_rule, // 207
  immI_255_rule, // 208
  immI_65535_rule, // 209
  rRegI_rule, // 210
  _LoadI_memory__rule, // 211
  rRegI_rule, // 212
  _LoadI_memory__rule, // 213
  immI_rule, // 214
  rRegI_rule, // 215
  _LoadI_memory__rule, // 216
  immI_rule, // 217
  rRegL_rule, // 218
  _LoadL_memory__rule, // 219
  immL32_rule, // 220
  0, // 221
  rRegL_rule, // 222
  _LoadL_memory__rule, // 223
  immL32_rule, // 224
  rRegL_rule, // 225
  _LoadL_memory__rule, // 226
  immL32_rule, // 227
  rRegI_rule, // 228
  rRegI_rule, // 229
  rRegI_rule, // 230
  _CmpLTMask_rRegI_rRegI_rule, // 231
  0, // 232
  0, // 233
  0, // 234
  0, // 235
  0, // 236
  0, // 237
  0, // 238
  0, // 239
  0, // 240
  rcx_RegI_rule, // 241
  rdx_RegI_rule, // 242
  rdx_RegI_rule, // 243
  rax_RegI_rule, // 244
  rsi_RegP_rule, // 245
  immI_rule, // 246
  0, // 247
  0, // 248
  immL32_rule, // 249
  rax_RegP_rule, // 250
  // last internally defined operand
  0, // 251
  0, // 252
  0, // 253
  0, // 254
  0, // 255
  0, // 256
  0, // 257
  0, // 258
  0, // 259
  0, // 260
  0, // 261
  0, // 262
  0, // 263
  0, // 264
  0, // 265
  0, // 266
  0, // 267
  0, // 268
  0, // 269
  0, // 270
  0, // 271
  0, // 272
  0, // 273
  0, // 274
  0, // 275
  0, // 276
  0, // 277
  0, // 278
  0, // 279
  0, // 280
  0, // 281
  0, // 282
  0, // 283
  0, // 284
  0, // 285
  0, // 286
  0, // 287
  0, // 288
  0, // 289
  0, // 290
  0, // 291
  0, // 292
  0, // 293
  0, // 294
  0, // 295
  0, // 296
  0, // 297
  0, // 298
  _Binary_rax_RegP_rRegP_rule, // 299
  _Binary_rax_RegI_rRegI_rule, // 300
  _Binary_rax_RegL_rRegL_rule, // 301
  regF_rule, // 302
  regF_rule, // 303
  _LoadF_memory__rule, // 304
  _LoadF_memory__rule, // 305
  immF_rule, // 306
  immF_rule, // 307
  regD_rule, // 308
  regD_rule, // 309
  _LoadD_memory__rule, // 310
  _LoadD_memory__rule, // 311
  immD_rule, // 312
  immD_rule, // 313
  0, // 314
  0, // 315
  0, // 316
  0, // 317
  0, // 318
  0, // 319
  0, // 320
  0, // 321
  0, // 322
  0, // 323
  0, // 324
  0, // 325
  0, // 326
  0, // 327
  0, // 328
  rRegI_rule, // 329
  immI_rule, // 330
  _LoadI_memory__rule, // 331
  immI0_rule, // 332
  immI0_rule, // 333
  immI0_rule, // 334
  immI0_rule, // 335
  rRegI_rule, // 336
  immI_rule, // 337
  _LoadI_memory__rule, // 338
  immI0_rule, // 339
  rRegP_rule, // 340
  _LoadP_memory__rule, // 341
  _LoadP_memory__rule, // 342
  immP0_rule, // 343
  immP0_rule, // 344
  immP0_rule, // 345
  rRegN_rule, // 346
  _LoadN_memory__rule, // 347
  immN_rule, // 348
  _LoadN_memory__rule, // 349
  immN0_rule, // 350
  immN0_rule, // 351
  immN0_rule, // 352
  rRegL_rule, // 353
  immL32_rule, // 354
  _LoadL_memory__rule, // 355
  immL0_rule, // 356
  immL0_rule, // 357
  immL0_rule, // 358
  immL0_rule, // 359
  immP0_rule, // 360
  rRegP_rule, // 361
  rax_RegP_rule, // 362
  0, // 363
  0, // 364
  0, // 365
  0, // 366
  0, // 367
  0, // 368
  0, // 369
  0, // 370
  immI_24_rule, // 371
  0, // 372
  0, // 373
  immI_24_rule, // 374
  0, // 375
  0, // 376
  0, // 377
  0, // 378
  immI_24_rule, // 379
  immI_255_rule, // 380
  immI_16_rule, // 381
  immI_65535_rule, // 382
  0, // 383
  0, // 384
  0, // 385
  0, // 386
  0, // 387
  0, // 388
  0, // 389
  0, // 390
  0, // 391
  0, // 392
  0, // 393
  0, // 394
  0, // 395
  0, // 396
  0, // 397
  0, // 398
  0, // 399
  0, // 400
  0, // 401
  0, // 402
  0, // 403
  0, // 404
  0, // 405
  0, // 406
  0, // 407
  0, // 408
  0, // 409
  rRegI_rule, // 410
  rRegI_rule, // 411
  rRegI_rule, // 412
  rRegL_rule, // 413
  any_RegP_rule, // 414
  immP0_rule, // 415
  immP31_rule, // 416
  rRegN_rule, // 417
  immN0_rule, // 418
  immN_rule, // 419
  immI0_rule, // 420
  immI_rule, // 421
  immL0_rule, // 422
  immL32_rule, // 423
  immI0_rule, // 424
  immI16_rule, // 425
  immI0_rule, // 426
  immI8_rule, // 427
  regD_rule, // 428
  regD_rule, // 429
  regD_rule, // 430
  immI0_rule, // 431
  immI0_rule, // 432
  regD_rule, // 433
  regF_rule, // 434
  immF0_rule, // 435
  immF_rule, // 436
  regD_rule, // 437
  immD0_rule, // 438
  immD0_rule, // 439
  0, // 440
  0, // 441
  0, // 442
  0, // 443
  0, // 444
  0, // 445
  0, // 446
  0, // 447
  0, // 448
  0, // 449
  0, // 450
  0, // 451
  0, // 452
  0, // 453
  0, // 454
  0, // 455
  0, // 456
  0, // 457
  0, // 458
  0, // 459
  0, // 460
  0, // 461
  _Binary_rRegI_rRegI_rule, // 462
  _Binary_rRegI_rRegI_rule, // 463
  _Binary_rRegI_rRegI_rule, // 464
  _Binary_rRegI__LoadI_memory__rule, // 465
  _Binary_rRegI__LoadI_memory__rule, // 466
  _Binary_rRegI__LoadI_memory__rule, // 467
  _Binary_rRegN_rRegN_rule, // 468
  _Binary_rRegN_rRegN_rule, // 469
  _Binary_rRegN_rRegN_rule, // 470
  _Binary_rRegP_rRegP_rule, // 471
  _Binary_rRegP_rRegP_rule, // 472
  _Binary_rRegP_rRegP_rule, // 473
  _Binary_rRegL_rRegL_rule, // 474
  _Binary_rRegL__LoadL_memory__rule, // 475
  _Binary_rRegL_rRegL_rule, // 476
  _Binary_rRegL_rRegL_rule, // 477
  _Binary_rRegL__LoadL_memory__rule, // 478
  _Binary_rRegL__LoadL_memory__rule, // 479
  _Binary_regF_regF_rule, // 480
  _Binary_regF_regF_rule, // 481
  _Binary_regF_regF_rule, // 482
  _Binary_regD_regD_rule, // 483
  _Binary_regD_regD_rule, // 484
  _Binary_regD_regD_rule, // 485
  rRegI_rule, // 486
  immI_rule, // 487
  _LoadI_memory__rule, // 488
  rRegI_rule, // 489
  _AddI__LoadI_memory__rRegI_rule, // 490
  _AddI_rRegI__LoadI_memory__rule, // 491
  _AddI__LoadI_memory__immI_rule, // 492
  immI1_rule, // 493
  _AddI__LoadI_memory__immI1_rule, // 494
  immI_M1_rule, // 495
  _AddI__LoadI_memory__immI_M1_rule, // 496
  immI_rule, // 497
  rRegL_rule, // 498
  immL32_rule, // 499
  _LoadL_memory__rule, // 500
  rRegL_rule, // 501
  _AddL__LoadL_memory__rRegL_rule, // 502
  _AddL_rRegL__LoadL_memory__rule, // 503
  _AddL__LoadL_memory__immL32_rule, // 504
  immL1_rule, // 505
  _AddL__LoadL_memory__immL1_rule, // 506
  immL_M1_rule, // 507
  _AddL__LoadL_memory__immL_M1_rule, // 508
  immL32_rule, // 509
  rRegL_rule, // 510
  immL32_rule, // 511
  immL32_rule, // 512
  0, // 513
  0, // 514
  _Binary_rax_RegP_rRegP_rule, // 515
  _Binary_rax_RegL_rRegL_rule, // 516
  _Binary_rax_RegI_rRegI_rule, // 517
  _Binary_rax_RegN_rRegN_rule, // 518
  rRegI_rule, // 519
  immI_rule, // 520
  _LoadI_memory__rule, // 521
  _SubI__LoadI_memory__rRegI_rule, // 522
  _SubI__LoadI_memory__immI_rule, // 523
  rRegL_rule, // 524
  immL32_rule, // 525
  _LoadL_memory__rule, // 526
  _SubL__LoadL_memory__rRegL_rule, // 527
  _SubL__LoadL_memory__immL32_rule, // 528
  _SubI_immI0_rRegI_rule, // 529
  rRegI_rule, // 530
  _SubI_immI0__LoadI_memory__rule, // 531
  rRegL_rule, // 532
  _SubL_immL0__LoadL_memory__rule, // 533
  rRegI_rule, // 534
  immI_rule, // 535
  _LoadI_memory__rule, // 536
  rRegI_rule, // 537
  immI_rule, // 538
  rRegL_rule, // 539
  immL32_rule, // 540
  _LoadL_memory__rule, // 541
  rRegL_rule, // 542
  immL32_rule, // 543
  rax_RegL_rule, // 544
  no_rax_rdx_RegI_rule, // 545
  no_rax_rdx_RegL_rule, // 546
  no_rax_rdx_RegI_rule, // 547
  no_rax_rdx_RegL_rule, // 548
  0, // 549
  0, // 550
  0, // 551
  0, // 552
  immL10_rule, // 553
  no_rax_rdx_RegI_rule, // 554
  no_rax_rdx_RegL_rule, // 555
  immI1_rule, // 556
  _LShiftI__LoadI_memory__immI1_rule, // 557
  immI8_rule, // 558
  _LShiftI__LoadI_memory__immI8_rule, // 559
  rcx_RegI_rule, // 560
  _LShiftI__LoadI_memory__rcx_RegI_rule, // 561
  immI1_rule, // 562
  _RShiftI__LoadI_memory__immI1_rule, // 563
  immI8_rule, // 564
  _RShiftI__LoadI_memory__immI8_rule, // 565
  rcx_RegI_rule, // 566
  _RShiftI__LoadI_memory__rcx_RegI_rule, // 567
  immI1_rule, // 568
  _URShiftI__LoadI_memory__immI1_rule, // 569
  immI8_rule, // 570
  _URShiftI__LoadI_memory__immI8_rule, // 571
  rcx_RegI_rule, // 572
  _URShiftI__LoadI_memory__rcx_RegI_rule, // 573
  immI1_rule, // 574
  _LShiftL__LoadL_memory__immI1_rule, // 575
  immI8_rule, // 576
  _LShiftL__LoadL_memory__immI8_rule, // 577
  rcx_RegI_rule, // 578
  _LShiftL__LoadL_memory__rcx_RegI_rule, // 579
  immI1_rule, // 580
  _RShiftL__LoadL_memory__immI1_rule, // 581
  immI8_rule, // 582
  _RShiftL__LoadL_memory__immI8_rule, // 583
  rcx_RegI_rule, // 584
  _RShiftL__LoadL_memory__rcx_RegI_rule, // 585
  immI1_rule, // 586
  _URShiftL__LoadL_memory__immI1_rule, // 587
  immI8_rule, // 588
  _URShiftL__LoadL_memory__immI8_rule, // 589
  rcx_RegI_rule, // 590
  _URShiftL__LoadL_memory__rcx_RegI_rule, // 591
  immI_24_rule, // 592
  immI_16_rule, // 593
  0, // 594
  0, // 595
  0, // 596
  _URShiftI_rRegI_immI_M1_rule, // 597
  _LShiftI_rRegI_immI1_rule, // 598
  _URShiftI_rRegI_immI8_rule, // 599
  _LShiftI_rRegI_immI8_rule, // 600
  _URShiftI_no_rcx_RegI__SubI_immI0_rcx_RegI_rule, // 601
  _LShiftI_no_rcx_RegI_rcx_RegI_rule, // 602
  _URShiftI_no_rcx_RegI__SubI_immI_32_rcx_RegI_rule, // 603
  _LShiftI_no_rcx_RegI_rcx_RegI_rule, // 604
  0, // 605
  0, // 606
  0, // 607
  _LShiftI_rRegI_immI_M1_rule, // 608
  _URShiftI_rRegI_immI1_rule, // 609
  _LShiftI_rRegI_immI8_rule, // 610
  _URShiftI_rRegI_immI8_rule, // 611
  _LShiftI_no_rcx_RegI__SubI_immI0_rcx_RegI_rule, // 612
  _URShiftI_no_rcx_RegI_rcx_RegI_rule, // 613
  _LShiftI_no_rcx_RegI__SubI_immI_32_rcx_RegI_rule, // 614
  _URShiftI_no_rcx_RegI_rcx_RegI_rule, // 615
  0, // 616
  0, // 617
  0, // 618
  _URShiftL_rRegL_immI_M1_rule, // 619
  _LShiftL_rRegL_immI1_rule, // 620
  _URShiftL_rRegL_immI8_rule, // 621
  _LShiftL_rRegL_immI8_rule, // 622
  _URShiftL_no_rcx_RegL__SubI_immI0_rcx_RegI_rule, // 623
  _LShiftL_no_rcx_RegL_rcx_RegI_rule, // 624
  _URShiftL_no_rcx_RegL__SubI_immI_64_rcx_RegI_rule, // 625
  _LShiftL_no_rcx_RegL_rcx_RegI_rule, // 626
  0, // 627
  0, // 628
  0, // 629
  _LShiftL_rRegL_immI_M1_rule, // 630
  _URShiftL_rRegL_immI1_rule, // 631
  _LShiftL_rRegL_immI8_rule, // 632
  _URShiftL_rRegL_immI8_rule, // 633
  _LShiftL_no_rcx_RegL__SubI_immI0_rcx_RegI_rule, // 634
  _URShiftL_no_rcx_RegL_rcx_RegI_rule, // 635
  _LShiftL_no_rcx_RegL__SubI_immI_64_rcx_RegI_rule, // 636
  _URShiftL_no_rcx_RegL_rcx_RegI_rule, // 637
  rRegI_rule, // 638
  immI_255_rule, // 639
  0, // 640
  immI_65535_rule, // 641
  0, // 642
  immI_rule, // 643
  _LoadI_memory__rule, // 644
  rRegI_rule, // 645
  _AndI__LoadI_memory__rRegI_rule, // 646
  _AndI_rRegI__LoadI_memory__rule, // 647
  _AndI__LoadI_memory__immI_rule, // 648
  rRegI_rule, // 649
  immI_rule, // 650
  _LoadI_memory__rule, // 651
  rRegI_rule, // 652
  _OrI__LoadI_memory__rRegI_rule, // 653
  _OrI_rRegI__LoadI_memory__rule, // 654
  _OrI__LoadI_memory__immI_rule, // 655
  rRegI_rule, // 656
  immI_M1_rule, // 657
  immI_rule, // 658
  _LoadI_memory__rule, // 659
  rRegI_rule, // 660
  _XorI__LoadI_memory__rRegI_rule, // 661
  _XorI_rRegI__LoadI_memory__rule, // 662
  _XorI__LoadI_memory__immI_rule, // 663
  rRegL_rule, // 664
  immL_255_rule, // 665
  immL_65535_rule, // 666
  immL32_rule, // 667
  _LoadL_memory__rule, // 668
  rRegL_rule, // 669
  _AndL__LoadL_memory__rRegL_rule, // 670
  _AndL_rRegL__LoadL_memory__rule, // 671
  _AndL__LoadL_memory__immL32_rule, // 672
  rRegL_rule, // 673
  _CastP2X_any_RegP__rule, // 674
  rRegL_rule, // 675
  immL32_rule, // 676
  _LoadL_memory__rule, // 677
  rRegL_rule, // 678
  _OrL__LoadL_memory__rRegL_rule, // 679
  _OrL_rRegL__LoadL_memory__rule, // 680
  _OrL__LoadL_memory__immL32_rule, // 681
  rRegL_rule, // 682
  immL_M1_rule, // 683
  immL32_rule, // 684
  _LoadL_memory__rule, // 685
  rRegL_rule, // 686
  _XorL__LoadL_memory__rRegL_rule, // 687
  _XorL_rRegL__LoadL_memory__rule, // 688
  _XorL__LoadL_memory__immL32_rule, // 689
  0, // 690
  0, // 691
  rRegI_rule, // 692
  immI0_rule, // 693
  _SubI_rRegI_rRegI_rule, // 694
  _AndI__CmpLTMask_rRegI_rRegI_rRegI_rule, // 695
  _SubI_rRegI_rRegI_rule, // 696
  _AndI_rRegI__CmpLTMask_rRegI_rRegI_rule, // 697
  regF_rule, // 698
  _LoadF_memory__rule, // 699
  immF_rule, // 700
  regD_rule, // 701
  _LoadD_memory__rule, // 702
  immD_rule, // 703
  regF_rule, // 704
  _LoadF_memory__rule, // 705
  regF_rule, // 706
  immF_rule, // 707
  regD_rule, // 708
  _LoadD_memory__rule, // 709
  regD_rule, // 710
  immD_rule, // 711
  regF_rule, // 712
  _LoadF_memory__rule, // 713
  immF_rule, // 714
  regD_rule, // 715
  _LoadD_memory__rule, // 716
  immD_rule, // 717
  regF_rule, // 718
  _LoadF_memory__rule, // 719
  regF_rule, // 720
  immF_rule, // 721
  regD_rule, // 722
  _LoadD_memory__rule, // 723
  regD_rule, // 724
  immD_rule, // 725
  regF_rule, // 726
  _LoadF_memory__rule, // 727
  immF_rule, // 728
  regD_rule, // 729
  _LoadD_memory__rule, // 730
  immD_rule, // 731
  0, // 732
  0, // 733
  0, // 734
  0, // 735
  0, // 736
  0, // 737
  0, // 738
  0, // 739
  0, // 740
  0, // 741
  0, // 742
  0, // 743
  0, // 744
  0, // 745
  0, // 746
  0, // 747
  0, // 748
  0, // 749
  0, // 750
  0, // 751
  0, // 752
  0, // 753
  0, // 754
  0, // 755
  0, // 756
  0, // 757
  0, // 758
  0, // 759
  0, // 760
  0, // 761
  immL_32bits_rule, // 762
  immL_32bits_rule, // 763
  immL_32bits_rule, // 764
  0, // 765
  0, // 766
  0, // 767
  0, // 768
  0, // 769
  0, // 770
  0, // 771
  0, // 772
  0, // 773
  0, // 774
  0, // 775
  0, // 776
  0, // 777
  0, // 778
  0, // 779
  0, // 780
  0, // 781
  0, // 782
  0, // 783
  0, // 784
  rdi_RegP_rule, // 785
  _Binary_rsi_RegP_rdx_RegI_rule, // 786
  _Binary_rsi_RegP_rax_RegI_rule, // 787
  rcx_RegI_rule, // 788
  rsi_RegP_rule, // 789
  rRegL_rule, // 790
  0, // 791
  rRegI_rule, // 792
  0, // 793
  rRegI_rule, // 794
  0, // 795
  rFlagsReg_rule, // 796
  rFlagsReg_rule, // 797
  rFlagsRegU_rule, // 798
  rFlagsRegUCF_rule, // 799
  rFlagsRegU_rule, // 800
  rFlagsRegUCF_rule, // 801
  rFlagsRegUCF_rule, // 802
  rax_RegP_rule, // 803
  0, // 804
  rFlagsReg_rule, // 805
  rFlagsReg_rule, // 806
  rFlagsRegU_rule, // 807
  rFlagsRegUCF_rule, // 808
  rFlagsRegU_rule, // 809
  rFlagsRegUCF_rule, // 810
  rFlagsRegUCF_rule, // 811
  0, // 812
  0, // 813
  0, // 814
  0, // 815
  0, // 816
  0, // 817
  0, // 818
  rbx_RegP_rule, // 819
  rax_RegP_rule, // 820
  0, // 821
  0, // 822
  0, // 823
  // last instruction
  0 // no trailing comma
};

const char        *ruleName[] = {
  "UNIVERSE", // 0
  "LABEL", // 1
  "SREGI", // 2
  "SREGP", // 3
  "SREGF", // 4
  "SREGD", // 5
  "SREGL", // 6
  "METHOD", // 7
  "IMMI", // 8
  "IMMI0", // 9
  "IMMI1", // 10
  "IMMI_M1", // 11
  "IMMI2", // 12
  "IMMI8", // 13
  "IMMI16", // 14
  "IMMI_32", // 15
  "IMMI_64", // 16
  "IMMP", // 17
  "IMMP0", // 18
  "IMMN", // 19
  "IMMN0", // 20
  "IMMP31", // 21
  "IMML", // 22
  "IMML8", // 23
  "IMMUL32", // 24
  "IMML32", // 25
  "IMML0", // 26
  "IMML1", // 27
  "IMML_M1", // 28
  "IMML10", // 29
  "IMML_127", // 30
  "IMML_32BITS", // 31
  "IMMF0", // 32
  "IMMF", // 33
  "IMMD0", // 34
  "IMMD", // 35
  "IMMI_16", // 36
  "IMMI_24", // 37
  "IMMI_255", // 38
  "IMMI_65535", // 39
  "IMML_255", // 40
  "IMML_65535", // 41
  "RREGI", // 42
  "RAX_REGI", // 43
  "RBX_REGI", // 44
  "RCX_REGI", // 45
  "RDX_REGI", // 46
  "RDI_REGI", // 47
  "NO_RCX_REGI", // 48
  "NO_RAX_RDX_REGI", // 49
  "ANY_REGP", // 50
  "RREGP", // 51
  "RREGN", // 52
  "NO_RAX_REGP", // 53
  "NO_RBP_REGP", // 54
  "NO_RAX_RBX_REGP", // 55
  "RAX_REGP", // 56
  "RAX_REGN", // 57
  "RBX_REGP", // 58
  "RSI_REGP", // 59
  "RDI_REGP", // 60
  "RBP_REGP", // 61
  "R15_REGP", // 62
  "RREGL", // 63
  "NO_RAX_RDX_REGL", // 64
  "NO_RAX_REGL", // 65
  "NO_RCX_REGL", // 66
  "RAX_REGL", // 67
  "RCX_REGL", // 68
  "RDX_REGL", // 69
  "RFLAGSREG", // 70
  "RFLAGSREGU", // 71
  "RFLAGSREGUCF", // 72
  "REGF", // 73
  "REGD", // 74
  "INDIRECT", // 75
  "INDOFFSET8", // 76
  "INDOFFSET32", // 77
  "INDINDEXOFFSET", // 78
  "INDINDEX", // 79
  "INDINDEXSCALE", // 80
  "INDINDEXSCALEOFFSET", // 81
  "INDPOSINDEXSCALEOFFSET", // 82
  "INDCOMPRESSEDOOPOFFSET", // 83
  "INDIRECTNARROW", // 84
  "INDOFFSET8NARROW", // 85
  "INDOFFSET32NARROW", // 86
  "INDINDEXOFFSETNARROW", // 87
  "INDINDEXNARROW", // 88
  "INDINDEXSCALENARROW", // 89
  "INDINDEXSCALEOFFSETNARROW", // 90
  "INDPOSINDEXSCALEOFFSETNARROW", // 91
  "STACKSLOTP", // 92
  "STACKSLOTI", // 93
  "STACKSLOTF", // 94
  "STACKSLOTD", // 95
  "STACKSLOTL", // 96
  "CMPOP", // 97
  "CMPOPU", // 98
  "CMPOPUCF", // 99
  "CMPOPUCF2", // 100
  // last operand
  "MEMORY", // 101
  // last operand class
  "_AddP_any_RegP_rRegL", // 102
  "_LShiftL_rRegL_immI2", // 103
  "_AddP_any_RegP__LShiftL_rRegL_immI2", // 104
  "_ConvI2L_rRegI_", // 105
  "_LShiftL__ConvI2L_rRegI__immI2", // 106
  "_AddP_any_RegP__LShiftL__ConvI2L_rRegI__immI2", // 107
  "_DecodeN_rRegN_", // 108
  "_AddP__DecodeN_rRegN__rRegL", // 109
  "_AddP__DecodeN_rRegN___LShiftL_rRegL_immI2", // 110
  "_AddP__DecodeN_rRegN___LShiftL__ConvI2L_rRegI__immI2", // 111
  "_LoadB_memory_", // 112
  "_LoadUB_memory_", // 113
  "_AndI__LoadUB_memory__immI8", // 114
  "_LoadS_memory_", // 115
  "_LShiftI__LoadS_memory__immI_24", // 116
  "_LoadUS_memory_", // 117
  "_LShiftI__LoadUS_memory__immI_24", // 118
  "_AndI__LoadUS_memory__immI_255", // 119
  "_AndI__LoadUS_memory__immI16", // 120
  "_LoadI_memory_", // 121
  "_LShiftI__LoadI_memory__immI_24", // 122
  "_LShiftI__LoadI_memory__immI_16", // 123
  "_AndI__LoadI_memory__immI_255", // 124
  "_AndI__LoadI_memory__immI_65535", // 125
  "_AndI__LoadI_memory__immI", // 126
  "_LoadL_memory_", // 127
  "_AddL__LShiftL_rRegL_immI2_immL32", // 128
  "_Binary_cmpOp_rFlagsReg", // 129
  "_Binary_rRegI_rRegI", // 130
  "_Binary_cmpOpU_rFlagsRegU", // 131
  "_Binary_cmpOpUCF_rFlagsRegUCF", // 132
  "_Binary_rRegI__LoadI_memory_", // 133
  "_Binary_rRegN_rRegN", // 134
  "_Binary_rRegP_rRegP", // 135
  "_Binary_rRegL_rRegL", // 136
  "_Binary_rRegL__LoadL_memory_", // 137
  "_Binary_regF_regF", // 138
  "_Binary_regD_regD", // 139
  "_AddI__LoadI_memory__rRegI", // 140
  "_AddI_rRegI__LoadI_memory_", // 141
  "_AddI__LoadI_memory__immI", // 142
  "_AddI__LoadI_memory__immI1", // 143
  "_AddI__LoadI_memory__immI_M1", // 144
  "_AddL__LoadL_memory__rRegL", // 145
  "_AddL_rRegL__LoadL_memory_", // 146
  "_AddL__LoadL_memory__immL32", // 147
  "_AddL__LoadL_memory__immL1", // 148
  "_AddL__LoadL_memory__immL_M1", // 149
  "_Binary_rax_RegP_rRegP", // 150
  "_Binary_rax_RegI_rRegI", // 151
  "_Binary_rax_RegL_rRegL", // 152
  "_Binary_rax_RegN_rRegN", // 153
  "_SubI__LoadI_memory__rRegI", // 154
  "_SubI__LoadI_memory__immI", // 155
  "_SubL__LoadL_memory__rRegL", // 156
  "_SubL__LoadL_memory__immL32", // 157
  "_SubI_immI0_rRegI", // 158
  "_SubI_immI0__LoadI_memory_", // 159
  "_SubL_immL0__LoadL_memory_", // 160
  "_LShiftI__LoadI_memory__immI1", // 161
  "_LShiftI__LoadI_memory__immI8", // 162
  "_LShiftI__LoadI_memory__rcx_RegI", // 163
  "_RShiftI__LoadI_memory__immI1", // 164
  "_RShiftI__LoadI_memory__immI8", // 165
  "_RShiftI__LoadI_memory__rcx_RegI", // 166
  "_URShiftI__LoadI_memory__immI1", // 167
  "_URShiftI__LoadI_memory__immI8", // 168
  "_URShiftI__LoadI_memory__rcx_RegI", // 169
  "_LShiftL__LoadL_memory__immI1", // 170
  "_LShiftL__LoadL_memory__immI8", // 171
  "_LShiftL__LoadL_memory__rcx_RegI", // 172
  "_RShiftL__LoadL_memory__immI1", // 173
  "_RShiftL__LoadL_memory__immI8", // 174
  "_RShiftL__LoadL_memory__rcx_RegI", // 175
  "_URShiftL__LoadL_memory__immI1", // 176
  "_URShiftL__LoadL_memory__immI8", // 177
  "_URShiftL__LoadL_memory__rcx_RegI", // 178
  "_LShiftI_rRegI_immI_24", // 179
  "_LShiftI_rRegI_immI_16", // 180
  "_LShiftI_rRegI_immI1", // 181
  "_URShiftI_rRegI_immI_M1", // 182
  "_LShiftI_rRegI_immI8", // 183
  "_URShiftI_rRegI_immI8", // 184
  "_LShiftI_no_rcx_RegI_rcx_RegI", // 185
  "_SubI_immI0_rcx_RegI", // 186
  "_URShiftI_no_rcx_RegI__SubI_immI0_rcx_RegI", // 187
  "_SubI_immI_32_rcx_RegI", // 188
  "_URShiftI_no_rcx_RegI__SubI_immI_32_rcx_RegI", // 189
  "_URShiftI_rRegI_immI1", // 190
  "_LShiftI_rRegI_immI_M1", // 191
  "_URShiftI_no_rcx_RegI_rcx_RegI", // 192
  "_LShiftI_no_rcx_RegI__SubI_immI0_rcx_RegI", // 193
  "_LShiftI_no_rcx_RegI__SubI_immI_32_rcx_RegI", // 194
  "_LShiftL_rRegL_immI1", // 195
  "_URShiftL_rRegL_immI_M1", // 196
  "_LShiftL_rRegL_immI8", // 197
  "_URShiftL_rRegL_immI8", // 198
  "_LShiftL_no_rcx_RegL_rcx_RegI", // 199
  "_URShiftL_no_rcx_RegL__SubI_immI0_rcx_RegI", // 200
  "_SubI_immI_64_rcx_RegI", // 201
  "_URShiftL_no_rcx_RegL__SubI_immI_64_rcx_RegI", // 202
  "_URShiftL_rRegL_immI1", // 203
  "_LShiftL_rRegL_immI_M1", // 204
  "_URShiftL_no_rcx_RegL_rcx_RegI", // 205
  "_LShiftL_no_rcx_RegL__SubI_immI0_rcx_RegI", // 206
  "_LShiftL_no_rcx_RegL__SubI_immI_64_rcx_RegI", // 207
  "_AndI_rRegI_immI_255", // 208
  "_AndI_rRegI_immI_65535", // 209
  "_AndI__LoadI_memory__rRegI", // 210
  "_AndI_rRegI__LoadI_memory_", // 211
  "_OrI__LoadI_memory__rRegI", // 212
  "_OrI_rRegI__LoadI_memory_", // 213
  "_OrI__LoadI_memory__immI", // 214
  "_XorI__LoadI_memory__rRegI", // 215
  "_XorI_rRegI__LoadI_memory_", // 216
  "_XorI__LoadI_memory__immI", // 217
  "_AndL__LoadL_memory__rRegL", // 218
  "_AndL_rRegL__LoadL_memory_", // 219
  "_AndL__LoadL_memory__immL32", // 220
  "_CastP2X_any_RegP_", // 221
  "_OrL__LoadL_memory__rRegL", // 222
  "_OrL_rRegL__LoadL_memory_", // 223
  "_OrL__LoadL_memory__immL32", // 224
  "_XorL__LoadL_memory__rRegL", // 225
  "_XorL_rRegL__LoadL_memory_", // 226
  "_XorL__LoadL_memory__immL32", // 227
  "_CmpLTMask_rRegI_rRegI", // 228
  "_AndI__CmpLTMask_rRegI_rRegI_rRegI", // 229
  "_SubI_rRegI_rRegI", // 230
  "_AndI_rRegI__CmpLTMask_rRegI_rRegI", // 231
  "_LoadF_memory_", // 232
  "_LoadD_memory_", // 233
  "_ConvF2D_regF_", // 234
  "_SqrtD__ConvF2D_regF__", // 235
  "_ConvF2D__LoadF_memory__", // 236
  "_SqrtD__ConvF2D__LoadF_memory___", // 237
  "_ConvF2D_immF_", // 238
  "_SqrtD__ConvF2D_immF__", // 239
  "_ConvI2L__LoadI_memory__", // 240
  "_Binary_rdi_RegP_rcx_RegI", // 241
  "_Binary_rsi_RegP_rdx_RegI", // 242
  "_Binary_rdi_RegP_rdx_RegI", // 243
  "_Binary_rsi_RegP_rax_RegI", // 244
  "_Binary_rdi_RegP_rsi_RegP", // 245
  "_AndI_rRegI_immI", // 246
  "_LoadP_memory_", // 247
  "_LoadN_memory_", // 248
  "_AndL_rRegL_immL32", // 249
  "_PartialSubtypeCheck_rsi_RegP_rax_RegP", // 250
  // last internally defined operand
  "storeSSI", // 251
  "storeSSL", // 252
  "storeSSP", // 253
  "storeSSF", // 254
  "storeSSD", // 255
  "leaP8", // 256
  "leaP32", // 257
  "leaPIdxOff", // 258
  "leaPIdxScale", // 259
  "leaPIdxScaleOff", // 260
  "leaPPosIdxScaleOff", // 261
  "leaPCompressedOopOffset", // 262
  "leaP8Narrow", // 263
  "leaP32Narrow", // 264
  "leaPIdxOffNarrow", // 265
  "leaPIdxScaleNarrow", // 266
  "leaPIdxScaleOffNarrow", // 267
  "leaPPosIdxScaleOffNarrow", // 268
  "loadConI", // 269
  "loadConI0", // 270
  "loadConL", // 271
  "loadConL0", // 272
  "loadConUL32", // 273
  "loadConL32", // 274
  "loadConP", // 275
  "loadConP0", // 276
  "loadConP31", // 277
  "loadConF", // 278
  "loadConN0", // 279
  "loadConN", // 280
  "loadConF0", // 281
  "loadConD", // 282
  "loadConD0", // 283
  "loadSSI", // 284
  "loadSSL", // 285
  "loadSSP", // 286
  "loadSSF", // 287
  "loadSSD", // 288
  "popCountI", // 289
  "popCountI_mem", // 290
  "popCountL", // 291
  "popCountL_mem", // 292
  "membar_volatile", // 293
  "castX2P", // 294
  "castP2X", // 295
  "checkCastPP", // 296
  "castPP", // 297
  "castII", // 298
  "storePConditional", // 299
  "storeIConditional", // 300
  "storeLConditional", // 301
  "cmpF_cc_reg", // 302
  "cmpF_cc_reg_CF", // 303
  "cmpF_cc_mem", // 304
  "cmpF_cc_memCF", // 305
  "cmpF_cc_imm", // 306
  "cmpF_cc_immCF", // 307
  "cmpD_cc_reg", // 308
  "cmpD_cc_reg_CF", // 309
  "cmpD_cc_mem", // 310
  "cmpD_cc_memCF", // 311
  "cmpD_cc_imm", // 312
  "cmpD_cc_immCF", // 313
  "absF_reg", // 314
  "absD_reg", // 315
  "negF_reg", // 316
  "negD_reg", // 317
  "roundFloat_nop", // 318
  "roundDouble_nop", // 319
  "MoveF2I_stack_reg", // 320
  "MoveI2F_stack_reg", // 321
  "MoveD2L_stack_reg", // 322
  "MoveL2D_stack_reg_partial", // 323
  "MoveL2D_stack_reg", // 324
  "MoveF2I_reg_reg", // 325
  "MoveD2L_reg_reg", // 326
  "MoveI2F_reg_reg", // 327
  "MoveL2D_reg_reg", // 328
  "compI_rReg", // 329
  "compI_rReg_imm", // 330
  "compI_rReg_mem", // 331
  "testI_reg", // 332
  "testI_reg_imm", // 333
  "testI_reg_mem", // 334
  "testI_reg_mem_0", // 335
  "compU_rReg", // 336
  "compU_rReg_imm", // 337
  "compU_rReg_mem", // 338
  "testU_reg", // 339
  "compP_rReg", // 340
  "compP_rReg_mem", // 341
  "compP_mem_rReg", // 342
  "testP_reg", // 343
  "testP_mem", // 344
  "testP_mem_reg0", // 345
  "compN_rReg", // 346
  "compN_rReg_mem", // 347
  "compN_rReg_imm", // 348
  "compN_mem_imm", // 349
  "testN_reg", // 350
  "testN_mem", // 351
  "testN_mem_reg0", // 352
  "compL_rReg", // 353
  "compL_rReg_imm", // 354
  "compL_rReg_mem", // 355
  "testL_reg", // 356
  "testL_reg_imm", // 357
  "testL_reg_mem", // 358
  "testL_reg_mem_0", // 359
  "partialSubtypeCheck_vs_Zero", // 360
  "cmpFastLock", // 361
  "cmpFastUnlock", // 362
  "safePoint_poll", // 363
  "tlsLoadP", // 364
  "loadB", // 365
  "loadB2L", // 366
  "loadUB", // 367
  "loadUB2L", // 368
  "loadUB2L_immI8", // 369
  "loadS", // 370
  "loadS2B", // 371
  "loadS2L", // 372
  "loadUS", // 373
  "loadUS2B", // 374
  "loadUS2L", // 375
  "loadUS2L_immI_255", // 376
  "loadUS2L_immI16", // 377
  "loadI", // 378
  "loadI2B", // 379
  "loadI2UB", // 380
  "loadI2S", // 381
  "loadI2US", // 382
  "loadI2L", // 383
  "loadI2L_immI_255", // 384
  "loadI2L_immI_65535", // 385
  "loadI2L_immI", // 386
  "loadUI2L", // 387
  "loadL", // 388
  "loadRange", // 389
  "loadP", // 390
  "loadN", // 391
  "loadKlass", // 392
  "loadNKlass", // 393
  "loadF", // 394
  "loadD_partial", // 395
  "loadD", // 396
  "loadA8B", // 397
  "loadA4S", // 398
  "loadA4C", // 399
  "load2IU", // 400
  "loadA2F", // 401
  "prefetchr", // 402
  "prefetchrNTA", // 403
  "prefetchrT0", // 404
  "prefetchrT2", // 405
  "prefetchw", // 406
  "prefetchwNTA", // 407
  "prefetchwT0", // 408
  "prefetchwT2", // 409
  "storeB", // 410
  "storeC", // 411
  "storeI", // 412
  "storeL", // 413
  "storeP", // 414
  "storeImmP0", // 415
  "storeImmP", // 416
  "storeN", // 417
  "storeImmN0", // 418
  "storeImmN", // 419
  "storeImmI0", // 420
  "storeImmI", // 421
  "storeImmL0", // 422
  "storeImmL", // 423
  "storeImmC0", // 424
  "storeImmI16", // 425
  "storeImmB0", // 426
  "storeImmB", // 427
  "storeA8B", // 428
  "storeA4C", // 429
  "storeA2I", // 430
  "storeImmCM0_reg", // 431
  "storeImmCM0", // 432
  "storeA2F", // 433
  "storeF", // 434
  "storeF0", // 435
  "storeF_imm", // 436
  "storeD", // 437
  "storeD0_imm", // 438
  "storeD0", // 439
  "bytes_reverse_int", // 440
  "bytes_reverse_long", // 441
  "bytes_reverse_unsigned_short", // 442
  "bytes_reverse_short", // 443
  "countLeadingZerosI", // 444
  "countLeadingZerosI_bsr", // 445
  "countLeadingZerosL", // 446
  "countLeadingZerosL_bsr", // 447
  "countTrailingZerosI", // 448
  "countTrailingZerosL", // 449
  "membar_acquire", // 450
  "membar_acquire_lock", // 451
  "membar_release", // 452
  "membar_release_lock", // 453
  "unnecessary_membar_volatile", // 454
  "encodeHeapOop", // 455
  "encodeHeapOop_not_null", // 456
  "decodeHeapOop", // 457
  "decodeHeapOop_not_null", // 458
  "jumpXtnd_offset", // 459
  "jumpXtnd_addr", // 460
  "jumpXtnd", // 461
  "cmovI_reg", // 462
  "cmovI_regU", // 463
  "cmovI_regUCF", // 464
  "cmovI_mem", // 465
  "cmovI_memU", // 466
  "cmovI_memUCF", // 467
  "cmovN_reg", // 468
  "cmovN_regU", // 469
  "cmovN_regUCF", // 470
  "cmovP_reg", // 471
  "cmovP_regU", // 472
  "cmovP_regUCF", // 473
  "cmovL_reg", // 474
  "cmovL_mem", // 475
  "cmovL_regU", // 476
  "cmovL_regUCF", // 477
  "cmovL_memU", // 478
  "cmovL_memUCF", // 479
  "cmovF_reg", // 480
  "cmovF_regU", // 481
  "cmovF_regUCF", // 482
  "cmovD_reg", // 483
  "cmovD_regU", // 484
  "cmovD_regUCF", // 485
  "addI_rReg", // 486
  "addI_rReg_imm", // 487
  "addI_rReg_mem", // 488
  "addI_rReg_mem_0", // 489
  "addI_mem_rReg", // 490
  "addI_mem_rReg_0", // 491
  "addI_mem_imm", // 492
  "incI_rReg", // 493
  "incI_mem", // 494
  "decI_rReg", // 495
  "decI_mem", // 496
  "leaI_rReg_immI", // 497
  "addL_rReg", // 498
  "addL_rReg_imm", // 499
  "addL_rReg_mem", // 500
  "addL_rReg_mem_0", // 501
  "addL_mem_rReg", // 502
  "addL_mem_rReg_0", // 503
  "addL_mem_imm", // 504
  "incL_rReg", // 505
  "incL_mem", // 506
  "decL_rReg", // 507
  "decL_mem", // 508
  "leaL_rReg_immL", // 509
  "addP_rReg", // 510
  "addP_rReg_imm", // 511
  "leaP_rReg_imm", // 512
  "loadPLocked", // 513
  "loadLLocked", // 514
  "compareAndSwapP", // 515
  "compareAndSwapL", // 516
  "compareAndSwapI", // 517
  "compareAndSwapN", // 518
  "subI_rReg", // 519
  "subI_rReg_imm", // 520
  "subI_rReg_mem", // 521
  "subI_mem_rReg", // 522
  "subI_mem_imm", // 523
  "subL_rReg", // 524
  "subL_rReg_imm", // 525
  "subL_rReg_mem", // 526
  "subL_mem_rReg", // 527
  "subL_mem_imm", // 528
  "subP_rReg", // 529
  "negI_rReg", // 530
  "negI_mem", // 531
  "negL_rReg", // 532
  "negL_mem", // 533
  "mulI_rReg", // 534
  "mulI_rReg_imm", // 535
  "mulI_mem", // 536
  "mulI_mem_0", // 537
  "mulI_mem_imm", // 538
  "mulL_rReg", // 539
  "mulL_rReg_imm", // 540
  "mulL_mem", // 541
  "mulL_mem_0", // 542
  "mulL_mem_imm", // 543
  "mulHiL_rReg", // 544
  "divI_rReg", // 545
  "divL_rReg", // 546
  "divModI_rReg_divmod", // 547
  "divModL_rReg_divmod", // 548
  "loadConL_0x6666666666666667", // 549
  "mul_hi", // 550
  "sarL_rReg_63", // 551
  "sarL_rReg_2", // 552
  "divL_10", // 553
  "modI_rReg", // 554
  "modL_rReg", // 555
  "salI_rReg_1", // 556
  "salI_mem_1", // 557
  "salI_rReg_imm", // 558
  "salI_mem_imm", // 559
  "salI_rReg_CL", // 560
  "salI_mem_CL", // 561
  "sarI_rReg_1", // 562
  "sarI_mem_1", // 563
  "sarI_rReg_imm", // 564
  "sarI_mem_imm", // 565
  "sarI_rReg_CL", // 566
  "sarI_mem_CL", // 567
  "shrI_rReg_1", // 568
  "shrI_mem_1", // 569
  "shrI_rReg_imm", // 570
  "shrI_mem_imm", // 571
  "shrI_rReg_CL", // 572
  "shrI_mem_CL", // 573
  "salL_rReg_1", // 574
  "salL_mem_1", // 575
  "salL_rReg_imm", // 576
  "salL_mem_imm", // 577
  "salL_rReg_CL", // 578
  "salL_mem_CL", // 579
  "sarL_rReg_1", // 580
  "sarL_mem_1", // 581
  "sarL_rReg_imm", // 582
  "sarL_mem_imm", // 583
  "sarL_rReg_CL", // 584
  "sarL_mem_CL", // 585
  "shrL_rReg_1", // 586
  "shrL_mem_1", // 587
  "shrL_rReg_imm", // 588
  "shrL_mem_imm", // 589
  "shrL_rReg_CL", // 590
  "shrL_mem_CL", // 591
  "i2b", // 592
  "i2s", // 593
  "rolI_rReg_imm1", // 594
  "rolI_rReg_imm8", // 595
  "rolI_rReg_CL", // 596
  "rolI_rReg_i1", // 597
  "rolI_rReg_i1_0", // 598
  "rolI_rReg_i8", // 599
  "rolI_rReg_i8_0", // 600
  "rolI_rReg_Var_C0", // 601
  "rolI_rReg_Var_C0_0", // 602
  "rolI_rReg_Var_C32", // 603
  "rolI_rReg_Var_C32_0", // 604
  "rorI_rReg_imm1", // 605
  "rorI_rReg_imm8", // 606
  "rorI_rReg_CL", // 607
  "rorI_rReg_i1", // 608
  "rorI_rReg_i1_0", // 609
  "rorI_rReg_i8", // 610
  "rorI_rReg_i8_0", // 611
  "rorI_rReg_Var_C0", // 612
  "rorI_rReg_Var_C0_0", // 613
  "rorI_rReg_Var_C32", // 614
  "rorI_rReg_Var_C32_0", // 615
  "rolL_rReg_imm1", // 616
  "rolL_rReg_imm8", // 617
  "rolL_rReg_CL", // 618
  "rolL_rReg_i1", // 619
  "rolL_rReg_i1_0", // 620
  "rolL_rReg_i8", // 621
  "rolL_rReg_i8_0", // 622
  "rolL_rReg_Var_C0", // 623
  "rolL_rReg_Var_C0_0", // 624
  "rolL_rReg_Var_C64", // 625
  "rolL_rReg_Var_C64_0", // 626
  "rorL_rReg_imm1", // 627
  "rorL_rReg_imm8", // 628
  "rorL_rReg_CL", // 629
  "rorL_rReg_i1", // 630
  "rorL_rReg_i1_0", // 631
  "rorL_rReg_i8", // 632
  "rorL_rReg_i8_0", // 633
  "rorL_rReg_Var_C0", // 634
  "rorL_rReg_Var_C0_0", // 635
  "rorL_rReg_Var_C64", // 636
  "rorL_rReg_Var_C64_0", // 637
  "andI_rReg", // 638
  "andI_rReg_imm255", // 639
  "andI2L_rReg_imm255", // 640
  "andI_rReg_imm65535", // 641
  "andI2L_rReg_imm65535", // 642
  "andI_rReg_imm", // 643
  "andI_rReg_mem", // 644
  "andI_rReg_mem_0", // 645
  "andI_mem_rReg", // 646
  "andI_mem_rReg_0", // 647
  "andI_mem_imm", // 648
  "orI_rReg", // 649
  "orI_rReg_imm", // 650
  "orI_rReg_mem", // 651
  "orI_rReg_mem_0", // 652
  "orI_mem_rReg", // 653
  "orI_mem_rReg_0", // 654
  "orI_mem_imm", // 655
  "xorI_rReg", // 656
  "xorI_rReg_im1", // 657
  "xorI_rReg_imm", // 658
  "xorI_rReg_mem", // 659
  "xorI_rReg_mem_0", // 660
  "xorI_mem_rReg", // 661
  "xorI_mem_rReg_0", // 662
  "xorI_mem_imm", // 663
  "andL_rReg", // 664
  "andL_rReg_imm255", // 665
  "andL_rReg_imm65535", // 666
  "andL_rReg_imm", // 667
  "andL_rReg_mem", // 668
  "andL_rReg_mem_0", // 669
  "andL_mem_rReg", // 670
  "andL_mem_rReg_0", // 671
  "andL_mem_imm", // 672
  "orL_rReg", // 673
  "orL_rReg_castP2X", // 674
  "orL_rReg_castP2X_0", // 675
  "orL_rReg_imm", // 676
  "orL_rReg_mem", // 677
  "orL_rReg_mem_0", // 678
  "orL_mem_rReg", // 679
  "orL_mem_rReg_0", // 680
  "orL_mem_imm", // 681
  "xorL_rReg", // 682
  "xorL_rReg_im1", // 683
  "xorL_rReg_imm", // 684
  "xorL_rReg_mem", // 685
  "xorL_rReg_mem_0", // 686
  "xorL_mem_rReg", // 687
  "xorL_mem_rReg_0", // 688
  "xorL_mem_imm", // 689
  "convI2B", // 690
  "convP2B", // 691
  "cmpLTMask", // 692
  "cmpLTMask0", // 693
  "cadd_cmpLTMask", // 694
  "cadd_cmpLTMask_1", // 695
  "cadd_cmpLTMask_0", // 696
  "cadd_cmpLTMask_2", // 697
  "cmpF_reg", // 698
  "cmpF_mem", // 699
  "cmpF_imm", // 700
  "cmpD_reg", // 701
  "cmpD_mem", // 702
  "cmpD_imm", // 703
  "addF_reg", // 704
  "addF_mem", // 705
  "addF_mem_0", // 706
  "addF_imm", // 707
  "addD_reg", // 708
  "addD_mem", // 709
  "addD_mem_0", // 710
  "addD_imm", // 711
  "subF_reg", // 712
  "subF_mem", // 713
  "subF_imm", // 714
  "subD_reg", // 715
  "subD_mem", // 716
  "subD_imm", // 717
  "mulF_reg", // 718
  "mulF_mem", // 719
  "mulF_mem_0", // 720
  "mulF_imm", // 721
  "mulD_reg", // 722
  "mulD_mem", // 723
  "mulD_mem_0", // 724
  "mulD_imm", // 725
  "divF_reg", // 726
  "divF_mem", // 727
  "divF_imm", // 728
  "divD_reg", // 729
  "divD_mem", // 730
  "divD_imm", // 731
  "sqrtF_reg", // 732
  "sqrtF_mem", // 733
  "sqrtF_imm", // 734
  "sqrtD_reg", // 735
  "sqrtD_mem", // 736
  "sqrtD_imm", // 737
  "cosD_reg", // 738
  "sinD_reg", // 739
  "tanD_reg", // 740
  "log10D_reg", // 741
  "logD_reg", // 742
  "convF2D_reg_reg", // 743
  "convF2D_reg_mem", // 744
  "convD2F_reg_reg", // 745
  "convD2F_reg_mem", // 746
  "convF2I_reg_reg", // 747
  "convF2L_reg_reg", // 748
  "convD2I_reg_reg", // 749
  "convD2L_reg_reg", // 750
  "convI2F_reg_reg", // 751
  "convI2F_reg_mem", // 752
  "convI2D_reg_reg", // 753
  "convI2D_reg_mem", // 754
  "convXI2F_reg", // 755
  "convXI2D_reg", // 756
  "convL2F_reg_reg", // 757
  "convL2F_reg_mem", // 758
  "convL2D_reg_reg", // 759
  "convL2D_reg_mem", // 760
  "convI2L_reg_reg", // 761
  "convI2L_reg_reg_zex", // 762
  "convI2L_reg_mem_zex", // 763
  "zerox_long_reg_reg", // 764
  "convL2I_reg_reg", // 765
  "MoveF2I_reg_stack", // 766
  "MoveI2F_reg_stack", // 767
  "MoveD2L_reg_stack", // 768
  "MoveL2D_reg_stack", // 769
  "Repl8B_reg", // 770
  "Repl8B_rRegI", // 771
  "Repl8B_immI0", // 772
  "Repl4S_reg", // 773
  "Repl4S_rRegI", // 774
  "Repl4S_immI0", // 775
  "Repl4C_reg", // 776
  "Repl4C_rRegI", // 777
  "Repl4C_immI0", // 778
  "Repl2I_reg", // 779
  "Repl2I_rRegI", // 780
  "Repl2I_immI0", // 781
  "Repl2F_reg", // 782
  "Repl2F_regF", // 783
  "Repl2F_immF0", // 784
  "rep_stos", // 785
  "string_compare", // 786
  "string_indexof", // 787
  "string_equals", // 788
  "array_equals", // 789
  "cmpL3_reg_reg", // 790
  "cmovI_reg_g", // 791
  "minI_rReg", // 792
  "cmovI_reg_l", // 793
  "maxI_rReg", // 794
  "jmpDir", // 795
  "jmpCon", // 796
  "jmpLoopEnd", // 797
  "jmpLoopEndU", // 798
  "jmpLoopEndUCF", // 799
  "jmpConU", // 800
  "jmpConUCF", // 801
  "jmpConUCF2", // 802
  "partialSubtypeCheck", // 803
  "jmpDir_short", // 804
  "jmpCon_short", // 805
  "jmpLoopEnd_short", // 806
  "jmpLoopEndU_short", // 807
  "jmpLoopEndUCF_short", // 808
  "jmpConU_short", // 809
  "jmpConUCF_short", // 810
  "jmpConUCF2_short", // 811
  "CallStaticJavaDirect", // 812
  "CallStaticJavaHandle", // 813
  "CallDynamicJavaDirect", // 814
  "CallRuntimeDirect", // 815
  "CallLeafDirect", // 816
  "CallLeafNoFPDirect", // 817
  "Ret", // 818
  "TailCalljmpInd", // 819
  "tailjmpInd", // 820
  "CreateException", // 821
  "RethrowException", // 822
  "ShouldNotReachHere", // 823
  // last instruction
  "no trailing comma"
};

const        bool  swallowed[] = {
  false, // 0
  false, // 1
  false, // 2
  false, // 3
  false, // 4
  false, // 5
  false, // 6
  false, // 7
  true, // 8
  true, // 9
  true, // 10
  true, // 11
  true, // 12
  true, // 13
  true, // 14
  true, // 15
  true, // 16
  true, // 17
  true, // 18
  true, // 19
  true, // 20
  true, // 21
  true, // 22
  true, // 23
  true, // 24
  true, // 25
  true, // 26
  true, // 27
  true, // 28
  true, // 29
  true, // 30
  true, // 31
  true, // 32
  true, // 33
  true, // 34
  true, // 35
  true, // 36
  true, // 37
  true, // 38
  true, // 39
  true, // 40
  true, // 41
  false, // 42
  false, // 43
  false, // 44
  false, // 45
  false, // 46
  false, // 47
  false, // 48
  false, // 49
  false, // 50
  false, // 51
  false, // 52
  false, // 53
  false, // 54
  false, // 55
  false, // 56
  false, // 57
  false, // 58
  false, // 59
  false, // 60
  false, // 61
  false, // 62
  false, // 63
  false, // 64
  false, // 65
  false, // 66
  false, // 67
  false, // 68
  false, // 69
  false, // 70
  false, // 71
  false, // 72
  false, // 73
  false, // 74
  false, // 75
  false, // 76
  false, // 77
  false, // 78
  false, // 79
  false, // 80
  false, // 81
  false, // 82
  false, // 83
  false, // 84
  false, // 85
  false, // 86
  false, // 87
  false, // 88
  false, // 89
  false, // 90
  false, // 91
  false, // 92
  false, // 93
  false, // 94
  false, // 95
  false, // 96
  true, // 97
  true, // 98
  true, // 99
  true, // 100
  // last operand
  false, // 101
  // last operand class
  false, // 102
  false, // 103
  false, // 104
  false, // 105
  false, // 106
  false, // 107
  false, // 108
  false, // 109
  false, // 110
  false, // 111
  false, // 112
  false, // 113
  false, // 114
  false, // 115
  false, // 116
  false, // 117
  false, // 118
  false, // 119
  false, // 120
  false, // 121
  false, // 122
  false, // 123
  false, // 124
  false, // 125
  false, // 126
  false, // 127
  false, // 128
  false, // 129
  false, // 130
  false, // 131
  false, // 132
  false, // 133
  false, // 134
  false, // 135
  false, // 136
  false, // 137
  false, // 138
  false, // 139
  false, // 140
  false, // 141
  false, // 142
  false, // 143
  false, // 144
  false, // 145
  false, // 146
  false, // 147
  false, // 148
  false, // 149
  false, // 150
  false, // 151
  false, // 152
  false, // 153
  false, // 154
  false, // 155
  false, // 156
  false, // 157
  false, // 158
  false, // 159
  false, // 160
  false, // 161
  false, // 162
  false, // 163
  false, // 164
  false, // 165
  false, // 166
  false, // 167
  false, // 168
  false, // 169
  false, // 170
  false, // 171
  false, // 172
  false, // 173
  false, // 174
  false, // 175
  false, // 176
  false, // 177
  false, // 178
  false, // 179
  false, // 180
  false, // 181
  false, // 182
  false, // 183
  false, // 184
  false, // 185
  false, // 186
  false, // 187
  false, // 188
  false, // 189
  false, // 190
  false, // 191
  false, // 192
  false, // 193
  false, // 194
  false, // 195
  false, // 196
  false, // 197
  false, // 198
  false, // 199
  false, // 200
  false, // 201
  false, // 202
  false, // 203
  false, // 204
  false, // 205
  false, // 206
  false, // 207
  false, // 208
  false, // 209
  false, // 210
  false, // 211
  false, // 212
  false, // 213
  false, // 214
  false, // 215
  false, // 216
  false, // 217
  false, // 218
  false, // 219
  false, // 220
  false, // 221
  false, // 222
  false, // 223
  false, // 224
  false, // 225
  false, // 226
  false, // 227
  false, // 228
  false, // 229
  false, // 230
  false, // 231
  false, // 232
  false, // 233
  false, // 234
  false, // 235
  false, // 236
  false, // 237
  false, // 238
  false, // 239
  false, // 240
  false, // 241
  false, // 242
  false, // 243
  false, // 244
  false, // 245
  false, // 246
  false, // 247
  false, // 248
  false, // 249
  false, // 250
  // last internally defined operand
  false, // 251
  false, // 252
  false, // 253
  false, // 254
  false, // 255
  false, // 256
  false, // 257
  false, // 258
  false, // 259
  false, // 260
  false, // 261
  false, // 262
  false, // 263
  false, // 264
  false, // 265
  false, // 266
  false, // 267
  false, // 268
  false, // 269
  false, // 270
  false, // 271
  false, // 272
  false, // 273
  false, // 274
  false, // 275
  false, // 276
  false, // 277
  false, // 278
  false, // 279
  false, // 280
  false, // 281
  false, // 282
  false, // 283
  false, // 284
  false, // 285
  false, // 286
  false, // 287
  false, // 288
  false, // 289
  false, // 290
  false, // 291
  false, // 292
  false, // 293
  false, // 294
  false, // 295
  false, // 296
  false, // 297
  false, // 298
  false, // 299
  false, // 300
  false, // 301
  false, // 302
  false, // 303
  false, // 304
  false, // 305
  false, // 306
  false, // 307
  false, // 308
  false, // 309
  false, // 310
  false, // 311
  false, // 312
  false, // 313
  false, // 314
  false, // 315
  false, // 316
  false, // 317
  false, // 318
  false, // 319
  false, // 320
  false, // 321
  false, // 322
  false, // 323
  false, // 324
  false, // 325
  false, // 326
  false, // 327
  false, // 328
  false, // 329
  false, // 330
  false, // 331
  false, // 332
  false, // 333
  false, // 334
  false, // 335
  false, // 336
  false, // 337
  false, // 338
  false, // 339
  false, // 340
  false, // 341
  false, // 342
  false, // 343
  false, // 344
  false, // 345
  false, // 346
  false, // 347
  false, // 348
  false, // 349
  false, // 350
  false, // 351
  false, // 352
  false, // 353
  false, // 354
  false, // 355
  false, // 356
  false, // 357
  false, // 358
  false, // 359
  false, // 360
  false, // 361
  false, // 362
  false, // 363
  false, // 364
  false, // 365
  false, // 366
  false, // 367
  false, // 368
  false, // 369
  false, // 370
  false, // 371
  false, // 372
  false, // 373
  false, // 374
  false, // 375
  false, // 376
  false, // 377
  false, // 378
  false, // 379
  false, // 380
  false, // 381
  false, // 382
  false, // 383
  false, // 384
  false, // 385
  false, // 386
  false, // 387
  false, // 388
  false, // 389
  false, // 390
  false, // 391
  false, // 392
  false, // 393
  false, // 394
  false, // 395
  false, // 396
  false, // 397
  false, // 398
  false, // 399
  false, // 400
  false, // 401
  false, // 402
  false, // 403
  false, // 404
  false, // 405
  false, // 406
  false, // 407
  false, // 408
  false, // 409
  false, // 410
  false, // 411
  false, // 412
  false, // 413
  false, // 414
  false, // 415
  false, // 416
  false, // 417
  false, // 418
  false, // 419
  false, // 420
  false, // 421
  false, // 422
  false, // 423
  false, // 424
  false, // 425
  false, // 426
  false, // 427
  false, // 428
  false, // 429
  false, // 430
  false, // 431
  false, // 432
  false, // 433
  false, // 434
  false, // 435
  false, // 436
  false, // 437
  false, // 438
  false, // 439
  false, // 440
  false, // 441
  false, // 442
  false, // 443
  false, // 444
  false, // 445
  false, // 446
  false, // 447
  false, // 448
  false, // 449
  false, // 450
  false, // 451
  false, // 452
  false, // 453
  false, // 454
  false, // 455
  false, // 456
  false, // 457
  false, // 458
  false, // 459
  false, // 460
  false, // 461
  false, // 462
  false, // 463
  false, // 464
  false, // 465
  false, // 466
  false, // 467
  false, // 468
  false, // 469
  false, // 470
  false, // 471
  false, // 472
  false, // 473
  false, // 474
  false, // 475
  false, // 476
  false, // 477
  false, // 478
  false, // 479
  false, // 480
  false, // 481
  false, // 482
  false, // 483
  false, // 484
  false, // 485
  false, // 486
  false, // 487
  false, // 488
  false, // 489
  false, // 490
  false, // 491
  false, // 492
  false, // 493
  false, // 494
  false, // 495
  false, // 496
  false, // 497
  false, // 498
  false, // 499
  false, // 500
  false, // 501
  false, // 502
  false, // 503
  false, // 504
  false, // 505
  false, // 506
  false, // 507
  false, // 508
  false, // 509
  false, // 510
  false, // 511
  false, // 512
  false, // 513
  false, // 514
  false, // 515
  false, // 516
  false, // 517
  false, // 518
  false, // 519
  false, // 520
  false, // 521
  false, // 522
  false, // 523
  false, // 524
  false, // 525
  false, // 526
  false, // 527
  false, // 528
  false, // 529
  false, // 530
  false, // 531
  false, // 532
  false, // 533
  false, // 534
  false, // 535
  false, // 536
  false, // 537
  false, // 538
  false, // 539
  false, // 540
  false, // 541
  false, // 542
  false, // 543
  false, // 544
  false, // 545
  false, // 546
  false, // 547
  false, // 548
  false, // 549
  false, // 550
  false, // 551
  false, // 552
  false, // 553
  false, // 554
  false, // 555
  false, // 556
  false, // 557
  false, // 558
  false, // 559
  false, // 560
  false, // 561
  false, // 562
  false, // 563
  false, // 564
  false, // 565
  false, // 566
  false, // 567
  false, // 568
  false, // 569
  false, // 570
  false, // 571
  false, // 572
  false, // 573
  false, // 574
  false, // 575
  false, // 576
  false, // 577
  false, // 578
  false, // 579
  false, // 580
  false, // 581
  false, // 582
  false, // 583
  false, // 584
  false, // 585
  false, // 586
  false, // 587
  false, // 588
  false, // 589
  false, // 590
  false, // 591
  false, // 592
  false, // 593
  false, // 594
  false, // 595
  false, // 596
  false, // 597
  false, // 598
  false, // 599
  false, // 600
  false, // 601
  false, // 602
  false, // 603
  false, // 604
  false, // 605
  false, // 606
  false, // 607
  false, // 608
  false, // 609
  false, // 610
  false, // 611
  false, // 612
  false, // 613
  false, // 614
  false, // 615
  false, // 616
  false, // 617
  false, // 618
  false, // 619
  false, // 620
  false, // 621
  false, // 622
  false, // 623
  false, // 624
  false, // 625
  false, // 626
  false, // 627
  false, // 628
  false, // 629
  false, // 630
  false, // 631
  false, // 632
  false, // 633
  false, // 634
  false, // 635
  false, // 636
  false, // 637
  false, // 638
  false, // 639
  false, // 640
  false, // 641
  false, // 642
  false, // 643
  false, // 644
  false, // 645
  false, // 646
  false, // 647
  false, // 648
  false, // 649
  false, // 650
  false, // 651
  false, // 652
  false, // 653
  false, // 654
  false, // 655
  false, // 656
  false, // 657
  false, // 658
  false, // 659
  false, // 660
  false, // 661
  false, // 662
  false, // 663
  false, // 664
  false, // 665
  false, // 666
  false, // 667
  false, // 668
  false, // 669
  false, // 670
  false, // 671
  false, // 672
  false, // 673
  false, // 674
  false, // 675
  false, // 676
  false, // 677
  false, // 678
  false, // 679
  false, // 680
  false, // 681
  false, // 682
  false, // 683
  false, // 684
  false, // 685
  false, // 686
  false, // 687
  false, // 688
  false, // 689
  false, // 690
  false, // 691
  false, // 692
  false, // 693
  false, // 694
  false, // 695
  false, // 696
  false, // 697
  false, // 698
  false, // 699
  false, // 700
  false, // 701
  false, // 702
  false, // 703
  false, // 704
  false, // 705
  false, // 706
  false, // 707
  false, // 708
  false, // 709
  false, // 710
  false, // 711
  false, // 712
  false, // 713
  false, // 714
  false, // 715
  false, // 716
  false, // 717
  false, // 718
  false, // 719
  false, // 720
  false, // 721
  false, // 722
  false, // 723
  false, // 724
  false, // 725
  false, // 726
  false, // 727
  false, // 728
  false, // 729
  false, // 730
  false, // 731
  false, // 732
  false, // 733
  false, // 734
  false, // 735
  false, // 736
  false, // 737
  false, // 738
  false, // 739
  false, // 740
  false, // 741
  false, // 742
  false, // 743
  false, // 744
  false, // 745
  false, // 746
  false, // 747
  false, // 748
  false, // 749
  false, // 750
  false, // 751
  false, // 752
  false, // 753
  false, // 754
  false, // 755
  false, // 756
  false, // 757
  false, // 758
  false, // 759
  false, // 760
  false, // 761
  false, // 762
  false, // 763
  false, // 764
  false, // 765
  false, // 766
  false, // 767
  false, // 768
  false, // 769
  false, // 770
  false, // 771
  false, // 772
  false, // 773
  false, // 774
  false, // 775
  false, // 776
  false, // 777
  false, // 778
  false, // 779
  false, // 780
  false, // 781
  false, // 782
  false, // 783
  false, // 784
  false, // 785
  false, // 786
  false, // 787
  false, // 788
  false, // 789
  false, // 790
  false, // 791
  false, // 792
  false, // 793
  false, // 794
  false, // 795
  false, // 796
  false, // 797
  false, // 798
  false, // 799
  false, // 800
  false, // 801
  false, // 802
  false, // 803
  false, // 804
  false, // 805
  false, // 806
  false, // 807
  false, // 808
  false, // 809
  false, // 810
  false, // 811
  false, // 812
  false, // 813
  false, // 814
  false, // 815
  false, // 816
  false, // 817
  false, // 818
  false, // 819
  false, // 820
  false, // 821
  false, // 822
  false, // 823
  // last instruction
  false // no trailing comma
};

// Mapping from machine-independent opcode to boolean
const        char must_clone[] = {
  0, // Node: 0
  0, // Set: 1
  0, // RegN: 2
  0, // RegI: 3
  0, // RegP: 4
  0, // RegF: 5
  0, // RegD: 6
  0, // RegL: 7
  0, // RegFlags: 8
  0, // _last_machine_leaf: 9
  0, // AbsD: 10
  0, // AbsF: 11
  0, // AbsI: 12
  0, // AddD: 13
  0, // AddF: 14
  0, // AddI: 15
  0, // AddL: 16
  0, // AddP: 17
  0, // Allocate: 18
  0, // AllocateArray: 19
  0, // AndI: 20
  0, // AndL: 21
  0, // AryEq: 22
  0, // AtanD: 23
  1, // Binary: 24
  1, // Bool: 25
  0, // BoxLock: 26
  0, // ReverseBytesI: 27
  0, // ReverseBytesL: 28
  0, // ReverseBytesUS: 29
  0, // ReverseBytesS: 30
  0, // CProj: 31
  0, // CallDynamicJava: 32
  0, // CallJava: 33
  0, // CallLeaf: 34
  0, // CallLeafNoFP: 35
  0, // CallRuntime: 36
  0, // CallStaticJava: 37
  0, // CastII: 38
  0, // CastX2P: 39
  0, // CastP2X: 40
  0, // CastPP: 41
  0, // Catch: 42
  0, // CatchProj: 43
  0, // CheckCastPP: 44
  0, // ClearArray: 45
  0, // ConstraintCast: 46
  0, // CMoveD: 47
  0, // CMoveF: 48
  0, // CMoveI: 49
  0, // CMoveL: 50
  0, // CMoveP: 51
  0, // CMoveN: 52
  1, // CmpN: 53
  1, // CmpD: 54
  0, // CmpD3: 55
  1, // CmpF: 56
  0, // CmpF3: 57
  1, // CmpI: 58
  1, // CmpL: 59
  0, // CmpL3: 60
  0, // CmpLTMask: 61
  1, // CmpP: 62
  1, // CmpU: 63
  0, // CompareAndSwapI: 64
  0, // CompareAndSwapL: 65
  0, // CompareAndSwapP: 66
  0, // CompareAndSwapN: 67
  0, // Con: 68
  0, // ConN: 69
  0, // ConD: 70
  0, // ConF: 71
  0, // ConI: 72
  0, // ConL: 73
  0, // ConP: 74
  0, // Conv2B: 75
  0, // ConvD2F: 76
  0, // ConvD2I: 77
  0, // ConvD2L: 78
  0, // ConvF2D: 79
  0, // ConvF2I: 80
  0, // ConvF2L: 81
  0, // ConvI2D: 82
  0, // ConvI2F: 83
  0, // ConvI2L: 84
  0, // ConvL2D: 85
  0, // ConvL2F: 86
  0, // ConvL2I: 87
  0, // CosD: 88
  0, // CountedLoop: 89
  0, // CountedLoopEnd: 90
  0, // CountLeadingZerosI: 91
  0, // CountLeadingZerosL: 92
  0, // CountTrailingZerosI: 93
  0, // CountTrailingZerosL: 94
  0, // CreateEx: 95
  0, // DecodeN: 96
  0, // DivD: 97
  0, // DivF: 98
  0, // DivI: 99
  0, // DivL: 100
  0, // DivMod: 101
  0, // DivModI: 102
  0, // DivModL: 103
  0, // EncodeP: 104
  0, // ExpD: 105
  1, // FastLock: 106
  1, // FastUnlock: 107
  0, // Goto: 108
  0, // Halt: 109
  0, // If: 110
  0, // IfFalse: 111
  0, // IfTrue: 112
  0, // Initialize: 113
  0, // JProj: 114
  0, // Jump: 115
  0, // JumpProj: 116
  0, // LShiftI: 117
  0, // LShiftL: 118
  0, // LoadB: 119
  0, // LoadUB: 120
  0, // LoadUS: 121
  0, // LoadD: 122
  0, // LoadD_unaligned: 123
  0, // LoadF: 124
  0, // LoadI: 125
  0, // LoadUI2L: 126
  0, // LoadKlass: 127
  0, // LoadNKlass: 128
  0, // LoadL: 129
  0, // LoadL_unaligned: 130
  0, // LoadPLocked: 131
  0, // LoadLLocked: 132
  0, // LoadP: 133
  0, // LoadN: 134
  0, // LoadRange: 135
  0, // LoadS: 136
  0, // Lock: 137
  0, // LogD: 138
  0, // Log10D: 139
  0, // Loop: 140
  0, // Mach: 141
  0, // MachProj: 142
  0, // MaxI: 143
  0, // MemBarAcquire: 144
  0, // MemBarCPUOrder: 145
  0, // MemBarRelease: 146
  0, // MemBarVolatile: 147
  0, // MergeMem: 148
  0, // MinI: 149
  0, // ModD: 150
  0, // ModF: 151
  0, // ModI: 152
  0, // ModL: 153
  0, // MoveI2F: 154
  0, // MoveF2I: 155
  0, // MoveL2D: 156
  0, // MoveD2L: 157
  0, // MulD: 158
  0, // MulF: 159
  0, // MulHiL: 160
  0, // MulI: 161
  0, // MulL: 162
  0, // Multi: 163
  0, // NegD: 164
  0, // NegF: 165
  0, // NeverBranch: 166
  0, // Opaque1: 167
  0, // Opaque2: 168
  0, // OrI: 169
  0, // OrL: 170
  0, // PCTable: 171
  0, // Parm: 172
  0, // PartialSubtypeCheck: 173
  0, // Phi: 174
  0, // PopCountI: 175
  0, // PopCountL: 176
  0, // PowD: 177
  0, // PrefetchRead: 178
  0, // PrefetchWrite: 179
  0, // Proj: 180
  0, // RShiftI: 181
  0, // RShiftL: 182
  0, // Region: 183
  0, // Rethrow: 184
  0, // Return: 185
  0, // Root: 186
  0, // RoundDouble: 187
  0, // RoundFloat: 188
  0, // SafePoint: 189
  0, // SafePointScalarObject: 190
  0, // SCMemProj: 191
  0, // SinD: 192
  0, // SqrtD: 193
  0, // Start: 194
  0, // StartOSR: 195
  0, // StoreB: 196
  0, // StoreC: 197
  0, // StoreCM: 198
  0, // StorePConditional: 199
  0, // StoreIConditional: 200
  0, // StoreLConditional: 201
  0, // StoreD: 202
  0, // StoreF: 203
  0, // StoreI: 204
  0, // StoreL: 205
  0, // StoreP: 206
  0, // StoreN: 207
  0, // StrComp: 208
  0, // StrEquals: 209
  0, // StrIndexOf: 210
  0, // SubD: 211
  0, // SubF: 212
  0, // SubI: 213
  0, // SubL: 214
  0, // TailCall: 215
  0, // TailJump: 216
  0, // TanD: 217
  0, // ThreadLocal: 218
  0, // Unlock: 219
  0, // URShiftI: 220
  0, // URShiftL: 221
  0, // XorI: 222
  0, // XorL: 223
  0, // Vector: 224
  0, // AddVB: 225
  0, // AddVC: 226
  0, // AddVS: 227
  0, // AddVI: 228
  0, // AddVL: 229
  0, // AddVF: 230
  0, // AddVD: 231
  0, // SubVB: 232
  0, // SubVC: 233
  0, // SubVS: 234
  0, // SubVI: 235
  0, // SubVL: 236
  0, // SubVF: 237
  0, // SubVD: 238
  0, // MulVF: 239
  0, // MulVD: 240
  0, // DivVF: 241
  0, // DivVD: 242
  0, // LShiftVB: 243
  0, // LShiftVC: 244
  0, // LShiftVS: 245
  0, // LShiftVI: 246
  0, // URShiftVB: 247
  0, // URShiftVC: 248
  0, // URShiftVS: 249
  0, // URShiftVI: 250
  0, // AndV: 251
  0, // OrV: 252
  0, // XorV: 253
  0, // VectorLoad: 254
  0, // Load16B: 255
  0, // Load8B: 256
  0, // Load4B: 257
  0, // Load8C: 258
  0, // Load4C: 259
  0, // Load2C: 260
  0, // Load8S: 261
  0, // Load4S: 262
  0, // Load2S: 263
  0, // Load4I: 264
  0, // Load2I: 265
  0, // Load2L: 266
  0, // Load4F: 267
  0, // Load2F: 268
  0, // Load2D: 269
  0, // VectorStore: 270
  0, // Store16B: 271
  0, // Store8B: 272
  0, // Store4B: 273
  0, // Store8C: 274
  0, // Store4C: 275
  0, // Store2C: 276
  0, // Store4I: 277
  0, // Store2I: 278
  0, // Store2L: 279
  0, // Store4F: 280
  0, // Store2F: 281
  0, // Store2D: 282
  0, // Pack: 283
  0, // PackB: 284
  0, // PackS: 285
  0, // PackC: 286
  0, // PackI: 287
  0, // PackL: 288
  0, // PackF: 289
  0, // PackD: 290
  0, // Pack2x1B: 291
  0, // Pack2x2B: 292
  0, // Replicate16B: 293
  0, // Replicate8B: 294
  0, // Replicate4B: 295
  0, // Replicate8S: 296
  0, // Replicate4S: 297
  0, // Replicate2S: 298
  0, // Replicate8C: 299
  0, // Replicate4C: 300
  0, // Replicate2C: 301
  0, // Replicate4I: 302
  0, // Replicate2I: 303
  0, // Replicate2L: 304
  0, // Replicate4F: 305
  0, // Replicate2F: 306
  0, // Replicate2D: 307
  0, // Extract: 308
  0, // ExtractB: 309
  0, // ExtractS: 310
  0, // ExtractC: 311
  0, // ExtractI: 312
  0, // ExtractL: 313
  0, // ExtractF: 314
  0 // no trailing comma // ExtractD: 315
};
//  The following instructions can cisc-spill
//  popCountI can cisc-spill operand 1 to popCountI_mem
//  popCountL can cisc-spill operand 1 to popCountL_mem
//  cmovI_reg can cisc-spill operand 4 to cmovI_mem
//  cmovI_regU can cisc-spill operand 4 to cmovI_memU
//  cmovI_regUCF can cisc-spill operand 4 to cmovI_memUCF
//  cmovL_reg can cisc-spill operand 4 to cmovL_mem
//  cmovL_regU can cisc-spill operand 4 to cmovL_memU
//  cmovL_regUCF can cisc-spill operand 4 to cmovL_memUCF
//  addI_rReg can cisc-spill operand 2 to addI_rReg_mem
//  addL_rReg can cisc-spill operand 2 to addL_rReg_mem
//  subI_rReg can cisc-spill operand 2 to subI_rReg_mem
//  subL_rReg can cisc-spill operand 2 to subL_rReg_mem
//  mulI_rReg can cisc-spill operand 2 to mulI_mem
//  mulI_rReg_imm can cisc-spill operand 1 to mulI_mem_imm
//  mulL_rReg can cisc-spill operand 2 to mulL_mem
//  mulL_rReg_imm can cisc-spill operand 1 to mulL_mem_imm
//  i2b can cisc-spill operand 1 to loadI2B
//  i2s can cisc-spill operand 1 to loadI2S
//  andI_rReg can cisc-spill operand 2 to andI_rReg_mem
//  andI2L_rReg_imm255 can cisc-spill operand 1 to loadI2L_immI_255
//  andI2L_rReg_imm65535 can cisc-spill operand 1 to loadI2L_immI_65535
//  orI_rReg can cisc-spill operand 2 to orI_rReg_mem
//  xorI_rReg can cisc-spill operand 2 to xorI_rReg_mem
//  andL_rReg can cisc-spill operand 2 to andL_rReg_mem
//  orL_rReg can cisc-spill operand 2 to orL_rReg_mem
//  xorL_rReg can cisc-spill operand 2 to xorL_rReg_mem
//  cmpF_cc_reg can cisc-spill operand 2 to cmpF_cc_mem
//  cmpF_cc_reg_CF can cisc-spill operand 2 to cmpF_cc_memCF
//  cmpD_cc_reg can cisc-spill operand 2 to cmpD_cc_mem
//  cmpD_cc_reg_CF can cisc-spill operand 2 to cmpD_cc_memCF
//  cmpF_reg can cisc-spill operand 2 to cmpF_mem
//  cmpD_reg can cisc-spill operand 2 to cmpD_mem
//  addF_reg can cisc-spill operand 2 to addF_mem
//  addD_reg can cisc-spill operand 2 to addD_mem
//  subF_reg can cisc-spill operand 2 to subF_mem
//  subD_reg can cisc-spill operand 2 to subD_mem
//  mulF_reg can cisc-spill operand 2 to mulF_mem
//  mulD_reg can cisc-spill operand 2 to mulD_mem
//  divF_reg can cisc-spill operand 2 to divF_mem
//  divD_reg can cisc-spill operand 2 to divD_mem
//  sqrtF_reg can cisc-spill operand 1 to sqrtF_mem
//  sqrtD_reg can cisc-spill operand 1 to sqrtD_mem
//  convF2D_reg_reg can cisc-spill operand 1 to convF2D_reg_mem
//  convD2F_reg_reg can cisc-spill operand 1 to convD2F_reg_mem
//  convL2F_reg_reg can cisc-spill operand 1 to convL2F_reg_mem
//  convL2D_reg_reg can cisc-spill operand 1 to convL2D_reg_mem
//  convI2L_reg_reg can cisc-spill operand 1 to loadI2L
//  convI2L_reg_reg_zex can cisc-spill operand 1 to convI2L_reg_mem_zex
//  MoveF2I_reg_reg can cisc-spill operand 1 to MoveF2I_stack_reg
//  MoveD2L_reg_reg can cisc-spill operand 1 to MoveD2L_stack_reg
//  MoveI2F_reg_reg can cisc-spill operand 1 to MoveI2F_stack_reg
//  compI_rReg can cisc-spill operand 2 to compI_rReg_mem
//  compU_rReg can cisc-spill operand 2 to compU_rReg_mem
//  compP_rReg can cisc-spill operand 2 to compP_rReg_mem
//  compL_rReg can cisc-spill operand 2 to compL_rReg_mem



// An array of character pointers to machine register names.
const char *Matcher::regName[REG_COUNT] = {
  "R10",
  "R10_H",
  "R11",
  "R11_H",
  "R8",
  "R8_H",
  "R9",
  "R9_H",
  "R12",
  "R12_H",
  "RCX",
  "RCX_H",
  "RBX",
  "RBX_H",
  "RDI",
  "RDI_H",
  "RDX",
  "RDX_H",
  "RSI",
  "RSI_H",
  "RAX",
  "RAX_H",
  "RBP",
  "RBP_H",
  "R13",
  "R13_H",
  "R14",
  "R14_H",
  "R15",
  "R15_H",
  "RSP",
  "RSP_H",
  "XMM0",
  "XMM0_H",
  "XMM1",
  "XMM1_H",
  "XMM2",
  "XMM2_H",
  "XMM3",
  "XMM3_H",
  "XMM4",
  "XMM4_H",
  "XMM5",
  "XMM5_H",
  "XMM6",
  "XMM6_H",
  "XMM7",
  "XMM7_H",
  "XMM8",
  "XMM8_H",
  "XMM9",
  "XMM9_H",
  "XMM10",
  "XMM10_H",
  "XMM11",
  "XMM11_H",
  "XMM12",
  "XMM12_H",
  "XMM13",
  "XMM13_H",
  "XMM14",
  "XMM14_H",
  "XMM15",
  "XMM15_H",
  "RFLAGS" // no trailing comma
};

// An array of character pointers to machine register names.
const VMReg OptoReg::opto2vm[REG_COUNT] = {
	r10->as_VMReg(),
	r10->as_VMReg()->next(),
	r11->as_VMReg(),
	r11->as_VMReg()->next(),
	r8->as_VMReg(),
	r8->as_VMReg()->next(),
	r9->as_VMReg(),
	r9->as_VMReg()->next(),
	r12->as_VMReg(),
	r12->as_VMReg()->next(),
	rcx->as_VMReg(),
	rcx->as_VMReg()->next(),
	rbx->as_VMReg(),
	rbx->as_VMReg()->next(),
	rdi->as_VMReg(),
	rdi->as_VMReg()->next(),
	rdx->as_VMReg(),
	rdx->as_VMReg()->next(),
	rsi->as_VMReg(),
	rsi->as_VMReg()->next(),
	rax->as_VMReg(),
	rax->as_VMReg()->next(),
	rbp->as_VMReg(),
	rbp->as_VMReg()->next(),
	r13->as_VMReg(),
	r13->as_VMReg()->next(),
	r14->as_VMReg(),
	r14->as_VMReg()->next(),
	r15->as_VMReg(),
	r15->as_VMReg()->next(),
	rsp->as_VMReg(),
	rsp->as_VMReg()->next(),
	xmm0->as_VMReg(),
	xmm0->as_VMReg()->next(),
	xmm1->as_VMReg(),
	xmm1->as_VMReg()->next(),
	xmm2->as_VMReg(),
	xmm2->as_VMReg()->next(),
	xmm3->as_VMReg(),
	xmm3->as_VMReg()->next(),
	xmm4->as_VMReg(),
	xmm4->as_VMReg()->next(),
	xmm5->as_VMReg(),
	xmm5->as_VMReg()->next(),
	xmm6->as_VMReg(),
	xmm6->as_VMReg()->next(),
	xmm7->as_VMReg(),
	xmm7->as_VMReg()->next(),
	xmm8->as_VMReg(),
	xmm8->as_VMReg()->next(),
	xmm9->as_VMReg(),
	xmm9->as_VMReg()->next(),
	xmm10->as_VMReg(),
	xmm10->as_VMReg()->next(),
	xmm11->as_VMReg(),
	xmm11->as_VMReg()->next(),
	xmm12->as_VMReg(),
	xmm12->as_VMReg()->next(),
	xmm13->as_VMReg(),
	xmm13->as_VMReg()->next(),
	xmm14->as_VMReg(),
	xmm14->as_VMReg()->next(),
	xmm15->as_VMReg(),
	xmm15->as_VMReg()->next(),
	VMRegImpl::Bad() // no trailing comma
	};

 OptoReg::Name OptoReg::vm2opto[ConcreteRegisterImpl::number_of_registers];

// An array of the machine register encode values
const unsigned char Matcher::_regEncode[REG_COUNT] = {
  (unsigned char)'\xA',  // R10
  (unsigned char)'\xA',  // R10_H
  (unsigned char)'\xB',  // R11
  (unsigned char)'\xB',  // R11_H
  (unsigned char)'\x8',  // R8
  (unsigned char)'\x8',  // R8_H
  (unsigned char)'\x9',  // R9
  (unsigned char)'\x9',  // R9_H
  (unsigned char)'\xC',  // R12
  (unsigned char)'\xC',  // R12_H
  (unsigned char)'\x1',  // RCX
  (unsigned char)'\x1',  // RCX_H
  (unsigned char)'\x3',  // RBX
  (unsigned char)'\x3',  // RBX_H
  (unsigned char)'\x7',  // RDI
  (unsigned char)'\x7',  // RDI_H
  (unsigned char)'\x2',  // RDX
  (unsigned char)'\x2',  // RDX_H
  (unsigned char)'\x6',  // RSI
  (unsigned char)'\x6',  // RSI_H
  (unsigned char)'\x0',  // RAX
  (unsigned char)'\x0',  // RAX_H
  (unsigned char)'\x5',  // RBP
  (unsigned char)'\x5',  // RBP_H
  (unsigned char)'\xD',  // R13
  (unsigned char)'\xD',  // R13_H
  (unsigned char)'\xE',  // R14
  (unsigned char)'\xE',  // R14_H
  (unsigned char)'\xF',  // R15
  (unsigned char)'\xF',  // R15_H
  (unsigned char)'\x4',  // RSP
  (unsigned char)'\x4',  // RSP_H
  (unsigned char)'\x0',  // XMM0
  (unsigned char)'\x0',  // XMM0_H
  (unsigned char)'\x1',  // XMM1
  (unsigned char)'\x1',  // XMM1_H
  (unsigned char)'\x2',  // XMM2
  (unsigned char)'\x2',  // XMM2_H
  (unsigned char)'\x3',  // XMM3
  (unsigned char)'\x3',  // XMM3_H
  (unsigned char)'\x4',  // XMM4
  (unsigned char)'\x4',  // XMM4_H
  (unsigned char)'\x5',  // XMM5
  (unsigned char)'\x5',  // XMM5_H
  (unsigned char)'\x6',  // XMM6
  (unsigned char)'\x6',  // XMM6_H
  (unsigned char)'\x7',  // XMM7
  (unsigned char)'\x7',  // XMM7_H
  (unsigned char)'\x8',  // XMM8
  (unsigned char)'\x8',  // XMM8_H
  (unsigned char)'\x9',  // XMM9
  (unsigned char)'\x9',  // XMM9_H
  (unsigned char)'\xA',  // XMM10
  (unsigned char)'\xA',  // XMM10_H
  (unsigned char)'\xB',  // XMM11
  (unsigned char)'\xB',  // XMM11_H
  (unsigned char)'\xC',  // XMM12
  (unsigned char)'\xC',  // XMM12_H
  (unsigned char)'\xD',  // XMM13
  (unsigned char)'\xD',  // XMM13_H
  (unsigned char)'\xE',  // XMM14
  (unsigned char)'\xE',  // XMM14_H
  (unsigned char)'\xF',  // XMM15
  (unsigned char)'\xF',  // XMM15_H
  (unsigned char)'\x10' // no trailing comma  // RFLAGS
};


//------------------Define classes derived from MachOper---------------------
MachOper  *labelOper::clone(Compile* C) const {
  return  new (C) labelOper(_label, _block_num);
}
uint labelOper::opcode() const { return LABEL; }

const RegMask *sRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

MachOper  *methodOper::clone(Compile* C) const {
  return  new (C) methodOper(_method);
}
uint methodOper::opcode() const { return METHOD; }

const RegMask *rRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask;
}

const RegMask *rax_RegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_RAX_REG_mask;
}

const RegMask *rbx_RegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_RBX_REG_mask;
}

const RegMask *rcx_RegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_RCX_REG_mask;
}

const RegMask *rdx_RegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_RDX_REG_mask;
}

const RegMask *rdi_RegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_RDI_REG_mask;
}

const RegMask *no_rcx_RegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_NO_RCX_REG_mask;
}

const RegMask *no_rax_rdx_RegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_NO_RAX_RDX_REG_mask;
}

const RegMask *any_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG_mask;
}

const RegMask *rRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask;
}

const RegMask *rRegNOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask;
}

const RegMask *no_rax_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_NO_RAX_REG_mask;
}

const RegMask *no_rbp_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_NO_RBP_REG_mask;
}

const RegMask *no_rax_rbx_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_NO_RAX_RBX_REG_mask;
}

const RegMask *rax_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_RAX_REG_mask;
}

const RegMask *rax_RegNOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_RAX_REG_mask;
}

const RegMask *rbx_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_RBX_REG_mask;
}

const RegMask *rsi_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_RSI_REG_mask;
}

const RegMask *rdi_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_RDI_REG_mask;
}

const RegMask *rbp_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_RBP_REG_mask;
}

const RegMask *r15_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_R15_REG_mask;
}

const RegMask *rRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &LONG_REG_mask;
}

const RegMask *no_rax_rdx_RegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &LONG_NO_RAX_RDX_REG_mask;
}

const RegMask *no_rax_RegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &LONG_NO_RAX_RDX_REG_mask;
}

const RegMask *no_rcx_RegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &LONG_NO_RCX_REG_mask;
}

const RegMask *rax_RegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &LONG_RAX_REG_mask;
}

const RegMask *rcx_RegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &LONG_RCX_REG_mask;
}

const RegMask *rdx_RegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &LONG_RDX_REG_mask;
}

const RegMask *rFlagsRegOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask;
}

const RegMask *rFlagsRegUOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask;
}

const RegMask *rFlagsRegUCFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask;
}

const RegMask *regFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FLOAT_REG_mask;
}

const RegMask *regDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &DOUBLE_REG_mask;
}

const RegMask *indirectOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask;
}

const RegMask *indOffset8Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG_mask;
}

const RegMask *indOffset32Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG_mask;
}

const RegMask *indIndexOffsetOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &ANY_REG_mask;
  case 1: return &LONG_REG_mask;
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indIndexOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &ANY_REG_mask;
  case 1: return &LONG_REG_mask;
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indIndexScaleOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &ANY_REG_mask;
  case 1: return &LONG_REG_mask;
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indIndexScaleOffsetOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &ANY_REG_mask;
  case 1: return &LONG_REG_mask;
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indPosIndexScaleOffsetOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &ANY_REG_mask;
  case 1: return &INT_REG_mask;
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indCompressedOopOffsetOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask;
}

const RegMask *indirectNarrowOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask;
}

const RegMask *indOffset8NarrowOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask;
}

const RegMask *indOffset32NarrowOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask;
}

const RegMask *indIndexOffsetNarrowOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &INT_REG_mask;
  case 1: return &LONG_REG_mask;
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indIndexNarrowOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &INT_REG_mask;
  case 1: return &LONG_REG_mask;
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indIndexScaleNarrowOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &INT_REG_mask;
  case 1: return &LONG_REG_mask;
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indIndexScaleOffsetNarrowOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &INT_REG_mask;
  case 1: return &LONG_REG_mask;
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indPosIndexScaleOffsetNarrowOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  return &INT_REG_mask;
}

const RegMask *stackSlotPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

//------------------Define members for classes derived from MachNode----------

void popCountINode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *popCountINode::cisc_version( int offset, Compile* C ) {
  popCountI_memNode *node = new (C) popCountI_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void popCountLNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask;
}

// Build CISC version of this instruction
MachNode *popCountLNode::cisc_version( int offset, Compile* C ) {
  popCountL_memNode *node = new (C) popCountL_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmovI_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmovI_regNode::cisc_version( int offset, Compile* C ) {
  cmovI_memNode *node = new (C) cmovI_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmovI_regUNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmovI_regUNode::cisc_version( int offset, Compile* C ) {
  cmovI_memUNode *node = new (C) cmovI_memUNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmovI_regUCFNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmovI_regUCFNode::cisc_version( int offset, Compile* C ) {
  cmovI_memUCFNode *node = new (C) cmovI_memUCFNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmovL_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmovL_regNode::cisc_version( int offset, Compile* C ) {
  cmovL_memNode *node = new (C) cmovL_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmovL_regUNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmovL_regUNode::cisc_version( int offset, Compile* C ) {
  cmovL_memUNode *node = new (C) cmovL_memUNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmovL_regUCFNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmovL_regUCFNode::cisc_version( int offset, Compile* C ) {
  cmovL_memUCFNode *node = new (C) cmovL_memUCFNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void addI_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *addI_rRegNode::cisc_version( int offset, Compile* C ) {
  addI_rReg_memNode *node = new (C) addI_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void addL_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask;
}

// Build CISC version of this instruction
MachNode *addL_rRegNode::cisc_version( int offset, Compile* C ) {
  addL_rReg_memNode *node = new (C) addL_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void subI_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *subI_rRegNode::cisc_version( int offset, Compile* C ) {
  subI_rReg_memNode *node = new (C) subI_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void subL_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask;
}

// Build CISC version of this instruction
MachNode *subL_rRegNode::cisc_version( int offset, Compile* C ) {
  subL_rReg_memNode *node = new (C) subL_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void mulI_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *mulI_rRegNode::cisc_version( int offset, Compile* C ) {
  mulI_memNode *node = new (C) mulI_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void mulI_rReg_immNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *mulI_rReg_immNode::cisc_version( int offset, Compile* C ) {
  mulI_mem_immNode *node = new (C) mulI_mem_immNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void mulL_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask;
}

// Build CISC version of this instruction
MachNode *mulL_rRegNode::cisc_version( int offset, Compile* C ) {
  mulL_memNode *node = new (C) mulL_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void mulL_rReg_immNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask;
}

// Build CISC version of this instruction
MachNode *mulL_rReg_immNode::cisc_version( int offset, Compile* C ) {
  mulL_mem_immNode *node = new (C) mulL_mem_immNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void i2bNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *i2bNode::cisc_version( int offset, Compile* C ) {
  loadI2BNode *node = new (C) loadI2BNode();
  node->_num_opnds = 3;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void i2sNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *i2sNode::cisc_version( int offset, Compile* C ) {
  loadI2SNode *node = new (C) loadI2SNode();
  node->_num_opnds = 3;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void andI_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *andI_rRegNode::cisc_version( int offset, Compile* C ) {
  andI_rReg_memNode *node = new (C) andI_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void andI2L_rReg_imm255Node::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *andI2L_rReg_imm255Node::cisc_version( int offset, Compile* C ) {
  loadI2L_immI_255Node *node = new (C) loadI2L_immI_255Node();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void andI2L_rReg_imm65535Node::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *andI2L_rReg_imm65535Node::cisc_version( int offset, Compile* C ) {
  loadI2L_immI_65535Node *node = new (C) loadI2L_immI_65535Node();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void orI_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *orI_rRegNode::cisc_version( int offset, Compile* C ) {
  orI_rReg_memNode *node = new (C) orI_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void xorI_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *xorI_rRegNode::cisc_version( int offset, Compile* C ) {
  xorI_rReg_memNode *node = new (C) xorI_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void andL_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask;
}

// Build CISC version of this instruction
MachNode *andL_rRegNode::cisc_version( int offset, Compile* C ) {
  andL_rReg_memNode *node = new (C) andL_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void orL_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask;
}

// Build CISC version of this instruction
MachNode *orL_rRegNode::cisc_version( int offset, Compile* C ) {
  orL_rReg_memNode *node = new (C) orL_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void xorL_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask;
}

// Build CISC version of this instruction
MachNode *xorL_rRegNode::cisc_version( int offset, Compile* C ) {
  xorL_rReg_memNode *node = new (C) xorL_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmpF_cc_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmpF_cc_regNode::cisc_version( int offset, Compile* C ) {
  cmpF_cc_memNode *node = new (C) cmpF_cc_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmpF_cc_reg_CFNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmpF_cc_reg_CFNode::cisc_version( int offset, Compile* C ) {
  cmpF_cc_memCFNode *node = new (C) cmpF_cc_memCFNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmpD_cc_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmpD_cc_regNode::cisc_version( int offset, Compile* C ) {
  cmpD_cc_memNode *node = new (C) cmpD_cc_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmpD_cc_reg_CFNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmpD_cc_reg_CFNode::cisc_version( int offset, Compile* C ) {
  cmpD_cc_memCFNode *node = new (C) cmpD_cc_memCFNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmpF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmpF_regNode::cisc_version( int offset, Compile* C ) {
  cmpF_memNode *node = new (C) cmpF_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmpD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_mask;
}

// Build CISC version of this instruction
MachNode *cmpD_regNode::cisc_version( int offset, Compile* C ) {
  cmpD_memNode *node = new (C) cmpD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void addF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_mask;
}

// Build CISC version of this instruction
MachNode *addF_regNode::cisc_version( int offset, Compile* C ) {
  addF_memNode *node = new (C) addF_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void addD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_mask;
}

// Build CISC version of this instruction
MachNode *addD_regNode::cisc_version( int offset, Compile* C ) {
  addD_memNode *node = new (C) addD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void subF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_mask;
}

// Build CISC version of this instruction
MachNode *subF_regNode::cisc_version( int offset, Compile* C ) {
  subF_memNode *node = new (C) subF_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void subD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_mask;
}

// Build CISC version of this instruction
MachNode *subD_regNode::cisc_version( int offset, Compile* C ) {
  subD_memNode *node = new (C) subD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void mulF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_mask;
}

// Build CISC version of this instruction
MachNode *mulF_regNode::cisc_version( int offset, Compile* C ) {
  mulF_memNode *node = new (C) mulF_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void mulD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_mask;
}

// Build CISC version of this instruction
MachNode *mulD_regNode::cisc_version( int offset, Compile* C ) {
  mulD_memNode *node = new (C) mulD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void divF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_mask;
}

// Build CISC version of this instruction
MachNode *divF_regNode::cisc_version( int offset, Compile* C ) {
  divF_memNode *node = new (C) divF_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void divD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_mask;
}

// Build CISC version of this instruction
MachNode *divD_regNode::cisc_version( int offset, Compile* C ) {
  divD_memNode *node = new (C) divD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void sqrtF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_mask;
}

// Build CISC version of this instruction
MachNode *sqrtF_regNode::cisc_version( int offset, Compile* C ) {
  sqrtF_memNode *node = new (C) sqrtF_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void sqrtD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_mask;
}

// Build CISC version of this instruction
MachNode *sqrtD_regNode::cisc_version( int offset, Compile* C ) {
  sqrtD_memNode *node = new (C) sqrtD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void convF2D_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_mask;
}

// Build CISC version of this instruction
MachNode *convF2D_reg_regNode::cisc_version( int offset, Compile* C ) {
  convF2D_reg_memNode *node = new (C) convF2D_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void convD2F_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_mask;
}

// Build CISC version of this instruction
MachNode *convD2F_reg_regNode::cisc_version( int offset, Compile* C ) {
  convD2F_reg_memNode *node = new (C) convD2F_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void convL2F_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask;
}

// Build CISC version of this instruction
MachNode *convL2F_reg_regNode::cisc_version( int offset, Compile* C ) {
  convL2F_reg_memNode *node = new (C) convL2F_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void convL2D_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask;
}

// Build CISC version of this instruction
MachNode *convL2D_reg_regNode::cisc_version( int offset, Compile* C ) {
  convL2D_reg_memNode *node = new (C) convL2D_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void convI2L_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *convI2L_reg_regNode::cisc_version( int offset, Compile* C ) {
  loadI2LNode *node = new (C) loadI2LNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void convI2L_reg_reg_zexNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *convI2L_reg_reg_zexNode::cisc_version( int offset, Compile* C ) {
  convI2L_reg_mem_zexNode *node = new (C) convI2L_reg_mem_zexNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void MoveF2I_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_mask;
}

// Build CISC version of this instruction
MachNode *MoveF2I_reg_regNode::cisc_version( int offset, Compile* C ) {
  MoveF2I_stack_regNode *node = new (C) MoveF2I_stack_regNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void MoveD2L_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_mask;
}

// Build CISC version of this instruction
MachNode *MoveD2L_reg_regNode::cisc_version( int offset, Compile* C ) {
  MoveD2L_stack_regNode *node = new (C) MoveD2L_stack_regNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void MoveI2F_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *MoveI2F_reg_regNode::cisc_version( int offset, Compile* C ) {
  MoveI2F_stack_regNode *node = new (C) MoveI2F_stack_regNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void compI_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *compI_rRegNode::cisc_version( int offset, Compile* C ) {
  compI_rReg_memNode *node = new (C) compI_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void compU_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask;
}

// Build CISC version of this instruction
MachNode *compU_rRegNode::cisc_version( int offset, Compile* C ) {
  compU_rReg_memNode *node = new (C) compU_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void compP_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_PTR_REG_mask;
}

// Build CISC version of this instruction
MachNode *compP_rRegNode::cisc_version( int offset, Compile* C ) {
  compP_rReg_memNode *node = new (C) compP_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void compL_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask;
}

// Build CISC version of this instruction
MachNode *compL_rRegNode::cisc_version( int offset, Compile* C ) {
  compL_rReg_memNode *node = new (C) compL_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}

// Build short branch version of this instruction
MachNode *jmpDirNode::short_branch_version(Compile* C) {
  jmpDir_shortNode *node = new (C) jmpDir_shortNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpConNode::short_branch_version(Compile* C) {
  jmpCon_shortNode *node = new (C) jmpCon_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpLoopEndNode::short_branch_version(Compile* C) {
  jmpLoopEnd_shortNode *node = new (C) jmpLoopEnd_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpLoopEndUNode::short_branch_version(Compile* C) {
  jmpLoopEndU_shortNode *node = new (C) jmpLoopEndU_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpLoopEndUCFNode::short_branch_version(Compile* C) {
  jmpLoopEndUCF_shortNode *node = new (C) jmpLoopEndUCF_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpConUNode::short_branch_version(Compile* C) {
  jmpConU_shortNode *node = new (C) jmpConU_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpConUCFNode::short_branch_version(Compile* C) {
  jmpConUCF_shortNode *node = new (C) jmpConUCF_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpConUCF2Node::short_branch_version(Compile* C) {
  jmpConUCF2_shortNode *node = new (C) jmpConUCF2_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  return node;
}


// Copy _idx, inputs and operands to new node
void MachNode::fill_new_machnode( MachNode* node, Compile* C) const {
  // New node must use same node index
  node->set_idx( _idx );
  // Copy machine-independent inputs
  for( uint j = 0; j < req(); j++ ) {
    node->add_req(in(j));
  }
  // Copy my operands, except for cisc position
  int nopnds = num_opnds();
  assert( node->num_opnds() == (uint)nopnds, "Must have same number of operands");
  MachOper **to = node->_opnds;
  for( int i = 0; i < nopnds; i++ ) {
    if( i != cisc_operand() ) 
      to[i] = _opnds[i]->clone(C);
  }
}

void jmpDirNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(1));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpConNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpLoopEndNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpLoopEndUNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpLoopEndUCFNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpConUNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpConUCFNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpConUCF2Node::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpDir_shortNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(1));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpCon_shortNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpLoopEnd_shortNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpLoopEndU_shortNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpLoopEndUCF_shortNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpConU_shortNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpConUCF_shortNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void jmpConUCF2_shortNode::label_set( Label& label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = &label;
  oper->_block_num = block_num;
}
void CallStaticJavaDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallStaticJavaHandleNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallDynamicJavaDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallRuntimeDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallLeafDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallLeafNoFPDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

int  loadBNode::reloc()   const {
  return  1;
}

int  loadB2LNode::reloc()   const {
  return  1;
}

int  loadUBNode::reloc()   const {
  return  1;
}

int  loadUB2LNode::reloc()   const {
  return  1;
}

int  loadUB2L_immI8Node::reloc()   const {
  return  1;
}

int  loadSNode::reloc()   const {
  return  1;
}

int  loadS2BNode::reloc()   const {
  return  1;
}

int  loadS2LNode::reloc()   const {
  return  1;
}

int  loadUSNode::reloc()   const {
  return  1;
}

int  loadUS2BNode::reloc()   const {
  return  1;
}

int  loadUS2LNode::reloc()   const {
  return  1;
}

int  loadUS2L_immI_255Node::reloc()   const {
  return  1;
}

int  loadUS2L_immI16Node::reloc()   const {
  return  1;
}

int  loadINode::reloc()   const {
  return  1;
}

int  loadI2BNode::reloc()   const {
  return  1;
}

int  loadI2UBNode::reloc()   const {
  return  1;
}

int  loadI2SNode::reloc()   const {
  return  1;
}

int  loadI2USNode::reloc()   const {
  return  1;
}

int  loadI2LNode::reloc()   const {
  return  1;
}

int  loadI2L_immI_255Node::reloc()   const {
  return  1;
}

int  loadI2L_immI_65535Node::reloc()   const {
  return  1;
}

int  loadI2L_immINode::reloc()   const {
  return  1;
}

int  loadUI2LNode::reloc()   const {
  return  1;
}

int  loadLNode::reloc()   const {
  return  1;
}

int  loadRangeNode::reloc()   const {
  return  1;
}

int  loadPNode::reloc()   const {
  return  1;
}

int  loadNNode::reloc()   const {
  return  1;
}

int  loadKlassNode::reloc()   const {
  return  1;
}

int  loadNKlassNode::reloc()   const {
  return  1;
}

int  loadFNode::reloc()   const {
  return  1;
}

int  loadD_partialNode::reloc()   const {
  return  1;
}

int  loadDNode::reloc()   const {
  return  1;
}

int  loadA8BNode::reloc()   const {
  return  1;
}

int  loadA4SNode::reloc()   const {
  return  1;
}

int  loadA4CNode::reloc()   const {
  return  1;
}

int  load2IUNode::reloc()   const {
  return  1;
}

int  loadA2FNode::reloc()   const {
  return  1;
}

int  loadConPNode::reloc()   const {
  return  1;
}

int  loadConP0Node::reloc()   const {
  return  1;
}

int  loadConP31Node::reloc()   const {
  return  1;
}

int  loadConFNode::reloc()   const {
  return  2;
}

int  loadConF0Node::reloc()   const {
  return  2;
}

int  loadConDNode::reloc()   const {
  return  2;
}

int  loadConD0Node::reloc()   const {
  return  2;
}

int  prefetchrNode::reloc()   const {
  return  1;
}

int  prefetchrNTANode::reloc()   const {
  return  1;
}

int  prefetchrT0Node::reloc()   const {
  return  1;
}

int  prefetchrT2Node::reloc()   const {
  return  1;
}

int  prefetchwNode::reloc()   const {
  return  1;
}

int  prefetchwNTANode::reloc()   const {
  return  1;
}

int  prefetchwT0Node::reloc()   const {
  return  1;
}

int  prefetchwT2Node::reloc()   const {
  return  1;
}

int  storeBNode::reloc()   const {
  return  2;
}

int  storeCNode::reloc()   const {
  return  2;
}

int  storeINode::reloc()   const {
  return  2;
}

int  storeLNode::reloc()   const {
  return  2;
}

int  storePNode::reloc()   const {
  return  2;
}

int  storeImmP0Node::reloc()   const {
  return  3;
}

int  storeImmPNode::reloc()   const {
  return  3;
}

int  storeNNode::reloc()   const {
  return  2;
}

int  storeImmN0Node::reloc()   const {
  return  2;
}

int  storeImmNNode::reloc()   const {
  return  2;
}

int  storeImmI0Node::reloc()   const {
  return  2;
}

int  storeImmINode::reloc()   const {
  return  2;
}

int  storeImmL0Node::reloc()   const {
  return  2;
}

int  storeImmLNode::reloc()   const {
  return  2;
}

int  storeImmC0Node::reloc()   const {
  return  2;
}

int  storeImmI16Node::reloc()   const {
  return  2;
}

int  storeImmB0Node::reloc()   const {
  return  2;
}

int  storeImmBNode::reloc()   const {
  return  2;
}

int  storeA8BNode::reloc()   const {
  return  2;
}

int  storeA4CNode::reloc()   const {
  return  2;
}

int  storeA2INode::reloc()   const {
  return  2;
}

int  storeImmCM0_regNode::reloc()   const {
  return  2;
}

int  storeImmCM0Node::reloc()   const {
  return  2;
}

int  storeA2FNode::reloc()   const {
  return  2;
}

int  storeFNode::reloc()   const {
  return  2;
}

int  storeF0Node::reloc()   const {
  return  3;
}

int  storeF_immNode::reloc()   const {
  return  3;
}

int  storeDNode::reloc()   const {
  return  2;
}

int  storeD0_immNode::reloc()   const {
  return  3;
}

int  storeD0Node::reloc()   const {
  return  3;
}

int  popCountI_memNode::reloc()   const {
  return  1;
}

int  popCountL_memNode::reloc()   const {
  return  1;
}

int  cmovI_memNode::reloc()   const {
  return  1;
}

int  cmovI_memUNode::reloc()   const {
  return  1;
}

int  cmovI_memUCFNode::reloc()   const {
  return  1;
}

int  cmovL_memNode::reloc()   const {
  return  1;
}

int  cmovL_memUNode::reloc()   const {
  return  1;
}

int  cmovL_memUCFNode::reloc()   const {
  return  1;
}

int  addI_rReg_memNode::reloc()   const {
  return  1;
}

int  addI_rReg_mem_0Node::reloc()   const {
  return  1;
}

int  addI_mem_rRegNode::reloc()   const {
  return  3;
}

int  addI_mem_rReg_0Node::reloc()   const {
  return  3;
}

int  addI_mem_immNode::reloc()   const {
  return  3;
}

int  incI_memNode::reloc()   const {
  return  3;
}

int  decI_memNode::reloc()   const {
  return  3;
}

int  addL_rReg_memNode::reloc()   const {
  return  1;
}

int  addL_rReg_mem_0Node::reloc()   const {
  return  1;
}

int  addL_mem_rRegNode::reloc()   const {
  return  3;
}

int  addL_mem_rReg_0Node::reloc()   const {
  return  3;
}

int  addL_mem_immNode::reloc()   const {
  return  3;
}

int  incL_memNode::reloc()   const {
  return  3;
}

int  decL_memNode::reloc()   const {
  return  3;
}

int  loadPLockedNode::reloc()   const {
  return  1;
}

int  loadLLockedNode::reloc()   const {
  return  1;
}

int  storePConditionalNode::reloc()   const {
  return  1;
}

int  storeIConditionalNode::reloc()   const {
  return  1;
}

int  storeLConditionalNode::reloc()   const {
  return  1;
}

int  compareAndSwapPNode::reloc()   const {
  return  1;
}

int  compareAndSwapLNode::reloc()   const {
  return  1;
}

int  compareAndSwapINode::reloc()   const {
  return  1;
}

int  compareAndSwapNNode::reloc()   const {
  return  1;
}

int  subI_rReg_memNode::reloc()   const {
  return  1;
}

int  subI_mem_rRegNode::reloc()   const {
  return  3;
}

int  subI_mem_immNode::reloc()   const {
  return  3;
}

int  subL_rReg_memNode::reloc()   const {
  return  1;
}

int  subL_mem_rRegNode::reloc()   const {
  return  3;
}

int  subL_mem_immNode::reloc()   const {
  return  3;
}

int  negI_memNode::reloc()   const {
  return  3;
}

int  negL_memNode::reloc()   const {
  return  3;
}

int  mulI_memNode::reloc()   const {
  return  1;
}

int  mulI_mem_0Node::reloc()   const {
  return  1;
}

int  mulI_mem_immNode::reloc()   const {
  return  1;
}

int  mulL_memNode::reloc()   const {
  return  1;
}

int  mulL_mem_0Node::reloc()   const {
  return  1;
}

int  mulL_mem_immNode::reloc()   const {
  return  1;
}

int  salI_mem_1Node::reloc()   const {
  return  3;
}

int  salI_mem_immNode::reloc()   const {
  return  3;
}

int  salI_mem_CLNode::reloc()   const {
  return  3;
}

int  sarI_mem_1Node::reloc()   const {
  return  3;
}

int  sarI_mem_immNode::reloc()   const {
  return  3;
}

int  sarI_mem_CLNode::reloc()   const {
  return  3;
}

int  shrI_mem_1Node::reloc()   const {
  return  3;
}

int  shrI_mem_immNode::reloc()   const {
  return  3;
}

int  shrI_mem_CLNode::reloc()   const {
  return  3;
}

int  salL_mem_1Node::reloc()   const {
  return  3;
}

int  salL_mem_immNode::reloc()   const {
  return  3;
}

int  salL_mem_CLNode::reloc()   const {
  return  3;
}

int  sarL_mem_1Node::reloc()   const {
  return  3;
}

int  sarL_mem_immNode::reloc()   const {
  return  3;
}

int  sarL_mem_CLNode::reloc()   const {
  return  3;
}

int  shrL_mem_1Node::reloc()   const {
  return  3;
}

int  shrL_mem_immNode::reloc()   const {
  return  3;
}

int  shrL_mem_CLNode::reloc()   const {
  return  3;
}

int  andI_rReg_memNode::reloc()   const {
  return  1;
}

int  andI_rReg_mem_0Node::reloc()   const {
  return  1;
}

int  andI_mem_rRegNode::reloc()   const {
  return  3;
}

int  andI_mem_rReg_0Node::reloc()   const {
  return  3;
}

int  andI_mem_immNode::reloc()   const {
  return  3;
}

int  orI_rReg_memNode::reloc()   const {
  return  1;
}

int  orI_rReg_mem_0Node::reloc()   const {
  return  1;
}

int  orI_mem_rRegNode::reloc()   const {
  return  3;
}

int  orI_mem_rReg_0Node::reloc()   const {
  return  3;
}

int  orI_mem_immNode::reloc()   const {
  return  3;
}

int  xorI_rReg_memNode::reloc()   const {
  return  1;
}

int  xorI_rReg_mem_0Node::reloc()   const {
  return  1;
}

int  xorI_mem_rRegNode::reloc()   const {
  return  3;
}

int  xorI_mem_rReg_0Node::reloc()   const {
  return  3;
}

int  xorI_mem_immNode::reloc()   const {
  return  3;
}

int  andL_rReg_memNode::reloc()   const {
  return  1;
}

int  andL_rReg_mem_0Node::reloc()   const {
  return  1;
}

int  andL_mem_rRegNode::reloc()   const {
  return  3;
}

int  andL_mem_rReg_0Node::reloc()   const {
  return  3;
}

int  andL_mem_immNode::reloc()   const {
  return  3;
}

int  orL_rReg_memNode::reloc()   const {
  return  1;
}

int  orL_rReg_mem_0Node::reloc()   const {
  return  1;
}

int  orL_mem_rRegNode::reloc()   const {
  return  3;
}

int  orL_mem_rReg_0Node::reloc()   const {
  return  3;
}

int  orL_mem_immNode::reloc()   const {
  return  3;
}

int  xorL_rReg_memNode::reloc()   const {
  return  1;
}

int  xorL_rReg_mem_0Node::reloc()   const {
  return  1;
}

int  xorL_mem_rRegNode::reloc()   const {
  return  3;
}

int  xorL_mem_rReg_0Node::reloc()   const {
  return  3;
}

int  xorL_mem_immNode::reloc()   const {
  return  3;
}

int  cmpF_cc_memNode::reloc()   const {
  return  1;
}

int  cmpF_cc_memCFNode::reloc()   const {
  return  1;
}

int  cmpF_cc_immNode::reloc()   const {
  return  1;
}

int  cmpF_cc_immCFNode::reloc()   const {
  return  1;
}

int  cmpD_cc_memNode::reloc()   const {
  return  1;
}

int  cmpD_cc_memCFNode::reloc()   const {
  return  1;
}

int  cmpD_cc_immNode::reloc()   const {
  return  1;
}

int  cmpD_cc_immCFNode::reloc()   const {
  return  1;
}

int  cmpF_memNode::reloc()   const {
  return  1;
}

int  cmpF_immNode::reloc()   const {
  return  1;
}

int  cmpD_memNode::reloc()   const {
  return  1;
}

int  cmpD_immNode::reloc()   const {
  return  1;
}

int  addF_memNode::reloc()   const {
  return  1;
}

int  addF_mem_0Node::reloc()   const {
  return  1;
}

int  addF_immNode::reloc()   const {
  return  1;
}

int  addD_memNode::reloc()   const {
  return  1;
}

int  addD_mem_0Node::reloc()   const {
  return  1;
}

int  addD_immNode::reloc()   const {
  return  1;
}

int  subF_memNode::reloc()   const {
  return  1;
}

int  subF_immNode::reloc()   const {
  return  1;
}

int  subD_memNode::reloc()   const {
  return  1;
}

int  subD_immNode::reloc()   const {
  return  1;
}

int  mulF_memNode::reloc()   const {
  return  1;
}

int  mulF_mem_0Node::reloc()   const {
  return  1;
}

int  mulF_immNode::reloc()   const {
  return  1;
}

int  mulD_memNode::reloc()   const {
  return  1;
}

int  mulD_mem_0Node::reloc()   const {
  return  1;
}

int  mulD_immNode::reloc()   const {
  return  1;
}

int  divF_memNode::reloc()   const {
  return  1;
}

int  divF_immNode::reloc()   const {
  return  1;
}

int  divD_memNode::reloc()   const {
  return  1;
}

int  divD_immNode::reloc()   const {
  return  1;
}

int  sqrtF_memNode::reloc()   const {
  return  1;
}

int  sqrtF_immNode::reloc()   const {
  return  1;
}

int  sqrtD_memNode::reloc()   const {
  return  1;
}

int  sqrtD_immNode::reloc()   const {
  return  1;
}

int  convF2D_reg_memNode::reloc()   const {
  return  1;
}

int  convD2F_reg_memNode::reloc()   const {
  return  1;
}

int  convI2F_reg_memNode::reloc()   const {
  return  1;
}

int  convI2D_reg_memNode::reloc()   const {
  return  1;
}

int  convL2F_reg_memNode::reloc()   const {
  return  1;
}

int  convL2D_reg_memNode::reloc()   const {
  return  1;
}

int  convI2L_reg_mem_zexNode::reloc()   const {
  return  1;
}

int  Repl2F_immF0Node::reloc()   const {
  return  1;
}

int  compI_rReg_memNode::reloc()   const {
  return  1;
}

int  testI_reg_memNode::reloc()   const {
  return  1;
}

int  testI_reg_mem_0Node::reloc()   const {
  return  1;
}

int  compU_rReg_memNode::reloc()   const {
  return  1;
}

int  compP_rReg_memNode::reloc()   const {
  return  1;
}

int  compP_mem_rRegNode::reloc()   const {
  return  1;
}

int  testP_regNode::reloc()   const {
  return  1;
}

int  testP_memNode::reloc()   const {
  return  2;
}

int  testP_mem_reg0Node::reloc()   const {
  return  2;
}

int  compN_rReg_memNode::reloc()   const {
  return  1;
}

int  compN_mem_immNode::reloc()   const {
  return  1;
}

int  testN_memNode::reloc()   const {
  return  1;
}

int  testN_mem_reg0Node::reloc()   const {
  return  1;
}

int  compL_rReg_memNode::reloc()   const {
  return  1;
}

int  testL_reg_memNode::reloc()   const {
  return  1;
}

int  testL_reg_mem_0Node::reloc()   const {
  return  1;
}

int  partialSubtypeCheck_vs_ZeroNode::reloc()   const {
  return  1;
}

int  safePoint_pollNode::reloc()   const {
  return  1;
}

int  CallStaticJavaDirectNode::reloc()   const {
  return  1;
}

int  CallStaticJavaHandleNode::reloc()   const {
  return  1;
}

int  CallDynamicJavaDirectNode::reloc()   const {
  return  1;
}

int  CallRuntimeDirectNode::reloc()   const {
  return  1;
}

int  CallLeafDirectNode::reloc()   const {
  return  1;
}

int  CallLeafNoFPDirectNode::reloc()   const {
  return  1;
}

int  RetNode::reloc()   const {
  return  1;
}

int  TailCalljmpIndNode::reloc()   const {
  return  1;
}

int  tailjmpIndNode::reloc()   const {
  return  1;
}

int  RethrowExceptionNode::reloc()   const {
  return  1;
}


void loadBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    MacroAssembler _masm(&cbuf);

#line 5854 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movsbl(opnd_array(0)->as_Register(ra_,this), Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 9115 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadB2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    MacroAssembler _masm(&cbuf);

#line 5869 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movsbq(opnd_array(0)->as_Register(ra_,this), Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 9130 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadUBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    MacroAssembler _masm(&cbuf);

#line 5884 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movzbl(opnd_array(0)->as_Register(ra_,this), Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 9145 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadUB2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    MacroAssembler _masm(&cbuf);

#line 5899 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movzbq(opnd_array(0)->as_Register(ra_,this), Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 9160 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadUB2L_immI8Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 5913 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this);
    __ movzbq(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
    __ andl(Rdst, opnd_array(2)->constant());
  
#line 9178 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    MacroAssembler _masm(&cbuf);

#line 5929 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movswl(opnd_array(0)->as_Register(ra_,this), Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 9193 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadS2BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 5942 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movsbl(opnd_array(0)->as_Register(ra_,this), Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 9209 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadS2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    MacroAssembler _masm(&cbuf);

#line 5956 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movswq(opnd_array(0)->as_Register(ra_,this), Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 9224 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadUSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    MacroAssembler _masm(&cbuf);

#line 5971 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movzwl(opnd_array(0)->as_Register(ra_,this), Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 9239 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadUS2BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 5984 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movsbl(opnd_array(0)->as_Register(ra_,this), Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 9255 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadUS2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    MacroAssembler _masm(&cbuf);

#line 5998 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movzwq(opnd_array(0)->as_Register(ra_,this), Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 9270 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadUS2L_immI_255Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 6010 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movzbq(opnd_array(0)->as_Register(ra_,this), Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 9286 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadUS2L_immI16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 6023 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this);
    __ movzwq(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
    __ andl(Rdst, opnd_array(2)->constant());
  
#line 9304 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    MacroAssembler _masm(&cbuf);

#line 6039 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movl(opnd_array(0)->as_Register(ra_,this), Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 9319 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadI2BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 6052 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movsbl(opnd_array(0)->as_Register(ra_,this), Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 9335 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadI2UBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 6064 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movzbl(opnd_array(0)->as_Register(ra_,this), Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 9351 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadI2SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 6076 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movswl(opnd_array(0)->as_Register(ra_,this), Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 9367 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadI2USNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 6088 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movzwl(opnd_array(0)->as_Register(ra_,this), Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 9383 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadI2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    MacroAssembler _masm(&cbuf);

#line 6102 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movslq(opnd_array(0)->as_Register(ra_,this), Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 9398 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadI2L_immI_255Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 6114 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movzbq(opnd_array(0)->as_Register(ra_,this), Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 9414 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadI2L_immI_65535Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 6125 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movzwq(opnd_array(0)->as_Register(ra_,this), Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 9430 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadI2L_immINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 6138 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this);
    __ movl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
    __ andl(Rdst, opnd_array(2)->constant());
  
#line 9448 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadUI2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    MacroAssembler _masm(&cbuf);

#line 6154 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movl(opnd_array(0)->as_Register(ra_,this), Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 9463 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    MacroAssembler _masm(&cbuf);

#line 6169 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movq(opnd_array(0)->as_Register(ra_,this), Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 9478 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadRangeNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 9518 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 9526 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 9542 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 9584 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 9592 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 9608 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    MacroAssembler _masm(&cbuf);

#line 6207 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

     __ movl(opnd_array(0)->as_Register(ra_,this), Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
   
#line 9623 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadKlassNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 9665 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 9673 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 9689 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadNKlassNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    MacroAssembler _masm(&cbuf);

#line 6233 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movl(opnd_array(0)->as_Register(ra_,this), Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 9704 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF3 /*primary()*/));
  
#line 9718 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 9752 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 9760 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x10 /*tertiary()*/));
  
#line 9768 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 9784 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadD_partialNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x66 /*primary()*/));
  
#line 9798 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 9832 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 9840 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x12 /*tertiary()*/));
  
#line 9848 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 9864 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF2 /*primary()*/));
  
#line 9878 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 9912 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 9920 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x10 /*tertiary()*/));
  
#line 9928 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 9944 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadA8BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 3339 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);
    __ movq(opnd_array(0)->as_XMMRegister(ra_,this), Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 9959 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadA4SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 3339 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);
    __ movq(opnd_array(0)->as_XMMRegister(ra_,this), Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 9974 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadA4CNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 3339 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);
    __ movq(opnd_array(0)->as_XMMRegister(ra_,this), Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 9989 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void load2IUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 3339 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);
    __ movq(opnd_array(0)->as_XMMRegister(ra_,this), Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 10004 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadA2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 3339 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);
    __ movq(opnd_array(0)->as_XMMRegister(ra_,this), Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 10019 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void leaP8Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 10061 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 10069 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 10085 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void leaP32Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 10127 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 10135 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 10151 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void leaPIdxOffNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 10193 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 10201 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 10217 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void leaPIdxScaleNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 10259 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 10267 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 10283 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void leaPIdxScaleOffNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 10325 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 10333 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 10349 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void leaPPosIdxScaleOffNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 10391 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 10399 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 10415 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void leaPCompressedOopOffsetNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 10457 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 10465 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 10481 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void leaP8NarrowNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 10523 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 10531 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 10547 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void leaP32NarrowNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 10589 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 10597 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 10613 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void leaPIdxOffNarrowNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 10655 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 10663 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 10679 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void leaPIdxScaleNarrowNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 10721 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 10729 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 10745 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void leaPIdxScaleOffNarrowNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 10787 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 10795 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 10811 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void leaPPosIdxScaleOffNarrowNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 10853 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 10861 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 10877 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadConINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2710 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(0)->reg(ra_,this);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    emit_opcode(cbuf, 0xB8 | dstenc);
    emit_d32(cbuf, opnd_array(1)->constant());
  
#line 10897 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadConI0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(0)->reg(ra_,this) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(0)->reg(ra_,this) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 10921 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 10929 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(0)->reg(ra_,this) & 7);
  
#line 10937 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadConLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2721 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(0)->reg(ra_,this);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    emit_opcode(cbuf, 0xB8 | dstenc);
    emit_d64(cbuf, opnd_array(1)->constantL());
  
#line 10959 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadConL0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(0)->reg(ra_,this) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(0)->reg(ra_,this) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 10983 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 10991 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(0)->reg(ra_,this) & 7);
  
#line 10999 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadConUL32Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2734 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // same as load_immI, but this time we care about zeroes in the high word
    int dstenc = opnd_array(0)->reg(ra_,this);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    emit_opcode(cbuf, 0xB8 | dstenc);
    emit_d32(cbuf, opnd_array(1)->constantL());
  
#line 11020 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadConL32Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2746 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(0)->reg(ra_,this);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    emit_opcode(cbuf, 0xC7);
    emit_rm(cbuf, 0x03, 0x00, dstenc);
    emit_d32(cbuf, opnd_array(1)->constantL());
  
#line 11043 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadConPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2772 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(0)->reg(ra_,this);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    emit_opcode(cbuf, 0xB8 | dstenc);
    // This next line should be generated from ADLC
    if (opnd_array(1)->constant_is_oop()) {
      emit_d64_reloc(cbuf, opnd_array(1)->constant(), relocInfo::oop_type, RELOC_IMM64);
    } else {
      emit_d64(cbuf, opnd_array(1)->constant());
    }
  
#line 11070 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadConP0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(0)->reg(ra_,this) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(0)->reg(ra_,this) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 11094 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 11102 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(0)->reg(ra_,this) & 7);
  
#line 11110 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadConP31Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2760 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // same as load_immI, but this time we care about zeroes in the high word
    int dstenc = opnd_array(0)->reg(ra_,this);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    emit_opcode(cbuf, 0xB8 | dstenc);
    
if ( opnd_array(1)->constant_is_oop() ) {
  emit_d32_reloc(cbuf, opnd_array(1)->constant(), relocInfo::oop_type, 0);
} else {
  emit_d32(cbuf, opnd_array(1)->constant());
};
  
#line 11136 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadConFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

#line 6582 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movflt(opnd_array(0)->as_XMMRegister(ra_,this), InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 11151 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadConFNode::eval_constant(Compile* C) {
  {

#line 6583 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"
    _constant = C->constant_table().add(opnd_array(1));
#line 11159 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadConN0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

#line 6592 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ xorq(opnd_array(0)->as_Register(ra_,this), opnd_array(0)->as_Register(ra_,this));
  
#line 11174 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadConNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

#line 6603 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    address con = (address)opnd_array(1)->constant();
    if (con == NULL) {
      ShouldNotReachHere();
    } else {
      __ set_narrow_oop(opnd_array(0)->as_Register(ra_,this), (jobject)opnd_array(1)->constant());
    }
  
#line 11194 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadConF0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(0)->reg(ra_,this) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(0)->reg(ra_,this) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 11218 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 11226 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x57 /*secondary()*/));
  
#line 11234 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(0)->reg(ra_,this) & 7);
  
#line 11242 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadConDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

#line 6630 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this), InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 11257 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadConDNode::eval_constant(Compile* C) {
  {

#line 6631 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"
    _constant = C->constant_table().add(opnd_array(1));
#line 11265 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadConD0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x66 /*primary()*/));
  
#line 11279 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(0)->reg(ra_,this) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(0)->reg(ra_,this) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 11297 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 11305 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x57 /*tertiary()*/));
  
#line 11313 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(0)->reg(ra_,this) & 7);
  
#line 11321 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadSSINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 11361 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 11369 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 11385 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadSSLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 11427 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 11435 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 11451 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadSSPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 11493 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 11501 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 11517 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadSSFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF3 /*primary()*/));
  
#line 11531 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 11565 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 11573 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x10 /*tertiary()*/));
  
#line 11581 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 11597 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadSSDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

#line 6698 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this), Address(rsp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 11612 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void prefetchrNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 2897 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 11636 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 11644 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0D /*secondary()*/));
  
#line 11652 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 11672 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void prefetchrNTANode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 2897 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 11696 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 11704 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x18 /*secondary()*/));
  
#line 11712 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 11732 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void prefetchrT0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 2897 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 11756 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 11764 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x18 /*secondary()*/));
  
#line 11772 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x01);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 11792 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void prefetchrT2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 2897 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 11816 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 11824 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x18 /*secondary()*/));
  
#line 11832 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x03);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 11852 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void prefetchwNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 2897 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 11876 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 11884 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0D /*secondary()*/));
  
#line 11892 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x01);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 11912 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void prefetchwNTANode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 2897 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 11936 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 11944 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x18 /*secondary()*/));
  
#line 11952 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 11972 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void prefetchwT0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 2897 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 11996 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 12004 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x18 /*secondary()*/));
  
#line 12012 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x01);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 12032 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void prefetchwT2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 2897 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 12056 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 12064 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x18 /*secondary()*/));
  
#line 12072 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x03);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 12092 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2954 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        } else if (opnd_array(2)->reg(ra_,this,idx2) >= 4) {
          emit_opcode(cbuf, Assembler::REX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 12135 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x88 /*primary()*/));
  
#line 12143 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 12159 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeCNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2170 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, 0x66);
  
#line 12174 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 12208 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x89 /*primary()*/));
  
#line 12216 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 12232 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 12273 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x89 /*primary()*/));
  
#line 12281 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 12297 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 12340 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x89 /*primary()*/));
  
#line 12348 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 12364 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storePNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 12407 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x89 /*primary()*/));
  
#line 12415 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 12431 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeImmP0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 6864 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movq(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), r12);
  
#line 12447 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeImmPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2912 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 12474 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xC7 /*primary()*/));
  
#line 12482 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 12502 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2852 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Output immediate
    
if ( opnd_array(2)->constant_is_oop() ) {
  emit_d32_reloc(cbuf, opnd_array(2)->constant(), relocInfo::oop_type, 0);
} else {
  emit_d32(cbuf, opnd_array(2)->constant());
};
  
#line 12516 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 6889 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movl(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), opnd_array(2)->as_Register(ra_,this,idx2));
  
#line 12532 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeImmN0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 6902 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movl(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), r12);
  
#line 12548 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeImmNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 6914 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    address con = (address)opnd_array(2)->constant();
    if (con == NULL) {
      __ movl(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), (int32_t)0);
    } else {
      __ set_narrow_oop(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), (jobject)opnd_array(2)->constant());
    }
  
#line 12569 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeImmI0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 6933 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movl(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), r12);
  
#line 12585 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeImmINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2897 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 12610 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xC7 /*primary()*/));
  
#line 12618 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 12638 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2852 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Output immediate
    emit_d32(cbuf, opnd_array(2)->constant());
  
#line 12647 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeImmL0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 6958 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movq(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), r12);
  
#line 12663 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeImmLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2912 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 12690 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xC7 /*primary()*/));
  
#line 12698 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 12718 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2852 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Output immediate
    emit_d32(cbuf, opnd_array(2)->constantL());
  
#line 12727 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeImmC0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 6983 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movw(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), r12);
  
#line 12743 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeImmI16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2170 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, 0x66);
  
#line 12758 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2897 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 12776 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xC7 /*primary()*/));
  
#line 12784 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 12804 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2872 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Output immediate
    emit_d16(cbuf, opnd_array(2)->constant());
  
#line 12813 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeImmB0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 7009 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movb(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), r12);
  
#line 12829 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeImmBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2897 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 12854 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xC6 /*primary()*/));
  
#line 12862 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 12882 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 12896 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeA8BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3344 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);
    __ movq(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), opnd_array(2)->as_XMMRegister(ra_,this,idx2));
  
#line 12912 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeA4CNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3344 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);
    __ movq(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), opnd_array(2)->as_XMMRegister(ra_,this,idx2));
  
#line 12928 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeA2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3344 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);
    __ movq(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), opnd_array(2)->as_XMMRegister(ra_,this,idx2));
  
#line 12944 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeImmCM0_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 7061 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movb(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), r12);
  
#line 12960 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeImmCM0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2897 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 12985 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xC6 /*primary()*/));
  
#line 12993 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 13013 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 13027 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeA2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3344 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);
    __ movq(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), opnd_array(2)->as_XMMRegister(ra_,this,idx2));
  
#line 13043 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF3 /*primary()*/));
  
#line 13058 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 13092 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 13100 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x11 /*tertiary()*/));
  
#line 13108 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 13124 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeF0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 7107 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movl(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), r12);
  
#line 13140 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeF_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2897 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 13165 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xC7 /*primary()*/));
  
#line 13173 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 13193 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2864 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Output Float immediate bits
    jfloat jf = opnd_array(2)->constantF();
    jint jf_as_bits = jint_cast(jf);
    emit_d32(cbuf, jf_as_bits);
  
#line 13204 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF2 /*primary()*/));
  
#line 13219 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 13253 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 13261 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x11 /*tertiary()*/));
  
#line 13269 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 13285 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeD0_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2912 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 13312 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xC7 /*primary()*/));
  
#line 13320 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 13340 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2864 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Output Float immediate bits
    jfloat jf = opnd_array(2)->constantD();
    jint jf_as_bits = jint_cast(jf);
    emit_d32(cbuf, jf_as_bits);
  
#line 13351 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeD0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 7156 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movq(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), r12);
  
#line 13367 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeSSINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 13407 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x89 /*primary()*/));
  
#line 13415 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int disp = opnd_array(0)->disp(ra_,this,0);
    bool disp_is_oop = opnd_array(0)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 13431 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeSSLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 13473 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x89 /*primary()*/));
  
#line 13481 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int disp = opnd_array(0)->disp(ra_,this,0);
    bool disp_is_oop = opnd_array(0)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 13497 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeSSPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 13539 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x89 /*primary()*/));
  
#line 13547 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int disp = opnd_array(0)->disp(ra_,this,0);
    bool disp_is_oop = opnd_array(0)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 13563 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeSSFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF3 /*primary()*/));
  
#line 13577 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 13611 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 13619 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x11 /*tertiary()*/));
  
#line 13627 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int disp = opnd_array(0)->disp(ra_,this,0);
    bool disp_is_oop = opnd_array(0)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 13643 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeSSDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF2 /*primary()*/));
  
#line 13657 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 13691 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 13699 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x11 /*tertiary()*/));
  
#line 13707 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int disp = opnd_array(0)->disp(ra_,this,0);
    bool disp_is_oop = opnd_array(0)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 13723 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void bytes_reverse_intNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2987 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 13739 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 13747 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2467 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // BSWAP
    emit_cc(cbuf, (0xC8 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1));
  
#line 13756 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void bytes_reverse_longNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2994 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 13774 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 13782 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2467 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // BSWAP
    emit_cc(cbuf, (0xC8 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1));
  
#line 13791 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void bytes_reverse_unsigned_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

#line 7242 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ bswapl(opnd_array(1)->as_Register(ra_,this,idx1));
    __ shrl(opnd_array(1)->as_Register(ra_,this,idx1), 16); 
  
#line 13807 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void bytes_reverse_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

#line 7254 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ bswapl(opnd_array(1)->as_Register(ra_,this,idx1));
    __ sarl(opnd_array(1)->as_Register(ra_,this,idx1), 16); 
  
#line 13823 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void countLeadingZerosINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

#line 7269 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ lzcntl(opnd_array(0)->as_Register(ra_,this), opnd_array(1)->as_Register(ra_,this,idx1));
  
#line 13838 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void countLeadingZerosI_bsrNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

#line 7286 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this);
    Register Rsrc = opnd_array(1)->as_Register(ra_,this,idx1);
    Label skip;
    __ bsrl(Rdst, Rsrc);
    __ jccb(Assembler::notZero, skip);
    __ movl(Rdst, -1);
    __ bind(skip);
    __ negl(Rdst);
    __ addl(Rdst, BitsPerInt - 1);
  
#line 13861 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void countLeadingZerosLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

#line 7306 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ lzcntq(opnd_array(0)->as_Register(ra_,this), opnd_array(1)->as_Register(ra_,this,idx1));
  
#line 13876 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void countLeadingZerosL_bsrNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

#line 7323 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this);
    Register Rsrc = opnd_array(1)->as_Register(ra_,this,idx1);
    Label skip;
    __ bsrq(Rdst, Rsrc);
    __ jccb(Assembler::notZero, skip);
    __ movl(Rdst, -1);
    __ bind(skip);
    __ negl(Rdst);
    __ addl(Rdst, BitsPerLong - 1);
  
#line 13899 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void countTrailingZerosINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

#line 7345 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this);
    Label done;
    __ bsfl(Rdst, opnd_array(1)->as_Register(ra_,this,idx1));
    __ jccb(Assembler::notZero, done);
    __ movl(Rdst, BitsPerInt);
    __ bind(done);
  
#line 13919 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void countTrailingZerosLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

#line 7364 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this);
    Label done;
    __ bsfq(Rdst, opnd_array(1)->as_Register(ra_,this,idx1));
    __ jccb(Assembler::notZero, done);
    __ movl(Rdst, BitsPerLong);
    __ bind(done);
  
#line 13939 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void popCountINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

#line 7383 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ popcntl(opnd_array(0)->as_Register(ra_,this), opnd_array(1)->as_Register(ra_,this,idx1));
  
#line 13954 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void popCountI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    MacroAssembler _masm(&cbuf);

#line 7394 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ popcntl(opnd_array(0)->as_Register(ra_,this), Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 13969 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void popCountLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

#line 7406 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ popcntq(opnd_array(0)->as_Register(ra_,this), opnd_array(1)->as_Register(ra_,this,idx1));
  
#line 13984 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void popCountL_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {
    MacroAssembler _masm(&cbuf);

#line 7418 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ popcntq(opnd_array(0)->as_Register(ra_,this), Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 13999 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void membar_acquireNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
uint  membar_acquireNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void membar_acquire_lockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
uint  membar_acquire_lockNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void membar_releaseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
uint  membar_releaseNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void membar_release_lockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
uint  membar_release_lockNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void membar_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

#line 7487 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ membar(Assembler::StoreLoad);
  
#line 14058 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void unnecessary_membar_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
uint  unnecessary_membar_volatileNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void castX2PNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2825 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int srcenc = opnd_array(1)->reg(ra_,this,idx1);
    int dstenc = opnd_array(0)->reg(ra_,this);

    if (dstenc != srcenc) {
      if (dstenc < 8) {
        if (srcenc < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WB);
          srcenc -= 8;
        }
      } else {
        if (srcenc < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRB);
          srcenc -= 8;
        }
        dstenc -= 8;
      }
      emit_opcode(cbuf, 0x8B);
      emit_rm(cbuf, 0x3, dstenc, srcenc);
    }
  
#line 14105 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void castP2XNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2825 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int srcenc = opnd_array(1)->reg(ra_,this,idx1);
    int dstenc = opnd_array(0)->reg(ra_,this);

    if (dstenc != srcenc) {
      if (dstenc < 8) {
        if (srcenc < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WB);
          srcenc -= 8;
        }
      } else {
        if (srcenc < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRB);
          srcenc -= 8;
        }
        dstenc -= 8;
      }
      emit_opcode(cbuf, 0x8B);
      emit_rm(cbuf, 0x3, dstenc, srcenc);
    }
  
#line 14141 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void encodeHeapOopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

#line 7532 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Register s = opnd_array(1)->as_Register(ra_,this,idx1);
    Register d = opnd_array(0)->as_Register(ra_,this);
    if (s != d) {
      __ movq(d, s);
    }
    __ encode_heap_oop(d);
  
#line 14161 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void encodeHeapOop_not_nullNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

#line 7548 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ encode_heap_oop_not_null(opnd_array(0)->as_Register(ra_,this), opnd_array(1)->as_Register(ra_,this,idx1));
  
#line 14176 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void decodeHeapOopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

#line 7560 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Register s = opnd_array(1)->as_Register(ra_,this,idx1);
    Register d = opnd_array(0)->as_Register(ra_,this);
    if (s != d) {
      __ movq(d, s);
    }
    __ decode_heap_oop(d);
  
#line 14196 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void decodeHeapOop_not_nullNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

#line 7577 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Register s = opnd_array(1)->as_Register(ra_,this,idx1);
    Register d = opnd_array(0)->as_Register(ra_,this);
    if (s != d) {
      __ decode_heap_oop_not_null(d, s);
    } else {
      __ decode_heap_oop_not_null(d);
    }
  
#line 14217 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void jumpXtnd_offsetNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  ra_->C->constant_table().fill_jump_table(cbuf, (MachConstantNode*) this, _index2label);
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 7601 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // We could use jump(ArrayAddress) except that the macro assembler needs to use r10
    // to do that and the compiler is using that register as one it can allocate.
    // So we build it all by hand.
    // Address index(noreg, switch_reg, (Address::ScaleFactor)$shift$$constant);
    // ArrayAddress dispatch(table, index);
    Address dispatch(opnd_array(3)->as_Register(ra_,this,idx3), opnd_array(1)->as_Register(ra_,this,idx1), (Address::ScaleFactor) opnd_array(2)->constant());
    __ lea(opnd_array(3)->as_Register(ra_,this,idx3), InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ jmp(dispatch);
  
#line 14242 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void jumpXtnd_offsetNode::eval_constant(Compile* C) {
  _constant = C->constant_table().allocate_jump_table(this);
  // User did not define an encode section.
}
void jumpXtnd_addrNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  ra_->C->constant_table().fill_jump_table(cbuf, (MachConstantNode*) this, _index2label);
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 7622 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // We could use jump(ArrayAddress) except that the macro assembler needs to use r10
    // to do that and the compiler is using that register as one it can allocate.
    // So we build it all by hand.
    // Address index(noreg, switch_reg, (Address::ScaleFactor) $shift$$constant, (int) $offset$$constant);
    // ArrayAddress dispatch(table, index);
    Address dispatch(opnd_array(4)->as_Register(ra_,this,idx4), opnd_array(1)->as_Register(ra_,this,idx1), (Address::ScaleFactor) opnd_array(2)->constant(), (int) opnd_array(3)->constantL());
    __ lea(opnd_array(4)->as_Register(ra_,this,idx4), InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ jmp(dispatch);
  
#line 14272 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void jumpXtnd_addrNode::eval_constant(Compile* C) {
  _constant = C->constant_table().allocate_jump_table(this);
  // User did not define an encode section.
}
void jumpXtndNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  ra_->C->constant_table().fill_jump_table(cbuf, (MachConstantNode*) this, _index2label);
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 7643 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // We could use jump(ArrayAddress) except that the macro assembler needs to use r10
    // to do that and the compiler is using that register as one it can allocate.
    // So we build it all by hand.
    // Address index(noreg, switch_reg, Address::times_1);
    // ArrayAddress dispatch(table, index);
    Address dispatch(opnd_array(2)->as_Register(ra_,this,idx2), opnd_array(1)->as_Register(ra_,this,idx1), Address::times_1);
    __ lea(opnd_array(2)->as_Register(ra_,this,idx2), InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ jmp(dispatch);
  
#line 14300 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void jumpXtndNode::eval_constant(Compile* C) {
  _constant = C->constant_table().allocate_jump_table(this);
  // User did not define an encode section.
}
void cmovI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(4)->reg(ra_,this,idx4) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(4)->reg(ra_,this,idx4) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 14331 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2504 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // CMOV
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 14341 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3) & 7, opnd_array(4)->reg(ra_,this,idx4) & 7);
  
#line 14349 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmovI_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(4)->reg(ra_,this,idx4) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(4)->reg(ra_,this,idx4) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 14376 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2504 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // CMOV
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 14386 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3) & 7, opnd_array(4)->reg(ra_,this,idx4) & 7);
  
#line 14394 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmovI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(4)->base(ra_,this,idx4) < 8) {
        if (opnd_array(4)->index(ra_,this,idx4) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(4)->base(ra_,this,idx4) < 8) {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 14437 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2504 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // CMOV
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 14447 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(3)->reg(ra_,this,idx3);
    int base = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int disp = opnd_array(4)->disp(ra_,this,idx4);
    bool disp_is_oop = opnd_array(4)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 14463 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmovI_memUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(4)->base(ra_,this,idx4) < 8) {
        if (opnd_array(4)->index(ra_,this,idx4) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(4)->base(ra_,this,idx4) < 8) {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 14506 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2504 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // CMOV
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 14516 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(3)->reg(ra_,this,idx3);
    int base = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int disp = opnd_array(4)->disp(ra_,this,idx4);
    bool disp_is_oop = opnd_array(4)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 14532 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmovN_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(4)->reg(ra_,this,idx4) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(4)->reg(ra_,this,idx4) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 14559 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2504 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // CMOV
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 14569 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3) & 7, opnd_array(4)->reg(ra_,this,idx4) & 7);
  
#line 14577 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmovN_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(4)->reg(ra_,this,idx4) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(4)->reg(ra_,this,idx4) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 14604 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2504 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // CMOV
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 14614 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3) & 7, opnd_array(4)->reg(ra_,this,idx4) & 7);
  
#line 14622 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmovP_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {

#line 3018 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(4)->reg(ra_,this,idx4) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(4)->reg(ra_,this,idx4) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 14651 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2504 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // CMOV
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 14661 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3) & 7, opnd_array(4)->reg(ra_,this,idx4) & 7);
  
#line 14669 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmovP_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {

#line 3018 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(4)->reg(ra_,this,idx4) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(4)->reg(ra_,this,idx4) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 14698 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2504 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // CMOV
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 14708 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3) & 7, opnd_array(4)->reg(ra_,this,idx4) & 7);
  
#line 14716 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmovL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {

#line 3018 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(4)->reg(ra_,this,idx4) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(4)->reg(ra_,this,idx4) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 14745 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2504 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // CMOV
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 14755 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3) & 7, opnd_array(4)->reg(ra_,this,idx4) & 7);
  
#line 14763 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmovL_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(4)->base(ra_,this,idx4) < 8) {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(4)->base(ra_,this,idx4) < 8) {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 14808 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2504 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // CMOV
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 14818 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(3)->reg(ra_,this,idx3);
    int base = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int disp = opnd_array(4)->disp(ra_,this,idx4);
    bool disp_is_oop = opnd_array(4)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 14834 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmovL_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {

#line 3018 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(4)->reg(ra_,this,idx4) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(4)->reg(ra_,this,idx4) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 14863 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2504 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // CMOV
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 14873 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3) & 7, opnd_array(4)->reg(ra_,this,idx4) & 7);
  
#line 14881 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmovL_memUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(4)->base(ra_,this,idx4) < 8) {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(4)->base(ra_,this,idx4) < 8) {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 14926 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2504 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // CMOV
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 14936 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(3)->reg(ra_,this,idx3);
    int base = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int disp = opnd_array(4)->disp(ra_,this,idx4);
    bool disp_is_oop = opnd_array(4)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 14952 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmovF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {

#line 2511 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Invert sense of branch from sense of cmov
    emit_cc(cbuf, 0x70, opnd_array(1)->ccode() ^ 1);
    emit_d8(cbuf, (opnd_array(3)->reg(ra_,this,idx3) < 8 && opnd_array(4)->reg(ra_,this,idx4) < 8)
                  ? (UseXmmRegToRegMoveAll ? 3 : 4)
                  : (UseXmmRegToRegMoveAll ? 4 : 5) ); // REX
    // UseXmmRegToRegMoveAll ? movaps(dst, src) : movss(dst, src)
    if (!UseXmmRegToRegMoveAll) emit_opcode(cbuf, 0xF3);
    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(4)->reg(ra_,this,idx4) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(4)->reg(ra_,this,idx4) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, UseXmmRegToRegMoveAll ? 0x28 : 0x10);
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3) & 7, opnd_array(4)->reg(ra_,this,idx4) & 7);
  
#line 14989 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmovF_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {

#line 2511 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Invert sense of branch from sense of cmov
    emit_cc(cbuf, 0x70, opnd_array(1)->ccode() ^ 1);
    emit_d8(cbuf, (opnd_array(3)->reg(ra_,this,idx3) < 8 && opnd_array(4)->reg(ra_,this,idx4) < 8)
                  ? (UseXmmRegToRegMoveAll ? 3 : 4)
                  : (UseXmmRegToRegMoveAll ? 4 : 5) ); // REX
    // UseXmmRegToRegMoveAll ? movaps(dst, src) : movss(dst, src)
    if (!UseXmmRegToRegMoveAll) emit_opcode(cbuf, 0xF3);
    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(4)->reg(ra_,this,idx4) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(4)->reg(ra_,this,idx4) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, UseXmmRegToRegMoveAll ? 0x28 : 0x10);
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3) & 7, opnd_array(4)->reg(ra_,this,idx4) & 7);
  
#line 15026 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmovD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {

#line 2536 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Invert sense of branch from sense of cmov
    emit_cc(cbuf, 0x70, opnd_array(1)->ccode() ^ 1);
    emit_d8(cbuf, opnd_array(3)->reg(ra_,this,idx3) < 8 && opnd_array(4)->reg(ra_,this,idx4) < 8 ? 4 : 5); // REX

    //  UseXmmRegToRegMoveAll ? movapd(dst, src) : movsd(dst, src)
    emit_opcode(cbuf, UseXmmRegToRegMoveAll ? 0x66 : 0xF2);
    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(4)->reg(ra_,this,idx4) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(4)->reg(ra_,this,idx4) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, UseXmmRegToRegMoveAll ? 0x28 : 0x10);
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3) & 7, opnd_array(4)->reg(ra_,this,idx4) & 7);
  
#line 15062 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmovD_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {

#line 2536 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Invert sense of branch from sense of cmov
    emit_cc(cbuf, 0x70, opnd_array(1)->ccode() ^ 1);
    emit_d8(cbuf, opnd_array(3)->reg(ra_,this,idx3) < 8 && opnd_array(4)->reg(ra_,this,idx4) < 8 ? 4 : 5); // REX

    //  UseXmmRegToRegMoveAll ? movapd(dst, src) : movsd(dst, src)
    emit_opcode(cbuf, UseXmmRegToRegMoveAll ? 0x66 : 0xF2);
    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(4)->reg(ra_,this,idx4) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(4)->reg(ra_,this,idx4) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, UseXmmRegToRegMoveAll ? 0x28 : 0x10);
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3) & 7, opnd_array(4)->reg(ra_,this,idx4) & 7);
  
#line 15098 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void addI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 15123 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x03 /*primary()*/));
  
#line 15131 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 15139 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void addI_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2398 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), dstenc);
  
#line 15169 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 15183 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void addI_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 15224 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x03 /*primary()*/));
  
#line 15232 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 15248 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void addI_rReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 15289 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x03 /*primary()*/));
  
#line 15297 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 15313 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void addI_mem_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 15354 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x01 /*primary()*/));
  
#line 15362 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 15378 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void addI_mem_rReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 15419 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x01 /*primary()*/));
  
#line 15427 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 15443 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void addI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2897 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 15468 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2386 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  
#line 15483 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 15503 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 15517 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void incI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2987 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 15534 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xFF /*primary()*/));
  
#line 15542 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 15551 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void incI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2897 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 15576 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xFF /*primary()*/));
  
#line 15584 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 15604 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void decI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2987 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 15621 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xFF /*primary()*/));
  
#line 15629 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x01 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 15638 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void decI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2897 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 15663 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xFF /*primary()*/));
  
#line 15671 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x01);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 15691 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void leaI_rReg_immINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x67));
  
#line 15706 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 15724 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 15732 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3129 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base         = opnd_array(1)->reg(ra_,this,idx1);      // 0xFFFFFFFF indicates no base
    int index        = 0x04;            // 0x04 indicates no index
    int scale        = 0x00;            // 0x00 indicates no scale
    int displace     = opnd_array(2)->constant(); // 0x00 indicates no displacement
    bool disp_is_oop = false;
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace,
                  disp_is_oop);
  
#line 15747 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void addL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3018 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 15774 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x03 /*primary()*/));
  
#line 15782 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 15790 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void addL_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2418 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), dstenc);
  
#line 15822 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  
#line 15836 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void addL_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 15879 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x03 /*primary()*/));
  
#line 15887 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 15903 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void addL_rReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 15946 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x03 /*primary()*/));
  
#line 15954 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 15970 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void addL_mem_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 16013 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x01 /*primary()*/));
  
#line 16021 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 16037 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void addL_mem_rReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 16080 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x01 /*primary()*/));
  
#line 16088 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 16104 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void addL_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2912 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 16131 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2386 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  
#line 16146 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 16166 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  
#line 16180 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void incL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2994 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 16199 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xFF /*primary()*/));
  
#line 16207 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 16216 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void incL_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2912 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 16243 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xFF /*primary()*/));
  
#line 16251 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 16271 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void decL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2994 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 16290 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xFF /*primary()*/));
  
#line 16298 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x01 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 16307 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void decL_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2912 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 16334 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xFF /*primary()*/));
  
#line 16342 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x01);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 16362 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void leaL_rReg_immLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3018 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 16389 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 16397 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3129 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base         = opnd_array(1)->reg(ra_,this,idx1);      // 0xFFFFFFFF indicates no base
    int index        = 0x04;            // 0x04 indicates no index
    int scale        = 0x00;            // 0x00 indicates no scale
    int displace     = opnd_array(2)->constantL(); // 0x00 indicates no displacement
    bool disp_is_oop = false;
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace,
                  disp_is_oop);
  
#line 16412 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void addP_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3018 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 16439 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x03 /*primary()*/));
  
#line 16447 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 16455 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void addP_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2418 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), dstenc);
  
#line 16487 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  
#line 16501 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void leaP_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3018 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 16528 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 16536 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3129 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this);
    int base         = opnd_array(1)->reg(ra_,this,idx1);      // 0xFFFFFFFF indicates no base
    int index        = 0x04;            // 0x04 indicates no index
    int scale        = 0x00;            // 0x00 indicates no scale
    int displace     = opnd_array(2)->constantL(); // 0x00 indicates no displacement
    bool disp_is_oop = false;
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace,
                  disp_is_oop);
  
#line 16551 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void checkCastPPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
uint  checkCastPPNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void castPPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
uint  castPPNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void castIINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
uint  castIINode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void loadPLockedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 16626 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 16634 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 16650 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadLLockedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 16692 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 16700 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 16716 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storePConditionalNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {

#line 2890 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (os::is_MP()) {
      emit_opcode(cbuf, 0xF0); // lock
    }
  
#line 16734 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 16770 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 16778 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB1 /*secondary()*/));
  
#line 16786 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(3)->reg(ra_,this,idx3);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 16802 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeIConditionalNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {

#line 2890 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (os::is_MP()) {
      emit_opcode(cbuf, 0xF0); // lock
    }
  
#line 16820 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 16854 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 16862 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB1 /*secondary()*/));
  
#line 16870 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(3)->reg(ra_,this,idx3);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 16886 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void storeLConditionalNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {

#line 2890 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (os::is_MP()) {
      emit_opcode(cbuf, 0xF0); // lock
    }
  
#line 16904 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 16940 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 16948 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB1 /*secondary()*/));
  
#line 16956 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(3)->reg(ra_,this,idx3);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 16972 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void compareAndSwapPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {

#line 2890 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (os::is_MP()) {
      emit_opcode(cbuf, 0xF0); // lock
    }
  
#line 16990 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 17026 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 17034 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB1 /*secondary()*/));
  
#line 17042 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(3)->reg(ra_,this,idx3);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 17058 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2930 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) >= 4) {
      emit_opcode(cbuf, opnd_array(0)->reg(ra_,this) < 8 ? Assembler::REX : Assembler::REX_B);
    }
  
#line 17068 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F));
  
#line 17076 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x94));
  
#line 17084 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2175 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, 0, opnd_array(0)->reg(ra_,this) & 7);
  
#line 17092 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2938 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(0)->reg(ra_,this) >= 4) {
        emit_opcode(cbuf, opnd_array(0)->reg(ra_,this) < 8 ? Assembler::REX : Assembler::REX_B);
      }
    } else {
      if (opnd_array(0)->reg(ra_,this) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 17110 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF));
  
#line 17118 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB6));
  
#line 17126 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(0)->reg(ra_,this) & 7);
  
#line 17134 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void compareAndSwapLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {

#line 2890 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (os::is_MP()) {
      emit_opcode(cbuf, 0xF0); // lock
    }
  
#line 17152 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 17188 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 17196 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB1 /*secondary()*/));
  
#line 17204 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(3)->reg(ra_,this,idx3);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 17220 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2930 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) >= 4) {
      emit_opcode(cbuf, opnd_array(0)->reg(ra_,this) < 8 ? Assembler::REX : Assembler::REX_B);
    }
  
#line 17230 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F));
  
#line 17238 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x94));
  
#line 17246 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2175 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, 0, opnd_array(0)->reg(ra_,this) & 7);
  
#line 17254 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2938 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(0)->reg(ra_,this) >= 4) {
        emit_opcode(cbuf, opnd_array(0)->reg(ra_,this) < 8 ? Assembler::REX : Assembler::REX_B);
      }
    } else {
      if (opnd_array(0)->reg(ra_,this) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 17272 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF));
  
#line 17280 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB6));
  
#line 17288 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(0)->reg(ra_,this) & 7);
  
#line 17296 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void compareAndSwapINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {

#line 2890 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (os::is_MP()) {
      emit_opcode(cbuf, 0xF0); // lock
    }
  
#line 17314 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 17348 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 17356 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB1 /*secondary()*/));
  
#line 17364 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(3)->reg(ra_,this,idx3);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 17380 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2930 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) >= 4) {
      emit_opcode(cbuf, opnd_array(0)->reg(ra_,this) < 8 ? Assembler::REX : Assembler::REX_B);
    }
  
#line 17390 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F));
  
#line 17398 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x94));
  
#line 17406 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2175 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, 0, opnd_array(0)->reg(ra_,this) & 7);
  
#line 17414 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2938 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(0)->reg(ra_,this) >= 4) {
        emit_opcode(cbuf, opnd_array(0)->reg(ra_,this) < 8 ? Assembler::REX : Assembler::REX_B);
      }
    } else {
      if (opnd_array(0)->reg(ra_,this) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 17432 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF));
  
#line 17440 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB6));
  
#line 17448 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(0)->reg(ra_,this) & 7);
  
#line 17456 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void compareAndSwapNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {

#line 2890 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (os::is_MP()) {
      emit_opcode(cbuf, 0xF0); // lock
    }
  
#line 17474 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 17508 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 17516 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB1 /*secondary()*/));
  
#line 17524 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(3)->reg(ra_,this,idx3);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 17540 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2930 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) >= 4) {
      emit_opcode(cbuf, opnd_array(0)->reg(ra_,this) < 8 ? Assembler::REX : Assembler::REX_B);
    }
  
#line 17550 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F));
  
#line 17558 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x94));
  
#line 17566 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2175 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, 0, opnd_array(0)->reg(ra_,this) & 7);
  
#line 17574 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2938 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(0)->reg(ra_,this) >= 4) {
        emit_opcode(cbuf, opnd_array(0)->reg(ra_,this) < 8 ? Assembler::REX : Assembler::REX_B);
      }
    } else {
      if (opnd_array(0)->reg(ra_,this) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 17592 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF));
  
#line 17600 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB6));
  
#line 17608 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(0)->reg(ra_,this) & 7);
  
#line 17616 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void subI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 17641 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x2B /*primary()*/));
  
#line 17649 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 17657 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void subI_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2398 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x05 /*secondary()*/), dstenc);
  
#line 17687 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 17701 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void subI_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 17742 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x2B /*primary()*/));
  
#line 17750 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 17766 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void subI_mem_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 17807 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x29 /*primary()*/));
  
#line 17815 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 17831 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void subI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2897 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 17856 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2386 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  
#line 17871 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x05);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 17891 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 17905 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void subL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3018 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 17932 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x2B /*primary()*/));
  
#line 17940 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 17948 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void subL_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2418 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x05 /*secondary()*/), dstenc);
  
#line 17980 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  
#line 17994 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void subL_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 18037 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x2B /*primary()*/));
  
#line 18045 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 18061 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void subL_mem_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 18104 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x29 /*primary()*/));
  
#line 18112 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 18128 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void subL_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2912 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 18155 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2386 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  
#line 18170 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x05);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 18190 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  
#line 18204 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void subP_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {

#line 3018 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(3)->reg(ra_,this,idx3) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 18232 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x2B /*primary()*/));
  
#line 18240 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(3)->reg(ra_,this,idx3) & 7);
  
#line 18248 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void negI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2987 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 18265 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 18273 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x03 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 18282 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void negI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2897 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 18307 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 18315 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x03 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 18335 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void negL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2994 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 18354 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 18362 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x03 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 18371 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void negL_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2912 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 18398 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 18406 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x03 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 18426 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void mulI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 18451 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 18459 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xAF /*secondary()*/));
  
#line 18467 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 18475 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void mulI_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 18500 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2386 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x69 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x69 /*primary()*/));
    }
  
#line 18515 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 18523 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 18537 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void mulI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 18578 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 18586 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xAF /*secondary()*/));
  
#line 18594 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 18610 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void mulI_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 18651 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 18659 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xAF /*secondary()*/));
  
#line 18667 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 18683 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void mulI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 18724 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2386 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x69 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x69 /*primary()*/));
    }
  
#line 18739 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 18755 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 18769 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void mulL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3018 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 18796 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 18804 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xAF /*secondary()*/));
  
#line 18812 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 18820 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void mulL_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3018 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 18847 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2386 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x69 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x69 /*primary()*/));
    }
  
#line 18862 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 18870 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  
#line 18884 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void mulL_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 18927 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 18935 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xAF /*secondary()*/));
  
#line 18943 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 18959 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void mulL_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 19002 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 19010 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xAF /*secondary()*/));
  
#line 19018 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 19034 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void mulL_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 19077 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2386 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x69 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x69 /*primary()*/));
    }
  
#line 19092 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 19108 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  
#line 19122 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void mulHiL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2994 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 19141 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 19149 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 19158 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void divI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2250 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Full implementation of Java idiv and irem; checks for
    // special case as described in JVM spec., p.243 & p.271.
    //
    //         normal case                           special case
    //
    // input : rax: dividend                         min_int
    //         reg: divisor                          -1
    //
    // output: rax: quotient  (= rax idiv reg)       min_int
    //         rdx: remainder (= rax irem reg)       0
    //
    //  Code sequnce:
    //
    //    0:   3d 00 00 00 80          cmp    $0x80000000,%eax
    //    5:   75 07/08                jne    e <normal>
    //    7:   33 d2                   xor    %edx,%edx
    //  [div >= 8 -> offset + 1]
    //  [REX_B]
    //    9:   83 f9 ff                cmp    $0xffffffffffffffff,$div
    //    c:   74 03/04                je     11 <done>
    // 000000000000000e <normal>:
    //    e:   99                      cltd
    //  [div >= 8 -> offset + 1]
    //  [REX_B]
    //    f:   f7 f9                   idiv   $div
    // 0000000000000011 <done>:

    // cmp    $0x80000000,%eax
    emit_opcode(cbuf, 0x3d);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x80);

    // jne    e <normal>
    emit_opcode(cbuf, 0x75);
    emit_d8(cbuf, opnd_array(2)->reg(ra_,this,idx2) < 8 ? 0x07 : 0x08);

    // xor    %edx,%edx
    emit_opcode(cbuf, 0x33);
    emit_d8(cbuf, 0xD2);

    // cmp    $0xffffffffffffffff,%ecx
    if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x83);
    emit_rm(cbuf, 0x3, 0x7, opnd_array(2)->reg(ra_,this,idx2) & 7);
    emit_d8(cbuf, 0xFF);

    // je     11 <done>
    emit_opcode(cbuf, 0x74);
    emit_d8(cbuf, opnd_array(2)->reg(ra_,this,idx2) < 8 ? 0x03 : 0x04);

    // <normal>
    // cltd
    emit_opcode(cbuf, 0x99);

    // idivl (note: must be emitted by the user of this rule)
    // <done>
  
#line 19232 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2987 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 19242 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 19250 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 19259 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void divL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2314 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Full implementation of Java ldiv and lrem; checks for
    // special case as described in JVM spec., p.243 & p.271.
    //
    //         normal case                           special case
    //
    // input : rax: dividend                         min_long
    //         reg: divisor                          -1
    //
    // output: rax: quotient  (= rax idiv reg)       min_long
    //         rdx: remainder (= rax irem reg)       0
    //
    //  Code sequnce:
    //
    //    0:   48 ba 00 00 00 00 00    mov    $0x8000000000000000,%rdx
    //    7:   00 00 80
    //    a:   48 39 d0                cmp    %rdx,%rax
    //    d:   75 08                   jne    17 <normal>
    //    f:   33 d2                   xor    %edx,%edx
    //   11:   48 83 f9 ff             cmp    $0xffffffffffffffff,$div
    //   15:   74 05                   je     1c <done>
    // 0000000000000017 <normal>:
    //   17:   48 99                   cqto
    //   19:   48 f7 f9                idiv   $div
    // 000000000000001c <done>:

    // mov    $0x8000000000000000,%rdx
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0xBA);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x80);

    // cmp    %rdx,%rax
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x39);
    emit_d8(cbuf, 0xD0);

    // jne    17 <normal>
    emit_opcode(cbuf, 0x75);
    emit_d8(cbuf, 0x08);

    // xor    %edx,%edx
    emit_opcode(cbuf, 0x33);
    emit_d8(cbuf, 0xD2);

    // cmp    $0xffffffffffffffff,$div
    emit_opcode(cbuf, opnd_array(2)->reg(ra_,this,idx2) < 8 ? Assembler::REX_W : Assembler::REX_WB);
    emit_opcode(cbuf, 0x83);
    emit_rm(cbuf, 0x3, 0x7, opnd_array(2)->reg(ra_,this,idx2) & 7);
    emit_d8(cbuf, 0xFF);

    // je     1e <done>
    emit_opcode(cbuf, 0x74);
    emit_d8(cbuf, 0x05);

    // <normal>
    // cqto
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x99);

    // idivq (note: must be emitted by the user of this rule)
    // <done>
  
#line 19340 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2994 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 19352 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 19360 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 19369 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void divModI_rReg_divmodNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2250 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Full implementation of Java idiv and irem; checks for
    // special case as described in JVM spec., p.243 & p.271.
    //
    //         normal case                           special case
    //
    // input : rax: dividend                         min_int
    //         reg: divisor                          -1
    //
    // output: rax: quotient  (= rax idiv reg)       min_int
    //         rdx: remainder (= rax irem reg)       0
    //
    //  Code sequnce:
    //
    //    0:   3d 00 00 00 80          cmp    $0x80000000,%eax
    //    5:   75 07/08                jne    e <normal>
    //    7:   33 d2                   xor    %edx,%edx
    //  [div >= 8 -> offset + 1]
    //  [REX_B]
    //    9:   83 f9 ff                cmp    $0xffffffffffffffff,$div
    //    c:   74 03/04                je     11 <done>
    // 000000000000000e <normal>:
    //    e:   99                      cltd
    //  [div >= 8 -> offset + 1]
    //  [REX_B]
    //    f:   f7 f9                   idiv   $div
    // 0000000000000011 <done>:

    // cmp    $0x80000000,%eax
    emit_opcode(cbuf, 0x3d);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x80);

    // jne    e <normal>
    emit_opcode(cbuf, 0x75);
    emit_d8(cbuf, opnd_array(2)->reg(ra_,this,idx2) < 8 ? 0x07 : 0x08);

    // xor    %edx,%edx
    emit_opcode(cbuf, 0x33);
    emit_d8(cbuf, 0xD2);

    // cmp    $0xffffffffffffffff,%ecx
    if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x83);
    emit_rm(cbuf, 0x3, 0x7, opnd_array(2)->reg(ra_,this,idx2) & 7);
    emit_d8(cbuf, 0xFF);

    // je     11 <done>
    emit_opcode(cbuf, 0x74);
    emit_d8(cbuf, opnd_array(2)->reg(ra_,this,idx2) < 8 ? 0x03 : 0x04);

    // <normal>
    // cltd
    emit_opcode(cbuf, 0x99);

    // idivl (note: must be emitted by the user of this rule)
    // <done>
  
#line 19443 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2987 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 19453 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 19461 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 19470 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void divModL_rReg_divmodNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2314 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Full implementation of Java ldiv and lrem; checks for
    // special case as described in JVM spec., p.243 & p.271.
    //
    //         normal case                           special case
    //
    // input : rax: dividend                         min_long
    //         reg: divisor                          -1
    //
    // output: rax: quotient  (= rax idiv reg)       min_long
    //         rdx: remainder (= rax irem reg)       0
    //
    //  Code sequnce:
    //
    //    0:   48 ba 00 00 00 00 00    mov    $0x8000000000000000,%rdx
    //    7:   00 00 80
    //    a:   48 39 d0                cmp    %rdx,%rax
    //    d:   75 08                   jne    17 <normal>
    //    f:   33 d2                   xor    %edx,%edx
    //   11:   48 83 f9 ff             cmp    $0xffffffffffffffff,$div
    //   15:   74 05                   je     1c <done>
    // 0000000000000017 <normal>:
    //   17:   48 99                   cqto
    //   19:   48 f7 f9                idiv   $div
    // 000000000000001c <done>:

    // mov    $0x8000000000000000,%rdx
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0xBA);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x80);

    // cmp    %rdx,%rax
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x39);
    emit_d8(cbuf, 0xD0);

    // jne    17 <normal>
    emit_opcode(cbuf, 0x75);
    emit_d8(cbuf, 0x08);

    // xor    %edx,%edx
    emit_opcode(cbuf, 0x33);
    emit_d8(cbuf, 0xD2);

    // cmp    $0xffffffffffffffff,$div
    emit_opcode(cbuf, opnd_array(2)->reg(ra_,this,idx2) < 8 ? Assembler::REX_W : Assembler::REX_WB);
    emit_opcode(cbuf, 0x83);
    emit_rm(cbuf, 0x3, 0x7, opnd_array(2)->reg(ra_,this,idx2) & 7);
    emit_d8(cbuf, 0xFF);

    // je     1e <done>
    emit_opcode(cbuf, 0x74);
    emit_d8(cbuf, 0x05);

    // <normal>
    // cqto
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x99);

    // idivq (note: must be emitted by the user of this rule)
    // <done>
  
#line 19551 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2994 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 19563 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 19571 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 19580 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadConL_0x6666666666666667Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2721 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(0)->reg(ra_,this);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    emit_opcode(cbuf, 0xB8 | dstenc);
    emit_d64(cbuf, (0x6666666666666667));
  
#line 19602 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void mul_hiNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2994 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 19621 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 19629 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 19638 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void sarL_rReg_63Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2695 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), dstenc);
    emit_d8(cbuf, (0x3F));
  
#line 19662 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void sarL_rReg_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2695 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), dstenc);
    emit_d8(cbuf, (0x2));
  
#line 19686 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void modI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2250 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Full implementation of Java idiv and irem; checks for
    // special case as described in JVM spec., p.243 & p.271.
    //
    //         normal case                           special case
    //
    // input : rax: dividend                         min_int
    //         reg: divisor                          -1
    //
    // output: rax: quotient  (= rax idiv reg)       min_int
    //         rdx: remainder (= rax irem reg)       0
    //
    //  Code sequnce:
    //
    //    0:   3d 00 00 00 80          cmp    $0x80000000,%eax
    //    5:   75 07/08                jne    e <normal>
    //    7:   33 d2                   xor    %edx,%edx
    //  [div >= 8 -> offset + 1]
    //  [REX_B]
    //    9:   83 f9 ff                cmp    $0xffffffffffffffff,$div
    //    c:   74 03/04                je     11 <done>
    // 000000000000000e <normal>:
    //    e:   99                      cltd
    //  [div >= 8 -> offset + 1]
    //  [REX_B]
    //    f:   f7 f9                   idiv   $div
    // 0000000000000011 <done>:

    // cmp    $0x80000000,%eax
    emit_opcode(cbuf, 0x3d);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x80);

    // jne    e <normal>
    emit_opcode(cbuf, 0x75);
    emit_d8(cbuf, opnd_array(2)->reg(ra_,this,idx2) < 8 ? 0x07 : 0x08);

    // xor    %edx,%edx
    emit_opcode(cbuf, 0x33);
    emit_d8(cbuf, 0xD2);

    // cmp    $0xffffffffffffffff,%ecx
    if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x83);
    emit_rm(cbuf, 0x3, 0x7, opnd_array(2)->reg(ra_,this,idx2) & 7);
    emit_d8(cbuf, 0xFF);

    // je     11 <done>
    emit_opcode(cbuf, 0x74);
    emit_d8(cbuf, opnd_array(2)->reg(ra_,this,idx2) < 8 ? 0x03 : 0x04);

    // <normal>
    // cltd
    emit_opcode(cbuf, 0x99);

    // idivl (note: must be emitted by the user of this rule)
    // <done>
  
#line 19760 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2987 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 19770 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 19778 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 19787 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void modL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2314 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Full implementation of Java ldiv and lrem; checks for
    // special case as described in JVM spec., p.243 & p.271.
    //
    //         normal case                           special case
    //
    // input : rax: dividend                         min_long
    //         reg: divisor                          -1
    //
    // output: rax: quotient  (= rax idiv reg)       min_long
    //         rdx: remainder (= rax irem reg)       0
    //
    //  Code sequnce:
    //
    //    0:   48 ba 00 00 00 00 00    mov    $0x8000000000000000,%rdx
    //    7:   00 00 80
    //    a:   48 39 d0                cmp    %rdx,%rax
    //    d:   75 08                   jne    17 <normal>
    //    f:   33 d2                   xor    %edx,%edx
    //   11:   48 83 f9 ff             cmp    $0xffffffffffffffff,$div
    //   15:   74 05                   je     1c <done>
    // 0000000000000017 <normal>:
    //   17:   48 99                   cqto
    //   19:   48 f7 f9                idiv   $div
    // 000000000000001c <done>:

    // mov    $0x8000000000000000,%rdx
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0xBA);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x80);

    // cmp    %rdx,%rax
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x39);
    emit_d8(cbuf, 0xD0);

    // jne    17 <normal>
    emit_opcode(cbuf, 0x75);
    emit_d8(cbuf, 0x08);

    // xor    %edx,%edx
    emit_opcode(cbuf, 0x33);
    emit_d8(cbuf, 0xD2);

    // cmp    $0xffffffffffffffff,$div
    emit_opcode(cbuf, opnd_array(2)->reg(ra_,this,idx2) < 8 ? Assembler::REX_W : Assembler::REX_WB);
    emit_opcode(cbuf, 0x83);
    emit_rm(cbuf, 0x3, 0x7, opnd_array(2)->reg(ra_,this,idx2) & 7);
    emit_d8(cbuf, 0xFF);

    // je     1e <done>
    emit_opcode(cbuf, 0x74);
    emit_d8(cbuf, 0x05);

    // <normal>
    // cqto
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x99);

    // idivq (note: must be emitted by the user of this rule)
    // <done>
  
#line 19868 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2994 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 19880 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 19888 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 19897 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void salI_rReg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2987 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 19914 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 19922 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 19931 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void salI_mem_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2897 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 19956 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 19964 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x4 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 19984 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void salI_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2682 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), dstenc);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 20007 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void salI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2897 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 20032 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xC1 /*primary()*/));
  
#line 20040 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x4 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 20060 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 20074 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void salI_rReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2987 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 20091 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 20099 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 20108 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void salI_mem_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2897 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 20133 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 20141 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x4 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 20161 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void sarI_rReg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2987 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 20178 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 20186 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 20195 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void sarI_mem_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2897 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 20220 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 20228 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x7 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 20248 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void sarI_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2682 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), dstenc);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 20271 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void sarI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2897 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 20296 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xC1 /*primary()*/));
  
#line 20304 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x7 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 20324 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 20338 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void sarI_rReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2987 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 20355 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 20363 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 20372 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void sarI_mem_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2897 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 20397 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 20405 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x7 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 20425 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void shrI_rReg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2987 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 20442 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 20450 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 20459 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void shrI_mem_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2897 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 20484 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 20492 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x5 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 20512 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void shrI_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2682 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), dstenc);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 20535 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void shrI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2897 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 20560 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xC1 /*primary()*/));
  
#line 20568 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x5 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 20588 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 20602 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void shrI_rReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2987 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 20619 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 20627 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 20636 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void shrI_mem_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2897 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 20661 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 20669 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x5 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 20689 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void salL_rReg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2994 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 20708 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 20716 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 20725 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void salL_mem_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2912 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 20752 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 20760 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x4 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 20780 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void salL_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2695 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), dstenc);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 20805 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void salL_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2912 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 20832 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xC1 /*primary()*/));
  
#line 20840 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x4 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 20860 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 20874 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void salL_rReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2994 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 20893 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 20901 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 20910 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void salL_mem_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2912 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 20937 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 20945 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x4 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 20965 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void sarL_rReg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2994 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 20984 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 20992 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 21001 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void sarL_mem_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2912 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 21028 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 21036 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x7 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 21056 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void sarL_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2695 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), dstenc);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 21081 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void sarL_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2912 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 21108 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xC1 /*primary()*/));
  
#line 21116 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x7 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 21136 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 21150 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void sarL_rReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2994 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 21169 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 21177 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 21186 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void sarL_mem_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2912 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 21213 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 21221 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x7 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 21241 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void shrL_rReg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2994 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 21260 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 21268 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 21277 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void shrL_mem_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2912 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 21304 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 21312 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x5 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 21332 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void shrL_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2695 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), dstenc);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 21357 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void shrL_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2912 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 21384 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xC1 /*primary()*/));
  
#line 21392 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x5 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 21412 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 21426 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void shrL_rReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2994 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 21445 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 21453 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 21462 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void shrL_mem_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2912 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 21489 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 21497 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x5 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 21517 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void i2bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2938 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 4) {
        emit_opcode(cbuf, opnd_array(1)->reg(ra_,this,idx1) < 8 ? Assembler::REX : Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 21542 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 21550 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xBE /*secondary()*/));
  
#line 21558 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 21566 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void i2sNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 21591 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 21599 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xBF /*secondary()*/));
  
#line 21607 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 21615 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void rolI_rReg_imm1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2987 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 21631 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 21639 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 21648 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void rolI_rReg_imm8Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2682 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), dstenc);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 21671 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void rolI_rReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2987 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 21688 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 21696 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 21705 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void rorI_rReg_imm1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2987 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 21721 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 21729 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 21738 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void rorI_rReg_imm8Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2682 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), dstenc);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 21761 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void rorI_rReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2987 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 21778 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 21786 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 21795 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void rolL_rReg_imm1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2994 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 21813 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 21821 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 21830 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void rolL_rReg_imm8Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2695 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), dstenc);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 21855 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void rolL_rReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2994 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 21874 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 21882 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 21891 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void rorL_rReg_imm1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2994 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 21909 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 21917 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 21926 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void rorL_rReg_imm8Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2695 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), dstenc);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 21951 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void rorL_rReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2994 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 21970 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 21978 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 21987 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void andI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 22012 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x23 /*primary()*/));
  
#line 22020 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 22028 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void andI_rReg_imm255Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2938 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 4) {
        emit_opcode(cbuf, opnd_array(1)->reg(ra_,this,idx1) < 8 ? Assembler::REX : Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 22053 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 22061 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB6 /*secondary()*/));
  
#line 22069 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 22077 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void andI2L_rReg_imm255Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2938 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 4) {
        emit_opcode(cbuf, opnd_array(1)->reg(ra_,this,idx1) < 8 ? Assembler::REX : Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 22102 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 22110 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB6 /*secondary()*/));
  
#line 22118 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 22126 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void andI_rReg_imm65535Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 22151 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 22159 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB7 /*secondary()*/));
  
#line 22167 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 22175 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void andI2L_rReg_imm65535Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 22200 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 22208 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB7 /*secondary()*/));
  
#line 22216 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 22224 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void andI_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2398 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x04 /*secondary()*/), dstenc);
  
#line 22254 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 22268 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void andI_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 22309 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x23 /*primary()*/));
  
#line 22317 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 22333 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void andI_rReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 22374 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x23 /*primary()*/));
  
#line 22382 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 22398 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void andI_mem_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 22439 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x21 /*primary()*/));
  
#line 22447 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 22463 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void andI_mem_rReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 22504 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x21 /*primary()*/));
  
#line 22512 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 22528 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void andI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2897 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 22553 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2386 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  
#line 22568 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x4 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 22588 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 22602 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void orI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 22627 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 22635 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 22643 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void orI_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2398 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x01 /*secondary()*/), dstenc);
  
#line 22673 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 22687 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void orI_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 22728 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 22736 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 22752 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void orI_rReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 22793 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 22801 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 22817 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void orI_mem_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 22858 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x09 /*primary()*/));
  
#line 22866 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 22882 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void orI_mem_rReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 22923 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x09 /*primary()*/));
  
#line 22931 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 22947 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void orI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2897 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 22972 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2386 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  
#line 22987 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x1 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 23007 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 23021 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void xorI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 23046 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 23054 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 23062 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void xorI_rReg_im1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 9856 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

     __ notl(opnd_array(1)->as_Register(ra_,this,idx1));
  
#line 23078 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void xorI_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2398 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x06 /*secondary()*/), dstenc);
  
#line 23108 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 23122 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void xorI_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 23163 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 23171 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 23187 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void xorI_rReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 23228 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 23236 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 23252 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void xorI_mem_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 23293 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x31 /*primary()*/));
  
#line 23301 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 23317 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void xorI_mem_rReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 23358 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x31 /*primary()*/));
  
#line 23366 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 23382 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void xorI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2897 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 23407 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2386 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  
#line 23422 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x6 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 23442 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 23456 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void andL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3018 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 23483 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x23 /*primary()*/));
  
#line 23491 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 23499 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void andL_rReg_imm255Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3018 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 23526 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 23534 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB6 /*secondary()*/));
  
#line 23542 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 23550 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void andL_rReg_imm65535Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3018 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 23577 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 23585 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB7 /*secondary()*/));
  
#line 23593 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 23601 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void andL_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2418 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x04 /*secondary()*/), dstenc);
  
#line 23633 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  
#line 23647 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void andL_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 23690 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x23 /*primary()*/));
  
#line 23698 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 23714 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void andL_rReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 23757 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x23 /*primary()*/));
  
#line 23765 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 23781 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void andL_mem_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 23824 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x21 /*primary()*/));
  
#line 23832 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 23848 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void andL_mem_rReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 23891 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x21 /*primary()*/));
  
#line 23899 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 23915 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void andL_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2912 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 23942 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2386 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  
#line 23957 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x4 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 23977 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  
#line 23991 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void orL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3018 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 24018 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 24026 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 24034 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void orL_rReg_castP2XNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3018 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 24061 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 24069 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 24077 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void orL_rReg_castP2X_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3018 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 24104 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 24112 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(2)->reg(ra_,this,idx2) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 24120 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void orL_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2418 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x01 /*secondary()*/), dstenc);
  
#line 24152 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  
#line 24166 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void orL_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 24209 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 24217 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 24233 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void orL_rReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 24276 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 24284 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 24300 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void orL_mem_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 24343 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x09 /*primary()*/));
  
#line 24351 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 24367 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void orL_mem_rReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 24410 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x09 /*primary()*/));
  
#line 24418 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 24434 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void orL_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2912 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 24461 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2386 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  
#line 24476 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x1 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 24496 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  
#line 24510 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void xorL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3018 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 24537 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 24545 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 24553 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void xorL_rReg_im1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 10099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

     __ notq(opnd_array(1)->as_Register(ra_,this,idx1));
  
#line 24569 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void xorL_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2418 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x06 /*secondary()*/), dstenc);
  
#line 24601 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  
#line 24615 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void xorL_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 24658 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 24666 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 24682 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void xorL_rReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 24725 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 24733 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 24749 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void xorL_mem_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 24792 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x31 /*primary()*/));
  
#line 24800 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 24816 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void xorL_mem_rReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 24859 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x31 /*primary()*/));
  
#line 24867 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 24883 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void xorL_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2912 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 24910 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2386 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  
#line 24925 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x6 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 24945 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  
#line 24959 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void convI2BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 24983 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2185 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x85));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 24992 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3182 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(0)->reg(ra_,this);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    } else if (dstenc >= 4) {
      emit_opcode(cbuf, Assembler::REX);
    }
    // SETNZ $dst
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x95);
    emit_rm(cbuf, 0x3, 0x0, dstenc);
  
#line 25010 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2938 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(0)->reg(ra_,this) >= 4) {
        emit_opcode(cbuf, opnd_array(0)->reg(ra_,this) < 8 ? Assembler::REX : Assembler::REX_B);
      }
    } else {
      if (opnd_array(0)->reg(ra_,this) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 25028 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F));
  
#line 25036 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB6));
  
#line 25044 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(0)->reg(ra_,this) & 7);
  
#line 25052 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void convP2BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3018 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 25078 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2185 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x85));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 25087 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3182 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(0)->reg(ra_,this);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    } else if (dstenc >= 4) {
      emit_opcode(cbuf, Assembler::REX);
    }
    // SETNZ $dst
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x95);
    emit_rm(cbuf, 0x3, 0x0, dstenc);
  
#line 25105 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2938 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(0)->reg(ra_,this) >= 4) {
        emit_opcode(cbuf, opnd_array(0)->reg(ra_,this) < 8 ? Assembler::REX : Assembler::REX_B);
      }
    } else {
      if (opnd_array(0)->reg(ra_,this) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 25123 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F));
  
#line 25131 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB6));
  
#line 25139 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(0)->reg(ra_,this) & 7);
  
#line 25147 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmpLTMaskNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 25172 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2185 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x3B));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 25181 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3167 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(0)->reg(ra_,this);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    } else if (dstenc >= 4) {
      emit_opcode(cbuf, Assembler::REX);
    }
    // SETLT $dst
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x9C);
    emit_rm(cbuf, 0x3, 0x0, dstenc);
  
#line 25199 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2938 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(0)->reg(ra_,this) >= 4) {
        emit_opcode(cbuf, opnd_array(0)->reg(ra_,this) < 8 ? Assembler::REX : Assembler::REX_B);
      }
    } else {
      if (opnd_array(0)->reg(ra_,this) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 25217 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F));
  
#line 25225 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB6));
  
#line 25233 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(0)->reg(ra_,this) & 7);
  
#line 25241 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3141 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(0)->reg(ra_,this);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    // NEG $dst
    emit_opcode(cbuf, 0xF7);
    emit_rm(cbuf, 0x3, 0x03, dstenc);
  
#line 25256 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmpLTMask0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2682 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), dstenc);
    emit_d8(cbuf, (0x1F));
  
#line 25279 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cadd_cmpLTMaskNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {

#line 3198 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // cadd_cmpLT

    int tmpReg = opnd_array(4)->reg(ra_,this,idx4);

    int penc = opnd_array(1)->reg(ra_,this,idx1);
    int qenc = opnd_array(2)->reg(ra_,this,idx2);
    int yenc = opnd_array(3)->reg(ra_,this,idx3);

    // subl $p,$q
    if (penc < 8) {
      if (qenc >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (qenc < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
    emit_opcode(cbuf, 0x2B);
    emit_rm(cbuf, 0x3, penc & 7, qenc & 7);

    // sbbl $tmp, $tmp
    emit_opcode(cbuf, 0x1B);
    emit_rm(cbuf, 0x3, tmpReg, tmpReg);

    // andl $tmp, $y
    if (yenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x23);
    emit_rm(cbuf, 0x3, tmpReg, yenc & 7);

    // addl $p,$tmp
    if (penc >= 8) {
        emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode(cbuf, 0x03);
    emit_rm(cbuf, 0x3, penc & 7, tmpReg);
  
#line 25335 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cadd_cmpLTMask_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {

#line 3198 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // cadd_cmpLT

    int tmpReg = opnd_array(4)->reg(ra_,this,idx4);

    int penc = opnd_array(1)->reg(ra_,this,idx1);
    int qenc = opnd_array(2)->reg(ra_,this,idx2);
    int yenc = opnd_array(3)->reg(ra_,this,idx3);

    // subl $p,$q
    if (penc < 8) {
      if (qenc >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (qenc < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
    emit_opcode(cbuf, 0x2B);
    emit_rm(cbuf, 0x3, penc & 7, qenc & 7);

    // sbbl $tmp, $tmp
    emit_opcode(cbuf, 0x1B);
    emit_rm(cbuf, 0x3, tmpReg, tmpReg);

    // andl $tmp, $y
    if (yenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x23);
    emit_rm(cbuf, 0x3, tmpReg, yenc & 7);

    // addl $p,$tmp
    if (penc >= 8) {
        emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode(cbuf, 0x03);
    emit_rm(cbuf, 0x3, penc & 7, tmpReg);
  
#line 25391 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cadd_cmpLTMask_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {

#line 3198 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // cadd_cmpLT

    int tmpReg = opnd_array(4)->reg(ra_,this,idx4);

    int penc = opnd_array(2)->reg(ra_,this,idx2);
    int qenc = opnd_array(3)->reg(ra_,this,idx3);
    int yenc = opnd_array(1)->reg(ra_,this,idx1);

    // subl $p,$q
    if (penc < 8) {
      if (qenc >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (qenc < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
    emit_opcode(cbuf, 0x2B);
    emit_rm(cbuf, 0x3, penc & 7, qenc & 7);

    // sbbl $tmp, $tmp
    emit_opcode(cbuf, 0x1B);
    emit_rm(cbuf, 0x3, tmpReg, tmpReg);

    // andl $tmp, $y
    if (yenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x23);
    emit_rm(cbuf, 0x3, tmpReg, yenc & 7);

    // addl $p,$tmp
    if (penc >= 8) {
        emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode(cbuf, 0x03);
    emit_rm(cbuf, 0x3, penc & 7, tmpReg);
  
#line 25447 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cadd_cmpLTMask_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {

#line 3198 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // cadd_cmpLT

    int tmpReg = opnd_array(4)->reg(ra_,this,idx4);

    int penc = opnd_array(1)->reg(ra_,this,idx1);
    int qenc = opnd_array(2)->reg(ra_,this,idx2);
    int yenc = opnd_array(3)->reg(ra_,this,idx3);

    // subl $p,$q
    if (penc < 8) {
      if (qenc >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (qenc < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
    emit_opcode(cbuf, 0x2B);
    emit_rm(cbuf, 0x3, penc & 7, qenc & 7);

    // sbbl $tmp, $tmp
    emit_opcode(cbuf, 0x1B);
    emit_rm(cbuf, 0x3, tmpReg, tmpReg);

    // andl $tmp, $y
    if (yenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x23);
    emit_rm(cbuf, 0x3, tmpReg, yenc & 7);

    // addl $p,$tmp
    if (penc >= 8) {
        emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode(cbuf, 0x03);
    emit_rm(cbuf, 0x3, penc & 7, tmpReg);
  
#line 25503 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmpF_cc_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 25528 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 25536 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x2E /*secondary()*/));
  
#line 25544 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 25552 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2191 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // jnp,s exit
    emit_opcode(cbuf, 0x7B);
    emit_d8(cbuf, 0x0A);

    // pushfq
    emit_opcode(cbuf, 0x9C);

    // andq $0xffffff2b, (%rsp)
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x81);
    emit_opcode(cbuf, 0x24);
    emit_opcode(cbuf, 0x24);
    emit_d32(cbuf, 0xffffff2b);

    // popfq
    emit_opcode(cbuf, 0x9D);

    // nop (target for branch to avoid branch to branch)
    emit_opcode(cbuf, 0x90);
  
#line 25578 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmpF_cc_reg_CFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 10275 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ ucomiss(opnd_array(1)->as_XMMRegister(ra_,this,idx1), opnd_array(2)->as_XMMRegister(ra_,this,idx2));
  
#line 25594 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmpF_cc_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 25635 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 25643 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x2E /*secondary()*/));
  
#line 25651 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 25667 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2191 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // jnp,s exit
    emit_opcode(cbuf, 0x7B);
    emit_d8(cbuf, 0x0A);

    // pushfq
    emit_opcode(cbuf, 0x9C);

    // andq $0xffffff2b, (%rsp)
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x81);
    emit_opcode(cbuf, 0x24);
    emit_opcode(cbuf, 0x24);
    emit_d32(cbuf, 0xffffff2b);

    // popfq
    emit_opcode(cbuf, 0x9D);

    // nop (target for branch to avoid branch to branch)
    emit_opcode(cbuf, 0x90);
  
#line 25693 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmpF_cc_memCFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 25734 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 25742 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x2E /*secondary()*/));
  
#line 25750 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 25766 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmpF_cc_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 10318 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Label L_exit;
    __ ucomiss(opnd_array(1)->as_XMMRegister(ra_,this,idx1), InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ jcc(Assembler::noParity, L_exit);
    __ pushf();
    __ andq(rsp, 0xffffff2b);
    __ popf();
    __ bind(L_exit);
    __ nop();
  
#line 25789 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmpF_cc_immNode::eval_constant(Compile* C) {
  {

#line 10320 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"
    _constant = C->constant_table().add(opnd_array(2));
#line 25797 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmpF_cc_immCFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 10335 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ ucomiss(opnd_array(1)->as_XMMRegister(ra_,this,idx1), InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 25813 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmpF_cc_immCFNode::eval_constant(Compile* C) {
  {

#line 10336 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"
    _constant = C->constant_table().add(opnd_array(2));
#line 25821 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmpD_cc_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x66 /*primary()*/));
  
#line 25836 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 25854 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 25862 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x2E /*tertiary()*/));
  
#line 25870 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 25878 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2191 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // jnp,s exit
    emit_opcode(cbuf, 0x7B);
    emit_d8(cbuf, 0x0A);

    // pushfq
    emit_opcode(cbuf, 0x9C);

    // andq $0xffffff2b, (%rsp)
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x81);
    emit_opcode(cbuf, 0x24);
    emit_opcode(cbuf, 0x24);
    emit_d32(cbuf, 0xffffff2b);

    // popfq
    emit_opcode(cbuf, 0x9D);

    // nop (target for branch to avoid branch to branch)
    emit_opcode(cbuf, 0x90);
  
#line 25904 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmpD_cc_reg_CFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 10363 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ ucomisd(opnd_array(1)->as_XMMRegister(ra_,this,idx1), opnd_array(2)->as_XMMRegister(ra_,this,idx2));
  
#line 25920 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmpD_cc_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x66 /*primary()*/));
  
#line 25935 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 25969 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 25977 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x2E /*tertiary()*/));
  
#line 25985 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 26001 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2191 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // jnp,s exit
    emit_opcode(cbuf, 0x7B);
    emit_d8(cbuf, 0x0A);

    // pushfq
    emit_opcode(cbuf, 0x9C);

    // andq $0xffffff2b, (%rsp)
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x81);
    emit_opcode(cbuf, 0x24);
    emit_opcode(cbuf, 0x24);
    emit_d32(cbuf, 0xffffff2b);

    // popfq
    emit_opcode(cbuf, 0x9D);

    // nop (target for branch to avoid branch to branch)
    emit_opcode(cbuf, 0x90);
  
#line 26027 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmpD_cc_memCFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x66 /*primary()*/));
  
#line 26042 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 26076 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 26084 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x2E /*tertiary()*/));
  
#line 26092 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 26108 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmpD_cc_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 10406 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Label L_exit;
    __ ucomisd(opnd_array(1)->as_XMMRegister(ra_,this,idx1), InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ jcc(Assembler::noParity, L_exit);
    __ pushf();
    __ andq(rsp, 0xffffff2b);
    __ popf();
    __ bind(L_exit);
    __ nop();
  
#line 26131 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmpD_cc_immNode::eval_constant(Compile* C) {
  {

#line 10408 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"
    _constant = C->constant_table().add(opnd_array(2));
#line 26139 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmpD_cc_immCFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 10423 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ ucomisd(opnd_array(1)->as_XMMRegister(ra_,this,idx1), InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 26155 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmpD_cc_immCFNode::eval_constant(Compile* C) {
  {

#line 10424 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"
    _constant = C->constant_table().add(opnd_array(2));
#line 26163 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmpF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 26188 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 26196 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x2E /*secondary()*/));
  
#line 26204 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 26212 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2214 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(0)->reg(ra_,this);

    // movl $dst, -1
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0xB8 | (dstenc & 7));
    emit_d32(cbuf, -1);

    // jp,s done
    emit_opcode(cbuf, 0x7A);
    emit_d8(cbuf, dstenc < 4 ? 0x08 : 0x0A);

    // jb,s done
    emit_opcode(cbuf, 0x72);
    emit_d8(cbuf, dstenc < 4 ? 0x06 : 0x08);

    // setne $dst
    if (dstenc >= 4) {
      emit_opcode(cbuf, dstenc < 8 ? Assembler::REX : Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x95);
    emit_opcode(cbuf, 0xC0 | (dstenc & 7));

    // movzbl $dst, $dst
    if (dstenc >= 4) {
      emit_opcode(cbuf, dstenc < 8 ? Assembler::REX : Assembler::REX_RB);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0xB6);
    emit_rm(cbuf, 0x3, dstenc & 7, dstenc & 7);
  
#line 26251 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmpF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 26292 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 26300 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x2E /*secondary()*/));
  
#line 26308 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 26324 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2214 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(0)->reg(ra_,this);

    // movl $dst, -1
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0xB8 | (dstenc & 7));
    emit_d32(cbuf, -1);

    // jp,s done
    emit_opcode(cbuf, 0x7A);
    emit_d8(cbuf, dstenc < 4 ? 0x08 : 0x0A);

    // jb,s done
    emit_opcode(cbuf, 0x72);
    emit_d8(cbuf, dstenc < 4 ? 0x06 : 0x08);

    // setne $dst
    if (dstenc >= 4) {
      emit_opcode(cbuf, dstenc < 8 ? Assembler::REX : Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x95);
    emit_opcode(cbuf, 0xC0 | (dstenc & 7));

    // movzbl $dst, $dst
    if (dstenc >= 4) {
      emit_opcode(cbuf, dstenc < 8 ? Assembler::REX : Assembler::REX_RB);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0xB6);
    emit_rm(cbuf, 0x3, dstenc & 7, dstenc & 7);
  
#line 26363 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmpF_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 10484 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Label L_done;
    Register Rdst = opnd_array(0)->as_Register(ra_,this);
    __ ucomiss(opnd_array(1)->as_XMMRegister(ra_,this,idx1), InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ movl(Rdst, -1);
    __ jcc(Assembler::parity, L_done);
    __ jcc(Assembler::below, L_done);
    __ setb(Assembler::notEqual, Rdst);
    __ movzbl(Rdst, Rdst);
    __ bind(L_done);
  
#line 26387 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmpF_immNode::eval_constant(Compile* C) {
  {

#line 10487 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"
    _constant = C->constant_table().add(opnd_array(2));
#line 26395 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmpD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x66 /*primary()*/));
  
#line 26410 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 26428 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 26436 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x2E /*tertiary()*/));
  
#line 26444 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 26452 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2214 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(0)->reg(ra_,this);

    // movl $dst, -1
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0xB8 | (dstenc & 7));
    emit_d32(cbuf, -1);

    // jp,s done
    emit_opcode(cbuf, 0x7A);
    emit_d8(cbuf, dstenc < 4 ? 0x08 : 0x0A);

    // jb,s done
    emit_opcode(cbuf, 0x72);
    emit_d8(cbuf, dstenc < 4 ? 0x06 : 0x08);

    // setne $dst
    if (dstenc >= 4) {
      emit_opcode(cbuf, dstenc < 8 ? Assembler::REX : Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x95);
    emit_opcode(cbuf, 0xC0 | (dstenc & 7));

    // movzbl $dst, $dst
    if (dstenc >= 4) {
      emit_opcode(cbuf, dstenc < 8 ? Assembler::REX : Assembler::REX_RB);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0xB6);
    emit_rm(cbuf, 0x3, dstenc & 7, dstenc & 7);
  
#line 26491 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmpD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x66 /*primary()*/));
  
#line 26506 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 26540 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 26548 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x2E /*tertiary()*/));
  
#line 26556 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 26572 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2214 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(0)->reg(ra_,this);

    // movl $dst, -1
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0xB8 | (dstenc & 7));
    emit_d32(cbuf, -1);

    // jp,s done
    emit_opcode(cbuf, 0x7A);
    emit_d8(cbuf, dstenc < 4 ? 0x08 : 0x0A);

    // jb,s done
    emit_opcode(cbuf, 0x72);
    emit_d8(cbuf, dstenc < 4 ? 0x06 : 0x08);

    // setne $dst
    if (dstenc >= 4) {
      emit_opcode(cbuf, dstenc < 8 ? Assembler::REX : Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x95);
    emit_opcode(cbuf, 0xC0 | (dstenc & 7));

    // movzbl $dst, $dst
    if (dstenc >= 4) {
      emit_opcode(cbuf, dstenc < 8 ? Assembler::REX : Assembler::REX_RB);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0xB6);
    emit_rm(cbuf, 0x3, dstenc & 7, dstenc & 7);
  
#line 26611 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmpD_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 10553 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this);
    Label L_done;
    __ ucomisd(opnd_array(1)->as_XMMRegister(ra_,this,idx1), InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ movl(Rdst, -1);
    __ jcc(Assembler::parity, L_done);
    __ jcc(Assembler::below, L_done);
    __ setb(Assembler::notEqual, Rdst);
    __ movzbl(Rdst, Rdst);
    __ bind(L_done);
  
#line 26635 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmpD_immNode::eval_constant(Compile* C) {
  {

#line 10556 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"
    _constant = C->constant_table().add(opnd_array(2));
#line 26643 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void addF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF3 /*primary()*/));
  
#line 26658 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 26676 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 26684 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x58 /*tertiary()*/));
  
#line 26692 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 26700 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void addF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF3 /*primary()*/));
  
#line 26715 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 26749 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 26757 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x58 /*tertiary()*/));
  
#line 26765 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 26781 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void addF_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF3 /*primary()*/));
  
#line 26796 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 26830 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 26838 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x58 /*tertiary()*/));
  
#line 26846 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 26862 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void addF_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 10593 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ addss(opnd_array(1)->as_XMMRegister(ra_,this,idx1), InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 26878 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void addF_immNode::eval_constant(Compile* C) {
  {

#line 10594 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"
    _constant = C->constant_table().add(opnd_array(2));
#line 26886 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void addD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF2 /*primary()*/));
  
#line 26901 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 26919 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 26927 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x58 /*tertiary()*/));
  
#line 26935 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 26943 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void addD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF2 /*primary()*/));
  
#line 26958 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 26992 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 27000 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x58 /*tertiary()*/));
  
#line 27008 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 27024 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void addD_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF2 /*primary()*/));
  
#line 27039 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 27073 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 27081 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x58 /*tertiary()*/));
  
#line 27089 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 27105 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void addD_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 10625 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ addsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1), InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 27121 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void addD_immNode::eval_constant(Compile* C) {
  {

#line 10626 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"
    _constant = C->constant_table().add(opnd_array(2));
#line 27129 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void subF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF3 /*primary()*/));
  
#line 27144 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 27162 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 27170 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x5C /*tertiary()*/));
  
#line 27178 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 27186 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void subF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF3 /*primary()*/));
  
#line 27201 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 27235 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 27243 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x5C /*tertiary()*/));
  
#line 27251 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 27267 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void subF_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 10657 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ subss(opnd_array(1)->as_XMMRegister(ra_,this,idx1), InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 27283 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void subF_immNode::eval_constant(Compile* C) {
  {

#line 10658 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"
    _constant = C->constant_table().add(opnd_array(2));
#line 27291 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void subD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF2 /*primary()*/));
  
#line 27306 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 27324 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 27332 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x5C /*tertiary()*/));
  
#line 27340 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 27348 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void subD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF2 /*primary()*/));
  
#line 27363 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 27397 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 27405 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x5C /*tertiary()*/));
  
#line 27413 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 27429 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void subD_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 10689 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ subsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1), InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 27445 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void subD_immNode::eval_constant(Compile* C) {
  {

#line 10690 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"
    _constant = C->constant_table().add(opnd_array(2));
#line 27453 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void mulF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF3 /*primary()*/));
  
#line 27468 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 27486 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 27494 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x59 /*tertiary()*/));
  
#line 27502 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 27510 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void mulF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF3 /*primary()*/));
  
#line 27525 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 27559 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 27567 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x59 /*tertiary()*/));
  
#line 27575 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 27591 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void mulF_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF3 /*primary()*/));
  
#line 27606 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 27640 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 27648 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x59 /*tertiary()*/));
  
#line 27656 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 27672 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void mulF_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 10721 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ mulss(opnd_array(1)->as_XMMRegister(ra_,this,idx1), InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 27688 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void mulF_immNode::eval_constant(Compile* C) {
  {

#line 10722 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"
    _constant = C->constant_table().add(opnd_array(2));
#line 27696 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void mulD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF2 /*primary()*/));
  
#line 27711 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 27729 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 27737 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x59 /*tertiary()*/));
  
#line 27745 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 27753 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void mulD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF2 /*primary()*/));
  
#line 27768 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 27802 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 27810 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x59 /*tertiary()*/));
  
#line 27818 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 27834 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void mulD_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF2 /*primary()*/));
  
#line 27849 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 27883 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 27891 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x59 /*tertiary()*/));
  
#line 27899 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 27915 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void mulD_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 10753 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ mulsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1), InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 27931 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void mulD_immNode::eval_constant(Compile* C) {
  {

#line 10754 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"
    _constant = C->constant_table().add(opnd_array(2));
#line 27939 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void divF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF3 /*primary()*/));
  
#line 27954 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 27972 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 27980 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x5E /*tertiary()*/));
  
#line 27988 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 27996 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void divF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF3 /*primary()*/));
  
#line 28011 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 28045 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 28053 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x5E /*tertiary()*/));
  
#line 28061 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 28077 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void divF_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 10785 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ divss(opnd_array(1)->as_XMMRegister(ra_,this,idx1), InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 28093 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void divF_immNode::eval_constant(Compile* C) {
  {

#line 10786 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"
    _constant = C->constant_table().add(opnd_array(2));
#line 28101 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void divD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF2 /*primary()*/));
  
#line 28116 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 28134 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 28142 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x5E /*tertiary()*/));
  
#line 28150 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 28158 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void divD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF2 /*primary()*/));
  
#line 28173 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 28207 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 28215 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x5E /*tertiary()*/));
  
#line 28223 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 28239 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void divD_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 10817 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ divsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1), InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 28255 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void divD_immNode::eval_constant(Compile* C) {
  {

#line 10818 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"
    _constant = C->constant_table().add(opnd_array(2));
#line 28263 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void sqrtF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF3 /*primary()*/));
  
#line 28277 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 28295 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 28303 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x51 /*tertiary()*/));
  
#line 28311 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 28319 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void sqrtF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF3 /*primary()*/));
  
#line 28333 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 28367 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 28375 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x51 /*tertiary()*/));
  
#line 28383 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 28399 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void sqrtF_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

#line 10849 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ sqrtss(opnd_array(0)->as_XMMRegister(ra_,this), InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 28414 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void sqrtF_immNode::eval_constant(Compile* C) {
  {

#line 10850 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"
    _constant = C->constant_table().add(opnd_array(1));
#line 28422 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void sqrtD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF2 /*primary()*/));
  
#line 28436 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 28454 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 28462 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x51 /*tertiary()*/));
  
#line 28470 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 28478 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void sqrtD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF2 /*primary()*/));
  
#line 28492 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 28526 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 28534 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x51 /*tertiary()*/));
  
#line 28542 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 28558 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void sqrtD_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

#line 10881 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ sqrtsd(opnd_array(0)->as_XMMRegister(ra_,this), InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 28573 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void sqrtD_immNode::eval_constant(Compile* C) {
  {

#line 10882 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"
    _constant = C->constant_table().add(opnd_array(1));
#line 28581 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void absF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3617 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    address signmask_address = (address) StubRoutines::x86::float_sign_mask();

    cbuf.set_insts_mark();
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
      dstenc -= 8;
    }
    // XXX reg_mem doesn't support RIP-relative addressing yet
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x54);
    emit_rm(cbuf, 0x0, dstenc, 0x5);  // 00 reg 101
    emit_d32_reloc(cbuf, signmask_address);
  
#line 28607 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void absD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3634 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    address signmask_address = (address) StubRoutines::x86::double_sign_mask();

    cbuf.set_insts_mark();
    emit_opcode(cbuf, 0x66);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
      dstenc -= 8;
    }
    // XXX reg_mem doesn't support RIP-relative addressing yet
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x54);
    emit_rm(cbuf, 0x0, dstenc, 0x5);  // 00 reg 101
    emit_d32_reloc(cbuf, signmask_address);
  
#line 28634 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void negF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3652 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    address signflip_address = (address) StubRoutines::x86::float_sign_flip();

    cbuf.set_insts_mark();
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
      dstenc -= 8;
    }
    // XXX reg_mem doesn't support RIP-relative addressing yet
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x57);
    emit_rm(cbuf, 0x0, dstenc, 0x5);  // 00 reg 101
    emit_d32_reloc(cbuf, signflip_address);
  
#line 28660 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void negD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3669 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    address signflip_address = (address) StubRoutines::x86::double_sign_flip();

    cbuf.set_insts_mark();
    emit_opcode(cbuf, 0x66);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
      dstenc -= 8;
    }
    // XXX reg_mem doesn't support RIP-relative addressing yet
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x57);
    emit_rm(cbuf, 0x0, dstenc, 0x5);  // 00 reg 101
    emit_d32_reloc(cbuf, signflip_address);
  
#line 28687 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cosD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3314 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int srcenc = opnd_array(1)->reg(ra_,this,idx1);

    // subq rsp,#8
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x83);
    emit_rm(cbuf, 0x3, 0x5, RSP_enc);
    emit_d8(cbuf, 0x8);

    // movsd [rsp],src
    emit_opcode(cbuf, 0xF2);
    if (srcenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x11);
    encode_RegMem(cbuf, srcenc, RSP_enc, 0x4, 0, 0, false);

    // fldd [rsp]
    emit_opcode(cbuf, 0x66);
    emit_opcode(cbuf, 0xDD);
    encode_RegMem(cbuf, 0x0, RSP_enc, 0x4, 0, 0, false);
  
#line 28721 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD9 /*primary()*/));
  
#line 28729 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xFF /*secondary()*/));
  
#line 28737 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3293 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(1)->reg(ra_,this,idx1);

    store_to_stackslot( cbuf, 0xDD, 0x03, 0 ); //FSTP [RSP]

    // UseXmmLoadAndClearUpper ? movsd dst,[rsp] : movlpd dst,[rsp]
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0xF2 : 0x66);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0x10 : 0x12 );
    encode_RegMem(cbuf, dstenc, RSP_enc, 0x4, 0, 0, false);

    // add rsp,8
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf,0x83);
    emit_rm(cbuf,0x3, 0x0, RSP_enc);
    emit_d8(cbuf,0x08);
  
#line 28762 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void sinD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3314 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int srcenc = opnd_array(1)->reg(ra_,this,idx1);

    // subq rsp,#8
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x83);
    emit_rm(cbuf, 0x3, 0x5, RSP_enc);
    emit_d8(cbuf, 0x8);

    // movsd [rsp],src
    emit_opcode(cbuf, 0xF2);
    if (srcenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x11);
    encode_RegMem(cbuf, srcenc, RSP_enc, 0x4, 0, 0, false);

    // fldd [rsp]
    emit_opcode(cbuf, 0x66);
    emit_opcode(cbuf, 0xDD);
    encode_RegMem(cbuf, 0x0, RSP_enc, 0x4, 0, 0, false);
  
#line 28796 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD9 /*primary()*/));
  
#line 28804 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xFE /*secondary()*/));
  
#line 28812 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3293 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(1)->reg(ra_,this,idx1);

    store_to_stackslot( cbuf, 0xDD, 0x03, 0 ); //FSTP [RSP]

    // UseXmmLoadAndClearUpper ? movsd dst,[rsp] : movlpd dst,[rsp]
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0xF2 : 0x66);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0x10 : 0x12 );
    encode_RegMem(cbuf, dstenc, RSP_enc, 0x4, 0, 0, false);

    // add rsp,8
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf,0x83);
    emit_rm(cbuf,0x3, 0x0, RSP_enc);
    emit_d8(cbuf,0x08);
  
#line 28837 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void tanD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3314 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int srcenc = opnd_array(1)->reg(ra_,this,idx1);

    // subq rsp,#8
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x83);
    emit_rm(cbuf, 0x3, 0x5, RSP_enc);
    emit_d8(cbuf, 0x8);

    // movsd [rsp],src
    emit_opcode(cbuf, 0xF2);
    if (srcenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x11);
    encode_RegMem(cbuf, srcenc, RSP_enc, 0x4, 0, 0, false);

    // fldd [rsp]
    emit_opcode(cbuf, 0x66);
    emit_opcode(cbuf, 0xDD);
    encode_RegMem(cbuf, 0x0, RSP_enc, 0x4, 0, 0, false);
  
#line 28871 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD9));
  
#line 28879 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF2));
  
#line 28887 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xDD));
  
#line 28895 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD8));
  
#line 28903 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3293 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(1)->reg(ra_,this,idx1);

    store_to_stackslot( cbuf, 0xDD, 0x03, 0 ); //FSTP [RSP]

    // UseXmmLoadAndClearUpper ? movsd dst,[rsp] : movlpd dst,[rsp]
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0xF2 : 0x66);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0x10 : 0x12 );
    encode_RegMem(cbuf, dstenc, RSP_enc, 0x4, 0, 0, false);

    // add rsp,8
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf,0x83);
    emit_rm(cbuf,0x3, 0x0, RSP_enc);
    emit_d8(cbuf,0x08);
  
#line 28928 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void log10D_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD9));
  
#line 28942 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xEC));
  
#line 28950 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3314 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int srcenc = opnd_array(1)->reg(ra_,this,idx1);

    // subq rsp,#8
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x83);
    emit_rm(cbuf, 0x3, 0x5, RSP_enc);
    emit_d8(cbuf, 0x8);

    // movsd [rsp],src
    emit_opcode(cbuf, 0xF2);
    if (srcenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x11);
    encode_RegMem(cbuf, srcenc, RSP_enc, 0x4, 0, 0, false);

    // fldd [rsp]
    emit_opcode(cbuf, 0x66);
    emit_opcode(cbuf, 0xDD);
    encode_RegMem(cbuf, 0x0, RSP_enc, 0x4, 0, 0, false);
  
#line 28978 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD9));
  
#line 28986 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF1));
  
#line 28994 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3293 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(1)->reg(ra_,this,idx1);

    store_to_stackslot( cbuf, 0xDD, 0x03, 0 ); //FSTP [RSP]

    // UseXmmLoadAndClearUpper ? movsd dst,[rsp] : movlpd dst,[rsp]
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0xF2 : 0x66);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0x10 : 0x12 );
    encode_RegMem(cbuf, dstenc, RSP_enc, 0x4, 0, 0, false);

    // add rsp,8
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf,0x83);
    emit_rm(cbuf,0x3, 0x0, RSP_enc);
    emit_d8(cbuf,0x08);
  
#line 29019 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void logD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD9));
  
#line 29033 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xED));
  
#line 29041 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3314 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int srcenc = opnd_array(1)->reg(ra_,this,idx1);

    // subq rsp,#8
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x83);
    emit_rm(cbuf, 0x3, 0x5, RSP_enc);
    emit_d8(cbuf, 0x8);

    // movsd [rsp],src
    emit_opcode(cbuf, 0xF2);
    if (srcenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x11);
    encode_RegMem(cbuf, srcenc, RSP_enc, 0x4, 0, 0, false);

    // fldd [rsp]
    emit_opcode(cbuf, 0x66);
    emit_opcode(cbuf, 0xDD);
    encode_RegMem(cbuf, 0x0, RSP_enc, 0x4, 0, 0, false);
  
#line 29069 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD9));
  
#line 29077 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF1));
  
#line 29085 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3293 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(1)->reg(ra_,this,idx1);

    store_to_stackslot( cbuf, 0xDD, 0x03, 0 ); //FSTP [RSP]

    // UseXmmLoadAndClearUpper ? movsd dst,[rsp] : movlpd dst,[rsp]
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0xF2 : 0x66);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode  (cbuf, 0x0F );
    emit_opcode  (cbuf, UseXmmLoadAndClearUpper ? 0x10 : 0x12 );
    encode_RegMem(cbuf, dstenc, RSP_enc, 0x4, 0, 0, false);

    // add rsp,8
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf,0x83);
    emit_rm(cbuf,0x3, 0x0, RSP_enc);
    emit_d8(cbuf,0x08);
  
#line 29110 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void roundFloat_nopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
void roundDouble_nopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
void convF2D_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF3 /*primary()*/));
  
#line 29138 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 29156 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 29164 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x5A /*tertiary()*/));
  
#line 29172 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 29180 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void convF2D_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF3 /*primary()*/));
  
#line 29194 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 29228 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 29236 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x5A /*tertiary()*/));
  
#line 29244 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 29260 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void convD2F_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF2 /*primary()*/));
  
#line 29274 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 29292 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 29300 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x5A /*tertiary()*/));
  
#line 29308 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 29316 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void convD2F_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF2 /*primary()*/));
  
#line 29330 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 29364 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 29372 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x5A /*tertiary()*/));
  
#line 29380 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 29396 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void convF2I_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF3 /*primary()*/));
  
#line 29410 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 29428 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 29436 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x2C /*tertiary()*/));
  
#line 29444 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 29452 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3687 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(0)->reg(ra_,this);
    int srcenc = opnd_array(1)->reg(ra_,this,idx1);

    // cmpl $dst, #0x80000000
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x81);
    emit_rm(cbuf, 0x3, 0x7, dstenc & 7);
    emit_d32(cbuf, 0x80000000);

    // jne,s done
    emit_opcode(cbuf, 0x75);
    if (srcenc < 8 && dstenc < 8) {
      emit_d8(cbuf, 0xF);
    } else if (srcenc >= 8 && dstenc >= 8) {
      emit_d8(cbuf, 0x11);
    } else {
      emit_d8(cbuf, 0x10);
    }

    // subq rsp, #8
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x83);
    emit_rm(cbuf, 0x3, 0x5, RSP_enc);
    emit_d8(cbuf, 8);

    // movss [rsp], $src
    emit_opcode(cbuf, 0xF3);
    if (srcenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x11);
    encode_RegMem(cbuf, srcenc, RSP_enc, 0x4, 0, 0, false); // 2 bytes

    // call f2i_fixup
    cbuf.set_insts_mark();
    emit_opcode(cbuf, 0xE8);
    emit_d32_reloc(cbuf,
                   (int)
                   (StubRoutines::x86::f2i_fixup() - cbuf.insts_end() - 4),
                   runtime_call_Relocation::spec(),
                   RELOC_DISP32);

    // popq $dst
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x58 | (dstenc & 7));

    // done:
  
#line 29511 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void convF2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF3 /*primary()*/));
  
#line 29525 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3018 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 29545 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 29553 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x2C /*tertiary()*/));
  
#line 29561 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 29569 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3743 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(0)->reg(ra_,this);
    int srcenc = opnd_array(1)->reg(ra_,this,idx1);
    address const_address = (address) StubRoutines::x86::double_sign_flip();

    // cmpq $dst, [0x8000000000000000]
    cbuf.set_insts_mark();
    emit_opcode(cbuf, dstenc < 8 ? Assembler::REX_W : Assembler::REX_WR);
    emit_opcode(cbuf, 0x39);
    // XXX reg_mem doesn't support RIP-relative addressing yet
    emit_rm(cbuf, 0x0, dstenc & 7, 0x5); // 00 reg 101
    emit_d32_reloc(cbuf, const_address);


    // jne,s done
    emit_opcode(cbuf, 0x75);
    if (srcenc < 8 && dstenc < 8) {
      emit_d8(cbuf, 0xF);
    } else if (srcenc >= 8 && dstenc >= 8) {
      emit_d8(cbuf, 0x11);
    } else {
      emit_d8(cbuf, 0x10);
    }

    // subq rsp, #8
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x83);
    emit_rm(cbuf, 0x3, 0x5, RSP_enc);
    emit_d8(cbuf, 8);

    // movss [rsp], $src
    emit_opcode(cbuf, 0xF3);
    if (srcenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x11);
    encode_RegMem(cbuf, srcenc, RSP_enc, 0x4, 0, 0, false); // 2 bytes

    // call f2l_fixup
    cbuf.set_insts_mark();
    emit_opcode(cbuf, 0xE8);
    emit_d32_reloc(cbuf,
                   (int)
                   (StubRoutines::x86::f2l_fixup() - cbuf.insts_end() - 4),
                   runtime_call_Relocation::spec(),
                   RELOC_DISP32);

    // popq $dst
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x58 | (dstenc & 7));

    // done:
  
#line 29630 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void convD2I_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF2 /*primary()*/));
  
#line 29644 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 29662 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 29670 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x2C /*tertiary()*/));
  
#line 29678 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 29686 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3801 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(0)->reg(ra_,this);
    int srcenc = opnd_array(1)->reg(ra_,this,idx1);

    // cmpl $dst, #0x80000000
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x81);
    emit_rm(cbuf, 0x3, 0x7, dstenc & 7);
    emit_d32(cbuf, 0x80000000);

    // jne,s done
    emit_opcode(cbuf, 0x75);
    if (srcenc < 8 && dstenc < 8) {
      emit_d8(cbuf, 0xF);
    } else if (srcenc >= 8 && dstenc >= 8) {
      emit_d8(cbuf, 0x11);
    } else {
      emit_d8(cbuf, 0x10);
    }

    // subq rsp, #8
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x83);
    emit_rm(cbuf, 0x3, 0x5, RSP_enc);
    emit_d8(cbuf, 8);

    // movsd [rsp], $src
    emit_opcode(cbuf, 0xF2);
    if (srcenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x11);
    encode_RegMem(cbuf, srcenc, RSP_enc, 0x4, 0, 0, false); // 2 bytes

    // call d2i_fixup
    cbuf.set_insts_mark();
    emit_opcode(cbuf, 0xE8);
    emit_d32_reloc(cbuf,
                   (int)
                   (StubRoutines::x86::d2i_fixup() - cbuf.insts_end() - 4),
                   runtime_call_Relocation::spec(),
                   RELOC_DISP32);

    // popq $dst
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x58 | (dstenc & 7));

    // done:
  
#line 29745 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void convD2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF2 /*primary()*/));
  
#line 29759 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3018 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 29779 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 29787 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x2C /*tertiary()*/));
  
#line 29795 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 29803 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3857 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(0)->reg(ra_,this);
    int srcenc = opnd_array(1)->reg(ra_,this,idx1);
    address const_address = (address) StubRoutines::x86::double_sign_flip();

    // cmpq $dst, [0x8000000000000000]
    cbuf.set_insts_mark();
    emit_opcode(cbuf, dstenc < 8 ? Assembler::REX_W : Assembler::REX_WR);
    emit_opcode(cbuf, 0x39);
    // XXX reg_mem doesn't support RIP-relative addressing yet
    emit_rm(cbuf, 0x0, dstenc & 7, 0x5); // 00 reg 101
    emit_d32_reloc(cbuf, const_address);


    // jne,s done
    emit_opcode(cbuf, 0x75);
    if (srcenc < 8 && dstenc < 8) {
      emit_d8(cbuf, 0xF);
    } else if (srcenc >= 8 && dstenc >= 8) {
      emit_d8(cbuf, 0x11);
    } else {
      emit_d8(cbuf, 0x10);
    }

    // subq rsp, #8
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x83);
    emit_rm(cbuf, 0x3, 0x5, RSP_enc);
    emit_d8(cbuf, 8);

    // movsd [rsp], $src
    emit_opcode(cbuf, 0xF2);
    if (srcenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_R);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x11);
    encode_RegMem(cbuf, srcenc, RSP_enc, 0x4, 0, 0, false); // 2 bytes

    // call d2l_fixup
    cbuf.set_insts_mark();
    emit_opcode(cbuf, 0xE8);
    emit_d32_reloc(cbuf,
                   (int)
                   (StubRoutines::x86::d2l_fixup() - cbuf.insts_end() - 4),
                   runtime_call_Relocation::spec(),
                   RELOC_DISP32);

    // popq $dst
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x58 | (dstenc & 7));

    // done:
  
#line 29864 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void convI2F_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF3 /*primary()*/));
  
#line 29878 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 29896 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 29904 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x2A /*tertiary()*/));
  
#line 29912 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 29920 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void convI2F_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF3 /*primary()*/));
  
#line 29934 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 29968 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 29976 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x2A /*tertiary()*/));
  
#line 29984 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 30000 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void convI2D_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF2 /*primary()*/));
  
#line 30014 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 30032 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 30040 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x2A /*tertiary()*/));
  
#line 30048 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 30056 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void convI2D_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF2 /*primary()*/));
  
#line 30070 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 30104 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 30112 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x2A /*tertiary()*/));
  
#line 30120 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 30136 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void convXI2F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

#line 11174 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this), opnd_array(1)->as_Register(ra_,this,idx1));
    __ cvtdq2ps(opnd_array(0)->as_XMMRegister(ra_,this), opnd_array(0)->as_XMMRegister(ra_,this));
  
#line 30152 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void convXI2D_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

#line 11188 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this), opnd_array(1)->as_Register(ra_,this,idx1));
    __ cvtdq2pd(opnd_array(0)->as_XMMRegister(ra_,this), opnd_array(0)->as_XMMRegister(ra_,this));
  
#line 30168 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void convL2F_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF3 /*primary()*/));
  
#line 30182 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3018 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 30202 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 30210 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x2A /*tertiary()*/));
  
#line 30218 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 30226 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void convL2F_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF3 /*primary()*/));
  
#line 30240 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 30276 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 30284 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x2A /*tertiary()*/));
  
#line 30292 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 30308 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void convL2D_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF2 /*primary()*/));
  
#line 30322 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3018 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 30342 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 30350 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x2A /*tertiary()*/));
  
#line 30358 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 30366 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void convL2D_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF2 /*primary()*/));
  
#line 30380 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 30416 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 30424 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x2A /*tertiary()*/));
  
#line 30432 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 30448 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void convI2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

#line 11241 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movslq(opnd_array(0)->as_Register(ra_,this), opnd_array(1)->as_Register(ra_,this,idx1));
  
#line 30463 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void convI2L_reg_reg_zexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2791 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    encode_copy(cbuf, opnd_array(0)->reg(ra_,this), opnd_array(1)->reg(ra_,this,idx1));
  
#line 30478 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void convI2L_reg_mem_zexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 30519 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 30527 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 30543 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void zerox_long_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2801 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int srcenc = opnd_array(1)->reg(ra_,this,idx1);
    int dstenc = opnd_array(0)->reg(ra_,this);

    if (dstenc < 8) {
      if (srcenc >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
        srcenc -= 8;
      }
    } else {
      if (srcenc < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
        srcenc -= 8;
      }
      dstenc -= 8;
    }

    emit_opcode(cbuf, 0x8B);
    emit_rm(cbuf, 0x3, dstenc, srcenc);
  
#line 30577 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void convL2I_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2801 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int srcenc = opnd_array(1)->reg(ra_,this,idx1);
    int dstenc = opnd_array(0)->reg(ra_,this);

    if (dstenc < 8) {
      if (srcenc >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
        srcenc -= 8;
      }
    } else {
      if (srcenc < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
        srcenc -= 8;
      }
      dstenc -= 8;
    }

    emit_opcode(cbuf, 0x8B);
    emit_rm(cbuf, 0x3, dstenc, srcenc);
  
#line 30610 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void MoveF2I_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 30650 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 30658 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 30674 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void MoveI2F_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF3 /*primary()*/));
  
#line 30688 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 30722 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 30730 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x10 /*tertiary()*/));
  
#line 30738 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 30754 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void MoveD2L_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 30796 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 30804 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 30820 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void MoveL2D_stack_reg_partialNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x66 /*primary()*/));
  
#line 30834 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 30868 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 30876 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x12 /*tertiary()*/));
  
#line 30884 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 30900 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void MoveL2D_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF2 /*primary()*/));
  
#line 30914 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 30948 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 30956 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x10 /*tertiary()*/));
  
#line 30964 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 30980 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void MoveF2I_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF3 /*primary()*/));
  
#line 30994 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 31028 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 31036 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x11 /*tertiary()*/));
  
#line 31044 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int disp = opnd_array(0)->disp(ra_,this,0);
    bool disp_is_oop = opnd_array(0)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 31060 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void MoveI2F_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 31100 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x89 /*primary()*/));
  
#line 31108 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int disp = opnd_array(0)->disp(ra_,this,0);
    bool disp_is_oop = opnd_array(0)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 31124 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void MoveD2L_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF2 /*primary()*/));
  
#line 31138 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 31172 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 31180 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2158 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x11 /*tertiary()*/));
  
#line 31188 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int disp = opnd_array(0)->disp(ra_,this,0);
    bool disp_is_oop = opnd_array(0)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 31204 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void MoveL2D_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 31246 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x89 /*primary()*/));
  
#line 31254 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int disp = opnd_array(0)->disp(ra_,this,0);
    bool disp_is_oop = opnd_array(0)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 31270 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void MoveF2I_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

#line 11411 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"
__ movdl(opnd_array(0)->as_Register(ra_,this), opnd_array(1)->as_XMMRegister(ra_,this,idx1)); 
#line 31283 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void MoveD2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

#line 11420 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"
__ movdq(opnd_array(0)->as_Register(ra_,this), opnd_array(1)->as_XMMRegister(ra_,this,idx1)); 
#line 31296 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void MoveI2F_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

#line 11430 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"
__ movdl(opnd_array(0)->as_XMMRegister(ra_,this), opnd_array(1)->as_Register(ra_,this,idx1)); 
#line 31309 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void MoveL2D_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {
    MacroAssembler _masm(&cbuf);

#line 11439 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"
__ movdq(opnd_array(0)->as_XMMRegister(ra_,this), opnd_array(1)->as_Register(ra_,this,idx1)); 
#line 31322 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void Repl8B_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3349 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);

    encode_CopyXD(cbuf, opnd_array(0)->reg(ra_,this), opnd_array(1)->reg(ra_,this,idx1));
    __ punpcklbw(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)));
    __ pshuflw(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)), 0x00);
  
#line 31340 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void Repl8B_rRegINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3375 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);

    __ movdl(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_Register(opnd_array(1)->reg(ra_,this,idx1)));
  
#line 31356 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3349 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);

    encode_CopyXD(cbuf, opnd_array(0)->reg(ra_,this), opnd_array(0)->reg(ra_,this));
    __ punpcklbw(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)));
    __ pshuflw(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)), 0x00);
  
#line 31368 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void Repl8B_immI0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3369 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);

    __ pxor(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)));
  
#line 31384 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void Repl4S_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3357 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);

    __ pshuflw(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(1)->reg(ra_,this,idx1)), 0x00);
  
#line 31400 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void Repl4S_rRegINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3375 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);

    __ movdl(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_Register(opnd_array(1)->reg(ra_,this,idx1)));
  
#line 31416 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3357 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);

    __ pshuflw(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)), 0x00);
  
#line 31426 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void Repl4S_immI0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3369 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);

    __ pxor(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)));
  
#line 31442 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void Repl4C_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3357 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);

    __ pshuflw(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(1)->reg(ra_,this,idx1)), 0x00);
  
#line 31458 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void Repl4C_rRegINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3375 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);

    __ movdl(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_Register(opnd_array(1)->reg(ra_,this,idx1)));
  
#line 31474 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3357 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);

    __ pshuflw(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)), 0x00);
  
#line 31484 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void Repl4C_immI0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3369 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);

    __ pxor(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)));
  
#line 31500 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void Repl2I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3363 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);

    __ pshufd(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(1)->reg(ra_,this,idx1)), (0x00));
  
#line 31516 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void Repl2I_rRegINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3375 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);

    __ movdl(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_Register(opnd_array(1)->reg(ra_,this,idx1)));
  
#line 31532 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3363 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);

    __ pshufd(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)), (0x00));
  
#line 31542 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void Repl2I_immI0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3369 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);

    __ pxor(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)));
  
#line 31558 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void Repl2F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3363 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);

    __ pshufd(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(1)->reg(ra_,this,idx1)), (0xe0));
  
#line 31574 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void Repl2F_regFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3363 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);

    __ pshufd(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(1)->reg(ra_,this,idx1)), (0xe0));
  
#line 31590 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void Repl2F_immF0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 3369 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);

    __ pxor(as_XMMRegister(opnd_array(0)->reg(ra_,this)), as_XMMRegister(opnd_array(0)->reg(ra_,this)));
  
#line 31606 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void rep_stosNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2185 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x33));
    emit_rm(cbuf, 0x3, (RAX_enc) & 7, (RAX_enc) & 7);
  
#line 31622 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF3));
  
#line 31630 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x48));
  
#line 31638 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xAB));
  
#line 31646 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void string_compareNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
    unsigned idx5 = idx4 + opnd_array(4)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 11593 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ string_compare(opnd_array(1)->as_Register(ra_,this,idx1), opnd_array(3)->as_Register(ra_,this,idx3),
                      opnd_array(2)->as_Register(ra_,this,idx2), opnd_array(4)->as_Register(ra_,this,idx4), opnd_array(0)->as_Register(ra_,this),
                      opnd_array(5)->as_XMMRegister(ra_,this,idx5));
  
#line 31667 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void string_indexofNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
    unsigned idx5 = idx4 + opnd_array(4)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 11609 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1), opnd_array(3)->as_Register(ra_,this,idx3),
                      opnd_array(2)->as_Register(ra_,this,idx2), opnd_array(4)->as_Register(ra_,this,idx4), opnd_array(0)->as_Register(ra_,this),
                      opnd_array(5)->as_XMMRegister(ra_,this,idx5), as_Register(RCX_enc));
  
#line 31688 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void string_equalsNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
    unsigned idx5 = idx4 + opnd_array(4)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 11625 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ char_arrays_equals(false, opnd_array(1)->as_Register(ra_,this,idx1), opnd_array(2)->as_Register(ra_,this,idx2),
                          opnd_array(3)->as_Register(ra_,this,idx3), opnd_array(0)->as_Register(ra_,this), as_Register(RBX_enc),
                          opnd_array(4)->as_XMMRegister(ra_,this,idx4), opnd_array(5)->as_XMMRegister(ra_,this,idx5));
  
#line 31709 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void array_equalsNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 11642 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ char_arrays_equals(true, opnd_array(1)->as_Register(ra_,this,idx1), opnd_array(2)->as_Register(ra_,this,idx2),
                          as_Register(RCX_enc), opnd_array(0)->as_Register(ra_,this), as_Register(RBX_enc),
                          opnd_array(3)->as_XMMRegister(ra_,this,idx3), opnd_array(4)->as_XMMRegister(ra_,this,idx4));
  
#line 31729 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void compI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 31754 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 31762 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 31770 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void compI_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2398 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x07 /*secondary()*/), dstenc);
  
#line 31800 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 31814 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void compI_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 31855 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 31863 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 31879 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void testI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 31904 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 31912 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 31920 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void testI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {

#line 2987 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 31938 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 31946 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 31955 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2852 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Output immediate
    emit_d32(cbuf, opnd_array(2)->constant());
  
#line 31964 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void testI_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 32006 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 32014 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 32030 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void testI_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 32072 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 32080 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 32096 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void compU_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 32121 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 32129 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 32137 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void compU_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2398 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x07 /*secondary()*/), dstenc);
  
#line 32167 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 32181 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void compU_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3035 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 32222 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 32230 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 32246 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void testU_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 32271 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 32279 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 32287 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void compP_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3018 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 32314 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 32322 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 32330 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void compP_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 32373 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 32381 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 32397 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void compP_mem_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 32440 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 32448 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 32464 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void testP_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3018 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 32491 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 32499 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 32507 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void testP_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2912 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 32534 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 32542 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3112 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 32562 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2879 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_d32(cbuf,(0xFFFFFFFF));
  
#line 32570 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void testP_mem_reg0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 11850 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ cmpq(r12, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 32586 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void compN_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 11861 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"
__ cmpl(opnd_array(1)->as_Register(ra_,this,idx1), opnd_array(2)->as_Register(ra_,this,idx2)); 
#line 32600 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void compN_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 11870 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ cmpl(opnd_array(1)->as_Register(ra_,this,idx1), Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()));
  
#line 32616 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void compN_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 11880 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ cmp_narrow_oop(opnd_array(1)->as_Register(ra_,this,idx1), (jobject)opnd_array(2)->constant());
  
#line 32632 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void compN_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 11891 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ cmp_narrow_oop(Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), (jobject)opnd_array(1)->constant());
  
#line 32648 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void testN_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 11901 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"
__ testl(opnd_array(1)->as_Register(ra_,this,idx1), opnd_array(1)->as_Register(ra_,this,idx1)); 
#line 32662 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void testN_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 11912 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ cmpl(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), (int)0xFFFFFFFF);
  
#line 32678 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void testN_mem_reg0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 11924 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ cmpl(r12, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 32694 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void compL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3018 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 32721 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 32729 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 32737 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void compL_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2418 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1);
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x07 /*secondary()*/), dstenc);
  
#line 32769 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2440 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  
#line 32783 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void compL_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 32826 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 32834 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 32850 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void testL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3018 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 32877 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 32885 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 32893 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void testL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {

#line 2994 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 32913 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 32921 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 32930 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2852 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Output immediate
    emit_d32(cbuf, opnd_array(2)->constantL());
  
#line 32939 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void testL_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 32983 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 32991 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1);
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 33007 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void testL_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 2;
    unsigned idx1 = 2;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {

#line 3066 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 33051 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 33059 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 3099 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2);
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 33075 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmpL3_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 3243 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int src1enc = opnd_array(1)->reg(ra_,this,idx1);
    int src2enc = opnd_array(2)->reg(ra_,this,idx2);
    int dstenc = opnd_array(0)->reg(ra_,this);

    // cmpq $src1, $src2
    if (src1enc < 8) {
      if (src2enc < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (src2enc < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
    emit_opcode(cbuf, 0x3B);
    emit_rm(cbuf, 0x3, src1enc & 7, src2enc & 7);

    // movl $dst, -1
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0xB8 | (dstenc & 7));
    emit_d32(cbuf, -1);

    // jl,s done
    emit_opcode(cbuf, 0x7C);
    emit_d8(cbuf, dstenc < 4 ? 0x06 : 0x08);

    // setne $dst
    if (dstenc >= 4) {
      emit_opcode(cbuf, dstenc < 8 ? Assembler::REX : Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x95);
    emit_opcode(cbuf, 0xC0 | (dstenc & 7));

    // movzbl $dst, $dst
    if (dstenc >= 4) {
      emit_opcode(cbuf, dstenc < 8 ? Assembler::REX : Assembler::REX_RB);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0xB6);
    emit_rm(cbuf, 0x3, dstenc & 7, dstenc & 7);
  
#line 33136 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmovI_reg_gNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 33162 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 33170 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x4F /*secondary()*/));
  
#line 33178 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 33186 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmovI_reg_lNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {

#line 3003 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2) >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2) < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 33212 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 33220 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2152 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x4C /*secondary()*/));
  
#line 33228 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2180 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1) & 7, opnd_array(2)->reg(ra_,this,idx2) & 7);
  
#line 33236 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void jmpDirNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xE9 /*primary()*/));
  
#line 33250 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2451 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // JMP, CALL
    Label* l = opnd_array(1)->label();
    emit_d32(cbuf, l ? (l->loc_pos() - (cbuf.insts_size() + 4)) : 0);
  
#line 33260 "../generated/adfiles/ad_x86_64.cpp"
  }
}
uint  jmpDirNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 5, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 5);
}
void jmpConNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {

#line 2485 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // JCC
    Label* l = opnd_array(3)->label();
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x80 /*secondary()*/), opnd_array(1)->ccode());
    emit_d32(cbuf, l ? (l->loc_pos() - (cbuf.insts_size() + 4)) : 0);
  
#line 33284 "../generated/adfiles/ad_x86_64.cpp"
  }
}
uint  jmpConNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 6);
}
void jmpLoopEndNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {

#line 2485 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // JCC
    Label* l = opnd_array(3)->label();
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x80 /*secondary()*/), opnd_array(1)->ccode());
    emit_d32(cbuf, l ? (l->loc_pos() - (cbuf.insts_size() + 4)) : 0);
  
#line 33308 "../generated/adfiles/ad_x86_64.cpp"
  }
}
uint  jmpLoopEndNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 6);
}
void jmpLoopEndUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {

#line 2485 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // JCC
    Label* l = opnd_array(3)->label();
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x80 /*secondary()*/), opnd_array(1)->ccode());
    emit_d32(cbuf, l ? (l->loc_pos() - (cbuf.insts_size() + 4)) : 0);
  
#line 33332 "../generated/adfiles/ad_x86_64.cpp"
  }
}
uint  jmpLoopEndUNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 6);
}
void jmpLoopEndUCFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {

#line 2485 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // JCC
    Label* l = opnd_array(3)->label();
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x80 /*secondary()*/), opnd_array(1)->ccode());
    emit_d32(cbuf, l ? (l->loc_pos() - (cbuf.insts_size() + 4)) : 0);
  
#line 33356 "../generated/adfiles/ad_x86_64.cpp"
  }
}
uint  jmpLoopEndUCFNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 6);
}
void jmpConUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {

#line 2485 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // JCC
    Label* l = opnd_array(3)->label();
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x80 /*secondary()*/), opnd_array(1)->ccode());
    emit_d32(cbuf, l ? (l->loc_pos() - (cbuf.insts_size() + 4)) : 0);
  
#line 33380 "../generated/adfiles/ad_x86_64.cpp"
  }
}
uint  jmpConUNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 6);
}
void jmpConUCFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {

#line 2485 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // JCC
    Label* l = opnd_array(3)->label();
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x80 /*secondary()*/), opnd_array(1)->ccode());
    emit_d32(cbuf, l ? (l->loc_pos() - (cbuf.insts_size() + 4)) : 0);
  
#line 33404 "../generated/adfiles/ad_x86_64.cpp"
  }
}
uint  jmpConUCFNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 6);
}
void jmpConUCF2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 12179 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Label* l = opnd_array(3)->label();
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x80 /*secondary()*/), Assembler::parity);
    int parity_disp = -1;
    if (opnd_array(1)->ccode()== Assembler::notEqual) {
       // the two jumps 6 bytes apart so the jump distances are too
       parity_disp = l ? (l->loc_pos() - (cbuf.insts_size() + 4)) : 0;
    } else if (opnd_array(1)->ccode()== Assembler::equal) {
       parity_disp = 6;
    } else {
       ShouldNotReachHere();
    }
    emit_d32(cbuf, parity_disp);
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x80 /*secondary()*/), opnd_array(1)->ccode());
    int disp = l ? (l->loc_pos() - (cbuf.insts_size() + 4)) : 0;
    emit_d32(cbuf, disp);
  
#line 33441 "../generated/adfiles/ad_x86_64.cpp"
  }
}
uint  jmpConUCF2Node::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 12, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 12);
}
void partialSubtypeCheckNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2560 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Register Rrdi = as_Register(RDI_enc); // result register
    Register Rrax = as_Register(RAX_enc); // super class
    Register Rrcx = as_Register(RCX_enc); // killed
    Register Rrsi = as_Register(RSI_enc); // sub class
    Label miss;
    const bool set_cond_codes = true;

    MacroAssembler _masm(&cbuf);
    __ check_klass_subtype_slow_path(Rrsi, Rrax, Rrcx, Rrdi,
                                     NULL, &miss,
                                     /*set_cond_codes:*/ true);
    if ((0x1 /*primary()*/)) {
      __ xorptr(Rrdi, Rrdi);
    }
    __ bind(miss);
  
#line 33474 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void partialSubtypeCheck_vs_ZeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {

#line 2560 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Register Rrdi = as_Register(RDI_enc); // result register
    Register Rrax = as_Register(RAX_enc); // super class
    Register Rrcx = as_Register(RCX_enc); // killed
    Register Rrsi = as_Register(RSI_enc); // sub class
    Label miss;
    const bool set_cond_codes = true;

    MacroAssembler _masm(&cbuf);
    __ check_klass_subtype_slow_path(Rrsi, Rrax, Rrcx, Rrdi,
                                     NULL, &miss,
                                     /*set_cond_codes:*/ true);
    if ((0x0 /*primary()*/)) {
      __ xorptr(Rrdi, Rrdi);
    }
    __ bind(miss);
  
#line 33504 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void jmpDir_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xEB /*primary()*/));
  
#line 33518 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2458 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // JMP, CALL
    Label* l = opnd_array(1)->label();
    int disp = l ? (l->loc_pos() - (cbuf.insts_size() + 1)) : 0;
    assert(-128 <= disp && disp <= 127, "Displacement too large for short jmp");
    emit_d8(cbuf, disp);
  
#line 33530 "../generated/adfiles/ad_x86_64.cpp"
  }
}
uint  jmpDir_shortNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void jmpCon_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {

#line 2494 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

  // JCC
    Label *l = opnd_array(3)->label();
    emit_cc(cbuf, (0x70 /*primary()*/), opnd_array(1)->ccode());
    int disp = l ? (l->loc_pos() - (cbuf.insts_size() + 1)) : 0;
    assert(-128 <= disp && disp <= 127, "Displacement too large for short jmp");
    emit_d8(cbuf, disp);
  
#line 33555 "../generated/adfiles/ad_x86_64.cpp"
  }
}
uint  jmpCon_shortNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void jmpLoopEnd_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {

#line 2494 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

  // JCC
    Label *l = opnd_array(3)->label();
    emit_cc(cbuf, (0x70 /*primary()*/), opnd_array(1)->ccode());
    int disp = l ? (l->loc_pos() - (cbuf.insts_size() + 1)) : 0;
    assert(-128 <= disp && disp <= 127, "Displacement too large for short jmp");
    emit_d8(cbuf, disp);
  
#line 33580 "../generated/adfiles/ad_x86_64.cpp"
  }
}
uint  jmpLoopEnd_shortNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void jmpLoopEndU_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {

#line 2494 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

  // JCC
    Label *l = opnd_array(3)->label();
    emit_cc(cbuf, (0x70 /*primary()*/), opnd_array(1)->ccode());
    int disp = l ? (l->loc_pos() - (cbuf.insts_size() + 1)) : 0;
    assert(-128 <= disp && disp <= 127, "Displacement too large for short jmp");
    emit_d8(cbuf, disp);
  
#line 33605 "../generated/adfiles/ad_x86_64.cpp"
  }
}
uint  jmpLoopEndU_shortNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void jmpLoopEndUCF_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {

#line 2494 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

  // JCC
    Label *l = opnd_array(3)->label();
    emit_cc(cbuf, (0x70 /*primary()*/), opnd_array(1)->ccode());
    int disp = l ? (l->loc_pos() - (cbuf.insts_size() + 1)) : 0;
    assert(-128 <= disp && disp <= 127, "Displacement too large for short jmp");
    emit_d8(cbuf, disp);
  
#line 33630 "../generated/adfiles/ad_x86_64.cpp"
  }
}
uint  jmpLoopEndUCF_shortNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void jmpConU_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {

#line 2494 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

  // JCC
    Label *l = opnd_array(3)->label();
    emit_cc(cbuf, (0x70 /*primary()*/), opnd_array(1)->ccode());
    int disp = l ? (l->loc_pos() - (cbuf.insts_size() + 1)) : 0;
    assert(-128 <= disp && disp <= 127, "Displacement too large for short jmp");
    emit_d8(cbuf, disp);
  
#line 33655 "../generated/adfiles/ad_x86_64.cpp"
  }
}
uint  jmpConU_shortNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void jmpConUCF_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {

#line 2494 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

  // JCC
    Label *l = opnd_array(3)->label();
    emit_cc(cbuf, (0x70 /*primary()*/), opnd_array(1)->ccode());
    int disp = l ? (l->loc_pos() - (cbuf.insts_size() + 1)) : 0;
    assert(-128 <= disp && disp <= 127, "Displacement too large for short jmp");
    emit_d8(cbuf, disp);
  
#line 33680 "../generated/adfiles/ad_x86_64.cpp"
  }
}
uint  jmpConUCF_shortNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 2);
}
void jmpConUCF2_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {
    MacroAssembler _masm(&cbuf);

#line 12385 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Label* l = opnd_array(3)->label();
    emit_cc(cbuf, (0x70 /*primary()*/), Assembler::parity);
    int parity_disp = -1;
    if (opnd_array(1)->ccode()== Assembler::notEqual) {
      parity_disp = l ? (l->loc_pos() - (cbuf.insts_size() + 1)) : 0;
    } else if (opnd_array(1)->ccode()== Assembler::equal) {
      parity_disp = 2;
    } else {
      ShouldNotReachHere();
    }
    emit_d8(cbuf, parity_disp);
    emit_cc(cbuf, (0x70 /*primary()*/), opnd_array(1)->ccode());
    int disp = l ? (l->loc_pos() - (cbuf.insts_size() + 1)) : 0;
    emit_d8(cbuf, disp);
    assert(-128 <= disp && disp <= 127, "Displacement too large for short jmp");
    assert(-128 <= parity_disp && parity_disp <= 127, "Displacement too large for short jmp");
  
#line 33716 "../generated/adfiles/ad_x86_64.cpp"
  }
}
uint  jmpConUCF2_shortNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 4);
}
void cmpFastLockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
    unsigned idx4 = idx3 + opnd_array(3)->num_edges();
  {

#line 3393 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Register objReg = as_Register((int)opnd_array(1)->reg(ra_,this,idx1));
    Register boxReg = as_Register((int)opnd_array(2)->reg(ra_,this,idx2));
    Register tmpReg = as_Register(opnd_array(3)->reg(ra_,this,idx3));
    Register scrReg = as_Register(opnd_array(4)->reg(ra_,this,idx4));
    MacroAssembler masm(&cbuf);

    // Verify uniqueness of register assignments -- necessary but not sufficient
    assert (objReg != boxReg && objReg != tmpReg &&
            objReg != scrReg && tmpReg != scrReg, "invariant") ;

    if (_counters != NULL) {
      masm.atomic_incl(ExternalAddress((address) _counters->total_entry_count_addr()));
    }
    if (EmitSync & 1) {
        // Without cast to int32_t a movptr will destroy r10 which is typically obj
        masm.movptr (Address(boxReg, 0), (int32_t)intptr_t(markOopDesc::unused_mark())) ;
        masm.cmpptr(rsp, (int32_t)NULL_WORD) ;
    } else
    if (EmitSync & 2) {
        Label DONE_LABEL;
        if (UseBiasedLocking) {
           // Note: tmpReg maps to the swap_reg argument and scrReg to the tmp_reg argument.
          masm.biased_locking_enter(boxReg, objReg, tmpReg, scrReg, false, DONE_LABEL, NULL, _counters);
        }
        // QQQ was movl...
        masm.movptr(tmpReg, 0x1);
        masm.orptr(tmpReg, Address(objReg, 0));
        masm.movptr(Address(boxReg, 0), tmpReg);
        if (os::is_MP()) {
          masm.lock();
        }
        masm.cmpxchgptr(boxReg, Address(objReg, 0)); // Updates tmpReg
        masm.jcc(Assembler::equal, DONE_LABEL);

        // Recursive locking
        masm.subptr(tmpReg, rsp);
        masm.andptr(tmpReg, 7 - os::vm_page_size());
        masm.movptr(Address(boxReg, 0), tmpReg);

        masm.bind(DONE_LABEL);
        masm.nop(); // avoid branch to branch
    } else {
        Label DONE_LABEL, IsInflated, Egress;

        masm.movptr(tmpReg, Address(objReg, 0)) ;
        masm.testl (tmpReg, 0x02) ;         // inflated vs stack-locked|neutral|biased
        masm.jcc   (Assembler::notZero, IsInflated) ;

        // it's stack-locked, biased or neutral
        // TODO: optimize markword triage order to reduce the number of
        // conditional branches in the most common cases.
        // Beware -- there's a subtle invariant that fetch of the markword
        // at [FETCH], below, will never observe a biased encoding (*101b).
        // If this invariant is not held we'll suffer exclusion (safety) failure.

        if (UseBiasedLocking && !UseOptoBiasInlining) {
          masm.biased_locking_enter(boxReg, objReg, tmpReg, scrReg, true, DONE_LABEL, NULL, _counters);
          masm.movptr(tmpReg, Address(objReg, 0)) ;        // [FETCH]
        }

        // was q will it destroy high?
        masm.orl   (tmpReg, 1) ;
        masm.movptr(Address(boxReg, 0), tmpReg) ;
        if (os::is_MP()) { masm.lock(); }
        masm.cmpxchgptr(boxReg, Address(objReg, 0)); // Updates tmpReg
        if (_counters != NULL) {
           masm.cond_inc32(Assembler::equal,
                           ExternalAddress((address) _counters->fast_path_entry_count_addr()));
        }
        masm.jcc   (Assembler::equal, DONE_LABEL);

        // Recursive locking
        masm.subptr(tmpReg, rsp);
        masm.andptr(tmpReg, 7 - os::vm_page_size());
        masm.movptr(Address(boxReg, 0), tmpReg);
        if (_counters != NULL) {
           masm.cond_inc32(Assembler::equal,
                           ExternalAddress((address) _counters->fast_path_entry_count_addr()));
        }
        masm.jmp   (DONE_LABEL) ;

        masm.bind  (IsInflated) ;
        // It's inflated

        // TODO: someday avoid the ST-before-CAS penalty by
        // relocating (deferring) the following ST.
        // We should also think about trying a CAS without having
        // fetched _owner.  If the CAS is successful we may
        // avoid an RTO->RTS upgrade on the $line.
        // Without cast to int32_t a movptr will destroy r10 which is typically obj
        masm.movptr(Address(boxReg, 0), (int32_t)intptr_t(markOopDesc::unused_mark())) ;

        masm.mov    (boxReg, tmpReg) ;
        masm.movptr (tmpReg, Address(tmpReg, ObjectMonitor::owner_offset_in_bytes()-2)) ;
        masm.testptr(tmpReg, tmpReg) ;
        masm.jcc    (Assembler::notZero, DONE_LABEL) ;

        // It's inflated and appears unlocked
        if (os::is_MP()) { masm.lock(); }
        masm.cmpxchgptr(r15_thread, Address(boxReg, ObjectMonitor::owner_offset_in_bytes()-2)) ;
        // Intentional fall-through into DONE_LABEL ...

        masm.bind  (DONE_LABEL) ;
        masm.nop   () ;                 // avoid jmp to jmp
    }
  
#line 33841 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmpFastUnlockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
    unsigned idx3 = idx2 + opnd_array(2)->num_edges();
  {

#line 3505 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"


    Register objReg = as_Register(opnd_array(1)->reg(ra_,this,idx1));
    Register boxReg = as_Register(opnd_array(2)->reg(ra_,this,idx2));
    Register tmpReg = as_Register(opnd_array(3)->reg(ra_,this,idx3));
    MacroAssembler masm(&cbuf);

    if (EmitSync & 4) {
       masm.cmpptr(rsp, 0) ;
    } else
    if (EmitSync & 8) {
       Label DONE_LABEL;
       if (UseBiasedLocking) {
         masm.biased_locking_exit(objReg, tmpReg, DONE_LABEL);
       }

       // Check whether the displaced header is 0
       //(=> recursive unlock)
       masm.movptr(tmpReg, Address(boxReg, 0));
       masm.testptr(tmpReg, tmpReg);
       masm.jcc(Assembler::zero, DONE_LABEL);

       // If not recursive lock, reset the header to displaced header
       if (os::is_MP()) {
         masm.lock();
       }
       masm.cmpxchgptr(tmpReg, Address(objReg, 0)); // Uses RAX which is box
       masm.bind(DONE_LABEL);
       masm.nop(); // avoid branch to branch
    } else {
       Label DONE_LABEL, Stacked, CheckSucc ;

       if (UseBiasedLocking && !UseOptoBiasInlining) {
         masm.biased_locking_exit(objReg, tmpReg, DONE_LABEL);
       }

       masm.movptr(tmpReg, Address(objReg, 0)) ;
       masm.cmpptr(Address(boxReg, 0), (int32_t)NULL_WORD) ;
       masm.jcc   (Assembler::zero, DONE_LABEL) ;
       masm.testl (tmpReg, 0x02) ;
       masm.jcc   (Assembler::zero, Stacked) ;

       // It's inflated
       masm.movptr(boxReg, Address (tmpReg, ObjectMonitor::owner_offset_in_bytes()-2)) ;
       masm.xorptr(boxReg, r15_thread) ;
       masm.orptr (boxReg, Address (tmpReg, ObjectMonitor::recursions_offset_in_bytes()-2)) ;
       masm.jcc   (Assembler::notZero, DONE_LABEL) ;
       masm.movptr(boxReg, Address (tmpReg, ObjectMonitor::cxq_offset_in_bytes()-2)) ;
       masm.orptr (boxReg, Address (tmpReg, ObjectMonitor::EntryList_offset_in_bytes()-2)) ;
       masm.jcc   (Assembler::notZero, CheckSucc) ;
       masm.movptr(Address (tmpReg, ObjectMonitor::owner_offset_in_bytes()-2), (int32_t)NULL_WORD) ;
       masm.jmp   (DONE_LABEL) ;

       if ((EmitSync & 65536) == 0) {
         Label LSuccess, LGoSlowPath ;
         masm.bind  (CheckSucc) ;
         masm.cmpptr(Address (tmpReg, ObjectMonitor::succ_offset_in_bytes()-2), (int32_t)NULL_WORD) ;
         masm.jcc   (Assembler::zero, LGoSlowPath) ;

         // I'd much rather use lock:andl m->_owner, 0 as it's faster than the
         // the explicit ST;MEMBAR combination, but masm doesn't currently support
         // "ANDQ M,IMM".  Don't use MFENCE here.  lock:add to TOS, xchg, etc
         // are all faster when the write buffer is populated.
         masm.movptr (Address (tmpReg, ObjectMonitor::owner_offset_in_bytes()-2), (int32_t)NULL_WORD) ;
         if (os::is_MP()) {
            masm.lock () ; masm.addl (Address(rsp, 0), 0) ;
         }
         masm.cmpptr(Address (tmpReg, ObjectMonitor::succ_offset_in_bytes()-2), (int32_t)NULL_WORD) ;
         masm.jcc   (Assembler::notZero, LSuccess) ;

         masm.movptr (boxReg, (int32_t)NULL_WORD) ;                   // box is really EAX
         if (os::is_MP()) { masm.lock(); }
         masm.cmpxchgptr(r15_thread, Address(tmpReg, ObjectMonitor::owner_offset_in_bytes()-2));
         masm.jcc   (Assembler::notEqual, LSuccess) ;
         // Intentional fall-through into slow-path

         masm.bind  (LGoSlowPath) ;
         masm.orl   (boxReg, 1) ;                      // set ICC.ZF=0 to indicate failure
         masm.jmp   (DONE_LABEL) ;

         masm.bind  (LSuccess) ;
         masm.testl (boxReg, 0) ;                      // set ICC.ZF=1 to indicate success
         masm.jmp   (DONE_LABEL) ;
       }

       masm.bind  (Stacked) ;
       masm.movptr(tmpReg, Address (boxReg, 0)) ;      // re-fetch
       if (os::is_MP()) { masm.lock(); }
       masm.cmpxchgptr(tmpReg, Address(objReg, 0)); // Uses RAX which is box

       if (EmitSync & 65536) {
          masm.bind (CheckSucc) ;
       }
       masm.bind(DONE_LABEL);
       if (EmitSync & 32768) {
          masm.nop();                      // avoid branch to branch
       }
    }
  
#line 33953 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void safePoint_pollNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 5;
    unsigned idx1 = 5;
  {

#line 3918 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // testl %rax, off(%rip) // Opcode + ModRM + Disp32 == 6 bytes
    // XXX reg_mem doesn't support RIP-relative addressing yet
    cbuf.set_insts_mark();
    cbuf.relocate(cbuf.insts_mark(), relocInfo::poll_type, 0); // XXX
    emit_opcode(cbuf, 0x85); // testl
    emit_rm(cbuf, 0x0, RAX_enc, 0x5); // 00 rax 101 == 0x5
    // cbuf.insts_mark() is beginning of instruction
    emit_d32_reloc(cbuf, os::get_polling_page());
//                    relocInfo::poll_type,
  
#line 33975 "../generated/adfiles/ad_x86_64.cpp"
  }
}
uint  safePoint_pollNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 6);
}
void CallStaticJavaDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2607 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // JAVA STATIC CALL
    // CALL to fixup routine.  Fixup routine uses ScopeDesc info to
    // determine who we intended to call.
    cbuf.set_insts_mark();
    emit_d8(cbuf, (0xE8 /*primary()*/));

    if (!_method) {
      emit_d32_reloc(cbuf,
                     (int) (opnd_array(1)->method() - ((intptr_t) cbuf.insts_end()) - 4),
                     runtime_call_Relocation::spec(),
                     RELOC_DISP32);
    } else if (_optimized_virtual) {
      emit_d32_reloc(cbuf,
                     (int) (opnd_array(1)->method() - ((intptr_t) cbuf.insts_end()) - 4),
                     opt_virtual_call_Relocation::spec(),
                     RELOC_DISP32);
    } else {
      emit_d32_reloc(cbuf,
                     (int) (opnd_array(1)->method() - ((intptr_t) cbuf.insts_end()) - 4),
                     static_call_Relocation::spec(),
                     RELOC_DISP32);
    }
    if (_method) {
      // Emit stub for static call
      emit_java_to_interp(cbuf);
    }
  
#line 34018 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 80 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/os_cpu/linux_x86/vm/linux_x86_64.ad"

    if (VerifyStackAtCalls) {
      // Check that stack depth is unchanged: find majik cookie on stack
      int framesize =
        ra_->reg2offset_unchecked(OptoReg::add(ra_->_matcher._old_SP, -3*VMRegImpl::slots_per_word));
      if (framesize) {
        if (framesize < 0x80) {
          emit_opcode(cbuf, Assembler::REX_W);
          emit_opcode(cbuf, 0x81); // cmpq [rsp+0],0xbadb1ood
          emit_d8(cbuf, 0x7C);
          emit_d8(cbuf, 0x24);
          emit_d8(cbuf, framesize); // Find majik cookie from ESP
          emit_d32(cbuf, 0xbadb100d);
        } else {
          emit_opcode(cbuf, Assembler::REX_W);
          emit_opcode(cbuf, 0x81); // cmpq [rsp+0],0xbadb1ood
          emit_d8(cbuf, 0xBC);
          emit_d8(cbuf, 0x24);
          emit_d32(cbuf, framesize); // Find majik cookie from ESP
          emit_d32(cbuf, 0xbadb100d);
        }
      }
      // jmp EQ around INT3
      // QQQ TODO
      const int jump_around = 5; // size of call to breakpoint, 1 for CC
      emit_opcode(cbuf, 0x74);
      emit_d8(cbuf, jump_around);
      // QQQ temporary
      emit_break(cbuf);
      // Die if stack mismatch
      // emit_opcode(cbuf,0xCC);
    }
  
#line 34056 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void CallStaticJavaHandleNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2591 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    debug_only(int off0 = cbuf.insts_size());
    MacroAssembler _masm(&cbuf);
    // RBP is preserved across all calls, even compiled calls.
    // Use it to preserve RSP in places where the callee might change the SP.
    __ movptr(rbp_mh_SP_save, rsp);
    debug_only(int off1 = cbuf.insts_size());
    assert(off1 - off0 == preserve_SP_size(), "correct size prediction");
  
#line 34076 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2607 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // JAVA STATIC CALL
    // CALL to fixup routine.  Fixup routine uses ScopeDesc info to
    // determine who we intended to call.
    cbuf.set_insts_mark();
    emit_d8(cbuf, (0xE8 /*primary()*/));

    if (!_method) {
      emit_d32_reloc(cbuf,
                     (int) (opnd_array(1)->method() - ((intptr_t) cbuf.insts_end()) - 4),
                     runtime_call_Relocation::spec(),
                     RELOC_DISP32);
    } else if (_optimized_virtual) {
      emit_d32_reloc(cbuf,
                     (int) (opnd_array(1)->method() - ((intptr_t) cbuf.insts_end()) - 4),
                     opt_virtual_call_Relocation::spec(),
                     RELOC_DISP32);
    } else {
      emit_d32_reloc(cbuf,
                     (int) (opnd_array(1)->method() - ((intptr_t) cbuf.insts_end()) - 4),
                     static_call_Relocation::spec(),
                     RELOC_DISP32);
    }
    if (_method) {
      // Emit stub for static call
      emit_java_to_interp(cbuf);
    }
  
#line 34109 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2601 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);
    __ movptr(rsp, rbp_mh_SP_save);
  
#line 34118 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 80 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/os_cpu/linux_x86/vm/linux_x86_64.ad"

    if (VerifyStackAtCalls) {
      // Check that stack depth is unchanged: find majik cookie on stack
      int framesize =
        ra_->reg2offset_unchecked(OptoReg::add(ra_->_matcher._old_SP, -3*VMRegImpl::slots_per_word));
      if (framesize) {
        if (framesize < 0x80) {
          emit_opcode(cbuf, Assembler::REX_W);
          emit_opcode(cbuf, 0x81); // cmpq [rsp+0],0xbadb1ood
          emit_d8(cbuf, 0x7C);
          emit_d8(cbuf, 0x24);
          emit_d8(cbuf, framesize); // Find majik cookie from ESP
          emit_d32(cbuf, 0xbadb100d);
        } else {
          emit_opcode(cbuf, Assembler::REX_W);
          emit_opcode(cbuf, 0x81); // cmpq [rsp+0],0xbadb1ood
          emit_d8(cbuf, 0xBC);
          emit_d8(cbuf, 0x24);
          emit_d32(cbuf, framesize); // Find majik cookie from ESP
          emit_d32(cbuf, 0xbadb100d);
        }
      }
      // jmp EQ around INT3
      // QQQ TODO
      const int jump_around = 5; // size of call to breakpoint, 1 for CC
      emit_opcode(cbuf, 0x74);
      emit_d8(cbuf, jump_around);
      // QQQ temporary
      emit_break(cbuf);
      // Die if stack mismatch
      // emit_opcode(cbuf,0xCC);
    }
  
#line 34156 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void CallDynamicJavaDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 2637 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // JAVA DYNAMIC CALL
    // !!!!!
    // Generate  "movq rax, -1", placeholder instruction to load oop-info
    // emit_call_dynamic_prologue( cbuf );
    cbuf.set_insts_mark();

    // movq rax, -1
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0xB8 | RAX_enc);
    emit_d64_reloc(cbuf,
                   (int64_t) Universe::non_oop_word(),
                   oop_Relocation::spec_for_immediate(), RELOC_IMM64);
    address virtual_call_oop_addr = cbuf.insts_mark();
    // CALL to fixup routine.  Fixup routine uses ScopeDesc info to determine
    // who we intended to call.
    cbuf.set_insts_mark();
    emit_d8(cbuf, (0xE8 /*primary()*/));
    emit_d32_reloc(cbuf,
                   (int) (opnd_array(1)->method() - ((intptr_t) cbuf.insts_end()) - 4),
                   virtual_call_Relocation::spec(virtual_call_oop_addr),
                   RELOC_DISP32);
  
#line 34190 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 80 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/os_cpu/linux_x86/vm/linux_x86_64.ad"

    if (VerifyStackAtCalls) {
      // Check that stack depth is unchanged: find majik cookie on stack
      int framesize =
        ra_->reg2offset_unchecked(OptoReg::add(ra_->_matcher._old_SP, -3*VMRegImpl::slots_per_word));
      if (framesize) {
        if (framesize < 0x80) {
          emit_opcode(cbuf, Assembler::REX_W);
          emit_opcode(cbuf, 0x81); // cmpq [rsp+0],0xbadb1ood
          emit_d8(cbuf, 0x7C);
          emit_d8(cbuf, 0x24);
          emit_d8(cbuf, framesize); // Find majik cookie from ESP
          emit_d32(cbuf, 0xbadb100d);
        } else {
          emit_opcode(cbuf, Assembler::REX_W);
          emit_opcode(cbuf, 0x81); // cmpq [rsp+0],0xbadb1ood
          emit_d8(cbuf, 0xBC);
          emit_d8(cbuf, 0x24);
          emit_d32(cbuf, framesize); // Find majik cookie from ESP
          emit_d32(cbuf, 0xbadb100d);
        }
      }
      // jmp EQ around INT3
      // QQQ TODO
      const int jump_around = 5; // size of call to breakpoint, 1 for CC
      emit_opcode(cbuf, 0x74);
      emit_d8(cbuf, jump_around);
      // QQQ temporary
      emit_break(cbuf);
      // Die if stack mismatch
      // emit_opcode(cbuf,0xCC);
    }
  
#line 34228 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void CallRuntimeDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 59 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/os_cpu/linux_x86/vm/linux_x86_64.ad"

    // No relocation needed

    // movq r10, <meth>
    emit_opcode(cbuf, Assembler::REX_WB);
    emit_opcode(cbuf, 0xB8 | (R10_enc - 8));
    emit_d64(cbuf, (int64_t) opnd_array(1)->method());

    // call (r10)
    emit_opcode(cbuf, Assembler::REX_B);
    emit_opcode(cbuf, 0xFF);
    emit_opcode(cbuf, 0xD0 | (R10_enc - 8));
  
#line 34252 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void CallLeafDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 59 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/os_cpu/linux_x86/vm/linux_x86_64.ad"

    // No relocation needed

    // movq r10, <meth>
    emit_opcode(cbuf, Assembler::REX_WB);
    emit_opcode(cbuf, 0xB8 | (R10_enc - 8));
    emit_d64(cbuf, (int64_t) opnd_array(1)->method());

    // call (r10)
    emit_opcode(cbuf, Assembler::REX_B);
    emit_opcode(cbuf, 0xFF);
    emit_opcode(cbuf, 0xD0 | (R10_enc - 8));
  
#line 34276 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void CallLeafNoFPDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  {

#line 59 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/os_cpu/linux_x86/vm/linux_x86_64.ad"

    // No relocation needed

    // movq r10, <meth>
    emit_opcode(cbuf, Assembler::REX_WB);
    emit_opcode(cbuf, 0xB8 | (R10_enc - 8));
    emit_d64(cbuf, (int64_t) opnd_array(1)->method());

    // call (r10)
    emit_opcode(cbuf, Assembler::REX_B);
    emit_opcode(cbuf, 0xFF);
    emit_opcode(cbuf, 0xD0 | (R10_enc - 8));
  
#line 34300 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void RetNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 5;
    unsigned idx1 = 5;
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xC3 /*primary()*/));
  
#line 34314 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void TailCalljmpIndNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 5;
    unsigned idx1 = 5;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2987 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 34331 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xFF /*primary()*/));
  
#line 34339 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 34348 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void tailjmpIndNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 5;
    unsigned idx1 = 5;
    unsigned idx2 = idx1 + opnd_array(1)->num_edges();
  {

#line 2164 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x5a));
  
#line 34363 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2987 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1) >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 34373 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2146 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xFF /*primary()*/));
  
#line 34381 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2479 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1) & 7);
  
#line 34390 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void CreateExceptionNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
uint  CreateExceptionNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void RethrowExceptionNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 5;
    unsigned idx1 = 5;
  {

#line 3607 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    cbuf.set_insts_mark();
    emit_opcode(cbuf, 0xE9); // jmp entry
    emit_d32_reloc(cbuf,
                   (int) (OptoRuntime::rethrow_stub() - cbuf.insts_end() - 4),
                   runtime_call_Relocation::spec(),
                   RELOC_DISP32);
  
#line 34420 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void tlsLoadPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 1;
    unsigned idx1 = 1;
  // User did not define which encode class to use.
}
uint  tlsLoadPNode::size(PhaseRegAlloc *ra_) const {
 assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
 return (VerifyOops ? MachNode::size(ra_) : 0);
}
void ShouldNotReachHereNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
    // Start at oper_input_base() and count operands
    unsigned idx0 = 5;
    unsigned idx1 = 5;
  {

#line 74 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/os_cpu/linux_x86/vm/linux_x86_64.ad"

    MacroAssembler* masm = new MacroAssembler(&cbuf);
    masm->call(RuntimeAddress(CAST_FROM_FN_PTR(address, os::breakpoint)));
  
#line 34446 "../generated/adfiles/ad_x86_64.cpp"
  }
}
const MachOper* loadBNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadB2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUBNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUB2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUB2L_immI8Node::memory_operand() const { return _opnds[1]; }
const MachOper* loadSNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadS2BNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadS2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUSNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUS2BNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUS2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUS2L_immI_255Node::memory_operand() const { return _opnds[1]; }
const MachOper* loadUS2L_immI16Node::memory_operand() const { return _opnds[1]; }
const MachOper* loadINode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2BNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2UBNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2SNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2USNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2L_immI_255Node::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2L_immI_65535Node::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2L_immINode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUI2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadLNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadRangeNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadPNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadNNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadKlassNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadNKlassNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadFNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadD_partialNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadDNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadA8BNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadA4SNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadA4CNode::memory_operand() const { return _opnds[1]; }
const MachOper* load2IUNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadA2FNode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchrNode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchrNTANode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchrT0Node::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchrT2Node::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchwNode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchwNTANode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchwT0Node::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchwT2Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeBNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeCNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeINode::memory_operand() const { return _opnds[1]; }
const MachOper* storeLNode::memory_operand() const { return _opnds[1]; }
const MachOper* storePNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmP0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmPNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeNNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmN0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmNNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmI0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmINode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmL0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmLNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmC0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmI16Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmB0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmBNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeA8BNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeA4CNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeA2INode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmCM0_regNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmCM0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeA2FNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeFNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeF0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeF_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeDNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeD0_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeD0Node::memory_operand() const { return _opnds[1]; }
const MachOper* popCountI_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* popCountL_memNode::memory_operand() const { return _opnds[1]; }
const TypePtr *membar_acquireNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* membar_acquireNode::memory_operand() const { return (MachOper*)-1; }
const TypePtr *membar_acquire_lockNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* membar_acquire_lockNode::memory_operand() const { return (MachOper*)-1; }
const TypePtr *membar_releaseNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* membar_releaseNode::memory_operand() const { return (MachOper*)-1; }
const TypePtr *membar_release_lockNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* membar_release_lockNode::memory_operand() const { return (MachOper*)-1; }
const MachOper* cmovI_memNode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovI_memUNode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovI_memUCFNode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovL_memNode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovL_memUNode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovL_memUCFNode::memory_operand() const { return _opnds[4]; }
const MachOper* addI_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addI_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addI_mem_rRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* addI_mem_rReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* incI_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* decI_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* addL_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addL_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addL_mem_rRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* addL_mem_rReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addL_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* incL_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* decL_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadPLockedNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadLLockedNode::memory_operand() const { return _opnds[1]; }
const MachOper* storePConditionalNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeIConditionalNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeLConditionalNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapPNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapLNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapINode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapNNode::memory_operand() const { return _opnds[1]; }
const MachOper* subI_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* subI_mem_rRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* subI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* subL_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* subL_mem_rRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* subL_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* negI_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* negL_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* mulI_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulI_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* mulL_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulL_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulL_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* salI_mem_1Node::memory_operand() const { return _opnds[1]; }
const MachOper* salI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* salI_mem_CLNode::memory_operand() const { return _opnds[1]; }
const MachOper* sarI_mem_1Node::memory_operand() const { return _opnds[1]; }
const MachOper* sarI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* sarI_mem_CLNode::memory_operand() const { return _opnds[1]; }
const MachOper* shrI_mem_1Node::memory_operand() const { return _opnds[1]; }
const MachOper* shrI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* shrI_mem_CLNode::memory_operand() const { return _opnds[1]; }
const MachOper* salL_mem_1Node::memory_operand() const { return _opnds[1]; }
const MachOper* salL_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* salL_mem_CLNode::memory_operand() const { return _opnds[1]; }
const MachOper* sarL_mem_1Node::memory_operand() const { return _opnds[1]; }
const MachOper* sarL_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* sarL_mem_CLNode::memory_operand() const { return _opnds[1]; }
const MachOper* shrL_mem_1Node::memory_operand() const { return _opnds[1]; }
const MachOper* shrL_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* shrL_mem_CLNode::memory_operand() const { return _opnds[1]; }
const MachOper* andI_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* andI_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* andI_mem_rRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* andI_mem_rReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* andI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* orI_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* orI_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* orI_mem_rRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* orI_mem_rReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* orI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* xorI_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* xorI_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* xorI_mem_rRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* xorI_mem_rReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* xorI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* andL_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* andL_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* andL_mem_rRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* andL_mem_rReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* andL_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* orL_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* orL_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* orL_mem_rRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* orL_mem_rReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* orL_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* xorL_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* xorL_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* xorL_mem_rRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* xorL_mem_rReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* xorL_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* cmpF_cc_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* cmpF_cc_memCFNode::memory_operand() const { return _opnds[2]; }
const MachOper* cmpD_cc_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* cmpD_cc_memCFNode::memory_operand() const { return _opnds[2]; }
const MachOper* cmpF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* cmpD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addF_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addD_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* subF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* subD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulF_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulD_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* divF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* divD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* sqrtF_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* sqrtD_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* convF2D_reg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* convD2F_reg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* convI2F_reg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* convI2D_reg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* convL2F_reg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* convL2D_reg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* convI2L_reg_mem_zexNode::memory_operand() const { return _opnds[1]; }
const MachOper* compI_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* testI_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* testI_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* compU_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* compP_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* compP_mem_rRegNode::memory_operand() const { return _opnds[2]; }
const MachOper* testP_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* testP_mem_reg0Node::memory_operand() const { return _opnds[1]; }
const MachOper* compN_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* compN_mem_immNode::memory_operand() const { return _opnds[2]; }
const MachOper* testN_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* testN_mem_reg0Node::memory_operand() const { return _opnds[1]; }
const MachOper* compL_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* testL_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* testL_reg_mem_0Node::memory_operand() const { return _opnds[1]; }


const bool Matcher::has_match_rule(int opcode) {
  assert(_last_machine_leaf < opcode && opcode < _last_opcode, "opcode in range");
  return _hasMatchRule[opcode];
}

const bool Matcher::_hasMatchRule[_last_opcode] = {
    false,  // Node
    false,  // Set
    false,  // RegN
    false,  // RegI
    false,  // RegP
    false,  // RegF
    false,  // RegD
    false,  // RegL
    false,  // RegFlags
    false,  // _last_machine_leaf
    true ,  // AbsD
    true ,  // AbsF
    false,  // AbsI
    true ,  // AddD
    true ,  // AddF
    true ,  // AddI
    true ,  // AddL
    true ,  // AddP
    false,  // Allocate
    false,  // AllocateArray
    true ,  // AndI
    true ,  // AndL
    true ,  // AryEq
    false,  // AtanD
    true ,  // Binary
    true ,  // Bool
    false,  // BoxLock
    true ,  // ReverseBytesI
    true ,  // ReverseBytesL
    true ,  // ReverseBytesUS
    true ,  // ReverseBytesS
    false,  // CProj
    true ,  // CallDynamicJava
    false,  // CallJava
    true ,  // CallLeaf
    true ,  // CallLeafNoFP
    true ,  // CallRuntime
    true ,  // CallStaticJava
    true ,  // CastII
    true ,  // CastX2P
    true ,  // CastP2X
    true ,  // CastPP
    false,  // Catch
    false,  // CatchProj
    true ,  // CheckCastPP
    true ,  // ClearArray
    false,  // ConstraintCast
    true ,  // CMoveD
    true ,  // CMoveF
    true ,  // CMoveI
    true ,  // CMoveL
    true ,  // CMoveP
    true ,  // CMoveN
    true ,  // CmpN
    true ,  // CmpD
    true ,  // CmpD3
    true ,  // CmpF
    true ,  // CmpF3
    true ,  // CmpI
    true ,  // CmpL
    true ,  // CmpL3
    true ,  // CmpLTMask
    true ,  // CmpP
    true ,  // CmpU
    true ,  // CompareAndSwapI
    true ,  // CompareAndSwapL
    true ,  // CompareAndSwapP
    true ,  // CompareAndSwapN
    false,  // Con
    true ,  // ConN
    true ,  // ConD
    true ,  // ConF
    true ,  // ConI
    true ,  // ConL
    true ,  // ConP
    true ,  // Conv2B
    true ,  // ConvD2F
    true ,  // ConvD2I
    true ,  // ConvD2L
    true ,  // ConvF2D
    true ,  // ConvF2I
    true ,  // ConvF2L
    true ,  // ConvI2D
    true ,  // ConvI2F
    true ,  // ConvI2L
    true ,  // ConvL2D
    true ,  // ConvL2F
    true ,  // ConvL2I
    true ,  // CosD
    false,  // CountedLoop
    true ,  // CountedLoopEnd
    true ,  // CountLeadingZerosI
    true ,  // CountLeadingZerosL
    true ,  // CountTrailingZerosI
    true ,  // CountTrailingZerosL
    true ,  // CreateEx
    true ,  // DecodeN
    true ,  // DivD
    true ,  // DivF
    true ,  // DivI
    true ,  // DivL
    false,  // DivMod
    true ,  // DivModI
    true ,  // DivModL
    true ,  // EncodeP
    false,  // ExpD
    true ,  // FastLock
    true ,  // FastUnlock
    true ,  // Goto
    true ,  // Halt
    true ,  // If
    false,  // IfFalse
    false,  // IfTrue
    false,  // Initialize
    false,  // JProj
    true ,  // Jump
    false,  // JumpProj
    true ,  // LShiftI
    true ,  // LShiftL
    true ,  // LoadB
    true ,  // LoadUB
    true ,  // LoadUS
    true ,  // LoadD
    false,  // LoadD_unaligned
    true ,  // LoadF
    true ,  // LoadI
    true ,  // LoadUI2L
    true ,  // LoadKlass
    true ,  // LoadNKlass
    true ,  // LoadL
    false,  // LoadL_unaligned
    true ,  // LoadPLocked
    true ,  // LoadLLocked
    true ,  // LoadP
    true ,  // LoadN
    true ,  // LoadRange
    true ,  // LoadS
    false,  // Lock
    true ,  // LogD
    true ,  // Log10D
    false,  // Loop
    false,  // Mach
    false,  // MachProj
    true ,  // MaxI
    true ,  // MemBarAcquire
    false,  // MemBarCPUOrder
    true ,  // MemBarRelease
    true ,  // MemBarVolatile
    false,  // MergeMem
    true ,  // MinI
    false,  // ModD
    false,  // ModF
    true ,  // ModI
    true ,  // ModL
    true ,  // MoveI2F
    true ,  // MoveF2I
    true ,  // MoveL2D
    true ,  // MoveD2L
    true ,  // MulD
    true ,  // MulF
    true ,  // MulHiL
    true ,  // MulI
    true ,  // MulL
    false,  // Multi
    true ,  // NegD
    true ,  // NegF
    false,  // NeverBranch
    false,  // Opaque1
    false,  // Opaque2
    true ,  // OrI
    true ,  // OrL
    false,  // PCTable
    false,  // Parm
    true ,  // PartialSubtypeCheck
    false,  // Phi
    true ,  // PopCountI
    true ,  // PopCountL
    false,  // PowD
    true ,  // PrefetchRead
    true ,  // PrefetchWrite
    false,  // Proj
    true ,  // RShiftI
    true ,  // RShiftL
    false,  // Region
    true ,  // Rethrow
    true ,  // Return
    false,  // Root
    true ,  // RoundDouble
    true ,  // RoundFloat
    true ,  // SafePoint
    false,  // SafePointScalarObject
    false,  // SCMemProj
    true ,  // SinD
    true ,  // SqrtD
    false,  // Start
    false,  // StartOSR
    true ,  // StoreB
    true ,  // StoreC
    true ,  // StoreCM
    true ,  // StorePConditional
    true ,  // StoreIConditional
    true ,  // StoreLConditional
    true ,  // StoreD
    true ,  // StoreF
    true ,  // StoreI
    true ,  // StoreL
    true ,  // StoreP
    true ,  // StoreN
    true ,  // StrComp
    true ,  // StrEquals
    true ,  // StrIndexOf
    true ,  // SubD
    true ,  // SubF
    true ,  // SubI
    true ,  // SubL
    true ,  // TailCall
    true ,  // TailJump
    true ,  // TanD
    true ,  // ThreadLocal
    false,  // Unlock
    true ,  // URShiftI
    true ,  // URShiftL
    true ,  // XorI
    true ,  // XorL
    false,  // Vector
    false,  // AddVB
    false,  // AddVC
    false,  // AddVS
    false,  // AddVI
    false,  // AddVL
    false,  // AddVF
    false,  // AddVD
    false,  // SubVB
    false,  // SubVC
    false,  // SubVS
    false,  // SubVI
    false,  // SubVL
    false,  // SubVF
    false,  // SubVD
    false,  // MulVF
    false,  // MulVD
    false,  // DivVF
    false,  // DivVD
    false,  // LShiftVB
    false,  // LShiftVC
    false,  // LShiftVS
    false,  // LShiftVI
    false,  // URShiftVB
    false,  // URShiftVC
    false,  // URShiftVS
    false,  // URShiftVI
    false,  // AndV
    false,  // OrV
    false,  // XorV
    false,  // VectorLoad
    false,  // Load16B
    true ,  // Load8B
    false,  // Load4B
    false,  // Load8C
    true ,  // Load4C
    false,  // Load2C
    false,  // Load8S
    true ,  // Load4S
    false,  // Load2S
    false,  // Load4I
    true ,  // Load2I
    false,  // Load2L
    false,  // Load4F
    true ,  // Load2F
    false,  // Load2D
    false,  // VectorStore
    false,  // Store16B
    true ,  // Store8B
    false,  // Store4B
    false,  // Store8C
    true ,  // Store4C
    false,  // Store2C
    false,  // Store4I
    true ,  // Store2I
    false,  // Store2L
    false,  // Store4F
    true ,  // Store2F
    false,  // Store2D
    false,  // Pack
    false,  // PackB
    false,  // PackS
    false,  // PackC
    false,  // PackI
    false,  // PackL
    false,  // PackF
    false,  // PackD
    false,  // Pack2x1B
    false,  // Pack2x2B
    false,  // Replicate16B
    true ,  // Replicate8B
    false,  // Replicate4B
    false,  // Replicate8S
    true ,  // Replicate4S
    false,  // Replicate2S
    false,  // Replicate8C
    true ,  // Replicate4C
    false,  // Replicate2C
    false,  // Replicate4I
    true ,  // Replicate2I
    false,  // Replicate2L
    false,  // Replicate4F
    true ,  // Replicate2F
    false,  // Replicate2D
    false,  // Extract
    false,  // ExtractB
    false,  // ExtractS
    false,  // ExtractC
    false,  // ExtractI
    false,  // ExtractL
    false,  // ExtractF
    false   // ExtractD
};


bool Matcher::stack_direction() const { return false; }

int Compile::sync_stack_slots() const { return 2; }

uint Matcher::stack_alignment_in_bytes() { return StackAlignmentInBytes; }

OptoReg::Name Matcher::return_addr() const { return OptoReg::stack2reg(- 2 +
              round_to(2 + 2 * VerifyStackAtCalls +
                       Compile::current()->fixed_slots(),
                       WordsPerLong * 2)); }

uint Compile::in_preserve_stack_slots() { return 4 + 2 * VerifyStackAtCalls; }

uint Compile::out_preserve_stack_slots() { return SharedRuntime::out_preserve_stack_slots(); }

uint Compile::varargs_C_out_slots_killed() const { return frame::arg_reg_save_area_bytes/BytesPerInt; }

void Matcher::calling_convention(BasicType *sig_bt, VMRegPair *regs, uint length, bool is_outgoing) {

#line 4048 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // No difference between ingoing/outgoing just pass false
    SharedRuntime::java_calling_convention(sig_bt, regs, length, false);
  
#line 35018 "../generated/adfiles/ad_x86_64.cpp"

}

void Matcher::c_calling_convention(BasicType *sig_bt, VMRegPair *regs, uint length) {

#line 4054 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // This is obviously always outgoing
    (void) SharedRuntime::c_calling_convention(sig_bt, regs, length);
  
#line 35029 "../generated/adfiles/ad_x86_64.cpp"

}

OptoRegPair Matcher::return_value(int ideal_reg, bool is_outgoing) {

#line 4061 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    assert(ideal_reg >= Op_RegI && ideal_reg <= Op_RegL,
           "only return normal values");

    static const int lo[Op_RegL + 1] = {
      0,
      0,
      RAX_num,  // Op_RegN
      RAX_num,  // Op_RegI
      RAX_num,  // Op_RegP
      XMM0_num, // Op_RegF
      XMM0_num, // Op_RegD
      RAX_num   // Op_RegL
    };
    static const int hi[Op_RegL + 1] = {
      0,
      0,
      OptoReg::Bad, // Op_RegN
      OptoReg::Bad, // Op_RegI
      RAX_H_num,    // Op_RegP
      OptoReg::Bad, // Op_RegF
      XMM0_H_num,   // Op_RegD
      RAX_H_num     // Op_RegL
    };
    assert(ARRAY_SIZE(hi) == _last_machine_leaf - 1, "missing type");
    return OptoRegPair(hi[ideal_reg], lo[ideal_reg]);
  
#line 35063 "../generated/adfiles/ad_x86_64.cpp"

}

OptoRegPair Matcher::c_return_value(int ideal_reg, bool is_outgoing) {

#line 4061 "/builddir/build/BUILD/icedtea6-1.11.11/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    assert(ideal_reg >= Op_RegI && ideal_reg <= Op_RegL,
           "only return normal values");

    static const int lo[Op_RegL + 1] = {
      0,
      0,
      RAX_num,  // Op_RegN
      RAX_num,  // Op_RegI
      RAX_num,  // Op_RegP
      XMM0_num, // Op_RegF
      XMM0_num, // Op_RegD
      RAX_num   // Op_RegL
    };
    static const int hi[Op_RegL + 1] = {
      0,
      0,
      OptoReg::Bad, // Op_RegN
      OptoReg::Bad, // Op_RegI
      RAX_H_num,    // Op_RegP
      OptoReg::Bad, // Op_RegF
      XMM0_H_num,   // Op_RegD
      RAX_H_num     // Op_RegL
    };
    assert(ARRAY_SIZE(hi) == _last_machine_leaf - 1, "missing type");
    return OptoRegPair(hi[ideal_reg], lo[ideal_reg]);
  
#line 35097 "../generated/adfiles/ad_x86_64.cpp"

}

OptoReg::Name Matcher::inline_cache_reg() { return OptoReg::Name(RAX_num); }

const RegMask &Matcher::inline_cache_reg_mask() { return INLINE_CACHE_REG_mask; }

int Matcher::inline_cache_reg_encode() { return _regEncode[inline_cache_reg()]; }

OptoReg::Name Matcher::interpreter_method_oop_reg() { return OptoReg::Name(RBX_num); }

const RegMask &Matcher::interpreter_method_oop_reg_mask() { return INTERPRETER_METHOD_OOP_REG_mask; }

int Matcher::interpreter_method_oop_reg_encode() { return _regEncode[interpreter_method_oop_reg()]; }

OptoReg::Name Matcher::interpreter_frame_pointer_reg() { return OptoReg::Name(RBP_num); }

const RegMask &Matcher::interpreter_frame_pointer_reg_mask() { return INTERPRETER_FRAME_POINTER_REG_mask; }

OptoReg::Name Matcher::c_frame_pointer() const { return OptoReg::Name(RSP_num); }

// Number of callee-save + always-save registers
int  Matcher::number_of_saved_registers() {
  return 0;
};

// Check consistency of C++ compilation with ADLC options:
// Check adlc -DLINUX=1
#ifndef LINUX
#  error "LINUX must be defined"
#endif // LINUX
// Check adlc -D_GNU_SOURCE=1
#ifndef _GNU_SOURCE
#  error "_GNU_SOURCE must be defined"
#endif // _GNU_SOURCE
// Check adlc -DAMD64=1
#ifndef AMD64
#  error "AMD64 must be defined"
#endif // AMD64
// Check adlc -D_LP64=1
#ifndef _LP64
#  error "_LP64 must be defined"
#endif // _LP64
